Package: artMS
Type: Package
Title: Analytical R tools for Mass Spectrometry
Version: 1.28.0
Date: 2021-07-13
Authors@R: c(person("David", "Jimenez-Morales", role = c("aut","cre"),
                     email = "biodavidjm@gmail.com",
                     comment = c(ORCID = "0000-0003-4356-6461")),
             person("Alexandre", "Rosa Campos", role = c("aut","ctb"),
                     comment = c(ORCID = "0000-0003-3988-7764")),
             person("John", "Von Dollen", role = c("aut")),
             person("Nevan", "Krogan", role = c("aut"),
             comment = c(ORCID = "0000-0003-4902-337X")),
             person("Danielle", "Swaney", role = c("aut", "ctb"),
                     comment = c(ORCID = "0000-0001-6119-6084")))
Description: artMS provides a set of tools for the analysis of proteomics 
    label-free datasets. It takes as input the MaxQuant search result 
    output (evidence.txt file) and performs quality control, 
    relative quantification using MSstats, downstream analysis 
    and integration. artMS also provides a set of functions to re-format 
    and make it compatible with other analytical tools, including, SAINTq, 
    SAINTexpress, Phosfate, and PHOTON. Check 
    [http://artms.org](http://artms.org) for details.
License: GPL (>= 3) + file LICENSE
URL: http://artms.org
BugReports: https://github.com/biodavidjm/artMS/issues
Depends: R (>= 4.1.0)
Imports: AnnotationDbi, bit64, circlize, cluster, corrplot, data.table,
        dplyr, getopt, ggdendro, ggplot2, gplots, ggrepel, graphics,
        grDevices, grid, limma, MSstats, openxlsx, org.Hs.eg.db,
        pheatmap, plotly, plyr, RColorBrewer, scales, seqinr, stats,
        stringr, tidyr, UpSetR, utils, VennDiagram, yaml
Suggests: BiocStyle, ComplexHeatmap, factoextra, FactoMineR, gProfileR,
        knitr, PerformanceAnalytics, org.Mm.eg.db, rmarkdown, testthat
Encoding: UTF-8
LazyLoad: yes
LazyData: true
biocViews: Proteomics, DifferentialExpression, BiomedicalInformatics,
        SystemsBiology, MassSpectrometry, Annotation, QualityControl,
        GeneSetEnrichment, Clustering, Normalization, ImmunoOncology,
        MultipleComparison
NeedsCompilation: no
VignetteBuilder: knitr
RoxygenNote: 7.1.1
Roxygen: list(markdown = TRUE)
git_url: https://git.bioconductor.org/packages/artMS
git_branch: RELEASE_3_22
git_last_commit: 9313c4c
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
Packaged: 2025-10-30 02:33:15 UTC; biocbuild
Author: David Jimenez-Morales [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-4356-6461>),
  Alexandre Rosa Campos [aut, ctb] (ORCID:
    <https://orcid.org/0000-0003-3988-7764>),
  John Von Dollen [aut],
  Nevan Krogan [aut] (ORCID: <https://orcid.org/0000-0003-4902-337X>),
  Danielle Swaney [aut, ctb] (ORCID:
    <https://orcid.org/0000-0001-6119-6084>)
Maintainer: David Jimenez-Morales <biodavidjm@gmail.com>

# Generated by roxygen2: do not edit by hand

export(artmsAnalysisQuantifications)
export(artmsAnnotateSpecie)
export(artmsAnnotationUniprot)
export(artmsAvgIntensityRT)
export(artmsChangeColumnName)
export(artmsConvertMetabolomics)
export(artmsDataPlots)
export(artmsEnrichLog2fc)
export(artmsEnrichProfiler)
export(artmsEvidenceToSAINTq)
export(artmsEvidenceToSaintExpress)
export(artmsFilterEvidenceContaminants)
export(artmsGeneratePhSiteExtended)
export(artmsIsEvidenceNewVersion)
export(artmsIsSpeciesSupported)
export(artmsLeaveOnlyUniprotEntryID)
export(artmsMapUniprot2Entrez)
export(artmsMergeEvidenceAndKeys)
export(artmsMsstatsSummary)
export(artmsPhosfateOutput)
export(artmsPhotonOutput)
export(artmsPlotHeatmapQuant)
export(artmsProtein2SiteConversion)
export(artmsQualityControlEvidenceBasic)
export(artmsQualityControlEvidenceExtended)
export(artmsQualityControlMetabolomics)
export(artmsQualityControlSummaryExtended)
export(artmsQuantification)
export(artmsResultsWide)
export(artmsSILACtoLong)
export(artmsSpectralCounts)
export(artmsVolcanoPlot)
export(artmsWriteConfigYamlFile)
import(AnnotationDbi, except = c(head))
import(MSstats)
import(RColorBrewer)
import(UpSetR)
import(VennDiagram)
import(bit64)
import(circlize)
import(data.table)
import(getopt)
import(ggdendro)
import(ggplot2)
import(ggrepel)
import(grid)
import(limma)
import(openxlsx)
import(org.Hs.eg.db)
import(pheatmap)
import(plotly, except = c(last_plot, mutate, arrange, 
rename, summarise, select, add_heatmap))
import(seqinr, except = c(zscore, count, a))
import(stringr)
import(yaml)
importFrom(cluster,pam)
importFrom(corrplot,corrplot)
importFrom(dplyr,arrange)
importFrom(dplyr,count)
importFrom(dplyr,desc)
importFrom(dplyr,everything)
importFrom(dplyr,mutate)
importFrom(dplyr,one_of)
importFrom(gplots,heatmap.2)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,dev.off)
importFrom(grDevices,pdf)
importFrom(graphics,barplot)
importFrom(graphics,hist)
importFrom(graphics,lines)
importFrom(graphics,pairs)
importFrom(graphics,panel.smooth)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,rect)
importFrom(graphics,strwidth)
importFrom(graphics,text)
importFrom(plyr,ddply)
importFrom(plyr,rename)
importFrom(plyr,summarise)
importFrom(scales,percent)
importFrom(stats,IQR)
importFrom(stats,aggregate)
importFrom(stats,as.dendrogram)
importFrom(stats,as.dist)
importFrom(stats,complete.cases)
importFrom(stats,cor)
importFrom(stats,dist)
importFrom(stats,fisher.test)
importFrom(stats,hclust)
importFrom(stats,kmeans)
importFrom(stats,median)
importFrom(stats,order.dendrogram)
importFrom(stats,phyper)
importFrom(stats,prcomp)
importFrom(stats,qt)
importFrom(stats,quantile)
importFrom(stats,sd)
importFrom(tidyr,pivot_longer)
importFrom(tidyr,pivot_wider)
importFrom(tidyr,unnest)
importFrom(utils,capture.output)
importFrom(utils,combn)
importFrom(utils,globalVariables)
importFrom(utils,head)
importFrom(utils,read.delim)
importFrom(utils,sessionInfo)
importFrom(utils,setTxtProgressBar)
importFrom(utils,txtProgressBar)
importFrom(utils,write.table)

#' @title Analysis of the Relative Quantifications
#'
#' @description Analysis of relative quantifications, including:
#'
#' - Annotations
#' - Summary files in different format (xls, txt) and shapes (long, wide)
#' - Numerous summary plots
#' - Enrichment analysis using Gprofiler
#' - PCA of quantifications
#' - Clustering analysis
#' - Basic imputation of missing values
#' 
#' To run this function, the following packages must be installed on your system:
#' - From bioconductor:
#' `BiocManager::install(c("ComplexHeatmap", "org.Mm.eg.db"))`
#' - From CRAN:
#' `install.packages(c("factoextra", "FactoMineR", "gProfileR", "PerformanceAnalytics"))`
#' 
#' @param log2fc_file (char) MSstats results file location
#' @param modelqc_file (char) MSstats modelqc file location
#' @param species (char) Select one species. Species currently supported for
#' a full analysis (including enrichment analysis):
#' - HUMAN
#' - MOUSE
#' To find out species supported only for annotation check 
#' `?artmsIsSpeciesSupported()`
#' 
#' @param output_dir (char) Name for the folder to output the results from the 
#' function. Default is current directory (recommended to provide a new folder
#' name).
#' @param outliers (char) It allows to keep or remove outliers. Options:
#' - `keep` (default): it keeps outliers 'keep', 'iqr', 'std'
#' - `iqr` (recommended): remove outliers +/- 6 x Interquartile Range (IQR)
#' - `std` : 6 x standard deviation
#' @param enrich (logical) Performed enrichment analysis using GprofileR?
#' Only available for species HUMAN and MOUSE. 
#' `TRUE` (default if "human" or "mouse" are the species) or `FALSE`
#' @param l2fc_thres (int) log2fc cutoff for enrichment analysis (default,
#' `l2fc_thres = 1.5`)
#' @param choosePvalue (char) specify whether `pvalue` or `adjpvalue` should 
#' use for the analysis. The default option is `adjpvalue`
#' (multiple testing correction).
#' But if the number of biological replicates for a given experiment is
#' too low (for example n = 2), then `choosePvalue = pvalue` is recommended.
#' @param isBackground (char) background of gene names for enrichment analysis.
#' `nobackground` (default) will use the total number of genes detected.
#' Alternatively provided the file path name to the background gene list.
#' @param isPtm (char) Is a ptm-site quantification? 
#' - `global` (default), 
#' - `ptmsites` (for site specific analysis), 
#' - `ptmph` (Jeff Johnson script output evidence file)
#' @param mnbr (int) **PARAMETER FOR NAIVE IMPUTATION**: 
#' "minimal number of biological replicates" for "naive 
#' imputation" and filtering. Default: `mnbr = 2`. Details:
#' Intensity values for proteins/PTMs that are completely missed in one 
#' of the two conditions compared ("condition A"), but are found in 
#' at least 2 biological replicates (`mnbr = 2`) of the other "condition B", 
#' are imputed (values artificially assigned) and the log2FC values calculated. 
#' The goal is to keep those proteins/PTMs that are consistently found in 
#' one of the two conditions (in this case "condition B") and facilitate the 
#' inclusion in downstream analysis (if wished). The imputed intensity values 
#' are sampled from the lowest intensity values detected in the experiment, 
#' and (**WARNING**) the p-values are just randomly assigned between 
#' 0.05 and 0.01 for illustration purposes (when generating a volcano 
#' plot with the output of `artmsAnalysisQuantifications`) or to include 
#' them when making a cutoff of p-value < 0.05 for enrichment analysis
#' **CAUTION**: `mnbr` would also add the constraint that any protein 
#' must be identified in at least `nmbr` biological replicates of the 
#' **same** condition or it will be filtered out. That is,
#' if `mnbr = 2`, a protein found in two conditions but only in one
#' biological replicate in each of them, it would be removed.
#' @param pathogen (char) Is there a pathogen in the dataset as well?
#' if it does not, then use `pathogen = nopathogen` (default).
#' Pathogens available: `tb` (Tuberculosis), `lpn` (Legionella)
#' @param plotPvaluesLog2fcDist (logical) If `TRUE` (default) plots pvalues
#' and log2fc distributions
#' @param plotAbundanceStats (logical) If `TRUE` (default) plots stats graphs
#' about abundance values
#' @param plotReproAbundance (logical) If `TRUE` plots reproducibility
#' based on normalized abundance values
#' @param plotCorrConditions (logical) If `TRUE` plots correlation  
#' between the different conditions
#' @param plotCorrQuant (logical) if `TRUE` plots correlation between the
#' available quantifications (comparisons)
#' @param plotPCAabundance (logical) if `TRUE` performs PCA analysis of
#' conditions using normalized abundance values
#' @param plotFinalDistributions (logical) if `TRUE` plots distribution of both
#' log2fc and pvalues
#' @param plotPropImputation (logical) if `TRUE` plots proportion of overall
#' imputation
#' @param plotHeatmapsChanges (logical) if `TRUE` plots heatmaps of quantified
#' changes (both all and significant only). Only if `printPDF` is also `TRUE`
#' @param plotTotalQuant (logical) if `TRUE` plots barplot of total number of
#' quantifications per comparison
#' @param plotClusteringAnalysis (logical) if `TRUE` performs clustering
#' analysis between quantified comparisons (more than 1 comparison required)
#' @param data_object (logical) flag to indicate whether the required files 
#' are data objects. Default is FALSE
#' @param printPDF If `TRUE` (default) prints out the pdfs. Warning: plot
#' objects are not returned due to the large number of them. 
#' @param verbose (logical) `TRUE` (default) shows function messages
#' @return (data.frame) summary of quantifications, including annotations, 
#' enrichments, etc
#' @keywords analysis, quantifications
#' @examples
#' # Testing that the files cannot be empty
#' artmsAnalysisQuantifications(log2fc_file = NULL,
#'                               modelqc_file = NULL,
#'                               species = NULL,
#'                               output_dir = NULL)
#' @export
artmsAnalysisQuantifications <- function(log2fc_file,
                                         modelqc_file,
                                         species,
                                         output_dir = "analysis_quant",
                                         outliers = c("keep", "iqr", "std"),
                                         enrich = TRUE,
                                         l2fc_thres = 1,
                                         choosePvalue = c("adjpvalue","pvalue"),
                                         isBackground = "nobackground",
                                         isPtm = "global",
                                         mnbr = 2,
                                         pathogen = "nopathogen",
                                         plotPvaluesLog2fcDist = TRUE,
                                         plotAbundanceStats = TRUE,
                                         plotReproAbundance = TRUE,
                                         plotCorrConditions = TRUE,
                                         plotCorrQuant = TRUE,
                                         plotPCAabundance = TRUE,
                                         plotFinalDistributions = TRUE,
                                         plotPropImputation = TRUE,
                                         plotHeatmapsChanges = TRUE,
                                         plotTotalQuant = TRUE,
                                         plotClusteringAnalysis = TRUE,
                                         data_object = FALSE,
                                         printPDF = TRUE,
                                         verbose = TRUE
                                         ) {

  Uniprot_PTM = imputed = Gene_Protein = NULL
  
  if(verbose){
    message("---------------------------------------------")
    message("artMS: ANALYSIS OF QUANTIFICATIONS")
    message("---------------------------------------------")
  }
  
  # Checking arguments-----
  
  if(any(missing(log2fc_file) | 
         missing(modelqc_file) |
         missing(species) ))
   stop("One (or many) of the required arguments missed. 
        Please, check the help for this function to find out more")
  
  if(is.null(log2fc_file) & 
     is.null(modelqc_file) & 
     is.null(species) & 
     is.null(output_dir)){
    return("The evidence_file, modelqc_file, species and output_dir arguments cannot be NULL")
  }

  # CHECK POINT: DO THE FILES EXIST?
  if (!is.logical(data_object)) {
    stop(" Argument <data_object> must be logical (TRUE or FALSE) ")
  }
  
  if(!data_object){
    if(!file.exists(log2fc_file)){
      stop("THE FILE ", log2fc_file, " DOES NOT EXIST ")
    }
    
    if(!file.exists(modelqc_file)){
      stop("THE FILE ", modelqc_file, " DOES NOT EXIST ")
    }
  }
  
  if (!is.logical(enrich)) {
    stop(" Argument <enrich> must be logical (TRUE or FALSE) ")
  }

  if (!is.numeric(l2fc_thres)) {
    stop(" Argument <l2fc_thres> must be numeric ")
  }
  
  if (!is.numeric(mnbr)) {
    stop(" Argument <mnbr> must be numeric ")
  }

  if(!(isPtm %in% c('global', 'ptmph', 'ptmsites'))){
    stop("The < isPtm > argument is wrong. 
         The valid options are: <global> or <ptmsites> ")
  }
  
  outliers <- tolower(outliers)
  outliers <- match.arg(outliers)
  
  choosePvalue <- tolower(choosePvalue)
  choosePvalue <- match.arg(choosePvalue)
  
  # Check supported species------
  if(isFALSE(artmsIsSpeciesSupported(species))){
    if(verbose) message("----(-) ", species, " is not supported. \n\tGene Symbol, Protein Name, and EntrezID won't be provided")
  }
  
  species <- tolower(species)
  if(!(species %in% c('human', 'mouse'))){
    if(enrich){
      message("--- Enrichment analysis turned off (only available for HUMAN and MOUSE)")
      enrich <- FALSE
    }
  }
  
  # Checking required packages-----

  ## Bioconductor-----
  cp <- 0
  if(species == "human"){
    if (!"org.Hs.eg.db" %in% installed.packages()){
      message(paste0('- Package < org.Hs.eg.db > not installed in your system. 
                     Please, install running < BiocManager::install(\"org.Hs.eg.db\") >'))
      cp = cp + 1
    }
  }else if(species == "mouse"){
    if (!"org.Mm.eg.db" %in% installed.packages()){
      message(paste0('- Package < org.Mm.eg.db > not installed in your system. 
                     Please, install running < BiocManager::install(\"org.Mm.eg.db\") >'))
      cp = cp + 1
    }
  }
  
  ## CRAN -----
  required_bioc_packages <- c("ComplexHeatmap")
  for(p in required_bioc_packages){
    if(!(p %in% installed.packages())){
      message(paste0('- Package < ', p, ' > not installed in your system. 
                     Please, install running < BiocManager::install(\"',p,'\") >'))
      cp <- cp + 1
    }
  }
  
  required_cran_packages <- c("factoextra", "FactoMineR", "gProfileR", "PerformanceAnalytics")
  
  for(p in required_cran_packages){
    if(!(p %in% installed.packages())){
      message(paste0('- Package < ', p, ' > not installed in your system. 
                     Please, install running < install.packages(\"',p,'\") >'))
      cp <- cp + 1
    }
  }

  if(cp > 0){
    stop("\nRequired packages to run artmsAnalysisQuantifications() not available. Please, install and run again")
  }
  
  
  # Check pathogens-----
  pathogen <- tolower(pathogen)
  if (pathogen == "nopathogen") {
    if(verbose) message("--- No Pathogen extra in these samples")
  } else if (pathogen == "tb") {
    # This should not work
    if(verbose) message("\nPATHOGEN IN SAMPLES: Tuberculosis (TB)\n")
    pathogen.ids <- artms_data_pathogen_TB
    names(pathogen.ids) <- c('Entry')
  } else if (pathogen == "lpn") {
    if(verbose) message("\nPATHOGEN IN SAMPLES: LEGIONELLA PNEUMOPHILA\n")
    pathogen.ids <- artms_data_pathogen_LPN
  } else{
    stop("This pathogen is not supported yet")
  }
  
  # Session info-----
  session <- sessionInfo()
  sink("artms_sessionInfo_analysisQuant.txt")
  print(session)
  sink()
  
  # Create dir-----
  log2dirname <- normalizePath(dirname(log2fc_file))
  output_dir <- file.path(log2dirname, paste0(output_dir, "_", choosePvalue))
  
  if(!dir.exists(file.path(output_dir))){
    dir.create(file.path(output_dir), recursive = TRUE)
  }
  
  # Create the ouput_directory in the results folder
  # dirname(log2fc_file)
  # getwd()

  # Open log2fc-----
  if(verbose) message(">> LOADING QUANTIFICATIONS (-results.txt from MSstats) ")
  if(data_object){
    dflog2fcraw <- log2fc_file
    log2fc_file <- "analysis-results.txt"
  }else{
    dflog2fcraw <- read.delim(log2fc_file,
                              header = TRUE,
                              sep = "\t",
                              stringsAsFactors = FALSE)
  }
  
  # use this as a template to generate the output file names
  log2fc_file <- basename(log2fc_file)
  
  # Open modelqc ----
  if(verbose) message(">> LOADING modelqc FILE (ABUNDANCE) ")
  if(data_object){
    dfmq <- modelqc_file
    remove(modelqc_file)
  }else{
    dfmq <- read.delim(modelqc_file,
                       header = TRUE,
                       sep = "\t",
                       stringsAsFactors = FALSE)
    colnames(dfmq) <- toupper(colnames(dfmq))
  }
  
  # ModelQC: fix old files-----
  if("GROUP_ORIGINAL" %in% colnames(dfmq)){
    if(verbose) message("\n------------------------------------------------------------")
    if(verbose) message("(!) WARNING!")
    if(verbose) message("Be aware that you are analyzing a dataset that was generated with an old version of MSstats (< 4.0)")
    if(verbose) message("Some changes will be applied")
    if(verbose) message(" ------------------------------------------------------------\n")
    dfmq$GROUP <- NULL
    dfmq$GROUP <- dfmq$GROUP_ORIGINAL
    dfmq$SUBJECT <- NULL
    dfmq$SUBJECT <- dfmq$SUBJECT_ORIGINAL
  }
  
  # Removing the empty protein names-----
  if (any(dfmq$PROTEIN == "")) {
    dfmq <- dfmq[-which(dfmq$PROTEIN == ""), ]
    if(verbose) message("--- Empty ID proteins removed from abundance data")
  }
  dfmq$PROTEIN <- gsub("(sp\\|)(.*)(\\|.*)", "\\2", dfmq$PROTEIN)
  dfmq$PROTEIN <- gsub("(.*)(\\|.*)", "\\1", dfmq$PROTEIN)
  
  # Check protein ID----
  if( length(dfmq$PROTEIN[grep("\\w{2}_\\d{1,}\\.\\d{1,}", dfmq$PROTEIN)]) > 100 ){
    if(verbose) message("----(-) Many RefSeq IDs detected in this dataset, which is not supported yet.
                        Gene Symbol, Protein Name, and EntrezID won't be provided")
  }
  
  # Remove outliers------
  if (outliers == "iqr" | outliers == "std"){

    if(outliers == "iqr"){
      if(verbose) message("--- Removing outliers (based on 6xIQR cutoff)")
      iqr <- IQR(dfmq$ABUNDANCE)
      m <- mean(dfmq$ABUNDANCE)
      uplim <- m+(6*iqr)
      lowlim <- m-(6*iqr)
    }else if(outliers == "std"){
      if(verbose) message("--- Removing outliers (based on 6xSTD cutoff)")
      std <- sd(dfmq$ABUNDANCE)
      m <- mean(dfmq$ABUNDANCE)
      uplim <- m+3*std
      lowlim <- m-3*std
    }

    # Record outliers for plotting
    dfmq$outliers <- ifelse(dfmq$ABUNDANCE > lowlim & dfmq$ABUNDANCE < uplim, "no", "yes")
    
    outliers_number <- length(dfmq$outliers[which(dfmq$outliers == "yes")])
    
    # Report outliers removed
    if (outliers_number == 0){
      if(verbose) message("------> No outliers have been removed")
    }else{
      if(verbose) message("------> Total outliers removed: ", outliers_number)
      out.outliers <- gsub(".txt", paste0("-outliers-removed-",outliers,".txt"), log2fc_file)
      out.outliers <- paste0(output_dir, "/", out.outliers)
      dfmq.outliers <- dfmq[which(dfmq$outliers == "yes"),]
      write.table(
        dfmq.outliers,
        out.outliers,
        quote = FALSE,
        sep = "\t",
        row.names = FALSE,
        col.names = TRUE
      )
      
      # PLOT Outliers
      k <- ggplot(dfmq, aes(x = SUBJECT, y = ABUNDANCE, color = outliers))
      k <- k + geom_jitter(width = 0.3, size = 0.5, na.rm = TRUE)
      k <- k + theme_minimal()
      k <- k + ggtitle("Distribution of ABUNDANCE with outliers")
      k <-k + theme(axis.text.x = element_text(
        angle = 90,
        hjust = 1,
        vjust = 0.5
      ))
      
      # Distribution without ouliers
      l <- ggplot(dfmq[which(dfmq$outliers == "no"),], aes(x = SUBJECT, y = ABUNDANCE))
      l <- l + geom_jitter(width = 0.3, size = 0.5, na.rm = TRUE)
      l <- l + theme_minimal()
      l <- l + ggtitle("Distribution of ABUNDANCE (outliers removed)")
      l <- l + theme(axis.text.x = element_text(
        angle = 90,
        hjust = 1,
        vjust = 0.5
      ))
      
      out.outliers <- gsub(".txt", paste0("-outliers-removed-",outliers,".pdf"), log2fc_file)
      out.outliers <- paste0(output_dir, "/", out.outliers)
      if(printPDF) pdf(out.outliers)
      plot(k)
      plot(l)
      if(printPDF) garbage <- dev.off()
      
      if(verbose) message("------> Check table and plot to find out more about outliers removed")

      # Remove outliers
      dfmq <- dfmq[which(dfmq$ABUNDANCE > lowlim & dfmq$ABUNDANCE < uplim), ]
    }
  }else{
    if(verbose) message("--- Outliers kept (user selection)")
    dfmq <- dfmq
  }

  # Conditions----
  conditions <- unique(dfmq$GROUP)
  numberConditions <- length(conditions)
  
  # KEY STEP: getting background gene list-----
  if (isBackground == "nobackground") {
    # If not list of background genes is provided,
    # then extract them from the modelqc file
    if (isPtm == "global") {
      suppressMessages(
        dfmq2Genes <- artmsAnnotationUniprot(dfmq, 'PROTEIN', species)
        )
      numberTotalGenes <- length(unique(dfmq2Genes$Gene))
      if(verbose) message("--- Total number of genes/proteins: ", numberTotalGenes, " ")
      listOfGenes <- unique(dfmq2Genes$Gene)
    } else if (grepl("ptm", isPtm)) {
      dfmq2Genes <- dfmq[c('PROTEIN', 'GROUP')] 
      names(dfmq2Genes)[grep('PROTEIN', names(dfmq2Genes))] <- 'Protein'
      # # Removing party sites
      # dfmq2Genes <- dfmq2Genes[grep(",", dfmq2Genes$Protein, invert = TRUE), ]
      
      dfmq2Genes <- .artmsExtractUniprotId(x = dfmq2Genes, 
                                           uniprotPtmColumn = "Protein", 
                                           newColumnName = "Protein")
      
      dfmq2Genes <- unique(dfmq2Genes)
      suppressMessages(
        dfmq2Genes <- artmsAnnotationUniprot(dfmq2Genes, 'Protein', species)
        )
      numberTotalGenes <- length(unique(dfmq2Genes$Gene))
      if(verbose) message("--- TOTAL NUMBER OF GENES/PROTEINS: ",
          numberTotalGenes, "  ")
      if (numberTotalGenes == 0) {
        stop("IDs are not recognized")
      }
      listOfGenes <- unique(dfmq2Genes$Gene)
    }
  } else{
    # No matter what list is provided, it must come with a "Gene" column
    backgroundList <- read.delim(isBackground,
                                 header = TRUE,
                                 sep = "\t",
                                 quote = "",
                                 stringsAsFactors = FALSE)
    listOfGenes <- unique(backgroundList$Gene)
  }
  
  backgroundNumber <- length(listOfGenes)
  
  
  # Process log2fc----
  
  if (any(dflog2fcraw$Protein == "")) {
    dflog2fcraw <- dflog2fcraw[-which(dflog2fcraw$Protein == ""), ]
    if(verbose) message("--- Empty ID proteins removed from log2fc data")
  }
  dflog2fcraw$Protein <- gsub("(sp\\|)(.*)(\\|.*)", "\\2", dflog2fcraw$Protein)
  dflog2fcraw$Protein <- gsub("(.*)(\\|.*)", "\\1", dflog2fcraw$Protein)
  
  dflog2fcfinites <- dflog2fcraw[is.finite(dflog2fcraw$log2FC), ]
  
  
  ## Removing log2fc outliers----
  cutofflog2fc <- 15
  if(verbose) message(
    "--- Removing log2fc outliers (",
    paste0("-", cutofflog2fc, " < log2fc < +", cutofflog2fc),
    ") "
  )
  
  filtermorethan10 <- length(dflog2fcfinites$log2FC[abs(dflog2fcfinites$log2FC) > cutofflog2fc])
  if (filtermorethan10 > 0) {
    if(verbose) message("------ (-) Removing [",
                        filtermorethan10,
                        "] protein ids with a abs(log2fc) >",
                        cutofflog2fc, 
                        " ")
    dflog2fcfinites <- dflog2fcfinites[-which(abs(dflog2fcfinites$log2FC) > cutofflog2fc), ]
  }
  
  ## IMPUTING MISSING VALUES-----
  # When a value is completely missed in one of the conditions,
  # the log2fc = Inf / -Inf. Here, we impute those values.
  # The imputation method works as follow. The assumption is that those
  # proteins are likely present as well in those conditions where are missed, 
  # but due to the
  # small sampling (usually 2 or 3 biological replicas) and other proteomics
  # related issue, those proteins didn't make it through the level of detection.
  # Therefore, a small intensity (sampled from the bottom 5%) will be assigned
  # to the protein/site in the missing condition, and the new log2fc is 
  # re-calculated out of the MSstats box. Two issues are addressed in this way
  # 1. If a protein has been consistently identified in one of the conditions, 
  # it will stay
  # 2. But if the intensity value in those conditions was too low, then the 
  # log2fc will be also low
  
  if(verbose) message(">> IMPUTING MISSING VALUES ")
  # Select infinite values (i.e., log2fc missed for that)
  dflog2fcinfinites <- dflog2fcraw[is.infinite(dflog2fcraw$log2FC), ]
  numberInfinites <- dim(dflog2fcinfinites)[1]
  
  # Control
  if (numberInfinites < 1) {
    if(verbose) message(" WARNING: O Infinite values (not very usual)")
  } else {
    if(verbose) message("--- Number of +/- INF values: ", dim(dflog2fcinfinites)[1], " ")
    
    imputedL2FCmelted <- .artms_imputeMissingValues(dflog2fcinfinites = dflog2fcinfinites, 
                                                    dfmq = dfmq)
    
    # Merge with the original log2fc values to impute...
    theImputedL2FC <- merge(dflog2fcinfinites,
                            imputedL2FCmelted,
                            by = c("Protein", "Label"),
                            all = FALSE)
    # check size of theImputedL2FC compared with dflog2fcinfinites to see if we lost rows during imputation
    # possibly we were unable to impute rows after removing outliers
    if (nrow(theImputedL2FC) != nrow (dflog2fcinfinites)){
      numMissing = nrow (dflog2fcinfinites) - nrow(theImputedL2FC)
      message(" WARNING:  Failed to impute values for ", numMissing, "rows. Possibly due to outlier removal")
    }
      
    theImputedL2FC$imputed <- "yes"
  }
  
  # Getting the data ready for merging
  dflog2fcfinites$imputed <- "no"
  dflog2fcfinites$iLog2FC <- dflog2fcfinites$log2FC
  
  ## Choose the pvalue or adjusted pvalue as the iPvalue-----
  if (choosePvalue == "pvalue") {
    dflog2fcfinites$iPvalue <- dflog2fcfinites$pvalue
  } else if (choosePvalue == "adjpvalue") {
    dflog2fcfinites$iPvalue <- dflog2fcfinites$adj.pvalue
  } else{
    stop("Only <pvalue> or <adjpvalue> for argument <choosePvalue>")
  }
  
  # Merging: NA values are thrown away at this point
  if (numberInfinites < 1) {
    dflog2fc <- dflog2fcfinites
  } else {
    dflog2fc <- rbind(dflog2fcfinites, theImputedL2FC)
  }
  
  ## Plot pvalue distributions-----
  if(plotPvaluesLog2fcDist){
    if(verbose) message("--- Plotting distributions of log2fc and pvalues ")
    
    plotDFdistColor <-
      ggplot(dflog2fc, aes(x = log2FC, fill = Label)) +
      geom_histogram(bins = 100,
                     alpha = .4,
                     col = "black", 
                     na.rm = TRUE) +
      labs(title = "Distribution log2FC", x = "log2FC")
    
    plotDFdistAll <- ggplot(dflog2fc, aes(x = log2FC)) +
      geom_histogram(bins = 100,
                     alpha = .4,
                     col = "black", 
                     na.rm = TRUE) +
      labs(title = "Distribution log2FC", x = "log2FC")
    
    plotDFdistiLog <- ggplot(dflog2fc, aes(x = iLog2FC)) +
      geom_histogram(bins = 100,
                     alpha = .4,
                     col = "black", 
                     na.rm = TRUE) +
      labs(title = "Distribution ilog2FC (imputed + nonimputed", x = "iLog2FC")
    
    plotPvalues <-
      ggplot(dflog2fc[is.finite(dflog2fc$pvalue), ], aes(x = pvalue)) +
      geom_histogram(bins = 50,
                     alpha = .4,
                     col = "black",
                     na.rm = TRUE) +
      labs(title = "Distribution p-values", x = "p-values")
    
    plotAdjustedPvalues <-
      ggplot(dflog2fc[-which(dflog2fc$adj.pvalue == 0), ], aes(x = adj.pvalue)) +
      geom_histogram(bins = 150,
                     alpha = .4,
                     col = "black",
                     na.rm = TRUE) +
      labs(title = "Distribution adj.pvalues", x = "adj.values")
    
    plotAdjustedIpvalues <- ggplot(dflog2fc, aes(x = iPvalue)) +
      geom_histogram(bins = 150,
                     alpha = .4,
                     col = "black",
                     na.rm = TRUE) +
      labs(title = "Distribution imputed p-values", x = "iPvalues")
    
    
    # DISTRIBUTION PRINT OUTS
    distributionsOut <- gsub(".txt", ".distributions.pdf", log2fc_file)
    distributionsOut <- paste0(output_dir, "/", distributionsOut)
    if(printPDF) pdf(distributionsOut)
    print(plotDFdistColor)
    print(plotDFdistAll)
    print(plotDFdistiLog)
    print(plotPvalues)
    print(plotAdjustedPvalues)
    print(plotAdjustedIpvalues)
    
    if (numberInfinites > 0) {
      hist(
        imputedL2FCmelted$iLog2FC,
        breaks = 100,
        main = paste0("Imputed Log2FC (all)\n n = ", dim(imputedL2FCmelted)[1]),
        xlab = "log2fc"
      )
      hist(
        theImputedL2FC$iLog2FC,
        breaks = 100,
        main = paste0("Imputed Log2FC merged\n n = ", dim(theImputedL2FC)[1]),
        xlab = "log2fc"
      )
    }
    
    hist(
      dflog2fcfinites$pvalue,
      breaks = 100,
      main = paste0("p-value distribution\n n = ", dim(dflog2fcfinites)[1]),
      xlab = "adj.pvalues"
    )
    hist(
      dflog2fcfinites$adj.pvalue,
      breaks = 100,
      main = paste0("Adjusted p-values distribution\n n = ", 
                    dim(dflog2fcfinites)[1]),
      xlab = "adj.pvalues"
    )
    hist(
      dflog2fcfinites$iLog2FC,
      breaks = 1000,
      main = paste0(
        "Non-imputed Log2FC distribution\n n = ",
        dim(dflog2fcfinites)[1]
      ),
      xlab = "log2FC"
    )
    hist(
      dflog2fc$iPvalue,
      breaks = 100,
      main = paste0(
        "(Imputed+NonImputed) adjusted pvalue distribution\n n = ",
        dim(dflog2fc)[1]
      ),
      xlab = "adj.pvalues"
    )
    hist(
      dflog2fc$iLog2FC,
      breaks = 1000,
      main = paste0(
        "(Imputed+NonImputed) log2fc distribution\n n = ",
        dim(dflog2fc)[1]
      ),
      xlab = "log2FC"
    )
    if(printPDF) garbage <- dev.off()
  }
  
  # Plot correlations based on log2fc------
  if(plotCorrQuant){
    if(verbose) message(">> PLOT: CORRELATION BETWEEN QUANTIFICATIONS (based on log2fc values)")
    if (length(unique(dflog2fc$Label)) > 1) {
      relaChanges <- gsub(".txt", ".correlationQuantifications.pdf", log2fc_file)
      relaChanges <- paste0("plot.", relaChanges)
      relaChanges <- paste0(output_dir, "/", relaChanges)
      if(printPDF) pdf(relaChanges)
      .artms_plotRatioLog2fc(dflog2fc, verbose = verbose)
      if(printPDF) garbage <- dev.off()
    } else{
      if(verbose) message("--- Only one Comparison is available (correlation is not possible) ")
    }
  }
  
  # ABUNDANCE-----
  ## Relationship between conditions------
  # Get the number of biological replicas based on the first condition
  theConditions <- unique(dfmq$GROUP)
  theFirstCond <- theConditions[2]
  condFirst <- dfmq[which(dfmq$GROUP == theFirstCond), ]
  theBiologicalReplicas <- unique(condFirst$SUBJECT)
  numberBioReplicas <- length(theBiologicalReplicas)

  ## ABUNDANCE PLOTS----
  
  if(plotAbundanceStats){
    if(verbose) message(">> PLOTS: ABUNDANCE PLOTS ")
    abundancesName <- gsub(".txt", ".relativeABUNDANCE.pdf", log2fc_file)
    abundancesName <- paste0("plot.", abundancesName)
    abundancesName <- paste0(output_dir, "/", abundancesName)
    if(printPDF) pdf(abundancesName)
    .artms_plotAbundanceBoxplots(df = dfmq)
    .artms_plotNumberProteinsAbundance(df = dfmq)
    if(printPDF) garbage <- dev.off()
  }
  
  ## Reproducibility plots-----
  if(plotReproAbundance){
    if(verbose) message(">> PLOTS: REPRODUCIBILITY PLOTS ")
    reproName <- gsub(".txt", ".reproducibilityAbundance.pdf", log2fc_file)
    reproName <- paste0("plot.", reproName)
    reproName <- paste0(output_dir, "/", reproName)
    if(printPDF) pdf(reproName)
    .artms_plotReproducibilityAbundance(x = dfmq, verbose = verbose)
    if(printPDF) garbage <- dev.off()
  }
  
  ## Correlation between Conditions------
  if(plotCorrConditions){
    if(verbose) message(">> PLOT: CORRELATION BETWEEN ALL COMPARISONS ")
    relaCond <- gsub(".txt", ".correlationConditions.pdf", log2fc_file)
    relaCond <- paste0("plot.", relaCond)
    relaCond <- paste0(output_dir, "/", relaCond)
    if(printPDF) pdf(relaCond)
    .artms_plotCorrelationConditions(x = dfmq, 
                                     numberBiologicalReplicas = numberBioReplicas)
    if(printPDF) garbage <- dev.off()
  }
  
  ## ABUNDANCE DATA, CREATE FILTERS-----
  abundance <- .artms_loadModelqcBasic(dfmq)
  names(abundance)[grep('Protein', names(abundance))] <- 'Prey'
  names(abundance)[grep('Condition', names(abundance))] <- 'Bait'
  # TECHNICAL REPLICAS: if there are technical replicas means that we will find
  # two values for the same protein in the same bioreplica, therefore we need to
  # aggregate first just in case:
  ##LEGACY
  # abundance <- aggregate(Abundance ~ Prey + Bait + BioReplicate,
  #                        data = abundance,
  #                        FUN = mean)
  # abundance_dcmean <- data.table::dcast(abundance,
  #                                       Prey ~ Bait,
  #                                       value.var = 'Abundance',
  #                                       fun.aggregate = mean,
  #                                       fill = 0)
  
  # Let's aggregate to get the sum of the abundance, we will use it later.
  # abundance_dcsum <- data.table::dcast(abundance,
  #                                      Prey ~ Bait,
  #                                      value.var = 'Abundance',
  #                                      fun.aggregate = sum,
  #                                      fill = 0)
  
  abundance_dcsum <- abundance %>% pivot_wider(names_from = Bait, 
                                                id_cols = Prey,
                                                values_from = Abundance, 
                                                values_fn = list(Abundance = sum))
  
  abundance_dcmean <- abundance %>% pivot_wider(names_from = Bait, 
                                                 id_cols = Prey,
                                                 values_from = Abundance, 
                                                 values_fn = list(Abundance = mean))
                                               
  # Melt again the sum and mean
  ##LEGACY
  # abundancelongsum <- data.table::melt(abundance_dcsum,
  #                                    id.vars = c('Prey'),
  #                                    value.name = 'Abundance',
  #                                    variable.name = 'Bait')
  # 
  # abundancelongmean <- data.table::melt(abundance_dcmean,
  #                                     id.vars = c('Prey'),
  #                                     value.name = 'Abundance',
  #                                     variable.name = 'Bait')
  
  abundancelongsum <- abundance_dcsum %>% pivot_longer(cols = -Prey, 
                                                        names_to = "Bait", 
                                                        values_to = "AbSum")
  
  abundancelongmean <- abundance_dcmean %>% pivot_longer(cols = -Prey, 
                                                           names_to = "Bait", 
                                                           values_to = "AbMean")
  
  # We dont need the 0 values
  abundancelongsum <- abundancelongsum[!(abundancelongsum$AbSum == 0), ]
  abundancelongmean <- abundancelongmean[!(abundancelongmean$AbMean == 0), ]
  
  abundancelongsum <- abundancelongsum[!(is.na(abundancelongsum$AbSum)), ]
  abundancelongmean <- abundancelongmean[!( is.na(abundancelongmean$AbMean) ), ]
  
  abundancelongsummean <- merge(abundancelongsum, abundancelongmean, by = c('Prey', 'Bait'))
  
  remove(abundancelongsum, abundancelongmean, abundance_dcsum, abundance_dcmean)
  
  # REPRODUCIBILITY AND SPECIFICY PARATEMERS
  
  # Get the number of bioreplicates based on abundance data
  
  ##LEGACY
  # nbr_wide <- data.table::dcast(abundance,
  #                               Prey ~ Bait,
  #                               value.var = 'Abundance',
  #                               fun.aggregate = length,
  #                               fill = 0)
  # nbr_long <- data.table::melt(nbr_wide,
  #                            id.vars = c('Prey'),
  #                            value.name = 'Abundance',
  #                            variable.name = 'Bait')
  # 
  # nbr_long <- nbr_long[!(nbr_long$BioRep == 0), ]
  # names(nbr_long)[grep('Abundance', names(nbr_long))] <- 'BioRep'
  
  nbr_wide <- abundance %>% pivot_wider(names_from = Bait, 
                                         id_cols = Prey,
                                         values_from = Abundance, 
                                         values_fn = list(Abundance = length))
  nbr_wide[(is.na(nbr_wide))] <- 0
  
  nbr_long <- nbr_wide %>% pivot_longer(cols = -Prey, 
                                        names_to = "Bait", 
                                        values_to = "BioRep")
  
  nbr_long <- nbr_long[!(nbr_long$BioRep == 0), ]
  nbr_long <- nbr_long[!( is.na(nbr_long$BioRep) ), ]
                                                       
  # Get the number of replicates in long format
  ##LEGACY
  # OUTreprod <- data.table::dcast(data = nbr_long, Prey ~ Bait, value.var = 'BioRep')
  # OUTreprod[is.na(OUTreprod)] <- 0
  
  OUTreprod <- abundance %>% pivot_wider(id_cols = Prey, 
                                        names_from = Bait, 
                                        values_from = Abundance, 
                                        values_fn = list(Abundance = length))
  
  OUTreprod[is.na(OUTreprod)] <- 0
  
  # Make a copy for conditions
  OUTreproCondition <- OUTreprod
  
  here <- dim(OUTreprod)[2]
  
  # Make a copy to use later
  bioReplicaInfo <- OUTreprod
  
  # And get the total number of biological replicates
  
  OUTreprod$BiorepCount <- rowSums(OUTreprod[, 2:here])
  
  # Get whether a protein is found in all conditions
  reprospec2merge <- subset(OUTreprod, select = c(Prey, BiorepCount))
  
  ##LEGACY
  # OUTreproCondition <- data.table::dcast(data = nbr_long, Prey ~ Bait, value.var = 'BioRep')
  # OUTreproCondition[is.na(OUTreproCondition)] <- 0
  
  thedim <- dim(OUTreproCondition)[2]
  thepreys <- subset(OUTreproCondition, select = c(Prey))
  thevalues <- subset(OUTreproCondition, select = -c(Prey))
  thevalues[thevalues > 0] <- 1
  thevalues$CondCount <- rowSums(thevalues)
  FinalReproCondition <- cbind(thepreys, thevalues)
  
  reprocondition2merge <- subset(FinalReproCondition, select = c(Prey, CondCount))
  
  # This version will be printed out below
  OUTreprodFinal <- merge(OUTreprod, reprocondition2merge, by = 'Prey')
  
  # PCA ANALYSIS-----
  # It requires a simplified version for modelqc
  if(plotPCAabundance){
    if(verbose) message(">> PRINCIPAL COMPONENT ANALYSIS BASED ON ABUNDANCE ")
    modelqcabundance <- .artms_loadModelQCstrict(df = dfmq,
                                                 species = species,
                                                 ptmis = isPtm,
                                                 verbose = verbose)
    out.pca <- gsub(".txt", "-pca", log2fc_file)
    out.pca <- paste0(output_dir, "/", out.pca)
    suppressWarnings(.artms_getPCAplots(x = modelqcabundance, 
                                        filename = out.pca, 
                                        allConditions = conditions))
  }

  # ANNOTATIONS-----
  modelqc_file_splc <- .artms_mergeAbNbr(df_input =  dfmq, 
                                         repro =  nbr_wide, 
                                         species =  species)
  
  if(verbose) message(">> ANNOTATIONS ")
  # Now get ready for annotation
  if(verbose) message("--- Abundance data ")
  if (grepl("ptm", isPtm)) {
    names(modelqc_file_splc)[grep('^Protein$', names(modelqc_file_splc))] <- 'Uniprot_PTM'
    # Take the Protein ID, but being very careful about the fluomics labeling
    
    modelqc_file_splc <- .artmsExtractUniprotId(x = modelqc_file_splc, 
                                                uniprotPtmColumn = "Uniprot_PTM", 
                                                newColumnName = "Protein")
    suppressMessages(
      modelqc_file_splc <- artmsAnnotationUniprot(modelqc_file_splc, 'Protein', species)
      )
  } else{
    suppressMessages(
      modelqc_file_splc <- artmsAnnotationUniprot(modelqc_file_splc, 'Protein', species)
    )
  }
  
  if(verbose) message("--- Relative Quantifications (Log2fc) ")
  
  if(choosePvalue == "adjpvalue"){
    choosePvalue <- "adj.pvalue"
  }
  # Prepare output of changes
  log2fc_file_splc <- .artms_mergeChangesNbr(df_input = dflog2fc, 
                                             repro =  nbr_wide, 
                                             species =  species,
                                             choosePvalue = choosePvalue)
  
  # Now get ready for annotation
  if (grepl("ptm", isPtm)) {
    names(log2fc_file_splc)[grep('^Protein$', names(log2fc_file_splc))] <- 'Uniprot_PTM'
    # Take the Protein ID, but being very careful about the fluomics labeling
    log2fc_file_splc <- .artmsExtractUniprotId(x = log2fc_file_splc, 
                                               uniprotPtmColumn = "Uniprot_PTM", 
                                               newColumnName = "Protein")
    suppressMessages(
      log2fc_file_splc <- artmsAnnotationUniprot(log2fc_file_splc, 'Protein', species)
    )
  } else{
    suppressMessages(
      log2fc_file_splc <- artmsAnnotationUniprot(log2fc_file_splc, 'Protein', species)
    )
  }
  
  if(verbose) message(">> FILTERING CHANGES BEFORE PRINTING OUT ")
  
  imputedDF <- dflog2fc[c('Protein',
                          'Label',
                          'log2FC',
                          'pvalue',
                          'adj.pvalue',
                          'imputed',
                          'iLog2FC',
                          'iPvalue'
                          )]
    
    
  if(verbose) message("--- Merging Changes with replication data ")
  imputedDF <- merge(imputedDF,
                     bioReplicaInfo,
                     by.x = 'Protein',
                     by.y = 'Prey',
                     all.x = TRUE)
  
  if(verbose) message("--- Removing NA ")
    imputedDF <- imputedDF[!is.na(imputedDF$log2FC), ]
  
  if(verbose) message("--- Add labeling of condition more abundant in the quantification ")
    imputedDF$CMA <- mapply(.artms_selectTheOneLog2fc,
                            imputedDF$iLog2FC,
                            imputedDF$Label)
  
  if(verbose) message("--- Removing proteins not found in a minimal number <",
                      mnbr,
                      "> (user selected) of biological replicates ")
  
  before <- dim(imputedDF)[1]
  imputedDF <- .artms_RemoveProtBelowThres(imputedDF, mnbr)
  after <- dim(imputedDF)[1]
  total_remove <- before - after
  
  if(verbose) message("\t", total_remove, " proteins removed (i.e., not found in more than <", mnbr, "> biological replicates in at least one condition)")
  
  if(verbose) message("--- Filtering is done! ")
  
  # PLOTS: GENERATING QC PLOTS ABOUT CHANGES-----
  if(plotFinalDistributions){
    if(verbose) message(">> GENERATING QC PLOTS ABOUT CHANGES (log2fc) ")
    if(verbose) message("--- Distribution of log2fc and pvalues ")
    distributionsFilteredOut <- gsub(".txt", ".distributionsFil.pdf", log2fc_file)
    distributionsFilteredOut <- paste0(output_dir, "/", distributionsFilteredOut)
    if(printPDF) pdf(distributionsFilteredOut)
      hist(imputedDF$iLog2FC,
           breaks = 1000,
           main = paste0("Filtered Log2FC (>2BR)\n n = ", dim(imputedDF)[1]),
           xlab = "log2fc")
      hist(imputedDF$iPvalue,
           breaks = 1000,
           main = paste0("Filtered p-values (>2BR)\n n = ", dim(imputedDF)[1]),
           xlab = "p-value")
    if(printPDF) garbage <- dev.off()
  }
  
  if(plotPropImputation){
    if(verbose) message("--- Proportion imputed values ")
    # Stats about imputed values
    yesimputed <- dim(imputedDF[which(imputedDF$imputed == 'yes'), ])[1]
    nonimputed <- dim(imputedDF[which(imputedDF$imputed == 'no'), ])[1]
    
    dat <- data.frame(count = c(yesimputed, nonimputed),
                      category = c("Imputed", "Non-Imputed"))
    # Add addition columns, needed for drawing with geom_rect.
    dat$fraction = dat$count / sum(dat$count)
    dat <- dat[order(dat$fraction),]
    dat$ymax <- cumsum(dat$fraction)
    dat$ymin <- c(0, head(dat$ymax, n = -1))
    
    p1 <- ggplot(dat, aes(fill = category,
                          ymax = ymax,
                          ymin = ymin,
                          xmax = 4,
                          xmin = 3)) +
      geom_rect(na.rm = TRUE) +
      coord_polar(theta = "y") +
      xlim(c(0, 4)) +
      labs(title = "Proportion of Imputed Intensity values")
    
    p2 <- ggplot(dat, aes(x = category, y = count, fill = category)) +
      geom_bar(stat = "identity",
               na.rm = TRUE) +
      labs(title = "Proportion of Imputed Intensity values")
    
    outImputation <- gsub(".txt", ".imputation.pdf", log2fc_file)
    outImputation <- paste0(output_dir, "/", outImputation)
    if(printPDF) pdf(outImputation)
    print(p1)
    print(p2)
    if(printPDF) garbage <- dev.off()
  }
  
  if(plotHeatmapsChanges){
    if(verbose) message(">> HEATMAPS OF CHANGES (log2fc) ")
    
    ##LEGACY
    # l2fcol <- data.table::dcast(data = imputedDF, 
    #                             Protein ~ Label, 
    #                             value.var = 'iLog2FC')
    l2fcol <- imputedDF %>% tidyr::pivot_wider(id_cols = Protein, 
                                                names_from = Label, 
                                                values_from = iLog2FC)
    
    l2fcol <- as.data.frame(l2fcol)
    rownames(l2fcol) <- l2fcol$Protein
    l2fcol <- within(l2fcol, rm(Protein))
    l2fcol[is.na(l2fcol)] <- 0
    
    if (numberConditions > 1) {
      l2fcolmatrix <- data.matrix(l2fcol)
      if(verbose) message("--- All changes ")
      
      if(printPDF){
        outHeatMapOverallL2fc <- gsub(".txt",
                                      ".clustering.log2fc.all-overview.pdf",
                                      log2fc_file)
        
        outHeatMapOverallL2fc <- paste0(output_dir, "/", outHeatMapOverallL2fc)
        plotPhLog1 <- pheatmap(
          l2fcolmatrix,
          filename = outHeatMapOverallL2fc,
          cellwidth = 20,
          main = "Clustering Log2FC",
          cluster_cols = FALSE,
          clustering_method = "average",
          fontfamily = "Helvetica",
          show_colnames = FALSE,
          fontsize = 6,
          fontsize_row = 3,
          fontsize_col = 10,
          border_color = NA
        )
        print(plotPhLog1)
      }else{
        plotPhLog2 <- pheatmap(
          l2fcolmatrix,
          cellwidth = 20,
          main = "Clustering Log2FC",
          cluster_cols = FALSE,
          clustering_method = "average",
          fontfamily = "Helvetica",
          show_colnames = FALSE,
          fontsize = 6,
          fontsize_row = 3,
          fontsize_col = 10,
          border_color = NA
        )
        print(plotPhLog2)
      }

      # Only significant pvalues
      if(verbose) message("--- Only significant changes ")
      imputedDFsig <- imputedDF[which(imputedDF$iPvalue < 0.05), ]
      ##LEGACY
      # l2fcolSignificants <- data.table::dcast(data = imputedDFsig, 
      #                                         Protein ~ Label, 
      #                                         value.var = 'iLog2FC')
      l2fcolSignificants <- imputedDFsig %>% tidyr::pivot_wider(id_cols = Protein, 
                                                 names_from = Label, 
                                                 values_from = iLog2FC)
      l2fcolSignificants <- as.data.frame(l2fcolSignificants)
      rownames(l2fcolSignificants) <- l2fcolSignificants$Protein
      l2fcolSignificants <- within(l2fcolSignificants, rm(Protein))
      l2fcolSignificants[is.na(l2fcolSignificants)] <- 0
      
      l2fcolSignificantsmatrix <- data.matrix(l2fcolSignificants)

      if(printPDF){
        outHeatMapOverallL2fc <- gsub(".txt",
                                      ".clustering.log2fcSign.all-overview.pdf",
                                      log2fc_file)
        outHeatMapOverallL2fc <- paste0(output_dir, "/", outHeatMapOverallL2fc)
        plotPh1 <- pheatmap(
          l2fcolSignificantsmatrix,
          filename = outHeatMapOverallL2fc,
          cellwidth = 20,
          main = "Clustering Log2FC (p-value < 0.05)",
          cluster_cols = FALSE,
          fontfamily = "Helvetica",
          labels_row = "",
          fontsize = 6,
          fontsize_row = 8,
          fontsize_col = 8,
          border_color = NA,
          fontfamily = "Helvetica")
        print(plotPh1)
      }else{
        plotPh2 <- pheatmap(
          l2fcolSignificantsmatrix,
          cellwidth = 20,
          main = "Clustering Log2FC (p-value < 0.05)",
          cluster_cols = FALSE,
          fontfamily = "Helvetica",
          labels_row = "",
          fontsize = 6,
          fontsize_row = 8,
          fontsize_col = 8,
          border_color = NA,
          fontfamily = "Helvetica")
        print(plotPh2)
      }
    }
  }

  # ENRICHMENT OF MOST ABUNDANT PROTEINS (from IMPUTED LOG2FC values)-----
  # Let's select first significance based on pvalue, by using the iPvalue
  # we are already including the imputed pvalues...
  
  ##LEGACY
  # l2fcol4enrichment <- data.table::dcast(data = imputedDF[which(imputedDF$iPvalue < 0.05), ], 
  #                                        Protein ~ Label, 
  #                                        value.var = 'iLog2FC')
  
  l2fcol4enrichment <- imputedDF %>% 
    dplyr::filter(iPvalue < 0.05) %>%
    tidyr::pivot_wider(id_cols = Protein, 
                       names_from = Label, 
                       values_from = iLog2FC)
  
  l2fcol4enrichment <- as.data.frame(l2fcol4enrichment)
  
  ## GPROFILER=====
  if (enrich == TRUE & dim(l2fcol4enrichment)[1] > 0) {
    if(verbose) message(">> ENRICHMENT ANALYSIS OF SELECTED CHANGES USING GPROFILER ")
    
    if (dim(l2fcol4enrichment)[1] > 0) {
      # Let's melt now for enrichment analysis
      ##LEGACY
      # l2fcol4enrichment <- data.table::melt(data = l2fcol4enrichment,
      #                                     id.vars = c('Protein'),
      #                                     variable.name = "Comparisons")
      
      l2fcol4enrichment <- l2fcol4enrichment %>%
        tidyr::pivot_longer(cols = -Protein, 
                            names_to = "Comparisons", 
                            values_to = "value")
      
      l2fcol4enrichment <- as.data.frame(l2fcol4enrichment)
      
      l2fcol4enrichment <- l2fcol4enrichment[complete.cases(l2fcol4enrichment), ]
      # Now get ready for annotation
      if (grepl("ptm", isPtm)) {
        names(l2fcol4enrichment)[grep('^Protein$', names(l2fcol4enrichment))] <- 'Uniprot_PTM'
        # Take the Protein ID, but being very careful about the fluomics labeling
        l2fcol4enrichment <- .artmsExtractUniprotId(x = l2fcol4enrichment, 
                                                    uniprotPtmColumn = "Uniprot_PTM", 
                                                    newColumnName = "Protein")
        suppressMessages(
          l2fcol4enrichment <- artmsAnnotationUniprot(l2fcol4enrichment, 
                                                      'Protein', 
                                                      species)
        )
      } else{
        suppressMessages(
          l2fcol4enrichment <- artmsAnnotationUniprot(l2fcol4enrichment, 
                                                      'Protein', 
                                                      species)
        )
      }
    }
    
    if (grepl("ptm", isPtm)) {
      # l2fcol4enrichment <- 
      # within(l2fcol4enrichment, rm(Gene,Uniprot_PTM,Protein.names))
      # # Remove parties for enrichment
      # l2fcol4enrichment <- l2fcol4enrichment[grep(",", l2fcol4enrichment$Protein, invert = TRUE),]
      # Select the Uniprot ID, but keep in mind that some of them might
      # have many _ph54_ph446 before
      # l2fcol4enrichment$Protein <- 
      # gsub("^(\\S+?)_.*", "\\1", l2fcol4enrichment$Protein, perl = TRUE)
      l2fcol4enrichment <- unique(l2fcol4enrichment[c("Protein", "Gene", "Comparisons", "value")])
    }
    
    # ALL SIGNIFICANT CHANGES log2fc
    # GPROFILER
    if(verbose) message("1) Enrichment of ALL significant Changes ")
      filallsig_log2fc_long <- l2fcol4enrichment[which(abs(l2fcol4enrichment$value) >= l2fc_thres),]
    
    if (dim(filallsig_log2fc_long)[1] > 0) {
      out.mac.allsig <- gsub(".txt", "-enrich-MAC-allsignificants.txt", log2fc_file)
      out.mac.allsig <- paste0(output_dir, "/", out.mac.allsig)
      mac.allsig <- NULL
      
      tryCatch(
        mac.allsig <- artmsEnrichLog2fc(
          dataset = filallsig_log2fc_long,
          output_name = out.mac.allsig,
          species = species,
          heatmaps = TRUE,
          background = listOfGenes,
          verbose = verbose
        ), error = function(e){
          message("\n\t(Error): Enrichment is not possible! ")
          message("\tgProfiler server night be down or your internet connection is not working")
          enrich <- FALSE
          }
      )
      
      if(!is.null(mac.allsig)){
        if (dim(mac.allsig)[1] > 0) {
          write.table(
            mac.allsig,
            out.mac.allsig,
            quote = FALSE,
            sep = "\t",
            row.names = FALSE,
            col.names = TRUE
          )
        }
      }

      
      if(verbose) message("---+ Corum Protein Complex Enrichment Analysis ")
      
      # CORUM
      allsigComplexEnriched <- .artms_enrichForComplexes(df = filallsig_log2fc_long,
                                                         backgroundNumber = backgroundNumber)
      
      if (dim(allsigComplexEnriched)[1] > 0) {
        out.mac.allsig.corum <-  gsub(".txt",
                                      "-enrich-MAC-allsignificants-corum.txt",
                                      log2fc_file)
        out.mac.allsig.corum <- paste0(output_dir, "/", out.mac.allsig.corum)
        
        write.table(
          allsigComplexEnriched,
          out.mac.allsig.corum,
          quote = FALSE,
          sep = "\t",
          row.names = FALSE,
          col.names = TRUE
        )
      }
      # And the heatmap
      if (dim(allsigComplexEnriched)[1] > 2) {
        out.mac.allsig.corum.pdf <- gsub(".txt",
                                         "-enrich-MAC-allsignificants-corum.pdf",
                                         log2fc_file)
          
        out.mac.allsig.corum.pdf <- paste0(output_dir, "/", out.mac.allsig.corum.pdf)
        
        .artms_plotCorumEnrichment(x =  allsigComplexEnriched, 
                                   outfile = out.mac.allsig.corum.pdf, 
                                   theTitle = "MAC ALL SIGNIFICANT Protein Complex Enrichment")
          
      } else{
        message("--- (-) Not enough negative corum complexes to plot ")
      }
    } else{
      message(" ----(-) No significant hits ")
      mac.allsig <- NULL
      allsigComplexEnriched <- NULL
    }
    
    # POSITIVE log2fc
    # GPROFILER
    if(verbose) message("2) Enrichment of selected POSITIVE significant changes ")
    
    filpos_log2fc_long <- l2fcol4enrichment[which(l2fcol4enrichment$value >= l2fc_thres),]
    
    if (dim(filpos_log2fc_long)[1] > 0) {
      out.mac.pos <- gsub(".txt", "-enrich-MAC-positives.txt", log2fc_file)
      out.mac.pos <- paste0(output_dir, "/", out.mac.pos)

      mac.pos <- NULL
      tryCatch(
          mac.pos <- artmsEnrichLog2fc(
            dataset = filpos_log2fc_long,
            species = species,
            heatmaps = TRUE,
            output_name = out.mac.pos,
            background = listOfGenes,
            verbose = verbose
          ), error = function(e){
            message("\n\t(Error): Enrichment is not possible! ")
            message("\tgProfiler server night be down or your internet connection is not working")
            enrich = FALSE
          }
      )
      
      if(!is.null(mac.pos)){
        if (dim(mac.pos)[1] > 0) {
          write.table(
            mac.pos,
            out.mac.pos,
            quote = FALSE,
            sep = "\t",
            row.names = FALSE,
            col.names = TRUE
          )
        }
      }
      
      if(verbose) message("---+ Corum Protein Complex Enrichment Analysis ")
      
      # CORUM
      positiveComplexEnriched <- .artms_enrichForComplexes(filpos_log2fc_long, backgroundNumber)
      
      if (dim(positiveComplexEnriched)[1] > 0) {
        out.mac.pos.corum <-
          gsub(".txt",
               "-enrich-MAC-positives-corum.txt",
               log2fc_file)
        out.mac.pos.corum <-
          paste0(output_dir, "/", out.mac.pos.corum)
        write.table(
          positiveComplexEnriched,
          out.mac.pos.corum,
          quote = FALSE,
          sep = "\t",
          row.names = FALSE,
          col.names = TRUE
        )
      }
      
      # And the heatmap
      if (dim(positiveComplexEnriched)[1] > 2) {
        out.mac.pos.corum.pdf <- gsub(".txt",
                                      "-enrich-MAC-positives-corum.pdf",
                                      log2fc_file)
          
        out.mac.pos.corum.pdf <- paste0(output_dir, "/", out.mac.pos.corum.pdf)
        # out.mac.pos.corum.pdf <- 'whatever.corum.positive.pdf'
        .artms_plotCorumEnrichment(
          positiveComplexEnriched,
          out.mac.pos.corum.pdf,
          "MAC+ Protein Complex Enrichment"
        )
      } else{
        if(verbose) message("\t----(-) Not enough positive corum complexes to plot ")
      }
    } else{
      if(verbose) message("\t ------ Nothing is significant in the selected Positive log2fc ")
      mac.pos <- NULL
      positiveComplexEnriched <- NULL
    }
    
    
    # NEGATIVE log2fc
    if(verbose) message("3) Enrichment of selected NEGATIVE significant changes ")
    
    filneg_log2fc_long <- l2fcol4enrichment[which(l2fcol4enrichment$value <= -l2fc_thres),]
    
    if (dim(filneg_log2fc_long)[1] > 0) {
      out.mac.neg <- gsub(".txt", "-enrich-MAC-negatives.txt", log2fc_file)
      out.mac.neg <- paste0(output_dir, "/", out.mac.neg)
      
      mac.neg <- NULL
      tryCatch(
        mac.neg <- artmsEnrichLog2fc(
          dataset = filneg_log2fc_long,
          output_name = out.mac.neg,
          species = species,
          heatmaps = TRUE,
          background = listOfGenes,
          verbose = verbose), 
        error = function(e){
          message("\n\t(Error): Enrichment is not possible! ")
          message("\tgProfiler server night be down or your internet connection is not working")
          enrich <- FALSE
        })
      
      if(!is.null(mac.neg)){
        if (dim(mac.neg)[1] > 0) {
          write.table(
            mac.neg,
            out.mac.neg,
            quote = FALSE,
            sep = "\t",
            row.names = FALSE,
            col.names = TRUE
          )
        }
      }
      
      if(verbose) message("---+ Corum Protein Complex Enrichment Analysis ")
      
      negativesComplexEnriched <- .artms_enrichForComplexes(filneg_log2fc_long, backgroundNumber)
      
      if (dim(negativesComplexEnriched)[1] > 0) {
        out.mac.neg.corum <- gsub(".txt",
                                  "-enrich-MAC-negatives-corum.txt",
                                  log2fc_file)
          
        out.mac.neg.corum <- paste0(output_dir, "/", out.mac.neg.corum)
        write.table(
          negativesComplexEnriched,
          out.mac.neg.corum,
          quote = FALSE,
          sep = "\t",
          row.names = FALSE,
          col.names = TRUE
        )
      }
      
      # And the heatmap
      if (dim(negativesComplexEnriched)[1] > 2) {
        out.mac.neg.corum.pdf <- gsub(".txt",
                                      "-enrich-MAC-negatives-corum.pdf",
                                      log2fc_file)
          
        out.mac.neg.corum.pdf <- paste0(output_dir, "/", out.mac.neg.corum.pdf)
        .artms_plotCorumEnrichment(x = negativesComplexEnriched, 
                                   outfile = out.mac.neg.corum.pdf, 
                                   theTitle = "MAC- Protein Complex Enrichment")
          
      } else{
        if(verbose) message("\t-----(-) Not enough negative corum complexes to plot ")
      }
    } else{
      if(verbose) message("\t------ Nothing is significant in the NEGATIVE site of things ")
      mac.neg <- NULL
      negativesComplexEnriched <- NULL
    }
  } else{
    if(verbose) message(">> NO ENRICHMENT of CHANGES (log2fc) SELECTED ")
    mac.allsig <- NULL
    mac.pos <- NULL
    mac.neg <- NULL
  }
  
  # Superunified data version-----
  superunified <- merge(abundancelongsummean,
                        nbr_long,
                        by = c('Bait', 'Prey'),
                        all = TRUE)
  superunified <- merge(superunified,
                        reprocondition2merge,
                        by = 'Prey',
                        all = TRUE)
  superunified <- merge(superunified,
                        reprospec2merge,
                        by = 'Prey',
                        all = TRUE)
  
  if (grepl("ptm", isPtm)) {
    names(superunified)[grep('^Prey$', names(superunified))] <- 'Uniprot_PTM'
    # Take the Protein ID, but being very careful about the fluomics labeling
    superunified <- .artmsExtractUniprotId(x = superunified, 
                                           uniprotPtmColumn = "Uniprot_PTM", 
                                           newColumnName = "Prey")
  }
  
  suppressMessages(
    superunified <- artmsAnnotationUniprot(superunified, 'Prey', species)
    )
  
  # Rename (before it was just a lazy way to use another code)
  names(superunified)[grep('Bait', names(superunified))] <- 'Condition'
  
  # ANNOTATE SPECIE
  if(verbose) message("--- Annotating species(s) in files ")
  superunified <- artmsAnnotateSpecie(superunified, pathogen, species)

  if (grepl("ptm", isPtm)) {
    if(verbose) message(">> GENERATING EXTENDED DETAILED VERSION OF PH-SITE ")
    imputedDFext <- artmsGeneratePhSiteExtended(df = imputedDF,
                                                pathogen = pathogen,
                                                species = species,
                                                ptmType = isPtm,
                                                output_name = paste0(output_dir, "/", log2fc_file))
  }
  
  # Final OUTPUT FILES------
  if(verbose) message(">> GENERATING FINAL OUTPUT FILES ")
  if (grepl("ptm", isPtm)) {
    names(imputedDF)[grep('Protein', names(imputedDF))] <- 'Uniprot_PTM'
    imputedDF$UniprotID <- imputedDF$Uniprot_PTM
    # The virus labeling has to be taken into account 
    # when getting the uniprot id:
    imputedDF <- .artmsExtractUniprotId(x = imputedDF, 
                                        uniprotPtmColumn = "Uniprot_PTM", 
                                        newColumnName = "UniprotID")
    suppressMessages(
      imputedDF <- artmsAnnotationUniprot(imputedDF, 
                                          'UniprotID', 
                                          species)
      )
                                                         
    names(imputedDF)[grep("Label", names(imputedDF))] <- 'Comparison'
    
    imputedDF <- artmsAnnotateSpecie(imputedDF, pathogen, species)
    
    # Wide version of imputedDF
    ## LEGACY NOT TESTED!
    # imputedDF_wide_log2fc <- data.table::dcast(
    #   data = imputedDF,
    #   Gene + Protein + EntrezID + Uniprot_PTM ~ Comparison,
    #   value.var = 'iLog2FC',
    #   fill = 0
    # )
    
    imputedDF_wide_log2fc <- imputedDF %>%
      tidyr::pivot_wider(id_cols = c(Gene, Protein, EntrezID, Uniprot_PTM), 
                         names_from = Comparison, 
                         values_from = iLog2FC)
      
    ##LEGACY NOT TESTED!
    # imputedDF_wide_pvalue <-
    #   data.table::dcast(
    #     data = imputedDF,
    #     Gene + Protein + EntrezID + Uniprot_PTM ~ Comparison,
    #     value.var = 'iPvalue',
    #     fill = 0
    #   )
    
    imputedDF_wide_pvalue <- imputedDF %>%
      tidyr::pivot_wider(id_cols = c(Gene, Protein, EntrezID, Uniprot_PTM), 
                         names_from = Comparison, 
                         values_from = iPvalue)
    
  } else if (isPtm == "global") {
    suppressMessages(
      imputedDF <- artmsAnnotationUniprot(x = imputedDF, 
                                          columnid = 'Protein', 
                                          species =  species)
      )
                                                         
    names(imputedDF)[grep("Label", names(imputedDF))] <- 'Comparison'
    
    imputedDF <- artmsAnnotateSpecie(imputedDF, pathogen, species)
    
    # Wide version of imputedDF
    ##LEGACY
    # imputedDF_wide_log2fc <- data.table::dcast(data = imputedDF,
    #                                            Gene + Protein + EntrezID ~ Comparison,
    #                                            value.var = 'iLog2FC',
    #                                            fill = 0)
    
    imputedDF_wide_log2fc <- imputedDF %>%
      tidyr::pivot_wider(id_cols = c(Gene, Protein, EntrezID), 
                         names_from = Comparison, 
                         values_from = iLog2FC)
    
    # LEGACY
    # imputedDF_wide_pvalue <- data.table::dcast(data = imputedDF,
    #                                            Gene + Protein + EntrezID ~ Comparison,
    #                                            value.var = 'iPvalue',
    #                                            fill = 0)
    
    imputedDF_wide_pvalue <- imputedDF %>%
      tidyr::pivot_wider(id_cols = c(Gene, Protein, EntrezID), 
                         names_from = Comparison, 
                         values_from = iPvalue)
      
      
  } else{
    stop(" WRONG isPTM SELECTED. 
         OPTIONS AVAILABLE: global, ptmph, ptmsites ")
  }

  # PLOTS: boxplot of relative quantification------
  if(plotTotalQuant){
    if(verbose) message(">> PLOT OUT: TOTAL NUMBER OF PROTEINS/SITES QUANTIFIED")
    numimputedfinal <- gsub(".txt", ".TotalQuantifications.pdf", log2fc_file)
    numimputedfinal <- paste0("plot.", numimputedfinal)
    numimputedfinal <- paste0(output_dir, "/", numimputedfinal)
    if(printPDF) pdf(numimputedfinal)
    .artms_plotNumberProteinsImputedLog2fc(imputedDF)
    if(printPDF) garbage <- dev.off()
  }
  
  # CLUSTERING analysis of quantifications-----
  if(plotClusteringAnalysis){
    if (isPtm == "global") {
      if(verbose) message(">> CLUSTERING ANALYSIS OF QUANTIFICATIONS ")
      
      # PRE-CLUSTERING
      # GET THE LIST OF SIGNIFICANTS FOR THE EXPERIMENT(S)
      list_of_significants <- unique(imputedDF$Protein[which(abs(imputedDF$iLog2FC > 1) &
                                                               imputedDF$iPvalue < 0.05)])
        
      
      # AND APPLY THE FILTER
      data.select <- imputedDF[which(imputedDF$Protein %in% list_of_significants), ]
      
      # GENE BASED -> heatmaps
      ##LEGACY
      # hasdc <- data.table::dcast(data = data.select[which(data.select$imputed == "no"), ],
      #                            Gene + Protein ~ Comparison,
      #                            value.var = "iLog2FC",
      #                            fun.aggregate = median,
      #                            fill = 0)
      # EXPERIMENT BASED -> PCA
      # hasdcexp <- data.table::dcast(data = data.select[which(data.select$imputed == "no"), ],
      #                               Comparison ~ Gene + Protein,
      #                               value.var = "iLog2FC",
      #                               fun.aggregate = median,
      #                               fill = 0)
      
     
      hasdc <- data.select %>% dplyr::filter(imputed == "no") %>%
        tidyr::pivot_wider(id_cols = c(Gene, Protein), 
                           names_from = Comparison, 
                           values_from = iLog2FC, 
                           values_fn = list(iLog2FC = median), 
                           values_fill = list(iLog2FC = 0))
      
      if(dim(hasdc)[1] > 50){
        hasdcexp <- data.select %>% dplyr::filter(imputed == "no") %>%
          dplyr::mutate(Gene_Protein = paste(Gene, Protein, sep = "_")) %>%
          tidyr::pivot_wider(id_cols = Comparison, 
                             names_from = Gene_Protein, 
                             values_from = iLog2FC, 
                             values_fn = list(iLog2FC = median),
                             values_fill = list(iLog2FC = 0))
        
        hasdc <- as.data.frame(hasdc)
        hasdcexp <- as.data.frame(hasdcexp)
        
        
        # CLUSTERING ANALYSIS
        # GENE BASED
        rownames(hasdc) <- paste0(hasdc$Gene, "_", hasdc$Protein)
        vamos <- within(hasdc, rm(Gene, Protein))
        
        # if this dataset only have one comparison,
        # this analysis does not makes sense: check it out:
        if ( dim(vamos)[2] > 2 ) {
          venga <- as.matrix(vamos)
          
          # EXPERIMENT BASED
          rownames(hasdcexp) <- hasdcexp$Comparison
          vamosexp <- within(hasdcexp, rm(Comparison))
          vengaexp <- as.matrix(vamosexp)
          
          # PCA AND CORRELATION ANALYSIS
          if(verbose) message("--- Correlation plots ")
          df.cor.matrix <- round(cor(venga, use = "pairwise.complete.obs"), 2)
          file_corr_l2fc <- gsub(".txt", ".log2fc-corr.pdf", log2fc_file)
          file_corr_l2fc <- paste0(output_dir, "/", file_corr_l2fc)
          if(printPDF) pdf(file_corr_l2fc, width = 12, height = 9)
            corrplot::corrplot(
            df.cor.matrix,
            type = "upper",
            tl.pos = "td",
            method = "circle",
            tl.cex = 0.9,
            tl.col = 'black',
            tl.srt = 45,
            # order = "hclust",
            diag = TRUE
          )
          PerformanceAnalytics::chart.Correlation(venga,
                                                  histogram = TRUE,
                                                  pch = 25,
                                                  main = "Correlation between Comparisons")
          if(printPDF) garbage <- dev.off()
          
          # BASED ON GROUPS
          pca.hasdcexp <- FactoMineR::PCA(
            hasdcexp[, -c(1)],
            scale.unit = FALSE,
            ncp = 4,
            graph = FALSE
          )
          
          pca_all <- factoextra::fviz_pca_ind(pca.hasdcexp,
                                              labelsize = 3,
                                              repel = TRUE,
                                              habillage = as.factor(hasdcexp$Comparison),
                                              addEllipses = FALSE,
                                              ellipse.level = 0.95
          )
          
          
          if(verbose) message("--- PCA, individuals plot ")

          if(printPDF){
            file_pca_l2fc <- gsub(".txt", ".log2fc-individuals-pca.pdf", log2fc_file)
            file_pca_l2fc <- paste0(output_dir, "/", file_pca_l2fc)
            pdf(file_pca_l2fc, width = 9, height = 7)
            print(pca_all)
            garbage <- dev.off()  
          } else{
            print(pca_all)
          }
          
          # Determine the OPTIMAL NUMBER OF CLUSTERS:
          
          # Elbow method
          e1 <- factoextra::fviz_nbclust(venga, kmeans, method = "wss") +
            geom_vline(xintercept = 4, linetype = 2) +
            labs(subtitle = "kmeans Elbow method")
          
          e2 <- factoextra::fviz_nbclust(venga, cluster::pam, method = "wss") +
            geom_vline(xintercept = 4, linetype = 2) +
            labs(subtitle = "PAM Elbow method")
          
          # Silhouette method
          k1 <- factoextra::fviz_nbclust(venga, kmeans, method = "silhouette") +
            labs(subtitle = "kmeans Silhouette method")
          
          k2 <- factoextra::fviz_nbclust(venga, cluster::pam, method = "silhouette") +
            labs(subtitle = "pam Silhouette method")
          
          # Create a dendrogram
          if(verbose) message("--- Dendrogram ")
          res.dist <- factoextra::get_dist(vamosexp, stand = TRUE, method = "minkowski")
          hc <- hclust(res.dist)
          file_dendro_l2fc <- gsub(".txt", ".log2fc-dendro.pdf", log2fc_file)
          file_dendro_l2fc <- paste0(output_dir, "/", file_dendro_l2fc)
          if(printPDF){
            pdf(file_dendro_l2fc, width = 9, height = 7)
            plot(hc)
            garbage <- dev.off()
          }else{
            plot(hc)
          }

          
          # COMPLEXHEATMAP Heatmap with a specified number of optimal clusters
          n = 10
          pam.res <- pam(vamos, k = n)
          
          cp1 <- factoextra::fviz_cluster(pam.res)
          cp2 <- factoextra::fviz_silhouette(pam.res, print.summary = FALSE)
          
          if(verbose) message("--- Plots to determine optimal number of clusters ")
          file_clusterplots_l2fc <- gsub(".txt", ".log2fc-clusters.pdf", log2fc_file)
          file_clusterplots_l2fc <- paste0(output_dir, "/", file_clusterplots_l2fc)
          
          if(printPDF){
            pdf(file_clusterplots_l2fc, width = 9, height = 7)
            print(e1)
            print(e2)
            print(k1)
            print(k2)
            print(cp1)
            print(cp2)
            garbage <- dev.off()
          } else{
            print(e1)
            print(e2)
            print(k1)
            print(k2)
            print(cp1)
            print(cp2)
          }
          
          if(verbose) message("--- Cluster heatmaps (10 clusters) ")
          
          hmap <- ComplexHeatmap::Heatmap(venga,
                                          name = paste0("Clusters ", "(n = ", n, ")"),
                                          col = circlize::colorRamp2(c(-3, 0, 3), c("firebrick1", "black", "olivedrab1")),
                                          heatmap_legend_param = list(color_bar = "continuous",
                                                                      legend_direction = "horizontal",
                                                                      legend_width = unit(5, "cm"),
                                                                      title_position = "topcenter",
                                                                      title_gp = gpar(fontsize = 15, fontface = "bold")),
                                          split = paste0("", pam.res$clustering),
                                          row_title = "Genes",
                                          row_title_side = "left",
                                          row_title_gp = gpar(fontsize = 15, fontface = "bold"),
                                          show_row_names = FALSE,
                                          column_title = "Relative Quantifications",
                                          column_title_side = "top",
                                          column_title_gp = gpar(fontsize = 10, fontface = "bold"),
                                          column_title_rot = 0,
                                          show_column_names = TRUE,
                                          cluster_columns = FALSE,
                                          clustering_distance_columns = function(x) as.dist(1 - cor(t(x))),
                                          clustering_method_columns = "ward.D2",
                                          clustering_distance_rows = "euclidean",
                                          clustering_method_rows = "ward.D2",
                                          row_dend_width = unit(30, "mm"),
                                          column_dend_height = unit(30, "mm"),
                                          # top_annotation=colAnn,
                                          # top_annotation_height = unit(1.75, "cm"),
                                          # bottom_annotation=sampleBoxplot,
                                          # bottom_annotation_height = unit(4, "cm"),
                                          column_names_gp = gpar(fontsize = 10)
          )
          
          file_clusterheat_l2fc <- gsub(".txt", ".log2fc-clusterheatmap.pdf", log2fc_file)
          file_clusterheat_l2fc <- paste0(output_dir, "/", file_clusterheat_l2fc)
          if(printPDF) pdf(file_clusterheat_l2fc,
              width = 12,
              height = 10)
          
          ComplexHeatmap::draw(hmap,
                               heatmap_legend_side = "top",
                               annotation_legend_side = "right")
          if(printPDF) garbage <- dev.off()
          
          # Pre enrichment of clusters
          cl_number <- pam.res$clustering
          dfclusters <- as.data.frame(cl_number)
          dfclusters$ids <- row.names(dfclusters)
          dfclusters$Gene <- gsub("(.*)(_)(.*)", "\\1", dfclusters$ids)
          dfclusters$Protein <- gsub("(.*)(_)(.*)", "\\3", dfclusters$ids)
          
          if(enrich){
            if(verbose) message("--- Enrichment analysis of the clusters ")
            # Making sure we have unique genes in each comparison 
            # (the PTM might bring redundancy)
            pretmp <- dfclusters[c('Gene', 'cl_number')]
            pretmp <- unique(pretmp)
            
            tmp <- split(pretmp$Gene, pretmp$cl_number, drop = TRUE)
            enrichgenes <- NULL
            
            if (species == "human") {
              tryCatch(enrichgenes <- artmsEnrichProfiler(tmp,
                                                          categorySource = c(
                                                            'GO:BP',
                                                            'GO:MF',
                                                            'GO:CC',
                                                            'KEGG',
                                                            'REAC',
                                                            'CORUM',
                                                            'HPA',
                                                            'OMIM'
                                                          ),
                                                          species = 'hsapiens',
                                                          listOfGenes,
                                                          verbose = verbose), 
                       error = function(e){
                         message("\n\t(Error): Enrichment is not possible! ")
                         message("\tgProfiler server night be down or your internet connection is not working")
                         enrich <- FALSE
                       }
              )
              
            } else if (species == "mouse") {
              tryCatch(enrichgenes <- artmsEnrichProfiler(tmp,
                                                          categorySource = c('GO:BP', 
                                                                             'GO:MF', 
                                                                             'GO:CC', 
                                                                             'KEGG', 
                                                                             'REAC', 
                                                                             'CORUM'),
                                                          species = 'mmusculus',
                                                          listOfGenes,
                                                          verbose = verbose),
                       error = function(e){
                         message("\n\t(Error): Enrichment is not possible! ")
                         message("\tgProfiler server night be down or your internet connection is not working")
                         enrich <- FALSE
                       }
              )
            } else{
              stop(species, " is currently not supported in the enrichment")
            }
            
            # Write the file
            if(!is.null(enrichgenes)){
              file_clusterheatenrich_l2fc <- gsub(".txt",
                                                  ".log2fc-clusterheatmap-enriched.txt",
                                                  log2fc_file)
              
              file_clusterheatenrich_l2fc <- paste0(output_dir, "/", file_clusterheatenrich_l2fc)
              write.table(enrichgenes,
                          file_clusterheatenrich_l2fc,
                          col.names = TRUE,
                          row.names = FALSE,
                          sep = "\t",
                          quote = FALSE)
            }
          } #if enrich = TRUE
          
          # Print out clusters
          file_clusterheatdata_l2fc <- gsub(".txt", ".log2fc-clusterheatmap.txt", log2fc_file)
          file_clusterheatdata_l2fc <- paste0(output_dir, "/", file_clusterheatdata_l2fc)
          write.table(
            dfclusters,
            file_clusterheatdata_l2fc,
            col.names = TRUE,
            row.names = FALSE,
            sep = "\t",
            quote = FALSE
          )
        }
        
      }else{
        message("--- (-) Not enought data available to perform clustering analysis")
      }
    } # if (isPtm == "global") {
  }
  # End of clustering analysis
  
  # END: WRITING OUTPUT FILES------
  if(verbose) message(">> WRITING THE OUTPUT FILES")
  
  # PRINT OUT IMPUTED
  outlog2fcImpute <- gsub(".txt", "-log2fc-long.txt", log2fc_file)
  outlog2fcImpute <- paste0(output_dir, "/", outlog2fcImpute)
  write.table(
    imputedDF,
    outlog2fcImpute,
    quote = FALSE,
    sep = "\t",
    row.names = FALSE,
    col.names = TRUE
  )
  
  outlog2fc <- gsub(".txt", "-log2fc-wide.txt", log2fc_file)
  outlog2fc <- paste0(output_dir, "/", outlog2fc)
  write.table(
    log2fc_file_splc,
    outlog2fc,
    quote = FALSE,
    sep = "\t",
    row.names = FALSE,
    col.names = TRUE
  )
  
  outexcel <- gsub(".txt", "-summary.xlsx", log2fc_file)
  outexcel <- paste0(output_dir, "/", outexcel)
  
  if(is.null(mac.allsig) | is.null(mac.pos) | is.null(mac.neg))
    enrich <- FALSE
  
  if (enrich) {
    # But now check whether is a PTM case:
    if (grepl("ptm", isPtm)) {
      list_of_datasets <- list(
        "log2fcImputed" = imputedDF,
        "log2fcImpExt" = imputedDFext,
        "wide_iLog2fc" = imputedDF_wide_log2fc,
        "wide_iPvalue" = imputedDF_wide_pvalue,
        "enrichALL" = mac.allsig,
        "enrichMACpos" = mac.pos,
        "enrichMACneg" = mac.neg,
        "enMACallCorum" = allsigComplexEnriched,
        "enMACposCorum" = positiveComplexEnriched,
        "enMACnegCorum" = negativesComplexEnriched
      )
    } else if (grepl("global", isPtm)) {
      list_of_datasets <- list(
        "log2fcImputed" = imputedDF,
        "wide_iLog2fc" = imputedDF_wide_log2fc,
        "wide_iPvalue" = imputedDF_wide_pvalue,
        "enrichALL" = mac.allsig,
        "enrich-MACpos" = mac.pos,
        "enrich-MACneg" = mac.neg,
        "enMACallCorum" = allsigComplexEnriched,
        "enMACposCorum" = positiveComplexEnriched,
        "enMACnegCorum" = negativesComplexEnriched
      )
      
    } else{
      stop("Wrong option:", isPtm)
    }
  } else if (!enrich) {
    if (grepl("ptm", isPtm)) {
      list_of_datasets <- list(
        "log2fcImputed" = imputedDF,
        "log2fcImpExt" = imputedDFext,
        "wide_iLog2fc" = imputedDF_wide_log2fc,
        "wide_iPvalue" = imputedDF_wide_pvalue
      )
    } else if (grepl("global", isPtm)) {
      list_of_datasets <- list(
        "log2fcImputed" = imputedDF,
        "wide_iLog2fc" = imputedDF_wide_log2fc,
        "wide_iPvalue" = imputedDF_wide_pvalue
      )
    } else{
      stop(isPtm, " is not a valid option")
    }
  } else{
    stop("Contact developers")
    # The script should have crashed by this point. 
    # If it gets up to here... it would be very weird
  }
  
  ## EXCEL: Defining style for the header-----
  hs <- openxlsx::createStyle(fontName = "Arial",
                              fontColour = "white",
                              fgFill = "#000000",
                              textDecoration = "Bold",
                              border = "Bottom")
  openxlsx::write.xlsx(
    list_of_datasets,
    file = outexcel,
    asTable = FALSE,
    headerStyle = hs, 
    overwrite = TRUE
  )
  
  if(verbose) message("Folder <", output_dir, "> ")
  if(verbose) message("- EXCEL: ", outexcel, " ")
  if(verbose) message("- Log2fc Wide: ", outlog2fc, " ")
  if(verbose) message("- Log2fc Impute: ", outlog2fc, " ")
  
  if (enrich == TRUE) {
    if(verbose) message("- ENRICHMENT files should also be out ")
  }
  
  if(verbose) message(">> SUPER ANALYSIS COMPLETED")
}


#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' @title Adding a column with the species name
#'
#' @description Adding the species name to every protein.
#' This makes more sense if there are more than one species in the dataset,
#' which must be specified in the `pathogen` option. Influenza is a special
#' case that it does not need to be specified, as far as the proteins were
#' originally annotated as `INFLUENZAGENE_STRAIN`
#' (strains covered `H1N1`, `H3N2`, `H5N1`), as for example, `NS1_H1N1`
#' @param df (data.frame) with a `Protein` column (of uniprot ids)
#' @param pathogen (char) Is there a pathogen in the dataset as well?
#' if it does not, then use `pathogen = nopathogen` (default). Supported`tb` 
#' (Tuberculosis),
#' `lpn` (Legionella)
#' @param species (char) Host organism (supported for now: `human` or `mouse`)
#' @return (data.frame) The same data.frame but with an extra column 
#' specifying the species
#' @keywords annotation, species
#' @examples
#' # Adding a new column with the main species of the data. Easy.
#' # But the main functionality is to add both the host-species and a pathogen,
#' # which is not illustrated in this example
#' data_with_specie <- artmsAnnotateSpecie(df = artms_data_ph_msstats_results,
#'                                          species = "human")
#' @export
artmsAnnotateSpecie <- function(df,
                                 pathogen = "nopathogen",
                                 species) {
  
  if(any(missing(df) | missing(species)))
    stop("Missed (one or many) required argument(s).
         Please, check the help of this function to find out more")
  
  if (pathogen == "nopathogen") {
    # Influenza is treated differently
    df$Species <-
      ifelse(grepl("_H1N1|_H3N2|_H5N1", df$Protein),
             "Influenza",
             species)
  } else if (pathogen == "tb") {
    pathogen.ids <- artms_data_pathogen_TB
    df$Species <-
      ifelse(df$Protein %in% pathogen.ids$Entry, pathogen, species)
  } else if (pathogen == "lpn") {
    pathogen.ids <- artms_data_pathogen_LPN
    df$Species <-
      ifelse(df$Protein %in% pathogen.ids$Entry, pathogen, species)
  }
  return(df)
}

# 
#' @title Generate ph-site specific detailed file
#'
#' @description Generate extended detailed ph-site file, where every line is a
#' ph site instead of a peptide. Therefore, if one peptide has multiple ph sites
#' it will be breaking down in each of the sites. This file will help generate
#' input files for tools as [Phosfate](http://phosfate.com/) or
#' [PHOTON](https://github.com/jdrudolph/photon)
#' @param df (data.frame) of log2fc and imputed values
#' @param pathogen (char) Is there a pathogen in the dataset as well? Available
#' pathogens are `tb` (Tuberculosis), `lpn` (Legionella). If it is not,
#' then use `nopathogen` (default).
#' @param species (char) Main organism (supported for now: `human` or `mouse`)
#' @param ptmType (char) It must be a ptm-site quantification dataset. Either:
#' yes: `ptmsites` (for site specific analysis), or
#' `ptmph` (Jeff's script output evidence file).
#' @param output_name (char) A output file name (extension `.txt` required)
#' @return (data.frame) extended version of the ph-site
#' @keywords external, tools, phosfate
#' @examples \dontrun{
#' artmsGeneratePhSiteExtended(df = dfobject, 
#'                              species = "mouse", 
#'                              ptmType = "ptmsites",
#'                              output_name = log2fc_file)
#' }
#' @export
artmsGeneratePhSiteExtended <- function(df,
                                        pathogen = "nopathogen", 
                                        species, 
                                        ptmType,
                                        output_name) {
                                         
  # #Debug
  # df = artms_data_ph_msstats_results
  # pathogen = "nopathogen"
  # species = "human"
  # ptmType = "ptmsites"
  # output_name = ""
  
  if(any(missing(df) | 
         missing(species) | 
         missing(ptmType) | 
         missing(output_name)))
    stop("Missed (one or many) required argument(s)
         Please, check the help of this function to find out more")
    
    imputedDFext <- NULL
    
    imputedDFext <- .artms_checkIfFile(input_file = df)
    
    if (ptmType == "ptmph") {
      # Change the Protein name to Uniprot_PTM (if it is not there already)
      if(!any(grepl("Uniprot_PTM", colnames(imputedDFext)))) 
        names(imputedDFext)[grep('^Protein$', names(imputedDFext))] <- 'Uniprot_PTM'
      
      # Take the Protein ID, but being very careful about the fluomics labeling
      imputedDFext <- 
        .artmsExtractUniprotId(x = imputedDFext, 
                               uniprotPtmColumn = "Uniprot_PTM", 
                               newColumnName = "Protein")
      
      # Extract sites from Uniprot_PTM
      imputedDFext$PTMsite <-
        gsub("^(\\S+?)(_ph.*)", "\\2", imputedDFext$Uniprot_PTM, perl = TRUE)
      imputedDFext$PTMsite <- gsub("^(_ph)", "", imputedDFext$PTMsite)
      imputedDFext$PTMsite <- gsub("_ph", ",", imputedDFext$PTMsite)
      
      # And create independent columns for each of them
      imputedDFext <- imputedDFext %>% mutate(PTMsite = strsplit(PTMsite, ",")) %>% tidyr::unnest(PTMsite)
      
      if( !any(grepl("Gene", colnames(imputedDFext))) ){
        suppressMessages(
          imputedDFext <- artmsAnnotationUniprot(imputedDFext, 
                                                 'Protein', 
                                                 species)
        )
      }
      
      if( any(grepl("Label", colnames(imputedDFext))) ){
        names(imputedDFext)[grep("^Label$", names(imputedDFext))] <- 'Comparison'
      }

      imputedDFext <- artmsAnnotateSpecie(imputedDFext, pathogen, species)
    } else if (ptmType == "ptmsites") {
      #1. Change the Protein name to Uniprot_PTM (if it is not there already)
      if(!any(grepl("Uniprot_PTM", colnames(imputedDFext)))) 
        names(imputedDFext)[grep('^Protein$', names(imputedDFext))] <- 
        'Uniprot_PTM'
      
      # 2. Make a copy of Uniprot_PTM to operate on it
      imputedDFext$PTMone <- imputedDFext$Uniprot_PTM
      
      # 3. Create independent columns for each of them
      imputedDFext <- imputedDFext %>% 
        dplyr::mutate(PTMone = strsplit(PTMone, ",")) %>% 
        tidyr::unnest(PTMone)
      
      # 4. And take the labels:
      imputedDFext$Protein <-
        ifelse(
          grepl("_H1N1|_H3N2|_H5N1", imputedDFext$PTMone),
          gsub("^(\\S+?_H[1,3,5]N[1,2])_.*", "\\1",
               imputedDFext$PTMone, perl = TRUE
          ) ,
          gsub("^(\\S+?)_.*", "\\1", imputedDFext$PTMone, perl = TRUE)
        )
      imputedDFext$PTMaa <-
        gsub("(\\S+)(_)([S,T,Y,K])(\\d+)", "\\3", imputedDFext$PTMone)
      imputedDFext$PTMsite <-
        gsub("(\\S+)(_)([S,T,Y,K])(\\d+)", "\\4", imputedDFext$PTMone)
      
      imputedDFext$PTMone <- NULL
      
      if( !any(grepl("Gene", colnames(imputedDFext))) ) {
        suppressMessages(
          imputedDFext <- artmsAnnotationUniprot(imputedDFext, 'Protein', species)
        )
      }
      
      if( any(grepl("Label", colnames(imputedDFext))) ) {
        names(imputedDFext)[grep("^Label$", names(imputedDFext))] <- 'Comparison'
      }
      
      # imputedDFext$Species <- ifelse(grepl("_H1N1|_H3N2|_H5N1", 
      # imputedDFext$Protein), "Influenza", species)
      # imputedDFext$Species <- ifelse(imputedDFext$Protein 
      # %in% pathogen.ids$Entry, pathogen, species)
      
      if( !any(grepl("Species", colnames(imputedDFext))) ){
        suppressMessages(
          imputedDFext <- artmsAnnotateSpecie(imputedDFext, "Protein", species)
        )
      }
    }else{
      stop( "--- (!) Only 'ptmph' or 'ptmsites' allowed for argument <ptmType> ")
    }
      
    outlog2fcImputext <- gsub(".txt", "-imputedL2fcExtended.txt", output_name)
    
    write.table(
      imputedDFext,
      outlog2fcImputext,
      quote = FALSE,
      sep = "\t",
      row.names = FALSE,
      col.names = TRUE
    )
    return(imputedDFext)
  }


# 
# @title Imputing missing values
#
# @description When a value is completely missed in one of the conditions,
# the `log2fc = Inf / -Inf`. This function imputes those values, i.e.,
# will assign 'artificial' values.
# The imputation method works as follow. The assumption is that those
# proteins are likely present as well in those conditions where are found
# missed, but due to the small sampling (usually 2 or 3 biological replicas)
# and other proteomics related issues, those proteins didn't make it through
# the level of detection.
# Therefore, a small intensity (sampled from the bottom 10 intensity values)
# will be assigned to the protein/site in the missing condition,
# and the new log2fc is re-calculated out of the MSstats box.
# Two issues are addressed as follows:
# 1. If a protein has been consistently identified in one of the conditions,
# it will stay
# 2. But if the intensity value in those conditions was too low,
# then the log2fc will be also low
# @param dflog2fcinfinites data.frame of proteins with only infinite
# values from the msstats results file
# @param dfmq Abundance data, which will be used to know the details of
# reproducibility
# @return Imputed missing values
# @keywords internal, imputation, log2fc, quantifications, missing values
.artms_imputeMissingValues <- function(dflog2fcinfinites, 
                                       dfmq) {
  
  # The comparsions
  contrast <- unique(dflog2fcinfinites$Label)
  
  # Select the IDs to impute
  ids2impute <- unique(dflog2fcinfinites$Protein)
  
  # Take the abundance values for all the proteins
  abu2imp <- .artms_loadModelqcBasic(dfmq)
  
  # Aggregate the technical replica by choosing the maximum value
  abu2imp <- aggregate(Abundance ~ Protein + Condition + BioReplicate,
                        data = abu2imp,
                        FUN = mean)
    
  
  # Check things that will be imputed
  # dfdc.ni <- data.table::dcast(data=abu2imp2, 
  # Protein~BioReplicate, value.var = "Abundance")
  
  # Two possible options here.
  # 1. Select based on the bottom x%
  # # Imputing the missing values by selecting randomly from the bottom 5%
  # theMin <- min(dfmq$ABUNDANCE)
  # # Select the 5% quartile as the maximum value to sample from
  # theMax <- quantile(dfmq$ABUNDANCE, probs = .05)
  #
  # 2. Select the bottom 20 intensities
  # Grab the bottom 30 intensities in the dataset
  dfmqOrdered <- dfmq[order(dfmq$ABUNDANCE, decreasing = FALSE), ]
  
  numberFromBottom <- 5
  abuBottom <- head(dfmqOrdered$ABUNDANCE, n = numberFromBottom)
  
  theMin <- abuBottom[1]
  theMax <- abuBottom[numberFromBottom]
  
  # Generating the numbers from which we are going to sample
  numbers2sample <- seq(from = theMin, to = theMax, by = .00001)
  
  ##LEGACY
  # # dcast on abundance and fill with random numbers between the minimum and q05
  # suppressWarnings(
  #   dfdc.im <- data.table::dcast(data = abu2imp2,
  #                                Protein ~ BioReplicate,
  #                                value.var = "Abundance",
  #                                fill = sample(numbers2sample, 
  #                                              replace = FALSE)
  #   )
  # )
  
  dfdc.im <- abu2imp %>% 
    dplyr::select(-Condition) %>%
    tidyr::pivot_wider(names_from = BioReplicate,
                       values_from = Abundance)
  
  dfdc.im <- as.data.frame(dfdc.im)

  dfdc.im[] <- Map(function(x, y) replace(x, is.na(x), y), 
                   dfdc.im, 
                   sample(numbers2sample, 
                          size = dim(dfdc.im)[2], 
                          replace = FALSE))

  # Needs to aggregate on biological replicas
  # 1. Melt on biological replicas
  # Legacy
  # dfdc.melt <- data.table::melt(dfdc.im,
  #                               id.vars = c('Protein'),
  #                               value.name = 'Abundance',
  #                               variable.name = 'BioReplicate')
  
  dfdc.melt <- dfdc.im %>% 
    tidyr::pivot_longer(cols = -c(Protein), 
                        names_to = "BioReplicate", 
                        values_to = "Abundance")

  # 2. Get the condition
  dfdc.melt$Condition <- gsub("(.*)(-)(.*)", "\\1", dfdc.melt$BioReplicate)
    
  # 3. Dcast and Aggregate on the condition, taking the mean
  ##LEGACY
  # dfdc.final <-
  #   data.table::dcast(
  #     data = dfdc.melt,
  #     Protein ~ Condition,
  #     value.var = 'Abundance',
  #     fun.aggregate = mean
  #   )
  
  dfdc.final <- dfdc.melt %>% 
    dplyr::select(-BioReplicate) %>%
    tidyr::pivot_wider(names_from = Condition, 
                values_from = Abundance,
                values_fn = list(Abundance = mean))
                                           
  # 4. Filter by proteins to impute
  dfdc.final <- dfdc.final[which(dfdc.final$Protein %in% ids2impute), ]
    
  
  for (c in contrast) {
    # message("\t",c," --> ")
    x <- gsub("(.*)(-)(.*)", "\\1", c)
    y <- gsub("(.*)(-)(.*)", "\\3", c)
    # message("log2fc(",x, " - ", y,") ")
    
    # Renaming the comparision name just for illustration purposes
    rnc <- paste0("l2fc_", c)
    
    dfdc.final[[rnc]] <- dfdc.final[[x]] - dfdc.final[[y]]
  }
  
  # Select only log2fc columns
  imputedL2FCValues <- dfdc.final[grepl("Protein|l2fc_", colnames(dfdc.final))]
  
  # Melt again
  # LEGACY
  # imputedL2FCmelted <- data.table::melt(imputedL2FCValues,
  #                                       id.vars = c('Protein'),
  #                                       variable.name = 'Label',
  #                                       value.name = 'iLog2FC')
  
  imputedL2FCmelted <- imputedL2FCValues %>%
    tidyr::pivot_longer(cols = -Protein,
                        names_to = "Label",
                        values_to = "iLog2FC")
    
    
  # Now let's get it ready for merging with the values to be 
  # imputed at dflog2fcinfinites
  imputedL2FCmelted$Label <- gsub("l2fc_", "", imputedL2FCmelted$Label)
    
  
  # And let's add p-values
  samplingPvalue <- seq(from = 0.01, to = 0.05, by = .0000001)
  # And add imputed pvalues
  imputedL2FCmelted$iPvalue <- sample(samplingPvalue,
                                      size = nrow(imputedL2FCmelted),
                                      replace = FALSE)
  
  imputedL2FCmelted <- as.data.frame(imputedL2FCmelted)
  
  return (imputedL2FCmelted)
}

# 
# @title Load limited columns from abundance (modelqc) annotated
#
# @description Load limited columns from abundance (modelqc) annotated
# @param df (data.frame) with the raw abundance data (modelqc)
# @param species (char) Species name for annotation purposes
# @param ptmis (char) Specify whether is a PTM dataset: `global`, `ptmsites`,
# `ptmph`
# @param verbose (logical) `TRUE` (default) shows function messages
# @return annotated data.frame of abundance data
# @keywords abundance, annotated
.artms_loadModelQCstrict <- function (df, 
                                      species, 
                                      ptmis,
                                      verbose = TRUE) {
  
  colnames(df) <- toupper(colnames(df))
  
  # Remove empty entries
  if (any(df$PROTEIN == "")) {
    df <- df[-which(df$PROTEIN == ""), ]
  }
  df$PROTEIN <- gsub("(^sp\\|)(.*)(\\|.*)", "\\2", df$PROTEIN)
  df$PROTEIN <- gsub("(.*)(\\|.*)", "\\1", df$PROTEIN)
  
  # Technical replicas: aggregate on the mean the technical replicas
  b <- aggregate(ABUNDANCE ~ PROTEIN + GROUP + SUBJECT,
                 data = df,
                 FUN = mean)
  
  ##LEGACY
  # datadc <- data.table::dcast(data = b,
  #                             PROTEIN ~ GROUP,
  #                             value.var = 'ABUNDANCE',
  #                             fun.aggregate = mean)
  
  datadc <- b %>% pivot_wider(id_cols = PROTEIN, 
                              names_from = GROUP, 
                              values_from = ABUNDANCE, 
                              values_fn = list(ABUNDANCE = mean))
                               
    
  
  names(datadc)[grep('PROTEIN', names(datadc))] <- 'Protein'
  
  if (grepl("ptm", ptmis)) {
    # if is a PTM dataset we don't need the real gene names for now,
    # we need to use the Uniprot_ptm notation
    datadc$Gene <- datadc$Protein
    send_back <- datadc
  } else{
    suppressMessages(
      send_back <- artmsAnnotationUniprot(x =  datadc, 
                                          columnid = 'Protein', 
                                          species = species)
      )
  }
  return(send_back)
}


#
# @title Load the basic ModelQC file
#
# @param x (data.frame) of the ModelQC file
# @return (data.frame) of the modelqc file with the columns Protein, Abundance,
# Condition, BioReplicate
# @keywords internal, loading
.artms_loadModelqcBasic <- function(x) {
  ## Not removing protein groups for now
  # if (length(grep(";", x$PROTEIN)) > 0)
  #   x <- x[-grep(";", x$PROTEIN), ]
  
  if ("PROTEIN" %in% colnames(x)) {
    names(x)[grep("PROTEIN", names(x))] <- 'Protein'
  } else{
    stop("<PROTEIN> column not found")
  }
  if ("ABUNDANCE" %in% colnames(x)) {
    names(x)[grep("ABUNDANCE", names(x))] <- 'Abundance'
  } else{
    stop("<ABUNDANCE> protein not found!")
  }
  if ("GROUP" %in% colnames(x)) {
    names(x)[grep("GROUP", names(x))] <- 'Condition'
  } else{
    stop("<GROUP> not found")
  }
  if ("SUBJECT" %in% colnames(x)) {
    names(x)[grep("SUBJECT", names(x))] <- 'BioReplicate'
  } else{
    stop("<SUBJECT> not found")
  }
  x <- subset(x, select = c(Protein, Abundance, Condition, BioReplicate))
  return(x)
}

# 
# @title Merge abundance and number of biological replicates per condition
#
# @description Merge abundance and number of biological replicates
# per condition
# @param df_input (data.frame) Abundance input file
# @param repro (data.frame) Reproducibility data.frame
# @param species (char) Species for annotation purposes
# @return (data.frame) of abundance merged with reproducibility info
# @keywords abundance, reproducibility, merging
.artms_mergeAbNbr <- function (df_input, repro, species) {
  # Remove empty entries
  if (any(df_input$PROTEIN == "")) {
    df_input <- df_input[-which(df_input$PROTEIN == ""), ]
  }
  df_input$PROTEIN <- gsub("(^sp\\|)(.*)(\\|.*)", "\\2", df_input$PROTEIN)
  df_input$PROTEIN <- gsub("(.*)(\\|.*)", "\\1", df_input$PROTEIN)
  
  # TECHNICAL REPLICAS: if there are technical replicas, 
  # this means that we will find
  # two values for the same protein in the same bioreplica, 
  # therefore we need to aggregate first just in case:
  df_input <- aggregate(ABUNDANCE ~ PROTEIN + GROUP + SUBJECT,
                        data = df_input,
                        FUN = mean)
    
  ##LEGACY
  # dc_input <- data.table::dcast( data = df_input[, c('PROTEIN', 'ABUNDANCE', 'GROUP')],
  #                                PROTEIN ~ GROUP,
  #                                value.var = 'ABUNDANCE',
  #                                fun.aggregate = mean,
  #                                fill = 0)
  
  dc_input <- df_input %>% pivot_wider(id_cols = PROTEIN, 
                                        names_from = GROUP, 
                                        values_from = ABUNDANCE, 
                                        values_fn = list(ABUNDANCE = mean), 
                                        values_fill = list(ABUNDANCE = 0))
   
    
  names(dc_input)[grep('PROTEIN', names(dc_input))] <- 'Protein'
  
  colnames(repro) <- paste("NumBR", colnames(repro), sep = "_")
  colnames(repro)[1] <- 'Protein'
  dc_input <- merge(dc_input, repro, by = c('Protein'))
  
  return(dc_input)
}

# 
# @title Merge changes (log2fc) and number of biological replicates per
# condition
#
# @description Merge changes, i.e., MSstats results file of quantified changes,
# with the number of biological replicates per condition
# @param df_input Changes data.frame
# @param repro Reproducibility data.frame
# @param species Species for annotation purposes
# @param choosePvalue pvalue to use when making wider the dataframe
# @return Merged data.frame of changes and reproducibility information
# @keywords changes, log2fc, reproducibility, merging
.artms_mergeChangesNbr <- function (df_input, repro, species, choosePvalue) {
  # # Remove the weird empty proteins
  # if(any(df_input$Protein == "")){ df_input <- 
  # df_input[-which(df_input$Protein == ""),]}
  # df_input$Protein <- gsub("(^sp\\|)(.*)(\\|.*)", "\\2", df_input$Protein )
  # df_input$Protein <- gsub("(.*)(\\|.*)", "\\1", df_input$Protein )
  
  ##LEGACY
  # input_melt = data.table::melt(data = df_input[, c('Protein', 
  #                                                 'Label', 
  #                                                 'log2FC', 
  #                                                 'adj.pvalue'), ], 
  #                               id.vars = c('Protein', 'Label'))
  
  input_melt <- df_input %>%
    dplyr::select(c(Protein, 
                    Label, 
                    log2FC, 
                    !!!choosePvalue)) %>%
    tidyr::pivot_longer(cols = -c(Protein, Label), 
                 names_to = "variable", 
                 values_to = "value")
  ##LEGACY
  # input_dcast <- data.table::dcast(Protein ~ Label + variable,
  #                                 data = input_melt,
  #                                 value.var = c('value'))
  
  input_dcast <- input_melt %>% 
    dplyr::mutate(Label_variable = paste(Label, variable, sep = "_")) %>%
    tidyr::pivot_wider(id_cols = c(Protein),
                       names_from = Label_variable, 
                       values_from = value)
  
  colnames(repro) <- paste("NumBR", colnames(repro), sep = "_")
  colnames(repro)[1] <- 'Protein'
  input_dcast <- merge(input_dcast, repro, by = c('Protein'))
  
  # Move Gene name to the left:
  return(input_dcast)
}


# @title Filter: Remove proteins below some threshold of minimal reproducibility
#
# @description If a protein is not found in a minimal number of
# biological replicates in at least one of the conditions, it is removed
# @param dfi (data.frame) Data.frame with biological replicates information
# @param mnbr (int) minimal number of biological replicates
# @return (data.frame) a filtered `dfi`
# @keywords internal, filter, bioreplicates, reproducibility
.artms_RemoveProtBelowThres <- function(dfi, mnbr) {
  theComparisons2check <- unique(dfi$Label)
  for (onlyonecomp in (theComparisons2check)) {
    ax <- gsub("(.*)(-)(.*)", "\\1", onlyonecomp)
    ay <- gsub("(.*)(-)(.*)", "\\3", onlyonecomp)
    
    # If the condition is not met, i.e., if all the proteins are
    # found in at least X biological replicas, then
    # it would remove the whole thing.
    if (dim(dfi[which((dfi[[ax]] < mnbr) & (dfi[[ay]] < mnbr)), ])[1] > 0) {
      dfi <- dfi[-which(((dfi[[ax]] < mnbr) & (dfi[[ay]] < mnbr)) & (dfi$Label == onlyonecomp)), ]
    }
  }
  return(dfi)
}

# 
# @title Select and label the condition more abundant in a quantification
#
# @description Select and label the condition more abundant in a quantification
# - If log2fc > 0 the condition on the left ('numerator') is the most abundant
# - If log2fc < 0 the condition on the right ('denominator') 
# is the most abundant
# @param a (char) log2fc column
# @param b (char) comparison column
# @return (char) One of the conditions from the comparison
# @keywords internal, selection, labeling
.artms_selectTheOneLog2fc <- function(a, b) {
  thrs <- 0
  # a should be the log2fc column
  if (a > thrs) {
    sb <- gsub("(.*)(-)(.*)", "\\1", b)
  } else if (a < -thrs) {
    sb <- gsub("(.*)(-)(.*)", "\\3", b)
  } else {
    sb <- 'NA'
  }
  return(sb)
}



# 
# @title Extract Uniprot ID from a UNIPROT_PTM  identifier
#
# @param x The data.frame with the uniprot_ptm column
# @param uniprotPtmColumn (char) Column in the data.frame with the Uniprot_PTM
# IDs
# @param newColumnName (char) Name of the new column that will contain the
# Uniprot ids
# @return (char) One of the conditions from the comparison
# @keywords internal, selection, labeling
.artmsExtractUniprotId <- function(x, uniprotPtmColumn, newColumnName){
  if(any(missing(x) | missing(newColumnName) | missing(uniprotPtmColumn)))
    stop("Missed (one or many) required argument(s)
         Please, check the help of this function to find out more")
  
  if(!is.character(newColumnName)) 
    stop("Argument 'newColumnName' must be a character")
  if(!is.character(uniprotPtmColumn)) 
    stop("Argument 'uniprotPtmColumn' must be a character")
  
  x[[newColumnName]] <- ifelse(
    grepl("_H1N1|_H3N2|_H5N1", 
          x[[uniprotPtmColumn]]),
    gsub("^(\\S+?_H[1,3,5]N[1,2])_.*",
         "\\1",
         x[[uniprotPtmColumn]],
         perl = TRUE) ,
    gsub("^(\\S+?)_.*", 
         "\\1", 
         x[[uniprotPtmColumn]], 
         perl = TRUE) 
  )
  return(x)
}

# ------------------------------------------------------------------------------
#' @title MaxQuant evidence file to SAINTq format
#'
#' @description Converts the MaxQuant evidence file to the required files
#' by SAINTq. The user can choose to use either peptides with `spectral counts`
#' (use `msspc`) or the all the peptides (use `all`) for the analysis. 
#' The quantitative can be also chosen (either MS Intensity or Spectral Counts)
#' @param evidence_file (char or data.frame) The evidence file path and name, 
#' or data.frame
#' @param keys_file (char) Keys file with a SAINT column specifying
#' test (`T`) and control (`C`) conditions
#' @param output_dir (char) New directory to create and save files. 
#' Default is current directory (recommended to provide a new folder name).
#' @param sc_option (char). Filter peptides with spectral counts only.
#' Two options:
#' - `msspc`: use only peptides with spectral_counts
#' - `all` (default): all peptides detected (including the one resulting from 
#' the MaxQuant 'Match between run' algorithm)
#' @param quant_variable (char) Select the quantitative variable. 
#' Two options available:
#' - `msint`: MS Intensity (default)
#' - `msspc`: MS.MS.count (Spectral Counts)
#' @param fractions (logical) `TRUE` for 2D proteomics (fractions). 
#' Default: `FALSE`
#' @param verbose (logical) `TRUE` (default) shows function messages
#' @return The input files requires to run SAINTq
#' @details After running the script, the new specified folder should contain
#' the folling files:
#' - saintq-config-peptides
#' - saintq-config-proteins
#' - saintq_input_peptides.txt
#' - saintq_input_proteins.txt
#' 
#' Then `cd` into the new folder and run either of the following two options
#' (assuming that `saintq` is installed in your linux/unix/mac os x system):
#' 
#' `> saintq config-saintq-peptides`
#' 
#' or
#' 
#' `> saintq config-saintq-proteins`
#' @keywords SAINT, SAINTq, APMS
#' @examples
#' # Testing that the files cannot be empty
#' artmsEvidenceToSAINTq   (evidence_file = NULL, 
#'                                    keys_file = NULL, 
#'                                    output_dir = NULL)
#' @export
artmsEvidenceToSAINTq    <- function(evidence_file,
                                     keys_file,
                                     output_dir = "artms_saintq",
                                     sc_option = c("all", "msspc"),
                                     fractions = FALSE,
                                     quant_variable = c('msint','msspc'),
                                     verbose = TRUE){

  Sequence = NULL

  if(verbose){
    message(">> GENERATING A SAINTq INPUT FILE ")
    message(">> CHECKING THE keys FILE FIRST ")
  }  
  
  if(is.null(evidence_file) & is.null(keys_file) & is.null(output_dir)){
    return("The evidence_file, keys_file and output_dir must not be NULL")
  }
  
  if(any(missing(evidence_file) | missing(keys_file)))
    stop("Missed (one or many) required argument(s)
         Please, check the help of this function to find out more")
  
  keys <- .artms_checkIfFile(keys_file)
  keys <- .artms_checkRawFileColumnName(keys)
  
  if(fractions){
    if(verbose) message("--- VERIFYING THAT THE INFORMATION ABOUT fractions IS AVAILABLE ")
    requiredColumns <- c('RawFile',
                         'IsotopeLabelType',
                         'Condition',
                         'BioReplicate',
                         'Run', 
                         'Fraction',
                         'SAINT')
    if(any(! requiredColumns %in% colnames(keys)))
      stop('Column names in keys not conform to schema. Required columns:', 
           sprintf('\t%s\n',requiredColumns))
  }else{
    requiredColumns <- c('RawFile','IsotopeLabelType','Condition',
                         'BioReplicate','Run', 'SAINT')
    if(any(! requiredColumns %in% colnames(keys)))
      stop('Column names in keys not conform to schema. Required columns:', 
           sprintf('\t%s\n', requiredColumns))
  }
  
  # EVIDENCE:
  datamerged <- artmsMergeEvidenceAndKeys(evidence_file, 
                                           keys_file,
                                           verbose = verbose)
  
  # SELECTING THE Leading.razor.protein
  
  datamerged <- subset(datamerged, select = -Proteins)
  if( ('Leading.razor.protein' %in% colnames(datamerged)) ) {
    if(verbose) message('--- Making the <Leading.Razor.Protein> the <Proteins> column ')
    names(datamerged)[grep('Leading.razor.protein', 
                           names(datamerged))] <- 'Proteins'
  } else if('Leading.Razor.Protein' %in% colnames(datamerged) ) {
    if(verbose) message('--- Making the <Leading.Razor.Protein> the <Proteins> column ')
    names(datamerged)[grep('Leading.Razor.Protein', names(datamerged))] <-
      'Proteins'
  } else{
    stop("there is no <leading.razor.protein> column in this evidence file.")
  }
  
  datamerged$Proteins <- gsub("(sp\\|)(.*)(\\|.*)", "\\2", datamerged$Proteins )

  ## ONLY VALUES WITH SPECTRAL COUNTS
  sc_option <- match.arg(sc_option)
  if(sc_option == "msspc"){
    if(verbose) message("--- Selecting peptides with spectral count only ")
    before <- dim(datamerged)[1]
    datamerged <- datamerged[which(datamerged$MS.MS.Count > 0),]
    after <- dim(datamerged)[1]
    keepingPercent <- (after*100)/before
    if(verbose){
      message("\t+--> Before:", before," ")
      message("\t+--> After:", after," (Keeping:", keepingPercent,"%) ")
    }
  }else if(sc_option == "all"){
    if(verbose)
      message("--- ALL peptides with intensities will be used to generate the 
      saintq input file (indepependently of the number of spectral counts ")
  }else{stop("<sc_option> argument must be either 'sc' or 'all'")
  }

  # Remove empty proteins
  if(verbose) message("--- Removing empty protein ids (if any) ")
  if(length(which(datamerged$Proteins==""))>0){
    datamerged <- datamerged[-which(datamerged$Proteins==""),]
  }
  
  # Remove protein groups
  if(verbose) message("--- Removing Protein Groups (if any) ")
  datamerged <- .artms_removeMaxQProteinGroups(datamerged)
  
  # Removing Contaminants
  if(verbose) message("--- Removing contaminants")
  data_f2 <- artmsFilterEvidenceContaminants(datamerged,
                                              verbose = verbose)
  
  quant_variable <- match.arg(quant_variable)
  if(quant_variable == "msint"){
    # Set the intensity as numeric to avoid overflow problems
    data_f2$Intensity = as.numeric(data_f2$Intensity)
    data_f2 <- data_f2[!is.na(data_f2$Intensity),]
  }else if(quant_variable == "msspc"){
    # hack to use spectral counts instead of intensities
    data_f2$Intensity <- NA
    data_f2$Intensity <- data_f2$MS.MS.Count
    data_f2$Intensity = as.numeric(data_f2$Intensity)
    data_f2 <- data_f2[!is.na(data_f2$Intensity),]
  }
  
  # create output directory if it doesn't exist
  if (!dir.exists(output_dir)) {
    dir.create(output_dir, recursive = TRUE)
  }
  
  # Combine all the fractions if this is a fractioning experiment by 
  # summing them up
  if (fractions){
    # Sum up all the fractions first
    data_f2_fa <- aggregate(
      Intensity~Sequence+Proteins+Condition+BioReplicate+Run, 
      data=data_f2, 
      FUN = sum)
  }else{
    data_f2_fa <- data_f2
  }

  # Take only unique values of both sequences and proteins.
  data_f2uniques <- unique(data_f2_fa[,c("Proteins", "Sequence")])
  protPep <- aggregate(Sequence ~ Proteins, 
                       data_f2uniques, 
                       FUN = paste, collapse="|" )
  
  # Using Intensities for both Peptides and Proteins
  ##LEGACY
  # protBiorepIntensity  <- data.table::dcast(data=data_f2_fa[,c("Proteins","BioReplicate","Intensity")], 
  #                                           Proteins~BioReplicate, 
  #                                           value.var = "Intensity", sum, na.rm = TRUE, fill=0 )
  
  protBiorepIntensity <- data_f2_fa[,c("Proteins","BioReplicate","Intensity")] %>%
    tidyr::pivot_wider(id_cols = c(Proteins, BioReplicate), 
                       names_from = BioReplicate, 
                       values_from = Intensity, 
                       values_fn = list(Intensity = sum), 
                       values_fill = list(Intensity = 0))
  protBiorepIntensity[is.na(protBiorepIntensity)] <- 0
  
  ##LEGACY
  # peptideBiorepIntensity <- data.table::dcast(
  #   data=data_f2_fa[,c("Proteins","Sequence","BioReplicate","Intensity")], 
  #   Proteins+Sequence~BioReplicate, 
  #   value.var = "Intensity", fun.aggregate = sum, na.rm = TRUE, fill=0 )
  
  peptideBiorepIntensity <- data_f2_fa[,c("Proteins", "Sequence", "BioReplicate","Intensity")] %>%
    tidyr::pivot_wider(id_cols = c(Proteins, Sequence), 
                       names_from = BioReplicate, 
                       values_from = Intensity, 
                       values_fn = list(Intensity = sum), 
                       values_fill = list(Intensity = 0))
  peptideBiorepIntensity[is.na(peptideBiorepIntensity)] <- 0

  # PROTEINS: extra step to add the information about peptides
  almost <- merge(protBiorepIntensity, protPep, by="Proteins")
  final_result <- almost[,c(1,dim(almost)[2],2:(dim(almost)[2]-1)) ]
  
  # SAINTQ HEADER (two extra rows on top)
  rekey <- keys[keys$BioReplicate %in% data_f2_fa$BioReplicate,]
  
  if(fractions){
    rekey <- rekey[c('Condition', 'BioReplicate', 'SAINT')]
    rekey <- unique(rekey)
  }
  
  x <- t(rekey[,c('SAINT','Condition','BioReplicate')])
  extra <- cbind(c('','','Proteins'),c('','','Sequence'))
  header <- t(cbind(extra, x))
  theader <- t(header)
  checkthis <- data.frame(theader, row.names = NULL, stringsAsFactors = FALSE)
  names(checkthis) = checkthis[3,]
  
  # PROTEINS: ADDING HEADER, merging based on row names
  proteinssaintqheader <- rbind(checkthis, final_result)
  # SEQUENCES: Adding HEADER:
  sequencesaintqheader <- rbind(checkthis, peptideBiorepIntensity)
  
  # Writing the files for PROTEIN
  output <- paste0(output_dir,'/saintq_input_proteins.txt')
  write.table(proteinssaintqheader, 
              output, sep='\t',
              row.names = FALSE, col.names= FALSE, quote = FALSE)
  
  outconfig_protein <- paste0(output_dir,'/config-saintq-proteins')
  cat ("
       ### SAINTq parameter file
       ## use # to mark a line as comment
       
       ## normalize control intensities
       normalize_control=false
       
       ## name of file with intensities
       input_filename=saintq_input_proteins.txt
       
       ## valid: protein, peptide, fragment
       input_level=protein
       
       ## column names
       protein_colname=Proteins
       
       ## control bait selection rules
       compress_n_ctrl=100
       
       ## test bait replicate selection rules
       compress_n_rep=100
       ", file=outconfig_protein)
  
  
  # WRITING the files for SEQUENCES
  outsequences <- paste0(output_dir,'/saintq_input_peptides.txt')
  write.table(sequencesaintqheader, 
              outsequences, 
              sep = '\t', 
              row.names = FALSE, 
              col.names = FALSE, 
              quote = FALSE)
  
  outconfig_peptide <- paste0(output_dir,'/config-saintq-peptides')
  cat ("
       ### SAINTq parameter file
       ## use # to mark a line as comment
       
       ## normalize control intensities
       normalize_control=false
       
       ## name of file with intensities
       input_filename=saintq_input_peptides.txt
       
       ## type of intensity
       ## valid: protein, peptide, fragment
       input_level=peptide
       
       ## column names
       protein_colname=Proteins
       pep_colname=Sequence
       
       ## control bait selection rules
       compress_n_ctrl=100
       
       ## test bait replicate selection rules
       compress_n_rep=100
       
       ## peptide selection rules
       min_n_pep=3
       best_prop_pep=0.5
       
       ", file=outconfig_peptide)

  if(verbose){
    message(">> NEW 4 FILES CREATED:")
    message("\t- saintq-config-peptides")
    message("\t- saintq-config-proteins")
    message("\t- saintq_input_peptides.txt")
    message("\t- saintq_input_proteins.txt")
    message(">> DONE! ")
  }
}

# ------------------------------------------------------------------------------
#' @title Summarize the MSStats results and data quantification
#'
#' @description Converts the MSStats results file to wide format
#' (unique Protein ID and columns are the comparisons), as well as adds
#' BioReplicate information about
#' - the Number of Unique Peptides,
#' - Spectral Counts
#' - Intensities
#' for each protein.
#' In cases where there are multiple values for a Protein-BioReplicate
#' pair due to minute changes in sequence, the maximum value is taken for the
#' pair. Any pairs without a value are assigned a value of NA.
#' @param evidence_file (char or data.frame) The filepath to the MaxQuant 
#' searched data (evidence) file (txt tab delimited file). 
#' Only works for the newer versions of the evidence file.
#' @param prot_group_file (char) The filepath to the MaxQuant
#' `proteinGroups.txt` file (txt tab delimited file) or data.frame
#' @param keys_file (char) The filepath to the keys file used with MSStats
#' (txt tab delimited file).
#' @param results_file (char) The filepath to the MSStats results file in t
#' he default long format (txt tab delimited file or data.frame).
#' @param return_df (data.frame) Whether or not to return the results
#' to the R environment upon completion. This is useful if this is being
#' used in an R pipeline and you want to feed the results directly into the
#' next stage of analysis via an R environment/terminal.
#' Regardless, the results will be written to file. Default = FALSE
#' @param verbose (logical) `TRUE` (default) shows function messages
#' @return (data.frame or txt file) with the summary
#' @keywords MaxQuant, evidence, MSStats, summary
#' @examples
#' # Testing warning if files are not submitted
#' test <- artmsMsstatsSummary(evidence_file = NULL,
#'                       prot_group_file = NULL,
#'                       keys_file = NULL,
#'                       results_file = NULL)
#' @export
artmsMsstatsSummary <- function(evidence_file,
                                prot_group_file,
                                keys_file,
                                results_file,
                                return_df = FALSE,
                                verbose = TRUE) {
  
  MS.MS.Count = NULL
  
  if(any(missing(evidence_file) | 
         missing(prot_group_file) |
         missing(keys_file) | 
         missing(results_file)))
    stop("Missed (one or many) required argument(s)
         Please, check the help of this function to find out more")
  
  
  # Check if passing in data or if passing in files
  if(verbose) message(">> GENERATING A GLOBAL SUMMARY")
  
  
  if(is.null(evidence_file) & 
     is.null(prot_group_file) & 
     is.null(keys_file)
     & is.null(results_file)){
    return("Files must not be NULL")
  }
  
  dat <- artmsMergeEvidenceAndKeys(evidence_file, 
                                    keys_file, 
                                    verbose = verbose)
  dat <- data.table(dat)
  pg <- .artms_checkIfFile(prot_group_file)
  pg <- data.table(pg)
  results <- .artms_checkIfFile(results_file)
  results <- data.table(results)
  
  # get SPECTRAL COUNTS
  message("--- Summarizing Spectral Counts ")
  if( any(grepl("MS.MS.count", colnames(dat))) ){
    dat <- artmsChangeColumnName(dataset = dat, "MS.MS.count", "MS.MS.Count")
  }
  # dat.sc <- data.table::dcast(data = dat,
  #                            Proteins ~ BioReplicate,
  #                            value.var = "MS.MS.Count",
  #                            max,
  #                            fill = NA_real_)
  
  dat.sc <- dat %>% 
    tidyr::pivot_wider(id_cols = Proteins, 
                       names_from = BioReplicate, 
                       values_from = MS.MS.Count, 
                       values_fn = list(MS.MS.Count = max))
  dat.sc <- as.data.frame(dat.sc)
  
    
  names(dat.sc)[-1] <- paste0(names(dat.sc)[-1], "_SC")
  
  # get INTENSITIES
  message("--- Summarizing Intensities ")
  ##LEGACY
  # dat.intensity1 <- data.table::dcast(data = dat,
  #                                    Proteins ~ BioReplicate,
  #                                    value.var = "Intensity",
  #                                    max,
  #                                    fill = NA_real_)
  
  dat.intensity <- dat %>%
    tidyr::pivot_wider(id_cols = Proteins, 
                       names_from = BioReplicate, 
                       values_from = Intensity, 
                       values_fn = list(Intensity = max))
  dat.intensity <- as.data.frame(dat.intensity)
  
  names(dat.intensity)[-1] <- paste0(names(dat.intensity)[-1], "_Intensity")
  
  # find the UNIQUE PEPTIDE columns
  message("--- Summarizing Unique Peptides ")
  idx <- grep("Peptide.counts..unique.", colnames(pg), fixed = TRUE)
  pg.uniqPep <- pg[, c(1, idx), with = FALSE]
  # # fix names to match the rest of the data and to merge smoothly
  # names(pg.uniqPep) <- gsub("Peptide counts (unique)", "", names(pg.uniqPep))
  # names(pg.uniqPep)[-1] <- paste0(names(pg.uniqPep)[-1], "_UniqPep")
  # names(pg.uniqPep)[1] <- "Proteins"
  
  names(pg.uniqPep)[grep("Peptide.counts..unique.", 
                         names(pg.uniqPep), 
                         fixed = TRUE)] <- "UniquePeptides"
  names(pg.uniqPep)[1] <- "Proteins"
  
  # convert RESULTS to WIDE format
  message(">> Converting Results to Wide format ")
  
  ##LEGACY
  # results_l1 = data.table::melt(data = results[, c("Protein", "Label", "log2FC", "adj.pvalue"), 
  #                                             with = FALSE], id.vars = c("Protein", "Label"))
  
  results_l <- results %>%
    dplyr::select(c(Protein, Label, log2FC, adj.pvalue)) %>%
    tidyr::pivot_longer(cols = -c(Protein, Label), 
                        values_to = "value", 
                        names_to = "variable")
                     
  ## then cast to get combinations of LFCV/PVAl and Label as columns
  ##LEGACY
  # results_w <- data.table::dcast(Protein ~ Label + variable,
  #                                data = results_l,
  #                                value.var = c("value"))
  
  results_w <- results_l %>%
    dplyr::mutate(Label_variable = paste(Label, variable, sep = "_")) %>%
    tidyr::pivot_wider(id_cols = Protein, 
                       names_from = Label_variable, 
                       values_from = value)
  
  
    
  names(results_w)[1] = "Proteins"
  
  # Combine them all together
  message(">> Bringing it all together ")
  results_summary = Reduce(function(...)
    merge(..., by = "Proteins", all.x = TRUE),
    list(pg.uniqPep, results_w, dat.sc, dat.intensity))
    
  names(results_summary)[grep("Proteins", names(results_summary))] = "Protein"
  
  # write out summary
  message(">> Writing out Summary. ")
  if (!is.data.frame(results_file) &
      !is.data.table(results_file)) {
    out_file <- gsub(".txt", "_summarized.txt", results_file)
    write.table(
      results_summary,
      out_file,
      quote = FALSE,
      row.names = FALSE,
      sep = "\t"
    )
  }
  
  if (return_df) {
    return(results_summary)
  }
  
  message(">> Summarization Completed ")
}

# artMS PLOT FUNCTIONS

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' @title Individual Normalized abundance dot plots for every protein
#'
#' @description Protein abundance dot plots for each unique uniprot id. It can
#' take a long time
#' @param input_file (char) File path and name to the `-normalized.txt` output
#' file from MSstats
#' @param output_file (char) Output file (path) name (add the `.pdf` extension)
#' @param verbose (logical) `TRUE` (default) shows function messages
#' @return (pdf) file with each individual protein abundance plot for each
#' conditions
#' @keywords abundance, dotplots, plot
#' @examples \dontrun{
#' artmsDataPlots(input_file = "results/ab-results-mss-normalized.txt",
#'                output_file = "results/ab-results-mss-normalized.pdf")
#' }
#' @export
artmsDataPlots <- function(input_file, 
                           output_file,
                           verbose = TRUE) {
                            
  if(any(missing(input_file) | 
         missing(output_file)))
    stop("Missed (one or many) required argument(s)
        Please, check the help of this function to find out more")
  
  
  data_mss = fread(input_file, integer64 = 'double')
  unique_subjects <- unique(data_mss$PROTEIN)
  condition_length <- length(unique(data_mss$GROUP))
  min_abu <- min(data_mss$ABUNDANCE, na.rm = TRUE)
  max_abu <- max(data_mss$ABUNDANCE, na.rm = TRUE)
  
  pdf(output_file, width = condition_length * 1.5, height = 3)
  if(verbose) message('>> PRINTING CONDITION PLOTS for every protein ')
  for (subject in unique_subjects) {
    subject_data <- data_mss[PROTEIN == subject, ]
    if(verbose) message(sprintf('%s ', subject))
    p <-
      ggplot(data = subject_data,
             aes(x = SUBJECT, y = ABUNDANCE, colour = FEATURE))
    p <- p + geom_point(size = 2, na.rm = TRUE) +
      facet_wrap(
        facets = ~ GROUP,
        drop = TRUE,
        scales = 'free_x',
        ncol = condition_length
      ) +
      ylim(min_abu, max_abu) +
      theme(axis.text.x = element_text(angle = -90, hjust = 1)) +
      guides(colour = FALSE) +
      xlab(NULL) +
      ggtitle(subject)
    print(p)
  }
  if(verbose) message("--- Done! ")
  garbarge <- dev.off()
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# @title Heatmap of significant values
#
# @description heatmap plot to represent proteins with significant changes
# @param mss_F (data.frame) with the significant values (log2fc, pvalues)
# @param out_file (char) Name for the output
# @param labelOrder (vector) Vector with the particular order for the IDs
# (default, `NULL` no order)
# @param names (char) Type of ID used. Default is `Protein` (uniprot entry id).
# Soon will be possible to use 'Gene' name ids.
# @param cluster_cols (logical) Select whether to cluster the columns.
# Options: `T` or `F`. Default `T`.
# @param display (char) Value used to genarate the heatmaps. Options:
# - `log2FC` (default)
# - `adj.pvalue`
# - `pvalue`
# @param verbose (logical) `TRUE` (default) shows function messages
# @return A heatmap of significant values
# @keywords significant, heatmap
.artms_plotHeat <- function(mss_F,
                            out_file,
                            labelOrder = NULL,
                            names = 'Protein',
                            cluster_cols = FALSE,
                            display = c('log2FC', 'adj.pvalue', 'pvalue'),
                            verbose = TRUE) {

  heat_data <- data.frame(mss_F, names = names)
    
  ## create matrix from log2FC or p-value as user defined
  display <- match.arg(display)
  if (display == 'log2FC') {
    # Issues with extreme_val later if we have Inf/-Inf values.
    if (sum(is.infinite(heat_data$log2FC)) > 0) {
      idx <- is.infinite(heat_data$log2FC)
      heat_data$log2FC[idx] <- NA
    }
    ##LEGACY
    # heat_data_w = data.table::dcast(names ~ Label, data = heat_data, value.var = 'log2FC')
    heat_data_w <- heat_data %>% 
      tidyr::pivot_wider(id_cols = names, 
                         names_from = Label, 
                         values_from = log2FC,
                         values_fn = list(log2FC = length))
  } else if (display == 'adj.pvalue') {
    heat_data$adj.pvalue = -log10(heat_data$adj.pvalue + 10 ^ -16)
    ##LEGACY
    # heat_data_w = data.table::dcast(names ~ Label,
    #                     data = heat_data, value.var = 'adj.pvalue')
    heat_data_w <- heat_data %>% 
      tidyr::pivot_wider(id_cols = names, 
                         names_from = Label, 
                         values_from = adj.pvalue,
                         values_fn = list(adj.pvalue = length))
  } else if (display == 'pvalue') {
    heat_data$pvalue = -log10(heat_data$pvalue + 10 ^ -16)
    ##LEGACY
    # heat_data_w = data.table::dcast(names ~ Label, data = heat_data, value.var = 'pvalue')
    
    heat_data_w <- heat_data %>% 
      tidyr::pivot_wider(id_cols = names, 
                         names_from = Label, 
                         values_from = pvalue,
                         values_fn = list(pvalue = length))
  }
  
  heat_data_w <- as.data.frame(heat_data_w)
  
  ## try
  #gene_names = uniprot_to_gene_replace(uniprot_ac=heat_data_w$Protein)
  rownames(heat_data_w) = heat_data_w$names
  heat_data_w = heat_data_w[, -1]
  heat_data_w[is.na(heat_data_w)] = 0
  max_val = ceiling(max(heat_data_w))
  min_val = floor(min(heat_data_w))
  extreme_val = max(max_val, abs(min_val))
  if (extreme_val %% 2 != 0)
    extreme_val = extreme_val + 1
  bin_size = 2
  signed_bins = (extreme_val / bin_size)
  colors_neg = rev(colorRampPalette(brewer.pal("Blues", n = extreme_val /
                                                 bin_size))(signed_bins))
  colors_pos = colorRampPalette(
    brewer.pal("Reds", n = extreme_val / bin_size))(signed_bins)
  colors_tot = c(colors_neg, colors_pos)
  
  if (is.null(labelOrder)) {
    pheatmap(
      heat_data_w,
      scale = "none",
      cellheight = 10,
      cellwidth = 10,
      filename = out_file,
      color = colors_tot,
      breaks = seq(
        from = -extreme_val,
        to = extreme_val,
        by = bin_size
      ),
      cluster_cols = cluster_cols,
      fontfamily = "mono"
    )
  } else{
    heat_data_w <- heat_data_w[, labelOrder]
    pheatmap(
      heat_data_w,
      scale = "none",
      cellheight = 10,
      cellwidth = 10,
      filename = out_file,
      color = colors_tot,
      breaks = seq(
        from = -extreme_val,
        to = extreme_val,
        by = bin_size
      ),
      cluster_cols = cluster_cols,
      fontfamily = "mono"
    )
    if(verbose) message("--- Heatmap is out ")
  }
  
  return(heat_data_w)
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' @title Outputs a heatmap of the MSStats results created using the log2fold
#' changes
#'
#' @description Heatmap of the Relative Quantifications (MSStats results)
#' @param input_file (char) MSstats `results.txt` file and location (or
#' data.frame of resuts)
#' @param output_file (char) Output file name (pdf format) and location.
#' Default:"quantifications_heatmap.pdf"
#' @param species (char). Specie name to be able to add the Gene name. To find
#' out more about the supported species check `?artmsMapUniprot2Entrez`
#' @param labels (vector) of uniprot ids if only specific labes would like to
#' be plotted. Default: all labels
#' @param cluster_cols (boolean) `True` or `False` to cluster columns.
#' Default: FALSE
#' @param lfc_lower (int) Lower limit for the log2fc. Default: -2
#' @param lfc_upper (int) Upper limit for the log2fc. Default: +2
#' @param whatPvalue (char) `pvalue` or `adj.pvalue` (default)
#' @param FDR (int) Upper limit false discovery rate (or pvalue). Default: 0.05
#' @param display Metric to be displayed. Options:
#' - `log2fc` (default)
#' - `adj.pvalue`
#' - `pvalue`
#' @param verbose (logical) `TRUE` (default) shows function messages
#' @return (pdf or ggplot2 object) heatmap of the MSStats results using the
#' selected metric
#' @keywords heatmap, log2fc
#' @examples
#' # Unfortunately, the example does not contain any significant hits
#' # Use for illustration purposes
#' artmsPlotHeatmapQuant(input_file = artms_data_ph_msstats_results,
#'                        species = "human",
#'                        output_file = NULL,
#'                        whatPvalue = "pvalue",
#'                        lfc_lower = -1,
#'                        lfc_upper = 1)
#' @export
artmsPlotHeatmapQuant <- function(input_file,
                                   output_file = "quantifications_heatmap.pdf",
                                   species,
                                   labels = '*',
                                   cluster_cols = FALSE,
                                   display = 'log2FC',
                                   lfc_lower = -2,
                                   lfc_upper = 2,
                                   whatPvalue = "adj.pvalue",
                                   FDR = 0.05,
                                   verbose = TRUE) {
  
  if(any(missing(input_file) | 
         missing(species)))
    stop("Missed (one or many) required argument(s)
        Please, check the help of this function to find out more")
  
  input <- .artms_checkIfFile(input_file)
  
  ## select data points  by LFC & FDR criterium in single condition and
  ## adding corresponding data points from the other conditions
  sign_hits <- .artms_significantHits(
    input,
    labels = labels,
    LFC = c(lfc_lower, lfc_upper),
    whatPvalue = whatPvalue,
    FDR = FDR
  )
  sign_hits <- sign_hits[complete.cases(sign_hits$log2FC), ]
  sign_hits <- sign_hits[is.finite(sign_hits$log2FC), ]
  if (dim(sign_hits)[1] == 0) {
    return(message("--- not enough significant hits!"))
  }
  
  sign_labels <- unique(sign_hits$Label)
  if(verbose) message(
    sprintf(
      ">> TOTAL NUMBER OF SELECTED HITS FOR PLOTS WITH LFC BETWEEN %s AND %s AT %s FDR:%s\n",
      lfc_lower,
      lfc_upper,
      FDR,
      nrow(sign_hits) / length(sign_labels)
    )
  )
  
  suppressMessages(
    sign_hits <- artmsAnnotationUniprot(
      x = sign_hits,
      columnid = "Protein",
      species = species
    )
  )
  
  ## REPRESENTING RESULTS AS HEATMAP
  ## plot heat map for all contrasts
  if (any(grepl('Gene', colnames(sign_hits)))) {
    heat_labels <- paste(sign_hits$Protein, sign_hits$Gene, sep = '_')
  } else{
    heat_labels <- sign_hits$Protein
  }
  
  heat_labels <- gsub('\\sNA$', '', heat_labels)
  
  # Old PlotHeat function:
  heat_data <- data.frame(sign_hits, heat_labels = heat_labels)
  
  ## create matrix from log2FC or p-value as user defined
  if (display == 'log2FC') {
    # Issues with extreme_val later if we have Inf/-Inf values.
    if (sum(is.infinite(heat_data$log2FC)) > 0) {
      idx <- is.infinite(heat_data$log2FC)
      heat_data$log2FC[idx] <- NA
    }
    
    ##LEGACY
    # heat_data_w <- data.table::dcast(heat_labels ~ Label, 
    #                                  data = heat_data, 
    #                                  value.var = 'log2FC')
    
    heat_data_w <- heat_data %>% 
      tidyr::pivot_wider(id_cols = heat_labels, 
                         names_from = Label, 
                         values_from = log2FC)
    
      
  } else if (display == 'adj.pvalue') {
    heat_data$adj.pvalue <- -log10(heat_data$adj.pvalue + 10 ^ -16)
    
    ##LEGACY
    # heat_data_w <- data.table::dcast(heat_labels ~ Label, 
    #                                  data = heat_data, 
    #                                  value.var = 'adj.pvalue')
    heat_data_w <- heat_data %>% 
      tidyr::pivot_wider(id_cols = heat_labels, 
                         names_from = Label, 
                         values_from = adj.pvalue)
      
  } else if (display == 'pvalue') {
    heat_data$pvalue <- -log10(heat_data$pvalue + 10 ^ -16)
    ##LEGACY
    # heat_data_w <- data.table::dcast(heat_labels ~ Label, 
    #                                  data = heat_data, 
    #                                  value.var = 'pvalue')
    heat_data_w <- heat_data %>% 
      tidyr::pivot_wider(id_cols = heat_labels, 
                         names_from = Label, 
                         values_from = pvalue)
  }
  
  #gene_names <- uniprot_to_gene_replace(uniprot_ac=heat_data_w$Protein)
  heat_data_w <- as.data.frame(heat_data_w)
  rownames(heat_data_w) <- heat_data_w$heat_labels
  heat_data_w <- heat_data_w[, -1]
  heat_data_w[is.na(heat_data_w)] = 0
  max_val <- ceiling(max(heat_data_w))
  min_val <- floor(min(heat_data_w))
  extreme_val <- max(max_val, abs(min_val))
  if (extreme_val %% 2 != 0)
    extreme_val = extreme_val + 1
  bin_size = 2
  signed_bins <- (extreme_val / bin_size)
  colors_neg <- rev(colorRampPalette(RColorBrewer::brewer.pal("Blues", n = extreme_val /
                                                    bin_size))(signed_bins))
  colors_pos <- colorRampPalette(RColorBrewer::brewer.pal("Reds", 
                                    n = extreme_val / bin_size))(signed_bins)
  colors_tot <- c(colors_neg, colors_pos)
  
  if (!is.null(output_file)) {
    pheatmap(
      heat_data_w,
      scale = "none",
      cellheight = 10,
      cellwidth = 10,
      filename = output_file,
      color = colors_tot,
      breaks = seq(
        from = -extreme_val,
        to = extreme_val,
        by = bin_size
      ),
      cluster_cols = cluster_cols,
      fontfamily = "mono"
    )
    if(verbose) message("--- Heatmap done ")
  } else{
    pheatmap(
      heat_data_w,
      scale = "none",
      cellheight = 10,
      cellwidth = 10,
      color = colors_tot,
      breaks = seq(
        from = -extreme_val,
        to = extreme_val,
        by = bin_size
      ),
      cluster_cols = cluster_cols,
      fontfamily = "mono"
    )
  }
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# @title Generate reproducibility plots based on raw intentities
# (from the evidence file)
#
# @description Generate reproducibility plots based on raw intentities
# (log tranformed) from the evidence file
# @param data (data.frame) clean processed evidence
# @param verbose (logical) `TRUE` (default) shows function messages
# @return (pdf) A reproducibility plot based on evidence values
# @keywords internal, plot, qc, quality, control
.artms_plotReproducibilityEvidence <- function(data,
                                               verbose = TRUE) {
  
  Feature = NULL
  
  data <- data[c('Feature',
                 'Proteins',
                 'Intensity',
                 'Condition',
                 'BioReplicate',
                 'Run')]
    
  condi <- unique(data$Condition)
  
  # Progress bar
  if(verbose) pb <- txtProgressBar(min = 0,
                       max = length(condi),
                       style = 3)
  
  for ( i in seq_len(length(condi)) ) {
    eCondition <- condi[i]
    # Progress bar
    if(verbose) setTxtProgressBar(pb, i)
    
    conditionOne <- data[which(data$Condition == eCondition), ]
    
    # FIRST CHECK FOR TECHNICAL REPLICAS
    bioreplicasAll <- unique(conditionOne$BioReplicate)
    
    for (eBioreplica in bioreplicasAll) {
      # message('\tChecking for technical replicas in ',eBioreplica, " ")
      biorepli <- conditionOne[conditionOne$BioReplicate == eBioreplica, ]
      here <- unique(biorepli$Run)
      
      if (length(here) > 1) {
        #Limit to 2 technical replicas: (length(here) > 1 & length(here) == 2)
        # We are expecting no more than 2 technical replicas. 
        # If there is more... it is worthy to double check
        # message('\t\t>>Reproducibility for technical replicas of',eBioreplica,":")
        #Need to change the RUN number to letters (TR: TECHNICAL REPLICA)
        biorepli$TR <- biorepli$Run
        biorepli$TR[biorepli$TR == here[1]] <- 'tr1'
        biorepli$TR[biorepli$TR == here[2]] <- 'tr2'
        
        # Let's select unique features per TECHNICAL REPLICAS, 
        # and sum them up (the same feature might have been many differnt times)
        biorepliaggregated <- aggregate(Intensity ~ Feature+Proteins+Condition+BioReplicate+Run+TR,
                                        data = biorepli,
                                        FUN = sum)
          
          
        biorepliaggregated$Intensity <- log2(biorepliaggregated$Intensity)
        ##LEGACY
        # bdc <- data.table::dcast(data = biorepliaggregated,
        #                          Feature + Proteins ~ TR,
        #                          value.var = 'Intensity')
        
        bdc <- biorepliaggregated %>%
          tidyr::pivot_wider(id_cols = c(Feature, Proteins), 
                             names_from = TR, 
                             values_from = Intensity)
        bdc <- as.data.frame(bdc)
          
        # Get the number of proteins
        np <- dim(bdc)[1]
        corr_coef <- round(cor(bdc$tr1, bdc$tr2, use = "pairwise.complete.obs"), digits = 2)
        # message("r:\t",corr_coef," ")
        p1 <- ggplot(bdc, aes(x = tr1, y = tr2))
        p1 <- p1 + geom_point(na.rm = TRUE) + geom_rug() + 
          geom_density_2d(colour = 'lightgreen')
        p1 <- p1 + geom_smooth(colour = "green",
                               fill = "lightblue",
                               method = 'lm',
                               formula = y ~ x,
                               na.rm = TRUE)
        p1 <- p1 + theme_light()
        p1 <- p1 + labs(title = paste(
          "Reproducibility between Technical Replicas\nBioReplica:\n",
          eBioreplica,
          "\n(n =  ",
          np,
          ", r = ",
          corr_coef,
          ")"
        )
        )
        
        print(p1)
        
      } 
      # else if (length(here) == 1) {
      #   # message("\t\tOnly one technical replica ")
      # } else{
      #   stop("More than 2 technical replicates found. This is very unusual.
      #   Please, revise it and if correct contact artms developers")
      # }
    } # Checking the reproducibility between Technical Replicas
    
    
    # NOW BETWEEN BIOREPLICAS
    # Before comparing the different biological replicas, 
    # aggregate the technical replicas
    # First choose the maximum for the technical replicas as before, 
    # but first check whether there are more than one
    if (length(here) > 1) {
      conditionOne <- aggregate(Intensity ~ Feature + Proteins + Condition + BioReplicate + Run,
                                data = conditionOne,
                                FUN = sum)
      b <- aggregate(Intensity ~ Feature + Proteins + Condition + BioReplicate,
                     data = conditionOne,
                     FUN = mean)
    } else{
      b <- aggregate(Intensity ~ Feature + Proteins + Condition + BioReplicate,
                     data = conditionOne,
                     FUN = sum)
    }
    
    b$Intensity <- log2(b$Intensity)
    blist <- unique(b$BioReplicate)
    if (length(blist) > 1) {
      
      # We need at least TWO BIOLOGICAL REPLICAS
      to <- length(blist) - 1
      
      for (i in seq_len(to)) {
        j <- i + 1
        for ( k in j:length(blist) ) {
          br1 <- blist[i]
          br2 <- blist[k]
          
          # message("\tChecking reproducibility between ",br1, "and ",br2 ,"\t")
          ##LEGACY
          # bcfinal <- data.table::dcast(data = b,
          #                              Feature + Proteins ~ BioReplicate,
          #                              value.var = 'Intensity')

          bcfinal <- b %>% tidyr::pivot_wider(id_cols = c(Feature, Proteins), 
                                              names_from = BioReplicate, 
                                              values_from = Intensity)
            
            
          bcfinal <- as.data.frame(bcfinal)
          bcfinal <- bcfinal[complete.cases(bcfinal),]
          
          # Let's check the total number of peptides here...
          checkTotalNumber <- subset(bcfinal, select = c(br1, br2))
          npt <- dim(checkTotalNumber)[1]
          
          corr_coef <- round(cor(bcfinal[[br1]], bcfinal[[br2]], use = "pairwise.complete.obs"), digits = 2)
          # message("r:\t",corr_coef," ")
          
          colnames(bcfinal) <- c("Feature", "Proteins", "br1", "br2")
          
          p2 <- ggplot(bcfinal, aes(x = br1, y = br2 ))
          p2 <- p2 + geom_point(na.rm = TRUE) + geom_rug() + geom_density_2d(colour = 'red')
          p2 <- p2 + geom_smooth(colour = "red",
                                 fill = "lightgreen",
                                 method = 'lm',
                                 formula = y ~ x,
                                 na.rm = TRUE)
          p2 <- p2 + theme_light()
          p2 <- p2 + labs(title = paste("Peptide Reproducibility between Bioreplicas\n (condition:",
                                        eCondition,
                                        ")",
                                        br1,
                                        "vs",
                                        br2,
                                        "\n(n =",
                                        npt,
                                        " r = ",
                                        corr_coef,
                                        ")"))
          p2 <- p2 + labs(x = "br1")
          p2 <- p2 + labs(y = "br2")
          print(p2)
        } #end for
      } #end for
    } else{
      message("\tONLY ONE BIOLOGICAL REPLICA AVAILABLE (plots are not possible) ")
    }
  } # all the conditions
  # Close Progress bar
  if(verbose) close(pb)
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# @title Plot abundance boxplots
#
# @description Plot abundance boxplots
# @param data (data.frame) processed modelqc
# @return Abundacen boxplot
# @keywords internal, plot, abundance
.artms_plotAbundanceBoxplots <- function(df) {
  p1 <- ggplot2::ggplot(df, 
                        aes(x = SUBJECT, 
                            y = ABUNDANCE, 
                            fill = ABUNDANCE))
  p1 <- p1 + geom_boxplot(aes(fill = SUBJECT),
                          na.rm = TRUE)
  p1 <- p1 + theme_linedraw()
  p1 <-
    p1 + theme(
      axis.text.x = element_text(
        angle = 90,
        hjust = 1,
        vjust = 0.5
      ),
      legend.position = "none"
    )
  p1 <- p1 + labs(x = "BIOREPLICATES")
  p1 <- p1 + ggtitle("Relative Abundance BioReplicates")
  print(p1)
  
  p2 <-
    ggplot2::ggplot(df, aes(
      x = as.factor(GROUP),
      y = ABUNDANCE,
      fill = ABUNDANCE
    ))
  p2 <- p2 + geom_boxplot(aes(fill = GROUP),
                          na.rm = TRUE)
  p2 <- p2 + theme_linedraw()
  p2 <-
    p2 + theme(
      axis.text.x = element_text(
        angle = 90,
        hjust = 1,
        vjust = 0.5
      ),
      legend.position = "none"
    )
  p2 <- p2 + labs(x = "CONDITIONS")
  p2 <- p2 + ggtitle("Relative Abundance Conditions")
  print(p2)
}


# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# @title Total Number of unique proteins based on abundance data
#
# @description Total Number of unique proteins per biological replicate and
# conditions
# @param df (data.frame) modelqc
# @return (pdf) Barplots with the number of proteins per br / condition
# @keywords internal, plots, abundance, counts
.artms_plotNumberProteinsAbundance <- function(df) {
  x <- df[,c('PROTEIN', 'SUBJECT')]
  y <- unique(x)
  names(y)[grep('SUBJECT', names(y))] <- 'BioReplicate'
  z <-
    ggplot2::ggplot(y, aes(x = BioReplicate, fill = BioReplicate))
  z <- z + geom_bar(stat = "count", na.rm = TRUE)
  z <- z + theme_linedraw()
  z <- z + theme(axis.text.x = element_text(angle = 90,
                                            hjust = 1,
                                            vjust = 0.5))
  z <- z + geom_text(stat = 'count',
                     aes(label = ..count..),
                     vjust = -0.5,
                     size = 2.7)
  z <- z + ggtitle("Unique Proteins in BioReplicates")
  print(z)
  
  a <- df[,c('PROTEIN', 'GROUP')]
  b <- unique(a)
  names(b)[grep('GROUP', names(b))] <- 'Condition'
  c <- ggplot2::ggplot(b, aes(x = Condition, fill = Condition))
  c <- c + geom_bar(stat = "count", na.rm = TRUE)
  c <- c + theme_linedraw()
  c <- c + theme(axis.text.x = element_text(angle = 90,
                                            hjust = 1,
                                            vjust = 0.5))
  c <- c + geom_text(stat = 'count',
                     aes(label = ..count..),
                     vjust = -0.5,
                     size = 2.7)

  c <- c + ggtitle("Unique Proteins in Conditions")
  print(c)
}

# 
# @title Plot the total number of quantified proteins in each condition
#
# @description
# @keys internal, plot, counts
# @param x (data.frame) Data frame of imputed log2fc
.artms_plotNumberProteinsImputedLog2fc <- function(x) {
  x <- x[c('Protein', 'Comparison')]
  y <- unique(x)
  z <- ggplot(y, aes(x = Comparison, fill = Comparison))
  z <- z + geom_bar(stat = "count",
                    na.rm = TRUE)
  z <- z + theme_linedraw()
  z <- z + theme(axis.text.x = element_text(angle = 90,
                                            hjust = 1,
                                            vjust = 0.5))
  
  
  z <- z + geom_text(stat = 'count',
                     aes(label = ..count..),
                     vjust = -0.5,
                     size = 2.7)
  
  z <- z + ggtitle("Unique Proteins in Comparisons")
  print(z)
}


# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# @title Generate reproducibility plots based on abundance data
# (normalized intensities from MSstats modelqc)
#
# @description Generate reproducibility plots based on abundance data
# (normalized intensities from MSstats modelqc)
# @param data (data.frame) Protein abundance (modelqc)
# @return Reproducibility plots based on abundance data 
# (normalized intensities)
# @keywords plot, reproducibility, abundance
.artms_plotReproducibilityAbundance <- function(x,
                                                verbose = verbose) {
  condi <- unique(x$GROUP)
  
  # Progress bar
  if(verbose) pb <- txtProgressBar(min = 0,
                       max = length(condi),
                       style = 3)
  
  for ( i in seq_len(length(condi)) ) {
    eCondition <- condi[i]
    
    # Progress bar
    if(verbose) setTxtProgressBar(pb, i)
    
    # message(" ################## CONDITION: ",eCondition," ############### ")
    # message("TECHNICAL REPLICAS --------------------------- ")
    # message("- ", eCondition," ")
    
    conditionOne <- x[which(x$GROUP == eCondition), ]
    
    # FIRST CHECK FOR TECHNICAL REPLICAS
    bioreplicasAll <- unique(conditionOne$SUBJECT)
    # plot_tr = list()
    for (eBioreplica in bioreplicasAll) {
      # message('\tChecking for technical replicas in ',eBioreplica, " ")
      biorepli <- conditionOne[conditionOne$SUBJECT == eBioreplica, ]
      here <- unique(biorepli$RUN)
      
      if (length(here) > 1) {
        # Check whether we have more than 2 technical replicas and let 
        # the user know:
        if (length(here) > 2) {
          message(
            "(-)----- WARNING: More than 2 technical replicas! 
            make sure that this is right  "
          )
        }
        # We are expecting no more than 2 technical replicas. 
        # If there is more... it is worthy to double check
        # message('\t\t>>Plotting Reproducibility between technical replicas ',
        # eBioreplica," ")
        #Need to change the RUN number to letters (TR: TECHNICAL REPLICA)
        biorepli$TR <- biorepli$RUN
        biorepli$TR[biorepli$TR == here[1]] <- 'tr1'
        biorepli$TR[biorepli$TR == here[2]] <- 'tr2'
        
        ##LEGACY
        # bdc <- data.table::dcast(data = biorepli, PROTEIN ~ TR, 
        #                          value.var = 'ABUNDANCE')
        bdc <- biorepli %>% 
          tidyr::pivot_wider(id_cols = PROTEIN, 
                             names_from = TR, 
                             values_from = ABUNDANCE)
        bdc <- as.data.frame(bdc)
        
        bdc <- bdc[complete.cases(bdc), ]
        # Get the number of proteins
        np <- length(unique(bdc$PROTEIN))
        corr_coef <- round(cor(bdc$tr1, bdc$tr2), digits = 2)
        p1 <- ggplot2::ggplot(bdc, aes(x = tr1, y = tr2))
        p1 <- p1 + geom_point(na.rm = TRUE)
        p1 <- p1 + geom_smooth(colour = "green",
                               fill = "lightblue",
                               method = 'lm',
                               formula = y ~ x,
                               na.rm = TRUE)
        p1 <- p1 + theme_light()
        p1 <- p1 + labs(
          title = paste(
            "Reproducibility between Technical Replicas\nBioReplica:\n",
            eBioreplica,
            "\n(n = ",
            np,
            ", r = ",
            corr_coef,
            ")"
          )
        )
        print(p1)
        # plot_tr[[eBioreplica]] <- p1
      } else if (length(here) < 1) {
        stop("More than 2 technical replicates found. This is very unusual.
             Please, revise it and contact artMS developers if right")
      }
    } # Checking the reproducibility between Technical Replicas
    
    
    # NOW BETWEEN BIOREPLICAS
    # Before comparing the different biological replicas, 
    # aggregate on the technical replicas
    # message(" BIOLOGICAL REPLICAS --------------------------- ")
    b <- aggregate(
      ABUNDANCE ~ PROTEIN + GROUP + SUBJECT,
      data = conditionOne,
      FUN = mean
    ) 
    
    # Remove the dash
    b$SUBJECT <- gsub("-", "_", b$SUBJECT)
      
    blist <- unique(b$SUBJECT)
    if ( length(blist) > 1 ) {
      # We need at least TWO BIOLOGICAL REPLICAS
      to <- length(blist) - 1
      for (i in seq_len(to) ) {
        j <- i + 1
        for ( k in j:length(blist) ) {
          br1 <- blist[i]
          br2 <- blist[k]
          # message("\tChecking reproducibility between ",br1, "and ",br2 ," ")
          
          ##LEGACY
          # bc <- data.table::dcast(data = b,
          #                        PROTEIN ~ SUBJECT,
          #                        value.var = 'ABUNDANCE')
          
          bc <- b %>% 
            tidyr::pivot_wider(id_cols = PROTEIN, 
                               names_from = SUBJECT, 
                               values_from = ABUNDANCE)
            
          bc <- bc[complete.cases(bc), ]
          
          npt <- length(unique(bc$PROTEIN))
          
          corr_coef <- round(cor(bc[[br1]], bc[[br2]]), digits = 2)
          
          p2 <- ggplot2::ggplot(bc, aes_string(x = paste(br1) , y = paste(br2) ))
          p2 <- p2 + geom_point(na.rm = TRUE)
          p2 <- p2 + geom_smooth(colour = "red",
                                 fill = "lightgreen",
                                 method = 'lm',
                                 formula = y ~ x,
                                 na.rm = TRUE)
          p2 <- p2 + theme_light()
          p2 <- p2 + labs(
            title = paste(
              "Reproducibility between Bioreplicas\n(condition:",
              eCondition, ") ", br1, "vs", br2, "\n(n =", npt, " r = ", corr_coef, ")"
            )
          )
          p2 <- p2 + labs(x = br1)
          p2 <- p2 + labs(y = br2)
          print(p2)
        }
      }
      if(verbose) message(" ")
    } else{
      if(verbose) message("\tONLY ONE BIOLOGICAL REPLICA AVAILABLE (plots are not possible) ")
    }
  } # all the conditions
  
  if(verbose) close(pb)
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# @title Plot correlation between conditions
#
# @description Plot correlation between conditions
# @param x (data.frame) of Protein Abundance (MSstats modelqc)
# @param numberBiologicalReplicas (int) Number of biological replicates
# @return (ggplot.object) A correlation plot between conditions
# @keywords internal, plot, correlation
.artms_plotCorrelationConditions <- function(x, numberBiologicalReplicas) {
  # Before jumping to merging biological replicas:
  # Technical replicas: aggregate on the technical replicas
  b <- aggregate(ABUNDANCE ~ PROTEIN + GROUP + SUBJECT,
                 data = x,
                 FUN = mean)
    
  # Aggregate now the CONDITIONS on the biological replicas:
  
  # One way to do this would be to be very stringent, 
  # requiring to find data in all biological replicas:
  # allBiologicalReplicas <- function(x){
  # ifelse(sum(!is.na(x)) == numberBiologicalReplicas, 
  # mean(x, na.rm = TRUE), NA)}
  # datadc <- data.table::dcast(data=b, 
  # PROTEIN~GROUP, value.var = 'ABUNDANCE', 
  # fun.aggregate = allBiologicalReplicas, fill = 0)
  
  # Or a most relaxed way:
  
  ##LEGACY
  # datadc <- data.table::dcast(data = b,
  #                             PROTEIN ~ GROUP,
  #                             value.var = 'ABUNDANCE',
  #                             fun.aggregate = mean) 
  
  datadc <- b %>% 
    tidyr::pivot_wider(id_cols = PROTEIN,
                       names_from = GROUP, 
                       values_from = ABUNDANCE, 
                       values_fn = list(ABUNDANCE = mean))
    
  
  # before <- dim(datadc)[1]
  # l <- dim(datadc)[2]
  # datadc <- datadc[apply(datadc[c(2:l)],1,function(z) !any(z==0)),]
  # evenafter <- dim(datadc)[1]
  # datadc <- datadc[complete.cases(datadc),]
  # after <- dim(datadc)[1]
  # message("Total proteins before: ", 
  # before, " Removing the 0s: ",evenafter, 
  # " Total proteins (only complete cases): ", after, "  ")
  
  blist <- unique(b$GROUP)
  if (length(blist) > 1) {
    # We need at least TWO CONDITIONS
    to <- length(blist) - 1
    for (i in seq_len(to) ) {
      j <- i + 1
      for (k in j:length(blist)) {
        br1 <- blist[i]
        br2 <- blist[k]
        # message("\t",br1,"-",br2,":")
        
        npt <- length(unique(datadc$PROTEIN))
        
        corr_coef <- round(cor(datadc[[br1]], 
                               datadc[[br2]], 
                               use = "complete.obs"), digits = 2)
        # cat ("r:",corr_coef,"\n")
        
        p2 <- ggplot2::ggplot(datadc, aes_string(x = paste(br1), y = paste(br2)))
        p2 <- p2 + geom_point(na.rm = TRUE)
        p2 <- p2 + geom_smooth(colour = "blue",
                               fill = "lightblue",
                               method = 'lm',
                               formula = y ~ x, 
                               na.rm = TRUE)
        p2 <- p2 + theme_light()
        p2 <- p2 + labs(title = paste("CORRELATION between CONDITIONS:\n",
                                      br1,
                                      "and",
                                      br2,
                                      "\n(n = ",
                                      npt,
                                      " r = ",
                                      corr_coef,
                                      ")"))
        p2 <- p2 + labs(x = br1)
        p2 <- p2 + labs(y = br2)
        print(p2)
      } # FOR loop
    } # For loop
    # message(" ")
  } else{
    message("\tONLY ONE BIOLOGICAL REPLICA AVAILABLE (plots are not possible) ")
  }
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# @title Plot correlation between quantifications different quantified
# comparisons
#
# @description Plot correlation between all quantifications, i.e., different
# quantified comparisons
# @param datai (data.frame) Processed MSstats results
# @param verbose (logical) `TRUE` (default) shows function messages
# @return (ggplot.object) Plot correlation between quantifications and r values
# @keywords internal, plot, correlation, log2fc
.artms_plotRatioLog2fc <- function(datai,
                                   verbose = TRUE) {
  ##LEGACY
  # datadc <- data.table::dcast(data = datai, Protein ~ Label, value.var = 'log2FC')
  
  datadc <- datai %>% 
    tidyr::pivot_wider(id_cols = Protein,
                       names_from = Label, 
                       values_from = log2FC)
  datadc <- as.data.frame(datadc)
  
  before <- dim(datadc)[1]
  l <- dim(datadc)[2]
  datadc <- do.call(data.frame, lapply(datadc, function(x) replace(x, is.infinite(x), NA)))
  datadc <- datadc[complete.cases(datadc), ]
  after <- dim(datadc)[1]
  if(verbose) message("---Total unique identifiers before: ",
    before,
    "\n---Total unique identifiers (only complete cases): ",
    after)
  
  blist <- unique(datai$Label)
  blist <- gsub("-", ".", blist)
  
  if (length(blist) > 1) {
    # We need at least TWO CONDITIONS
    to <- length(blist) - 1
    # Progress bar
    if(verbose) pb <- txtProgressBar(min = 0,
                         max = length(blist) - 1,
                         style = 3)
    for (i in seq_len(to)) {
      if(verbose) setTxtProgressBar(pb, i)
      j <- i + 1
      for (k in j:length(blist)) {
        br1 <- blist[i]
        br2 <- blist[k]
        # message("\tChecking relation between conditions ",br1, "and ",br2 ,":")
        
        npt <- length(unique(datadc$Protein))
        
        corr_coef <- round(cor(datadc[[br1]], datadc[[br2]]), digits = 2)
        # cat ("r: ",corr_coef," ")
        
        p3 <- ggplot2::ggplot(datadc, aes_string(x = paste(br1), y = paste(br2)))
        p3 <- p3 + geom_point(na.rm = TRUE) + geom_rug() + geom_density_2d()
        p3 <- p3 + geom_smooth(colour = "red",
                               fill = "lightblue",
                               method = 'lm',
                               formula = y ~ x,
                               na.rm = TRUE)
        p3 <- p3 + theme_light()
        p3 <- p3 + labs(title = paste0("log2fc(",
                                       br1,
                                       ") vs log2fc(",
                                       br2,
                                       ") \n(n = ",
                                       npt,
                                       " r = ",
                                       corr_coef,
                                       ")"))
        p3 <- p3 + labs(x = br1)
        p3 <- p3 + labs(y = br2)
        print(p3)
      } # FOR loop
    } # For loop
    if(verbose) close(pb)
  } else{
    message("\tONLY ONE BIOLOGICAL REPLICA AVAILABLE (plots are not possible) ")
  }
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# @title Pretty Labels for Heatmaps
#
# @description Generates pretty labels for the heatmaps.
# @param uniprot_acs (char) Uniprot accession id
# @param uniprot_ids (char) Uniprot entry id
# @param gene_names (car) Gene symbol
# @return Pretty labels for a heatmap
# @keywords internal, plots, pretty
.artms_prettyPrintHeatmapLabels <- function(uniprot_acs, uniprot_ids, gene_names) {
  result = paste(uniprot_acs, uniprot_ids, gene_names, sep = ' ')
  return(result)
}


# @title Correlation heatmaps of all the individual features
# @description Correlation heatmap using intensity values across all the
# conditions
# @param data_w (data.frame) resulting from the `.artms_castMaxQToWidePTM`
# function
# @param keys (data.frame) of the keys
# @param config (yaml.object) Configuration object (yaml loaded)
# @return (pdf) A correlation heatmap (suffix `-heatmap.pdf`)
# @keywords internal, heatmap, intensity, comparisons
.artms_sampleCorrelationHeatmap <- function (data_w, keys, config) {
  # data_w <- data.table::as.data.table(data_w2)
  # mat = log2(data_w[, 4:ncol(data_w), with = TRUE])
  # mat[is.na(mat)] = 0
  # mat_cor = cor(mat, method = 'pearson', use = 'everything')
  
  data_w <- log2(data_w[,4:ncol(data_w)])
  data_w[is.na(data_w)] <- 0
  mat_cor <- cor(data_w, method = 'pearson', use = 'everything')
  
  ## we want to make informarive row names so order by 
  ## RawFile because that's how data_w is ordered
  keys <- as.data.frame(keys)
  ordered_keys <- keys[with(keys, order(RawFile)), ] 
  mat_names <- paste(ordered_keys$Condition,
                    ordered_keys$BioReplicate,
                    ordered_keys$Run)
  colnames(mat_cor) <- mat_names
  rownames(mat_cor) <- mat_names
  colors_pos <- colorRampPalette(RColorBrewer::brewer.pal("Blues", n = 5))(10)
  colors_neg <- rev(colorRampPalette(RColorBrewer::brewer.pal("Reds", n = 5))(10))
  colors_tot <- c(colors_neg, colors_pos)
  pheatmap(
    mat = mat_cor,
    cellwidth = 10,
    cellheight = 10,
    scale = 'none',
    filename = gsub('.txt', '-heatmap.pdf', config$files$output),
    color = colors_tot,
    breaks = seq(from = -1, to = 1, by = .1),
    fontfamily = "mono"
  )
}



# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# @title Barplot of peptide counts per biological replicate
#
# @description Total number of unique peptide identified per biological
# replicate
# @param data_f (char) Evidence file (same structure as the original)
# @param config (yaml.object) Configuration object
# @return (pdf) Barplot of peptide counts
# @keywords barplot, counts, peptides
.artms_samplePeptideBarplot <- function(data_f, config) {
  # set up data into ggplot compatible format
  data_f <- data.table(data_f,
                       labels = paste(data_f$RawFile, 
                                      data_f$Condition, 
                                      data_f$BioReplicate))
  
  data_f <- data_f[with(data_f, order(labels, decreasing = TRUE)), ]
  
  # plot the peptide counts for all the samples TOGETHER
  p <- ggplot(data = data_f, aes(x = labels))
  p <- p + geom_bar(na.rm = TRUE) + theme(axis.text.x = element_text(angle = 90,
                                                         hjust = 1,
                                                         family = 'mono'
  )) + ggtitle('Unique peptides per run\n after filtering') + coord_flip()
  
  ggsave(filename = gsub('.txt', '-peptidecounts.pdf', config$files$output),
         plot = p,
         width = 8,
         height = 10)
  
  
  w <- 10
  h <- ceiling((7 / 5 + 2) * ceiling(length(unique(data_f$Condition)) / 5))
  # plot the peptide counts for all the samples PER BAIT
  p <- ggplot(data = data_f, aes(x = as.factor(BioReplicate)))
  p <- p + geom_bar(na.rm = TRUE) + theme(axis.text.x = element_text(angle = 90,
                                                         hjust = 1,
                                                         family = 'mono' )) + 
    ggtitle('Unique peptides per run\n after filtering') + 
    facet_wrap( ~ Condition, scales = 'free', ncol = 5)  + 
    theme(axis.text.x = element_text(angle = 45, hjust = 1))
  
  ggsave(filename = gsub('.txt', '-peptidecounts-perBait.pdf', config$files$output),
         plot = p,
         width = w,
         height = h)
}


# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# @title Select significant hits
#
# @description Filtered data.frame with significant values (log2fc > 2 |
# log2fc < -2; adj.pvalue < 0.05) from the MSstats results
# @param mss_results (data.frame) of MSstats results
# @param labels (vector) of selected labels. Default: all (`*`)
# @param LFC (vector, int) with the negative and positive threshold. Default:
# c(-2, 2)
# @param whatPvalue (char) `pvalue` or `adj.pvalue` (default)?
# @param FDR (int) false discovery rate (adj.pvalue) threshold. Default: 0.05
# @return (data.frame) only with significant hits
# @keywords internal, significant, selections
.artms_significantHits <- function(mss_results,
                                   labels = '*',
                                   LFC = c(-2, 2),
                                   whatPvalue = 'adj.pvalue',
                                   FDR = 0.05) {
  selected_results <- mss_results[grep(labels, mss_results$Label),]
  
  if (whatPvalue == "adj.pvalue") {
    significant_proteins <-
      selected_results[(
        !is.na(selected_results$log2FC) &
          selected_results$adj.pvalue <= FDR &
          (
            selected_results$log2FC >= LFC[2] |
              selected_results$log2FC <= LFC[1]
          )
      ), 'Protein']
  } else if (whatPvalue == "pvalue") {
    significant_proteins <-
      selected_results[(
        !is.na(selected_results$log2FC) &
          selected_results$pvalue <= FDR &
          (
            selected_results$log2FC >= LFC[2] |
              selected_results$log2FC <= LFC[1]
          )
      ), 'Protein']
  } else{
    stop("The whatPvalue argument is wrong. Valid options: pvalue or adj.pvalue")
  }
  
  significant_results <-
    selected_results[selected_results$Protein %in% significant_proteins,]
  return(significant_results)
}



# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# @title Summary stats for plots
#
# @description Gives count, mean, standard deviation, standard error of the mean, 
# and confidence interval (default 95%). 
# Borrowed from Cookbook for R 
# (http://www.cookbook-r.com/Manipulating_data/Summarizing_data/)
#
# @param data: a data frame.
# @param measurevar: the name of a column that contains the variable to be summariezed
# @param groupvars: a vector containing names of columns that contain grouping variables
# @param na.rm: a boolean that indicates whether to ignore NA's
# @param conf.interval: the percent range of the confidence interval (default is 95%)
# @return data.frame with summary statistics
# @keywords internal, plot, stats
.artms_summarySE <- function(data=NULL, 
                             measurevar, 
                             groupvars=NULL, 
                             na.rm=FALSE,
                             conf.interval=.95, 
                             .drop=TRUE) {
  
  # New version of length which can handle NA's: if na.rm==T, don't count them
  length2 <- function (x, na.rm=FALSE) {
    if (na.rm) sum(!is.na(x))
    else       length(x)
  }
  
  # This does the summary. For each group's data frame, return a vector with
  # N, mean, and sd
  datac <- plyr::ddply(data, groupvars, .drop=.drop,
                       .fun = function(xx, col) {
                         c(N    = length2(xx[[col]], na.rm=na.rm),
                           mean = mean   (xx[[col]], na.rm=na.rm),
                           sd   = sd     (xx[[col]], na.rm=na.rm)
                         )
                       },
                       measurevar
  )
  
  # Rename the "mean" column    
  datac <- rename(datac, c("mean" = measurevar))
  
  datac$se <- datac$sd / sqrt(datac$N)  # Calculate standard error of the mean
  
  # Confidence interval multiplier for standard error
  # Calculate t-statistic for confidence interval: 
  # e.g., if conf.interval is .95, use .975 (above/below), and use df=N-1
  ciMult <- qt(conf.interval/2 + .5, datac$N-1)
  datac$ci <- datac$se * ciMult
  
  return(datac)
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# @title Generate PCA plots based on abundance data
#
# @description Generate PCA plots based on abundance data
# @param x Data.frame output from `artms_loadModelQCstrict`
# @param filename Prefix to generate output names (WITH NO EXTENSION)
# @param allConditions Conditions selected to generate the plots
# @return PCA plots based on abundance data (pdf format)
# @keywords internal, plot, pca
.artms_getPCAplots <- function(x, filename, allConditions) {
  # PRINCIPAL COMPONENT ANALYSIS
  # Using the following packages:
  # FactoMineR, factoextra, corrplot, PerformanceAnalytics
  
  # Remove NA
  x <- as.data.frame(x)
  nogenename2 <- x[duplicated(x$Gene), ]
  if (dim(nogenename2)[1] > 0) {
    # message("\t---Removing proteins without a gene name: ")
    x <- x[complete.cases(x), ]
  }
  
  x <- x[!duplicated(x[, c("Gene")]),]
  rownames(x) <- x$Gene
  df <- x[, allConditions]
  
  # Correlation matrix
  df.cor.matrix <- round(cor(df, use = "complete.obs"), 2)
  
  # Correlation plots:
  out.correlation <- paste0(filename, "-correlations.pdf")
  pdf(out.correlation)
  corrplot::corrplot(
    df.cor.matrix,
    type = "upper",
    tl.col = "black",
    tl.srt = 45,
    diag = FALSE,
    addCoef.col = TRUE
  )
  PerformanceAnalytics::chart.Correlation(df,
                                          histogram = TRUE,
                                          pch = 19,
                                          main = "Correlation between Conditions")
  garbage <- dev.off()
  
  # PCA 1----
  
  res.pca <- FactoMineR::PCA(df,
                             scale.unit = TRUE,
                             ncp = 4,
                             graph = FALSE)
    
  eigenvalues <- res.pca$eig
  
  out.pca01 <- paste0(filename, "-pca01.pdf")
  pdf(out.pca01)
  # par(mfrow = c(1, 1))
  here1 <- plot(res.pca, choix = "var", new.plot = FALSE)
  print(here1)
  # This is equivalent to
  # PCA(df, scale.unit = TRUE, ncp = 4, graph = TRUE)
  garbage <- dev.off()
  
  # PCA 2----
  out.pca02 <- paste0(filename, "-pca02.pdf")
  pdf(out.pca02)
  barplot(
    eigenvalues[, 2],
    names.arg = seq_len(nrow(eigenvalues)),
    main = "Variances",
    xlab = "Principal Components",
    ylab = "Percentage of variances",
    col = "steelblue"
  )
  lines(
    x = seq_len(nrow(eigenvalues)),
    eigenvalues[, 2],
    type = "b",
    pch = 19,
    col = "red"
  )
  garbage <- dev.off()
  
  # PCA 3----
  h <- factoextra::fviz_pca_var(res.pca, 
                                col.var = "contrib") + theme_minimal()
  i <- factoextra::fviz_pca_biplot(res.pca,  
                                   labelsize = 3, 
                                   pointsize = 0.8) + theme_minimal()
  j <- factoextra::fviz_contrib(res.pca, choice = "var", axes = 1)
  l <- factoextra::fviz_contrib(res.pca, choice = "var", axes = 2)
  
  out.pca03 <- paste0(filename, "-pca03.pdf")
  pdf(out.pca03)
  print(h)
  print(i)
  print(j)
  print(l)
  garbage <- dev.off()
  
  # PCA 4
  df <- df[complete.cases(df),]
  prot.pca <- prcomp(t(df))
  prot.pcax <- as.data.frame(prot.pca$x)
  
  prot.pcax$Condition <- row.names(prot.pcax)
  
  p.pca.group <- ggplot(prot.pcax, 
                        aes(x = PC1, 
                            y = PC2, 
                            color = Condition)) + #, color=condition, shape=condition
    geom_point(alpha = .8, size = 3) +
    theme_light() + 
    labs(title = "PCA") 
  
  out.pca04 <- paste0(filename, "-pca04.pdf")
  pdf(out.pca04, width = 9, height = 7)
    plot(p.pca.group)
  garbage <- dev.off()
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' @title Volcano plot (log2fc / pvalues)
#'
#' @description It generates a scatter-plot used to quickly identify changes
#' @param mss_results (data.frame or file) Selected MSstats results
#' @param lfc_upper (numeric) log2fc upper threshold (positive value)
#' @param lfc_lower (numeric) log2fc lower threshold (negative value)
#' @param whatPvalue (char) `pvalue` or `adj.pvalue` (default)
#' @param FDR (numeric) False Discovery Rate threshold
#' @param output_name (char) Name for the output file (don't forget the `.pdf`
#' extension)
#' @param PDF (logical) Option to generate pdf format. Default: `T`
#' @param decimal_threshold (numeric) Decimal threshold for the pvalue.
#' Default: 16 (10^-16)
#' @param verbose (logical) `TRUE` (default) shows function messages
#' @keywords plot, volcano
#' @return (pdf) of a volcano plot
#' @examples
#' artmsVolcanoPlot(mss_results = artms_data_ph_msstats_results,
#'                   whatPvalue = "pvalue",
#'                   PDF = FALSE)
#' @export
artmsVolcanoPlot <- function(mss_results,
                             output_name = "volcano_plot.pdf",
                             lfc_upper = 1,
                             lfc_lower = -1,
                             whatPvalue = "adj.pvalue",
                             FDR = 0.05,
                             PDF = TRUE,
                             decimal_threshold = 16,
                             verbose = TRUE) {
  
  if(any(missing(mss_results)))
    stop("One (or many) of the required arguments missed. 
         Please, check the help for this function to find out more")
  
  if (PDF) {
    if (!grepl("\\.pdf", output_name)) {
      stop("File extension '.pdf' is missed for < output_name >")
    }
  }
  
  mss_results <- .artms_checkIfFile(mss_results)
  
  # handle cases where log2FC is Inf. There are no pvalues or other 
  # information for these cases :(
  # Issues with extreme_val later if we have Inf/-Inf values.
  if (sum(is.infinite(mss_results$log2FC)) > 0) {
    idx <- is.infinite(mss_results$log2FC)
    mss_results$log2FC[idx] <- NA
  }
  
  min_x <- -ceiling(max(abs(mss_results$log2FC), na.rm = TRUE))
  max_x <- ceiling(max(abs(mss_results$log2FC), na.rm = TRUE))
  
  # Deal with special cases in the data where we have pvalues = Inf,NA,0
  if (whatPvalue == "adj.pvalue") {
    if (sum(is.na(mss_results$adj.pvalue)) > 0)
      mss_results <- mss_results[!is.na(mss_results$adj.pvalue), ]
    if (nrow(mss_results[mss_results$adj.pvalue == 0 |
                         mss_results$adj.pvalue == -Inf, ]) > 0)
      mss_results[!is.na(mss_results$adj.pvalue) &
                    (mss_results$adj.pvalue == 0 |
                       mss_results$adj.pvalue == -Inf), ]$adj.pvalue = 10 ^ -decimal_threshold
    max_y = ceiling(-log10(min(mss_results[mss_results$adj.pvalue > 0, ]$adj.pvalue, na.rm = TRUE))) + 1
  } else if (whatPvalue == "pvalue") {
    if (sum(is.na(mss_results$pvalue)) > 0)
      mss_results <- mss_results[!is.na(mss_results$pvalue), ]
    if (nrow(mss_results[mss_results$pvalue == 0 |
                         mss_results$pvalue == -Inf, ]) > 0)
      mss_results[!is.na(mss_results$pvalue) &
                    (mss_results$pvalue == 0 |
                       mss_results$pvalue == -Inf), ]$pvalue = 10 ^ -decimal_threshold
    max_y = ceiling(-log10(min(mss_results[mss_results$pvalue > 0, ]$pvalue, na.rm = TRUE))) + 1
  } else{
    stop("The whatPvalue argument is wrong. Valid options: < pvalue > or < adj.pvalue >")
  }
  l <- length(unique(mss_results$Label))
  w_base <- 7
  h_base <- 7
  
  if (l <= 2) {
    w <- w_base * l
  } else{
    w <- w_base * 2
  }
  h <- h_base * ceiling(l / 2)
  
  if (whatPvalue == "adj.pvalue") {
    p <- ggplot(mss_results, aes(x = log2FC, y = -log10(adj.pvalue)))
    p <- p + geom_point(colour = 'grey', na.rm = TRUE) +
      geom_point(
        data = mss_results[mss_results$adj.pvalue <= FDR &
                             mss_results$log2FC >= lfc_upper, ],
        aes(x = log2FC, y = -log10(adj.pvalue)),
        colour = 'red',
        size = 1,
        na.rm = TRUE
      ) +
      geom_point(
        data = mss_results[mss_results$adj.pvalue <= FDR &
                             mss_results$log2FC <= lfc_lower, ],
        aes(x = log2FC, y = -log10(adj.pvalue)),
        colour = 'blue',
        size = 1,
        na.rm = TRUE
      ) +
      geom_vline(xintercept = c(lfc_lower, lfc_upper),
                 lty = 'dashed') +
      geom_hline(yintercept = -log10(FDR), lty = 'dashed') +
      xlim(min_x, max_x) +
      ylim(0, max_y) +
      facet_wrap(facets = ~ Label,
                 ncol = 2,
                 scales = 'fixed')
  } else{
    p <- ggplot(mss_results, aes(x = log2FC, y = -log10(pvalue)))
    p <- p + geom_point(colour = 'grey', na.rm = TRUE) +
      geom_point(
        data = mss_results[mss_results$pvalue <= FDR &
                             mss_results$log2FC >= lfc_upper, ],
        aes(x = log2FC, y = -log10(pvalue)),
        colour = 'red',
        size = 1,
        na.rm = TRUE
      ) +
      geom_point(
        data = mss_results[mss_results$pvalue <= FDR &
                             mss_results$log2FC <= lfc_lower, ],
        aes(x = log2FC, y = -log10(pvalue)),
        colour = 'blue',
        size = 1,
        na.rm = TRUE
      ) +
      geom_vline(xintercept = c(lfc_lower, lfc_upper),
                 lty = 'dashed') +
      geom_hline(yintercept = -log10(FDR), lty = 'dashed') +
      xlim(min_x, max_x) +
      ylim(0, max_y) +
      facet_wrap(facets = ~ Label,
                 ncol = 2,
                 scales = 'fixed')
  }
  
  if (PDF) {
    pdf(output_name, width = w, height = h)
    print(p)
    garbage <- dev.off()
    if(verbose) message(output_name, " is ready")
  } else{
    print(p)
  }
}

# ------------------------------------------------------------------------------
#' @title MaxQuant evidence file to SAINTexpress format
#'
#' @description Converts the MaxQuant evidence file to the 3 required files
#' by SAINTexpress. One can choose to either use the `spectral counts`
#' (use `msspc`) or the `intensities` (use `msint`) for the analysis.
#' @param evidence_file (char) The evidence file path and name
#' @param keys_file (char) Keys file with a SAINT column 
#' specifying test (`T`) and control (`C`) conditions
#' @param output_file (char) Output file name (must have extension .txt)
#' @param ref_proteome_file (char) Reference proteome path file name in
#' fasta format
#' @param quant_variable (char) choose either
#' - `msspc` (spectral counts, default) or
#' - `msint` (MS Intensity)
#' @param verbose (logical) `TRUE` (default) shows function messages
#' @return The 3 required files by SAINTexpress:
#' - `interactions.txt`
#' - `preys.txt`
#' - `baits.txt`
#' @keywords SAINT, SAINTexpress, APMS
#' @examples
#' # Testing that the files cannot be empty
#' artmsEvidenceToSaintExpress(evidence_file = NULL, 
#' keys_file = NULL, ref_proteome_file = NULL)
#' @export
artmsEvidenceToSaintExpress <- function(evidence_file,
                                           keys_file,
                                           ref_proteome_file,
                                           quant_variable = c('msspc','msint'),
                                           output_file, 
                                           verbose = TRUE) {
  if(verbose) message(">> CONVERTING TO SAINTexpress FORMAT ")
  
  if(is.null(evidence_file) & is.null(keys_file) & is.null(ref_proteome_file)){
    return("The evidence_file, keys_file, and ref_proteome_file 
           must not be empty")
  }

  if(any(missing(evidence_file) | 
         missing(keys_file) |
         missing(ref_proteome_file) | 
         missing(output_file)))
    stop("Missed (one or many) required argument(s)
         Please, check the help of this function to find out more")
    
  if (!grepl(".txt", output_file)) {
    stop(
      "Argument <output_file> must have the extension '.txt'"
    )
  }

  if(!file.exists(ref_proteome_file)){
    stop("The file ", ref_proteome_file, " does not exist ")
  }
  
  x <- fread(evidence_file, integer64 = 'double')
  keys <- fread(keys_file, integer64 = 'double')
  
  # Check Raw.file column
  x <- .artms_checkRawFileColumnName(x)
  keys <- .artms_checkRawFileColumnName(keys)
  
  # Check MS.MS
  x <- .artms_checkMSMSColumnName(x)
  
  
  if(verbose) message('>> VERIFYING DATA AND KEYS ')
  if (any(
    !c(
      'RawFile',
      'IsotopeLabelType',
      'Condition',
      'BioReplicate',
      'Run',
      'SAINT'
    ) %in% colnames(keys)
  )) {
    stop(
      'colnames in keys not conform to schema
      \tRawFile\tIsotopeLabelType\tCondition\tBioReplicate\tRun\tSAINT '
    )
  }
  
  saint_baits <- keys[, c('BioReplicate', 'Condition', 'SAINT'), with = FALSE]
  
  x <- artmsMergeEvidenceAndKeys(x, 
                                 keys, 
                                 by = c('RawFile'),
                                 verbose = verbose)
  
  data_f <- artmsFilterEvidenceContaminants(x = x, verbose = verbose)
  data_f <- .artms_removeMaxQProteinGroups(data_f)
  
  quant_variable <- match.arg(quant_variable)
  if(verbose) message(">> AGGREGATING ON ", quant_variable, " VALUES... ")
  ## aggregate over technical replicates if necessary
  if (quant_variable == 'msspc') {
    setnames(data_f, 'MS/MS Count', 'spectral_counts')
    data_f_agg <-
      aggregate(
        spectral_counts ~ BioReplicate+Condition+Proteins+Sequence+Charge,
        data = data_f,
        FUN = max
      )
    data_f_agg <-
      aggregate(
        spectral_counts ~ BioReplicate + Condition + Proteins,
        data = data_f_agg,
        FUN = sum
      )
  } else if (quant_variable == 'msint') {
    data_f_agg <-
      aggregate(
        Intensity ~ BioReplicate + Condition + Proteins + Sequence + Charge,
        data = data_f,
        FUN = max
      )
    data_f_agg <-
      aggregate(Intensity ~ BioReplicate + Condition + Proteins,
                data = data_f_agg,
                FUN = sum)
  } else{
    stop(" Wrong value for variable to quantify. 
         Please use 'msspc' or 'msint'")
  }
  
  ## IP name, bait name, prey name, and spectral counts or intensity values
  saint_interactions <- data_f_agg
  
  ref_proteome <- read.fasta(
    file = ref_proteome_file,
    seqtype = "AA",
    as.string = TRUE,
    set.attributes = TRUE,
    legacy.mode = TRUE,
    seqonly = FALSE,
    strip.desc = FALSE
  )
  p_lengths <- c()
  p_names <- c()
  for (e in ref_proteome) {
    p_lengths <- c(p_lengths, nchar(e[1]))
    p_names <- c(p_names, attr(e, 'name'))
  }
  ref_table <- data.table(names = p_names, lengths = p_lengths)
  ref_table[, uniprot_ac := gsub('([a-z,0-9,A-Z]+\\|{1})([A-Z,0-9,\\_]+)(\\|[A-Z,a-z,0-9,_]+)',
                                 '\\2',
                                 names)]
  ref_table[, uniprot_id := gsub('([a-z,0-9,A-Z]+\\|{1})([a-z,0-9,A-Z]+\\|{1})([A-Z,a-z,0-9,_]+)',
                                 '\\3',
                                 names)]
  
  unique_preys <- data.table(uniprot_ac = unique(data_f_agg$Proteins))
  saint_preys <- ref_table[, c('uniprot_ac', 'lengths', 'uniprot_id'), 
                          with = FALSE]
  saint_preys <- merge(unique_preys, saint_preys, by = 'uniprot_ac', 
                      all.x = TRUE)
  missing_lengths <- nrow(saint_preys[is.na(saint_preys$uniprot_id), ])
  saint_preys[is.na(saint_preys$uniprot_id), ]$uniprot_id = saint_preys[is.na(saint_preys$uniprot_id), ]$uniprot_ac
  if (missing_lengths > 0) {
    if(verbose)     
      message(
      sprintf(
        "--- WARNING! COMPUTING %s MISSING LENGTHS WITH THE MEDIAN LENGTH FROM THE DATASET ",
        missing_lengths))
    saint_preys[is.na(saint_preys$lengths), ]$lengths = median(saint_preys$lengths, na.rm = TRUE)
  }
  
  ## WRITE
  write.table(
    saint_baits,
    file = gsub('.txt', '-saint-baits.txt', output_file),
    eol = '\n',
    sep = '\t',
    quote = FALSE,
    row.names = FALSE,
    col.names = FALSE
  )
  write.table(
    saint_preys,
    file = gsub('.txt', '-saint-preys.txt', output_file),
    eol = '\n',
    sep = '\t',
    quote = FALSE,
    row.names = FALSE,
    col.names = FALSE
  )
  write.table(
    saint_interactions,
    file = gsub('.txt', '-saint-interactions.txt', output_file),
    eol = '\n',
    sep = '\t',
    quote = FALSE,
    row.names = FALSE,
    col.names = FALSE
  )
  if(verbose){
    message(">> OUTPUT FILES: ")
    message("--- ", gsub('.txt', '-saint-baits.txt', output_file))
    message("--- ", gsub('.txt', '-saint-preys.txt', output_file))
    message("--- ", gsub('.txt', '-saint-interactions.txt', output_file))
  }
}

# MANUSCRIPT DATASETS

######################################################
# Case Study: PHOSPHORYLATION
######################################################

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# Dataset 1: PHOSPHORYLATION DANIELLE
# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

setwd('~/artMS_Manuscript/CaseStudy_PH_Danielle/')

# PH GLOBAL
artmsQuantification("phglobal/phglobal_config.yaml")

setwd("phglobal/")

artmsAnalysisQuantifications(log2fc_file = "phglobal-results.txt", 
                             modelqc_file = "phglobal-results_ModelQC.txt", 
                             species = "human",
                             output_dir = "analysisPhglobalTest")

setwd("analysisPhglobal_adjpvalue/")

artmsPhosfateOutput(inputFile = "phglobal-results-log2fc-long.txt", output_dir = "phosfate")

# PH SITES
artmsProtein2SiteConversion(evidence_file = "evidence.txt", 
                            ref_proteome_file = "uniprot_canonical.fasta", 
                            column_name = "Leading razor protein",
                            output_file = "phsites-evidence.txt",
                            mod_type = "PH")

artmsQuantification("phsites/phsites_config.yaml")

setwd("phsites/")

artmsAnalysisQuantifications(log2fc_file = "phsites-results.txt", 
                             modelqc_file = "phsites-results_ModelQC.txt", 
                             output_dir = "analysisQuant", 
                             isPtm = "ptmsites", 
                             species = "human")

artmsVolcanoPlot(mss_results = "phsites-results.txt")


# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# Dataset 2: PHOSPHORYLATION PAPER OR8, OR9
# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

setwd("~/artMS_Manuscript/CaseStudy_PH_deGraaf_etal/")
evidence.ph <- read.delim("evidence.txt", stringsAsFactors = FALSE)
keys.ph <- read.delim("ph.keys.txt")
contrast.ph <- read.delim("ph.contrast.txt")

artmsProtein2SiteConversion(evidence_file = "evidence.txt", 
                            ref_proteome_file = "HUMAN_sprot-2012-09.fasta",
                            column_name = "Leading razor protein", 
                            output_file = "phsite-evidence.txt", 
                            mod_type = "PH")


# Create config files
# artmsWriteConfigYamlFile(config_file_name = "ph_global.yaml")
# artmsWriteConfigYamlFile(config_file_name = "ph_sites.yaml")

# PHGLOBAL

artmsQualityControlEvidenceBasic(evidence_file = "evidence.txt", 
                                 keys_file = "ph.option_1.keys.txt", 
                                 prot_exp = "PH", 
                                 output_name = "qc.phglobal.o1")

artmsQualityControlEvidenceExtended(evidence_file = "evidence.txt", 
                                    keys_file = "ph.option_1.keys.txt")


artmsQuantification(yaml_config_file = "phglobal.option_1.config.yaml")


setwd("phglobal_results_option1")

artmsAnalysisQuantifications(log2fc_file = "phglobal.results.txt", 
                             modelqc_file = "phglobal.results_ModelQC.txt", 
                             species = "human", 
                             output_dir = "analysisQuant")

setwd("../")

artmsQuantification(yaml_config_file = "ph.option_2.config.yaml")
setwd("results_phglobal_option2/")
artmsAnalysisQuantifications(log2fc_file = "phglobal.results.txt", 
                             modelqc_file = "phglobal.results_ModelQC.txt", 
                             species = "human", output_dir = "analysisQuant")


# checking normalized data
normalized <- read.delim("phglobal.results-mss-normalized.txt", stringsAsFactors = FALSE)

p1 <- ggplot2::ggplot(normalized, 
                      aes(x = GROUP_ORIGINAL, 
                          y = ABUNDANCE, 
                          fill = ABUNDANCE))
p1 <- p1 + geom_boxplot(aes(fill = GROUP_ORIGINAL))
p1 <- p1 + theme_linedraw()
p1 <-
  p1 + theme(
    axis.text.x = element_text(
      angle = 90,
      hjust = 1,
      vjust = 0.5
    ),
    legend.position = "none"
  )
p1 <- p1 + labs(x = "BIOREPLICATES")
p1 <- p1 + ggtitle("Relative Abundance BioReplicates")
print(p1)


artmsQuantification(yaml_config_file = "ph.OR8.config.yaml")


setwd("phglobal_OR8_results/")
artmsAnalysisQuantifications(log2fc_file = "phglobal.OR8.results.txt", 
                             modelqc_file = "phglobal.OR8.results_ModelQC.txt", 
                             output_dir = "analysisQuant", species = "human")





setwd("../")
artmsQuantification(yaml_config_file = "ph.OR9.config.yaml")

setwd("OR9_results_phglobal/")
artmsAnalysisQuantifications(log2fc_file = "phglobal.OR9.results.txt", 
                             modelqc_file = "phglobal.OR9.results_ModelQC.txt", 
                             output_dir = "analysisQuant", species = "human")




# PH SITES: 
artmsProtein2SiteConversion(evidence_file = "evidence.txt", 
                            ref_proteome_file = "HUMAN_sprot-2012-09.fasta", 
                            output_file = "phsite-evidence.txt", 
                            mod_type = "PH")

artmsQuantification(yaml_config_file = "ph.option_1.config.yaml")






##############################################################################
# CASE STUDY: CHANGES IN PROTEIN ABUNDANCE
##############################################################################

setwd("~/artMS_Manuscript/CaseStudy_ProteinAbundance/")

artmsWriteConfigYamlFile(config_file_name = "globalAbundance-config.yaml")

artmsQualityControlEvidenceBasic()

evidence_file = "ab-evidence.txt"
keys_file = "ab-keys.txt"
prot_exp =  "PH"
isSILAC = FALSE
plotINTDIST = TRUE
plotREPRO = TRUE
plotCORMAT = TRUE
plotINTMISC = TRUE
plotPTMSTATS = TRUE
printPDF = TRUE
verbose = TRUE

artmsQuantification(yaml_config_file = "globalAbundance-config.yaml")

setwd("results/")
artmsAnalysisQuantifications(log2fc_file = "ab-results.txt", 
                             modelqc_file = "ab-results_ModelQC.txt", 
                             species = "mouse", 
                             output_dir = "analysisQuant")


##############################################################################
# CASE STUDY: APMS
##############################################################################

# Regarding the controls, I prepared 4 replicates for each control control_HIVwt, 
# control_HIVwt_MG132 etc. However when I performed the final analysis 
# I combined the three baits (CUL5, ELOB and CBFB) for running the SAINT 
# analysis. Since I had 4 replicates of each control for each bait 
# (in total 12 replicates, I decided to use only 2 replicates that I 
#   prepared in parallel with each bait). Then I ran SAINT together 
# for all the three baits and had 6 controls for each condition.
# Please let me know if this makes sense. 

# The CUL5mock condition is not a control, only the ones that are labeled
# with control. CUL5mock condition was used to determine interactors of CUL5 
# in the absence of HIV infection. 
# So this is the way how I analyzed the data:
# 1. Use SAINT to determine specific interactions of CUL5 int he presence
# and absence of HIV infection
# 2. Using MSstats to determine differences in interactors between CUL5mock
# and CUL5wt.


setwd("~/artMS_Manuscript/CaseStudy_APMS_PXD009012/")

artmsQualityControlEvidenceBasic(evidence_file = "APMS-evidence.txt", 
                                 keys_file = "APMS-keys.txt", 
                                 prot_exp = "APMS")
artmsQualityControlEvidenceExtended(evidence_file = "APMS-evidence.txt", 
                                    keys_file = "APMS-keys.txt")

#-------------------------------------------------------------------------------
#' @title Generate Phosfate Input file
#' 
#' @description It takes as input the `imputedL2fcExtended.txt` results
#' generated by the `artmsAnalysisQuantifications()` function and generates
#' the [Phosfate](http://phosfate.com/) input file (or data.frame)
#' Please, notice that the only species suported by Phosfate is humans.
#' @param inputFile (char) the `imputedL2fcExtended.txt` file name and location
#' @param output_dir (char) Name of the folder to output results 
#' (Default: current directory. Recommended: `phosfate_input`)
#' @param verbose (logical) `TRUE` (default) to show function messages
#' @return Multiple output files (inputs of phosfate)
#' @keywords generate, outputs, files
#' @examples \dontrun{
#' artmsPhosfateOutput(inputFile)
#' }
#' @export
artmsPhosfateOutput <- function(inputFile, 
                                output_dir = ".",
                                verbose = TRUE){

  df <- .artms_checkIfFile(inputFile)
  
  # Check the existance of the columns that are needed.
  checkColumns <- c('Protein', 'PTMsite', 'iLog2FC', 'Comparison')
  
  if (any( !checkColumns %in% colnames(df)) ) {
    stop("One (or many) column names are not found: ",
         sprintf('%s, ',checkColumns))
  }
    
  # create output directory if it doesn't exist
  if(!dir.exists(output_dir)){
    dir.create(output_dir, recursive = TRUE)
  }
  
  conditions <- unique(df$Comparison)
  
  for ( i in seq_len(length(conditions)) ){
    if(verbose) message("+---", i, conditions[i], appendLF = FALSE)
    df.select <- df[which(df$Comparison == conditions[i]),]
    df.out <- df.select[c('Protein','PTMsite','iLog2FC')]
    fileout <- gsub(".txt","", inputFile)
    fileout <- paste0(fileout,"-",conditions[i],".txt")
    fileout <- paste0(output_dir,"/",fileout)
    write.table(df.out, fileout, col.names = FALSE, 
                row.names = FALSE, 
                quote = FALSE, sep = ",")
    if(verbose) message(":", fileout, " is out ")
  }
}

#-------------------------------------------------------------------------------
#' @title Generate PHOTON Input file
#' 
#' @description It takes as input the `imputedL2fcExtended.txt` results
#' generated by the `artmsAnalysisQuantifications()` function and generates
#' the [PHOTON](https://github.com/jdrudolph/photon) input file.
#' Please, notice that the only species suported by PHOTON is humans.
#' @param inputFile (char) the `imputedL2fcExtended.txt` file name and location
#' @param output_dir (char) Name of the folder to output results 
#' (Default: current. Recommended: "photon_input_files" or similar)
#' @param verbose (logical) `TRUE` (default) to show function messages
#' @return Multiple output files (inputs of phosfate)
#' @keywords generate, outputs, files
#' @examples \dontrun{
#' artmsPhotonOutput(inputFile)
#' }
#' @export
artmsPhotonOutput <- function(inputFile,
                              output_dir = ".",
                              verbose = TRUE){
  
  df <- .artms_checkIfFile(inputFile)
  
  # Check the existance of the columns that are needed.
  checkColumns <- c('Gene', 'PTMsite', 'iLog2FC', 'EntrezID')
  
  if (any( !checkColumns %in% colnames(df)) ) {
    stop("One (or many) column names are not found: ",sprintf('%s, ',checkColumns))
  }
  
  # create output directory if it doesn't exist
  if(!dir.exists(output_dir)){
    dir.create(output_dir, recursive = TRUE)
  }
  
  conditions <- unique(df$Comparison)
  
  for ( i in seq_len(length(conditions)) ){
    if(verbose) message("+---", i, conditions[i], appendLF = FALSE)
    df.select <- df[which(df$Comparison == conditions[i]),]
    
    # first, filter by pvalue
    df.select <- df.select[which(df.select$iPvalue < 0.05),]
    
    # Photon takes this input columns: GeneID,Amino.Acid,Position,avg,Symbol
    df.out <- df.select[c('Gene','PTMsite','iLog2FC','EntrezID')]
    df.out$Amino.Acid <- df.select$PTMaa
    df.out <- artmsChangeColumnName(df.out, "EntrezID", "GeneID")
    df.out <- artmsChangeColumnName(df.out, "PTMsite", "Position")
    df.out <- artmsChangeColumnName(df.out, "iLog2FC", "avg")
    df.out <- artmsChangeColumnName(df.out, "Gene", "Symbol")
    
    df.out <- df.out[c('GeneID', 'Amino.Acid', 'Position', 'avg', 'Symbol')]
    if(any(grep(";", df.out$Symbol))){
      if(verbose) message("\t---(-) Removed multiple ids from column 'Protein' ")
      df.out <- df.out[-grep(";", df.out$Symbol),]
    }
    fileout <- gsub(".txt","", inputFile)
    fileout <- paste0("photon.",conditions[i],".csv")
    fileout <- paste0(output_dir,"/",fileout)
    write.table(df.out, fileout, 
                col.names = TRUE, row.names = FALSE, 
                quote = FALSE, sep = ",")
    if(verbose) message(":",fileout," is out ")
  }
}

# ------------------------------------------------------------------------------
#' @title Annotate table with Gene Symbol and Name based on Uniprot ID(s)
#'
#' @description Annotate gene name and symbol based on uniprot ids. It will 
#' take the column from your data.frame specified by the `columnid` argument,
#' search for the gene symbol, name, and entrez based on the species (`species`
#' argument) and merge the information back to the input data.frame
#' @param x (data.frame) to be annotated (or file path and name)
#' @param columnid (char) The column with the uniprotkb ids
#' @param species (char) The species name. Check `?artmsMapUniprot2Entrez`
#' to find out more about supported species.
#' @param verbose (logical) `TRUE` (default) shows function messages
#' @return (data.frame) with two new columns: `Gene` and `Protein.name`
#' @keywords annotation, uniprot
#' @examples
#' # This example adds annotations to the example evidence file included in
#' # artMS, based on the column 'Proteins'.
#'
#' evidence_anno <- artmsAnnotationUniprot(x = artms_data_ph_evidence,
#'                                          columnid = 'Proteins',
#'                                          species = 'human')
#' @export
artmsAnnotationUniprot <- function(x, 
                                   columnid, 
                                   species,
                                   verbose = TRUE) {
  
  if(any(missing(x) | 
         missing(columnid) |
         missing(species)))
    stop("Missed (one or many) required argument(s)
         Please, check the help of this function to find out more")
  
  if(!is.character(columnid)) stop("Argument 'columnid' must be a character")
  if(!is.character(species)) stop("Argument <species> must be a character")
  
  x <- .artms_checkIfFile(x)
  
  # Deal with isoforms UNIPROT-1 etc
  if(!columnid %in% colnames(x)){
    stop("The column id <",columnid,"> was not found in this dataset")
  }
  x$EntryRoot <- x[[columnid]]
  
  # Check first if it is uniprot:
  isUniprot <- TRUE
  if( length(x$EntryRoot[grep("\\w{2}_\\d{1,}\\.\\d{1,}", x$EntryRoot)]) > 100 ){
    message("\nWarning!
    It looks like this dataset is using RefSeq as protein ID.
    RefSeq is not (yet) supported for gene annotation.\n")
    isUniprot <- FALSE
  }
  
  x$EntryRoot <- gsub("(\\w+)(-)(\\d+)", "\\1", x$EntryRoot)
  
  # Deal with protein groups and PTMs: select the first ID 
  # 1. strsplit based on ;
  x$EntryRoot <- unlist(lapply(x$EntryRoot, function(x) unlist(strsplit(x, "\\;"))[1]))
  # 2. now based on PTM: either STYK
  x$EntryRoot <- unlist(lapply(x$EntryRoot, function(x) unlist(strsplit(x, "_[S|T|Y|K]"))[1]))
  
  theUniprots <- as.character(unique(x$EntryRoot))
  
  if( isFALSE(artmsIsSpeciesSupported(species = species)) | isFALSE(isUniprot) ){
    if(isFALSE(artmsIsSpeciesSupported(species = species))) 
      message("---(-) Species ", species," not supported: no info about proteins will be provided")
    keepSearchName <- paste0(columnid, "Copy")
    y <- artmsChangeColumnName(x, columnid, keepSearchName)
    y <- artmsChangeColumnName(y, "EntryRoot", "Protein")
    y$Gene <- y$Protein
    y$ProteinName <- NA
    y$EntrezID <- NA
    send_back <- y[c("Protein", "Gene", "ProteinName", "EntrezID")]
    send_back <- unique(send_back)
    y <- subset(y, select = -c(Gene, ProteinName, EntrezID))
    send_back <- merge(send_back, y, by = "Protein")
    return(send_back)
  }else{
    preload <- artmsMapUniprot2Entrez(uniprotkb = theUniprots, 
                                      species = species)
    
    dc_merged <-merge(x, 
                      preload,
                      by.x = "EntryRoot",
                      by.y = "UNIPROT",
                      all.x = TRUE)
    
    # Move Gene name to the left:
    gene_first <- preload[, c("SYMBOL", "UNIPROT", "GENENAME", "ENTREZID")]
    dc_merged <- subset(dc_merged, select = -c(SYMBOL, GENENAME, ENTREZID))
    send_back <-
      merge(
        gene_first,
        dc_merged,
        by.x = "UNIPROT",
        by.y = "EntryRoot",
        all.y = TRUE
      )
    keepSearchName <- paste0(columnid,"Ref")
    send_back <- artmsChangeColumnName(send_back, columnid, keepSearchName)
    names(send_back)[grep("^UNIPROT$", names(send_back))] <- "Protein"
    names(send_back)[grep("^SYMBOL$", names(send_back))] <- "Gene"
    names(send_back)[grep("^GENENAME$", names(send_back))] <- "ProteinName"
    names(send_back)[grep("^ENTREZID$", names(send_back))] <- "EntrezID"
    # Some uniprot entries might not have yet a gene name, 
    # which will be an empty value. Replace with Entry name:
    send_back$Gene[which(send_back$Gene == "")] <- NA
    send_back$Gene[is.na(send_back$Gene)] <- send_back$Protein[is.na(send_back$Gene)]
    return(send_back)
  }
}


# ------------------------------------------------------------------------------
# @title Select the entry 
#
# @description Selet the Uniprot Entry ID from a full Uniprot id. For example:
#
# From `sp|P55011|S12A2_HUMAN` will select `P55011`
#
# @param x (vector) of protein ids. If the id is a uniprot full entry, it will
# extract the ENTRY id
# @return (vector) with only the Entry ID (if found)
# @keywords internal, Protein ID, Uniprot, Entry
.artms_selectEntryFromFullUniprot <- function(x){
  isolateIt <-  unlist(strsplit(x, ";"))
  replaceIt <- as.character(sapply(isolateIt, function(y) gsub("(^sp|tr)(\\|)(.*)(\\|.*)", "\\3", y)))
  if(length(replaceIt) > 1){
    replaceIt <- paste(replaceIt, collapse = ";")
  }
  return(replaceIt)
}


# ------------------------------------------------------------------------------
#' @title Leave only the Entry ID from a typical full Uniprot IDs in a 
#' given column 
#'
#' @description Downloading a Reference Uniprot fasta database includes several 
#' Uniprot IDs for every protein. If the regular expression available in 
#' Maxquant is not activated, the full id will be used in the Proteins,
#' Lead Protein, and Leading Razor Protein columns. This script leaves only the
#' Entry ID.
#' 
#' For example, values in a Protein column like this:
#' 
#' `sp|P12345|Entry_name;sp|P54321|Entry_name2` 
#' 
#'  will be replace by
#'  
#' `P12345;P54321``
#' 
#' @param x (data.frame) that contains the `columnid`
#' @param columnid (char) Column name with the full uniprot ids
#' @return (data.frame) with only Entry IDs.
#' 
#' @keywords annotation, ids
#' @examples
#' # Example of data frame with full uniprot ids and sequences
#' p <- c("sp|A6NIE6|RN3P2_HUMAN;sp|Q9NYV6|RRN3_HUMAN", 
#'        "sp|A7E2V4|ZSWM8_HUMAN", 
#'        "sp|A5A6H4|ROA1_PANTR;sp|P09651|ROA1_HUMAN;sp|Q32P51|RA1L2_HUMAN", 
#'        "sp|A0FGR8|ESYT2_HUMAN")
#' s <- c("ALENDFFNSPPRK", "GWGSPGRPK", "SSGPYGGGGQYFAK", "VLVALASEELAK")
#' evidence <- data.frame(Proteins = p, Sequences = s, stringsAsFactors = FALSE)
#' 
#' # Replace the Proteins column with only Entry ids
#' evidence <- artmsLeaveOnlyUniprotEntryID(x = evidence, columnid = "Proteins")
#' @export
artmsLeaveOnlyUniprotEntryID <- function(x, columnid){
  
  if(!(is.data.frame(x) | is.data.table(x))){
    stop("<x> must be a data.frame or data.table")
  }
  
  if(!is.vector(columnid)){
    stop("<columnid> must be a valid column name")
  }
  
  if(!(columnid %in% colnames(x))){
    stop("The <columnid> is not a column of <x>. Check the name again")
  }
  
  x[[columnid]] <- unlist(lapply(x[[columnid]], 
                                 function(z) sapply(z, .artms_selectEntryFromFullUniprot)))
  
  return(x)
}


# ------------------------------------------------------------------------------
#' @title Map GENE SYMBOL, NAME, AND ENTREZID to a vector of Uniprot IDS
#'
#' @description Map GENE SYMBOL, NAME, AND ENTREZID to a vector of Uniprot IDS
#' @param uniprotkb (vector) Vector of UniprotKB IDs
#' @param species (char) The species name. Species currently supported 
#' as part of artMS: check `?artmsIsSpeciesSupported()` to find out the
#' list of supported species`
#' @return (data.frame) with ENTREZID and GENENAMES mapped on UniprotKB ids
#' @keywords annotation, ids
#' @examples
#' # Load an example with human proteins
#' exampleID <- c("Q6P996", "B1N8M6")
#' artmsMapUniprot2Entrez(uniprotkb = exampleID, 
#'                        species = "HUMAN")
#' @export
artmsMapUniprot2Entrez <- function(uniprotkb, 
                                   species) {
  
  if( !("org.Hs.eg.db" %in% installed.packages()) ){
    stop("Package <org.Hs.eg.db> required to run this function. Please, install and run it again")
  }
  
  if(any(missing(uniprotkb) | 
         missing(species)))
    stop("Missed (one or many) required argument(s)
         Please, check the help of this function to find out more")
  
  if(!is.vector(uniprotkb)) stop("Argument <uniprotkb> is not a vector")
  if(!is.character(species)) stop("Argument <species> is not a vector")

  if(isFALSE(artmsIsSpeciesSupported(species = species))){
    stop("Species ", species," not supported")
  }else{
    thePack <- artmsIsSpeciesSupported(species = species)
  }

  suppressMessages(
    mappings <-
      AnnotationDbi::select(
        eval(as.symbol(thePack)),
        uniprotkb,
        c("UNIPROT", "SYMBOL",
          "GENENAME", "ENTREZID"),
        keytype = "UNIPROT"
      )
  )

  mappings <- unique(mappings)
  # It migth come with 1 uniprot to many gene names: 
  # take the first one, which should be the main gene name
  mappings <- mappings[!duplicated(mappings$UNIPROT),]
  return(mappings)
}


# ------------------------------------------------------------------------------
#' @title Check if a species is supported and available
#'
#' @description Given a species name, it checkes whether is supported, and 
#' if supported, check whether the annotation package is installed.
#' @param species (char) The species name. Species currently supported 
#' as part of artMS:
#' - HUMAN
#' - MOUSE
#' 
#' And the following species can be used as well, but the user needs to 
#' install the corresponding org.db package:
#' - ANOPHELES (`install.packages(org.Ag.eg.db)`)
#' - BOVINE (`install.packages(org.Bt.eg.db)`)
#' - WORM (`install.packages(org.Ce.eg.db)`)
#' - CANINE (`install.packages(org.Cf.eg.db)`)
#' - FLY (`install.packages(org.Dm.eg.db)`)
#' - ZEBRAFISH (`install.packages(org.Dr.eg.db)`)
#' - CHICKEN (`install.packages(org.Gg.eg.db)`)
#' - RHESUS (`install.packages(org.Mmu.eg.db)`)
#' - CHIMP (`install.packages(org.Pt.eg.db)`)
#' - RAT (`install.packages(org.Rn.eg.db)`)
#' - YEAST (`install.packages(org.Sc.sgd.db)`)
#' - PIG (`install.packages(org.Ss.eg.db)`)
#' - XENOPUS (`install.packages(org.Xl.eg.db)`)
#' @param verbose (logical) `TRUE` (default) shows function messages
#' @return (string) Name of the package for the given species
#' @keywords annotation, species
#' @examples
#' # Should return TRUE
#' artmsIsSpeciesSupported(species = "HUMAN")
#' artmsIsSpeciesSupported(species = "CHIMP")
#' @export
artmsIsSpeciesSupported <- function(species, 
                                    verbose = TRUE) {
  
  if(any(missing(species)))
    stop("Missed (one or many) required argument(s)
         Please, check the help of this function to find out more")
  
  if(!is.character(species)) stop("Argument <species> is not a vector")
  
  species <- toupper(species)
  
  LongName <- c(
    "ANOPHELES", 
    "BOVINE",
    "WORM",
    "CANINE",
    "FLY",
    "ZEBRAFISH",
    "CHICKEN",
    "HUMAN",
    "MOUSE",
    "RHESUS",
    "CHIMP",
    "RAT",
    "YEAST",
    "PIG",
    "XENOPUS")
  PackageName <- c(
    "org.Ag.eg.db", 
    "org.Bt.eg.db",
    "org.Ce.eg.db",
    "org.Cf.eg.db",
    "org.Dm.eg.db",
    "org.Dr.eg.db",
    "org.Gg.eg.db",
    "org.Hs.eg.db",
    "org.Mm.eg.db",
    "org.Mmu.eg.db",
    "org.Pt.eg.db",
    "org.Rn.eg.db",
    "org.Sc.sgd.db",
    "org.Ss.eg.db",
    "org.Xl.eg.db")
  
  OrgDB <- data.frame("LongName" = LongName, 
                      "PackageName" = PackageName, 
                      stringsAsFactors = FALSE)
  
  if(species %in% OrgDB$LongName){
    thePack <- OrgDB$PackageName[which(OrgDB$LongName == species)]
    
    if( !(thePack %in% rownames(installed.packages())) ){
      if(verbose) message("---(-) The package <",thePack,"> is not installed in your system.
           Just run: install.packages('", thePack,"') and try again")
      return(FALSE)
    } else{
      return(thePack)
    }
  }else{
    return(FALSE)
  }
}

#' @title Extended Quality Control of the MaxQuant evidence.txt file
#'
#' @description Performs quality control based on the information available in
#' the MaxQuant `evidence.txt` file.
#' @param evidence_file (char or data.frame) The evidence file path and name, or
#' data.frame
#' @param keys_file (char or data.frame) The keys file path and name or
#' data.frame
#' @param output_dir (char) Name for the folder to output the results plots. 
#' Default is "qc_extended".
#' @param output_name (char) prefix output name (no extension).
#' Default: "qcExtended_evidence"
#' @param isSILAC if `TRUE` processes SILAC input files. Default is `FALSE`
#' @param plotPSM (logical) `TRUE` generates peptide-spectrum-matches (PSMs) 
#' statistics plot: Page 1 shows the number of PSMs confidently identified 
#' in each BioReplicate. If replicates are present, Page 2 shows the mean 
#' number of PSMs per condition with error bar showing the standard error 
#' of the mean. Note that potential contaminant proteins are plotted separately. 
#' @param plotIONS (logical) `TRUE` generates peptide ion statistics plot: 
#' A peptide ion is defined in the context of m/z, in other words, an unique 
#' peptide sequence may give rise to multiple ions with different charge state 
#' and/or amino acid modification. Page 1 shows the number of ions confidently
#'  identified in each BioReplicate . If replicates are present, Page 2 shows 
#'  the mean number of peptide ions per condition with error bar showing the 
#'  standard error of the mean. Note that potential contaminant proteins are 
#'  plotted separately.  
#' @param plotTYPE (logical) `TRUE` generates identification type statistics 
#' plot: MaxQuant classifies each peptide identification into different 
#' categories (e.g., MSMS, MULTI-MSMS, MULTI-SECPEP). 
#' Page 1 shows the distribution of identification type in each BioReplicate
#' @param plotPEPTIDES (logical) `TRUE` generates peptide statistics plot: 
#' Page 1 shows the number of unique peptide sequences (disregard the charge 
#' state or amino acid modifications) confidently identified in each 
#' BioReplicate. If replicates are present, Page 2 shows the mean number of 
#' peptides per condition with error bar showing the standard error of the 
#' mean. Note that potential contaminant proteins are plotted separately. 
#' Pages 3 and 4 show peptide identification intersection between 
#' BioReplicates (the bars are ordered by degree or frequency, respectively),
#'  and Page 4 shows the intersections across conditions instead of 
#'  BioReplicates.
#' @param plotPEPTOVERLAP (logical) `TRUE` Show peptide identification 
#' intersection between BioReplicates and Conditions
#' @param plotPROTEINS (logical) `TRUE` generates protein statistics plot: 
#' Page 1 shows the number of protein groups confidently identified in each 
#' BioReplicate. If replicates are present, Page 2 shows the mean number of 
#' protein groups per condition with error bar showing the standard error of 
#' the mean. Note that potential contaminant proteins are plotted separately.
#' Pages 3 and 4 show peptide identification intersection between 
#' BioReplicates (the bars are ordered by degree or frequency, respectively), 
#' and Page 4 shows the intersections across conditions instead of 
#' BioReplicates.
#' @param plotPROTOVERLAP (logical) `TRUE` Show protein identification 
#' intersection between BioReplicates and Conditions
#' @param plotPIO (logical) `TRUE` generates oversampling statistics plot: 
#' Page 1 shows the proportion of all peptide ions (including peptides 
#' matched across runs) fragmented once, twice and thrice or more. 
#' Page 2 shows the  proportion of peptide ions (with intensity detected) 
#' fragmented once, twice and thrice or more. Page 3 shows the proportion of 
#' peptide ions (with intensity detected and MS/MS identification) fragmented 
#' once, twice and thrice or more
#' @param plotCS (logical) `TRUE` generates charge state plot: Page 1 shows 
#' the charge state distribution of PSMs confidently identified in each 
#' BioReplicate.
#' @param plotME (logical) `TRUE` generates precursor mass error plot: 
#' Page 1 shows the distribution of precursor error for all PSMs confidently 
#' identified in each BioReplicate.
#' @param plotMOCD (logical) `TRUE` generates precursor mass-over-charge plot: 
#' Page 1 shows the distribution of precursor mass-over-charge for all PSMs 
#' confidently identified in each BioReplicate. 
#' @param plotPEPICV (logical) `TRUE` generates peptide intensity coefficient 
#' of variance (CV) plot: The CV is calculated for each feature (peptide ion) 
#' identified in more than one replicate. Page 1 shows the distribution of 
#' CV's for each condition, while Page 2 shows the distribution of CV's 
#' within 4 bins of intensity (i.e., 4 quantiles of average intensity).
#' @param plotPEPDETECT (logical) `TRUE` generates peptide detection 
#' frequency plot: Page 1 summarizes the frequency that each peptide is 
#' detected across BioReplicates of each condition, showing the percentage 
#' of peptides detected once, twice, thrice, and so on (for whatever number 
#' of replicates each condition has).
#' @param plotPROTICV (logical) `TRUE` generates protein intensity coefficient 
#' of variance (CV) plot: The CV is calculated for each protein (after summing 
#' the peptide intensities) identified in more than one replicate. 
#' Page 1 shows the distribution of CV's for each condition, while Page 2 
#' shows the distribution of CV's within 4 bins of intensity (i.e., 4 
#' quantiles of average intensity).
#' @param plotPROTDETECT (logical) `TRUE` generates protein detection 
#' frequency plot: Page 1 summarizes the frequency that each protein group 
#' is detected across BioReplicates of each condition, showing the 
#' percentage of proteins detected once, twice, thrice, and so on (for 
#' whatever number of replicates each condition has). Page 2 shows the feature 
#' (peptide ion) intensity distribution within each BioReplicate (potential 
#' contaminant proteins are plot separately). Page 3 shows the density of 
#' feature intensity for different feature types (i.e., MULTI-MSMS, 
#' MULTI-SECPEP).
#' @param plotIDoverlap (logical) `TRUE` generates pairwise identification 
#' heatmap overlap: Pages 1 and 2 show pairwise peptide and protein overlap 
#' between any 2 BioReplicates, respectively. 
#' @param plotPCA (logical) `TRUE` generates PCA and pairwise intensity correlation: 
#' Page 1 and 3 show pairwise peptide and protein intensity correlation and 
#' scatter plot between any 2 BioReplicates, respectively. Page 2 and 4 show 
#' Principal Component Analysis at the intensity level for both peptide and 
#' proteins, respectively. 
#' @param plotSP (logical)  `TRUE` generates sample quality metrics: Page 1 
#' shows missing cleavage distribution of all peptides confidently identified 
#' in each BioReplicate. Page 2 shows the fraction of peptides with at least 
#' one methionine oxidized in each BioReplicate. 
#' @param printPDF If `TRUE` (default) prints out the pdfs. Warning: plot
#' objects are not returned due to the large number of them. 
#' @param verbose (logical) `TRUE` (default) shows function messages
#' @details all the plots are generated by default
#' @return A number of QC plots based on the evidence file
#' @keywords qc, evidence, keys
#' @examples
#' # Testing warning if files are not submitted
#' test <- artmsQualityControlEvidenceExtended(evidence_file = NULL,
#' keys_file = NULL)
#' @export
artmsQualityControlEvidenceExtended <- function(evidence_file,
                                                keys_file,
                                                output_dir = "qc_extended",
                                                output_name = "qcExtended_evidence",
                                                isSILAC = FALSE,
                                                plotPSM = TRUE,
                                                plotIONS = TRUE,
                                                plotTYPE = TRUE,
                                                plotPEPTIDES = TRUE,
                                                plotPEPTOVERLAP = TRUE,
                                                plotPROTEINS = TRUE,
                                                plotPROTOVERLAP = TRUE,
                                                plotPIO = TRUE,
                                                plotCS = TRUE,
                                                plotME = TRUE,
                                                plotMOCD = TRUE,
                                                plotPEPICV = TRUE,
                                                plotPEPDETECT = TRUE,
                                                plotPROTICV = TRUE,
                                                plotPROTDETECT = TRUE,
                                                plotIDoverlap = TRUE,
                                                plotPCA = TRUE,
                                                plotSP = TRUE,
                                                printPDF = TRUE,
                                                verbose = TRUE){
  
  # Local variables
  keysilac = NULL

  if(verbose){
    message("---------------------------------------------")
    message("artMS: EXTENDED QUALITY CONTROL (-evidence.txt based)")
    message("---------------------------------------------")
  }
  
  if (is.null(evidence_file) & is.null(keys_file)) {
    return("Evidence and keys cannot be NULL")
  }
  
  if(is.null(output_dir)){
    return("The output_dir argument cannot be NULL")
  }
  
  hmcol <- colorRampPalette(RColorBrewer::brewer.pal(10, "RdBu"))(256)
  
  # DATA PREPARATION----
  
  # OPEN KEYS
  keys <- .artms_checkIfFile(keys_file)
  keys <- .artms_checkRawFileColumnName(keys)
  
  # Check SILAC
  if(isSILAC){
    evidence_silac  <- artmsSILACtoLong(evidence_file,
                                        output = NULL,
                                        verbose = verbose)
    
    evidencekeys <- .artmsMergeSilacEvidenceKeys(evisilac = evidence_silac,
                                                 keysilac = keys)
  }else{
    evidencekeys <- artmsMergeEvidenceAndKeys(evidence_file, 
                                              keys,
                                              verbose = verbose)
  }
  
  colnames(evidencekeys) <- tolower(colnames(evidencekeys))
  
  if ( any(grepl("+", evidencekeys$reverse)) ) {
    evidencekeys <- subset(evidencekeys, reverse != "+")
  }
  
  # Add potential.contaminant column in case search was performed without it
  `%ni%` <- Negate(`%in%`)
  if ("potential.contaminant" %ni% colnames(evidencekeys)) {
    evidencekeys$potential.contaminant <- "no"
  }
  
  evidencekeys$potential.contaminant <- gsub("\\+", "yes", evidencekeys$potential.contaminant)
  evidencekeys$potential.contaminant <- gsub("^$", "no", evidencekeys$potential.contaminant)
  
  isFRACTION <- FALSE
  if("fraction" %in% colnames(evidencekeys)){
    if( length(unique(evidencekeys$fraction)) > 1 ){
      isFRACTION = TRUE
    }
  }
  
  evidencekeys.dt <- data.table::data.table(evidencekeys)
  
  evidence2 <- plyr::ddply(
    evidencekeys,
    c("bioreplicate", "condition", "potential.contaminant"),
    plyr::summarise,
    #!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    PSMs = sum(ms.ms.count),
    Ions = length(sequence[ms.ms.count > 0]),
    Peptides = length(unique(sequence[ms.ms.count >
                                        0])),
    Proteins = length(unique(proteins[ms.ms.count >
                                        0])),
    Intensity = as.numeric(format(
      sum(intensity, na.rm = TRUE),
      scientific = TRUE,
      digits = 2
    )),
    PeakWidth.mean = mean(retention.length, na.rm =
                            TRUE)
  )
  
  if (isFRACTION) {
    evidence2fx <- plyr::ddply(
      evidencekeys,
      c(
        "bioreplicate",
        "condition",
        "potential.contaminant",
        "fraction"
      ),
      plyr::summarise,
      PSMs = sum(ms.ms.count),
      Ions = length(sequence[ms.ms.count > 0]),
      Peptides = length(unique(sequence[ms.ms.count >
                                          0])),
      Proteins = length(unique(proteins[ms.ms.count >
                                          0])),
      Intensity = as.numeric(format(
        sum(intensity, na.rm = TRUE),
        scientific = TRUE,
        digits = 2
      )),
      PeakWidth.mean = mean(retention.length, na.rm =
                              TRUE)
    )
  }
  
  evidence3 <- plyr::ddply(evidence2,
                           c("condition", "potential.contaminant"),
                           plyr::summarise,
                           PSMs.mean = mean(PSMs),
                           PSMs.max = max(PSMs),
                           PSMs.min = min(PSMs),
                           PSMs.sem = sd(PSMs) / sqrt(length(PSMs)),
                           Ions.mean = mean(Ions),
                           Ions.max = max(Ions),
                           Ions.min = min(Ions),
                           Ions.sem = sd(Ions) / sqrt(length(Ions)),
                           Peptides.mean = mean(Peptides),
                           Peptides.max = max(Peptides),
                           Peptides.min = min(Peptides),
                           Peptides.sem = sd(Peptides) / sqrt(length(Peptides)),
                           Proteins.mean = mean(Proteins),
                           Proteins.max = max(Proteins),
                           Proteins.min = min(Proteins),
                           Proteins.sem = sd(Proteins) / sqrt(length(Proteins)),
                           Intensity.mean = mean(Intensity),
                           Intensity.max = max(Intensity),
                           Intensity.min = min(Intensity),
                           Intensity.sem = sd(Intensity) / sqrt(length(Intensity)),
                           PeakWidth.mean.mean = mean(PeakWidth.mean),
                           PeakWidth.mean.max = max(PeakWidth.mean),
                           PeakWidth.mean.min = min(PeakWidth.mean),
                           PeakWidth.mean.sem = sd(PeakWidth.mean) / sqrt(length(PeakWidth.mean))
                           )
    
  
  # - Oversampling
  ## Oversampling is calculated for peptides identified by MS/MS and detected
  ## in MS1 (i.e., peptides with intensity calculated)
  oversampling0 <- evidencekeys.dt[, .N, by = list(ms.ms.count, bioreplicate)]
  oversampling0$MSMS.counts <- findInterval(oversampling0$ms.ms.count, seq(1, 3, by = 1))
  oversampling0 <- oversampling0[, list(N = sum(N)), by = list(MSMS.counts, bioreplicate)]
  oversampling0$MSMS.counts <- paste("n=", oversampling0$MSMS.counts, sep = "")
  oversampling0$MSMS.counts <- gsub("n=3", "n=3+", oversampling0$MSMS.counts)
  oversampling0.total <- evidencekeys.dt[, .N, by = list(bioreplicate)]
  oversampling0 <- merge(oversampling0,
                         oversampling0.total,
                         by = "bioreplicate",
                         all = TRUE)
    
  oversampling0$FxOverSamp <- as.numeric(format(100 * (oversampling0$N.x / oversampling0$N.y), digits = 3))
  
  oversampling1 <- evidencekeys.dt[intensity > 0, .N, by = list(ms.ms.count, bioreplicate)]
  oversampling1$MSMS.counts <- findInterval(oversampling1$ms.ms.count, seq(1, 3, by = 1))
  oversampling1 <- oversampling1[, list(N = sum(N)), by = list(MSMS.counts, bioreplicate)]
  oversampling1$MSMS.counts <- paste("n=", oversampling1$MSMS.counts, sep = "")
  oversampling1$MSMS.counts <- gsub("n=3", "n=3+", oversampling1$MSMS.counts)
  oversampling1.total <- evidencekeys.dt[intensity > 0, .N, by = list(bioreplicate)]
  oversampling1 <- merge(oversampling1,
                         oversampling1.total,
                         by = "bioreplicate",
                         all = TRUE)
    
  oversampling1$FxOverSamp <- as.numeric(format(100 * (oversampling1$N.x / oversampling1$N.y), digits = 3))
  
  oversampling2 <- evidencekeys.dt[intensity > 0 &
                                     ms.ms.count > 0, .N, by = list(ms.ms.count, bioreplicate)]
    
  oversampling2$MSMS.counts <- findInterval(oversampling2$ms.ms.count, seq(1, 3, by = 1))
  oversampling2 <- oversampling2[, list(N = sum(N)), by = list(MSMS.counts, bioreplicate)]
  oversampling2$MSMS.counts <- paste("n=", oversampling2$MSMS.counts, sep = "")
  oversampling2$MSMS.counts <- gsub("n=3", "n=3+", oversampling2$MSMS.counts)
  oversampling2.total <- evidencekeys.dt[intensity > 0 & ms.ms.count > 0, .N, by = list(bioreplicate)]
  oversampling2 <- merge(oversampling2,
                         oversampling2.total,
                         by = "bioreplicate",
                         all = TRUE)
  oversampling2$FxOverSamp <- as.numeric(format(100 * (oversampling2$N.x / oversampling2$N.y), digits = 3))
  
  # Charge state
  chargeState <- evidencekeys.dt[, .N, by = list(charge, bioreplicate)]
  chargeState$charge <- paste("z=", chargeState$charge, sep = "")
  chargeState.total <- evidencekeys.dt[, .N, by = list(bioreplicate)]
  chargeState <- merge(chargeState,
                       chargeState.total,
                       by = "bioreplicate",
                       all = TRUE)
    
  chargeState$FxOverSamp <- as.numeric(format(100 * (chargeState$N.x / chargeState$N.y), digits = 1))
  
  chargeStateCond <- evidencekeys.dt[, .N, by = list(charge, condition)]
  chargeStateCond$charge <- paste("z=", chargeStateCond$charge, sep = "")
  chargeStateCond.total <- evidencekeys.dt[, .N, by = list(condition)]
  chargeStateCond <- merge(chargeStateCond,
                           chargeStateCond.total,
                           by = "condition",
                           all = TRUE)
  chargeStateCond$FxOverSamp <- as.numeric(format(100 * (chargeStateCond$N.x / chargeStateCond$N.y), digits = 1))
    
  
  # Type
  mstype <- evidencekeys.dt[, .N, by = list(type, bioreplicate, condition)]
  mstype.total <- evidencekeys.dt[, .N, by = list(bioreplicate)]
  mstype <- merge(mstype, mstype.total, by = "bioreplicate", all = TRUE)
  mstype$FxOverSamp <- as.numeric(format(100 * (mstype$N.x / mstype$N.y), digits = 2))
  
  # PLOTS-----
  if(verbose) message(">> GENERATING QC PLOTS ")
  
  # create output directory if it doesn't exist
  if(printPDF){
    if (!dir.exists(output_dir)) {
      if(verbose) message("-- Output folder created: ", output_dir)
      dir.create(output_dir, recursive = TRUE)
    }
  }
  
  nsamples <- length(unique(evidencekeys$bioreplicate))
  nconditions <- length(unique(evidencekeys$condition))
  
  ### PSM
  if (plotPSM) {
    if(verbose) message("--- Plot PSM", appendLF = FALSE)
    if(printPDF) pdf(
      paste0(output_dir, "/", output_name,'.qcplot.PSM.pdf'),
      width = 10, #nsamples * 3
      height = 6,
      onefile = TRUE
    )
    
    aa <- ggplot(evidence2,
                 aes(
                   x = bioreplicate,
                   y = PSMs,
                   fill = factor(condition)
                 )) +
      geom_bar(stat = "identity",
               position = position_dodge(width = 1),
               alpha = 0.7, na.rm = TRUE) +
      geom_text(aes(label = round(PSMs, digits = 0)),
                hjust = 0.5,
                vjust = -0.5,
                size = 2,
                angle = 0,
                position = position_dodge(width = 1)
                ) +
      facet_wrap( ~ potential.contaminant, ncol = 1) +
      xlab("Experiment") + ylab("Counts") +
      labs(title = "Number of Peptide-spectrum-matches (PSMs)",           
           caption = "bottom = Potential contaminants; top = non-contaminants",
           fill = "") +
      theme_linedraw() +
      theme(legend.text = element_text(size = 8)) +
      theme(axis.text.x = element_text(
        angle = 90,
        hjust = 0,
        size = 8
      )) +
      theme(axis.text.y = element_text(size = 10)) +
      theme(axis.title.y = element_text(size = 10)) +
      theme(plot.title = element_text(size = 12)) +
      
      scale_fill_brewer(palette = "Spectral")
    
    print(aa)
    
    ab <- ggplot(evidence3,
                 aes(
                   x = condition,
                   y = PSMs.mean,
                   fill = factor(potential.contaminant)
                 )) +
      geom_bar(stat = "identity",
               position = position_dodge(width = 1),
               alpha = 0.7, na.rm = TRUE) +
      geom_errorbar(
        aes(ymin = PSMs.mean - PSMs.sem, ymax = PSMs.mean + PSMs.sem),
        width = .2,
        position = position_dodge(.9)
      ) +
      geom_text(
        aes(label = round(PSMs.mean, digits = 0)),
        hjust = 0.5,
        vjust = -0.5,
        size = 2,
        angle = 0,
        position = position_dodge(width = 1)
      ) +
      xlab("Condition") + ylab("Counts") +
      theme_linedraw() +
      labs(title = "Mean number of PSMs per condition",
           caption = "yes = Potential contaminants; no = non-contaminants",
           fill = "Conditions") +
      theme(legend.text = element_text(size = 8)) +
      theme(axis.text.x = element_text(
        angle = 90,
        hjust = 1,
        size = 8
      )) +
      theme(axis.text.y = element_text(size = 10)) +
      theme(axis.title.y = element_text(size = 10)) +
      theme(plot.title = element_text(size = 12)) +
      scale_fill_brewer(palette = "Spectral")
    
    print(ab)
    
    if (isFRACTION) {
      ac <- ggplot(evidence2fx, aes(
        x = bioreplicate,
        y = PSMs,
        fill = factor(fraction)
      )) +
        geom_bar(stat = "identity", alpha = 0.7, na.rm = TRUE) +
        geom_text(
          aes(label = round(PSMs, digits = 0)),
          # hjust = 0.5,
          # vjust = 1.5,
          # size = 2,
          # position = position_stack()
          hjust = 1,
          size = 2.7,
          angle = 90,
          position = position_dodge(width = 1)
        ) +
        facet_wrap( ~ potential.contaminant,
                    ncol = 1,
                    scales = "free") +
        xlab("Experiment") + ylab("Counts") +
        labs(title = "Number of PSMs per Fraction",
             caption = "bottom = Potential contaminants; top = non-contaminants",
             fill = "") +
        theme(legend.text = element_text(size = 10)) +
        theme(axis.text.x = element_text(
          angle = 90,
          hjust = 0,
          size = 10
        )) +
        theme(axis.text.y = element_text(size = 10)) +
        theme(axis.title.y = element_text(size = 10)) +
        theme(plot.title = element_text(size = 12))
        
      print(ac)
    }
    
    if(printPDF) garbage <- dev.off()
    if(verbose) message(" done ")
  }
  
  
  ### IONS
  if (plotIONS) {
    if(verbose) message("--- Plot IONS", appendLF = FALSE)
    
    if(printPDF) pdf(
      paste0(output_dir, "/", output_name,'.qcplot.Ions.pdf'),
      width = 10, #nsamples * 3
      height = 6,
      onefile = TRUE
    )
    
    ba <- ggplot(evidence2,
                 aes(
                   x = bioreplicate,
                   y = Ions,
                   fill = factor(condition)
                 )) +
      geom_bar(stat = "identity",
               position = position_dodge(width = 1),
               alpha = 0.7, na.rm = TRUE) +
      geom_text(
        aes(label = round(Ions, digits = 0)),
        hjust = 0.5,
        vjust = -0.5,
        size = 2,
        position = position_dodge(width = 1)
      ) +
      facet_wrap( ~ potential.contaminant, ncol = 1) +
      xlab("Experiment") + ylab("Counts") +
      labs(title = "Number of unique Peptide Ions",
           caption = "bottom = Potential contaminants; top = non-contaminants",
           fill = "") +
      theme_linedraw() +
      theme(legend.text = element_text(size = 8)) +
      theme(axis.text.x = element_text(
        angle = 90,
        hjust = 1,
        size = 8
      )) +
      theme(axis.text.y = element_text(size = 10)) +
      theme(axis.title.y = element_text(size = 10)) +
      theme(plot.title = element_text(size = 12)) +
      scale_fill_brewer(palette = "Spectral")
    print(ba)
    
    bb <- ggplot(evidence3, aes(
      x = condition,
      y = Ions.mean,
      fill = factor(potential.contaminant)
    )) +
      geom_bar(stat = "identity",
               position = position_dodge(width = 1),
               alpha = 0.7, na.rm = TRUE) +
      geom_errorbar(
        aes(ymin = Ions.mean - Ions.sem, ymax = Ions.mean + Ions.sem),
        width = .2,
        position = position_dodge(.9)
      ) +
      geom_text(
        aes(label = round(Ions.mean, digits = 0)),
        hjust = 0.5,
        vjust = -0.5,
        size = 2,
        position = position_dodge(width = 1)
      ) +
      xlab("Condition") + ylab("Counts") +
      labs(title = "Mean number of unique Peptide Ions",
           subtitle = "Error bar = std error of the mean",
           fill = "Contaminants") +
      theme_linedraw() +
      theme(legend.text = element_text(size = 8)) +
      theme(axis.text.x = element_text(
        angle = 90,
        hjust = 1,
        size = 8
      )) +
      theme(axis.text.y = element_text(size = 10)) +
      theme(axis.title.y = element_text(size = 10)) +
      theme(plot.title = element_text(size = 12)) +
      scale_fill_brewer(palette = "Set1")
      
    print(bb)
    
    if (isFRACTION) {
      bc <- ggplot(evidence2fx, aes(
        x = bioreplicate,
        y = Ions,
        fill = factor(fraction)
      )) +
        geom_bar(stat = "identity", alpha = 0.7, na.rm = TRUE) +
        geom_text(
          aes(label = round(Ions, digits = 0)),
          hjust = 0.5,
          vjust = 1.5,
          size = 2,
          angle = 90,
          position = position_dodge(width = 1)
        ) +
        facet_wrap( ~ potential.contaminant,
                    ncol = 1,
                    scales = "free") +
        xlab("Experiment") + ylab("Counts") +
        labs(title = "Number of unique Peptide Ions in each Fraction",
             caption = "bottom = Potential contaminants; top = non-contaminants",
             fill = "Fractions") +
        theme_linedraw() +
        theme(legend.text = element_text(size = 10)) +
        theme(axis.text.x = element_text(
          angle = 90,
          hjust = 0,
          size = 8
        )) +
        theme(axis.text.y = element_text(size = 10)) +
        theme(axis.title.y = element_text(size = 10)) +
        theme(plot.title = element_text(size = 12))
        
      print(bc)
    }
    
    if(printPDF) garbage <- dev.off()
    
    if(verbose) message(" done ")
  }
  
  #### TYPE ######
  if (plotTYPE) {
    
    if(verbose) message("--- Plot TYPE", appendLF = FALSE)
    
    if(printPDF) pdf(
      paste0(output_dir, "/", output_name,'.qcplot.Type.pdf'),
      width = 10, #nsamples * 3
      height = 6,
      onefile = TRUE
    )
    
    ca <- ggplot(mstype, aes(
      x = bioreplicate,
      y = FxOverSamp,
      fill = factor(type))) +
      geom_bar(stat = "identity",
               position = position_stack(),
               alpha = 0.7, na.rm = TRUE) +
      ggrepel::geom_text_repel(
        aes(label = round(FxOverSamp, digits = 1)),
        vjust = 1,
        size = 2,
        position = position_stack()
      ) +
      xlab("Experiment") + ylab("Fraction") +
      labs(title = "Type of MaxQuant Feature Identification",
           fill = "Type") +
      theme_linedraw() +
      theme(legend.text = element_text(size = 8)) +
      theme(axis.text.x = element_text(
        angle = 90,
        hjust = 1,
        size = 8
      )) +
      theme(axis.text.y = element_text(size = 10)) +
      theme(axis.title.y = element_text(size = 10)) +
      theme(plot.title = element_text(size = 12)) +
      scale_fill_brewer(palette = "Set1")
      
    print(ca)
    
    if(printPDF) garbage <- dev.off()
    
    if(verbose) message(" done ")
  }
  
  ### PEPTIDES ###
  if (plotPEPTIDES) {
    if(verbose) message("--- Plot PEPTIDES", appendLF = FALSE)
    
    if(printPDF) pdf(
      paste0(output_dir, "/", output_name,'.qcplot.Peptides.pdf'),
      width = 10, #nsamples * 3
      height = 6,
      onefile = TRUE
    )
    
    da <- ggplot(evidence2, aes(
      x = bioreplicate,
      y = Peptides,
      fill = factor(condition))) +
      geom_bar(stat = "identity",
               position = position_dodge(width = 1),
               alpha = 0.7, na.rm = TRUE) +
      geom_text(
        aes(label = round(Peptides, digits = 0)),
        hjust = 0.5,
        vjust = -0.5,
        size = 2,
        angle = 0,
        position = position_dodge(width = 1)
      ) +
      facet_wrap( ~ potential.contaminant, ncol = 1) +
      xlab("Experiment") + ylab("Counts") +
      labs(title = "Number of unique Peptides",
           caption = "bottom = Potential contaminants; top = non-contaminants",
           fill = "Condition") +
      theme_linedraw() +
      theme(legend.text = element_text(size = 8)) +
      theme(axis.text.x = element_text(
        angle = 90,
        hjust = 1,
        size = 8
      )) +
      theme(axis.text.y = element_text(size = 10)) +
      theme(axis.title.y = element_text(size = 10)) +
      theme(plot.title = element_text(size = 12)) +
      scale_fill_brewer(palette = "Spectral")
      
    print(da)
    
    db <- ggplot(evidence3,
                 aes(
                   x = condition,
                   y = Peptides.mean,
                   fill = factor(potential.contaminant)
                 )) +
      geom_bar(stat = "identity",
               position = position_dodge(width = 1),
               alpha = 0.7, na.rm = TRUE) +
      geom_errorbar(
        aes(
          ymin = Peptides.mean - Peptides.sem,
          ymax = Peptides.mean + Peptides.sem
        ),
        width = .2,
        position = position_dodge(.9)
      ) +
      geom_text(
        aes(label = round(Peptides.mean, digits = 0)),
        hjust = 0.5,
        vjust = -0.5,
        size = 2,
        position = position_dodge(width = 1)
      ) +
      xlab("Condition") + ylab("Counts") +
      labs(title = "Number of unique Peptides",
           fill = "Contaminants") +
      theme_linedraw() +
      theme(legend.text = element_text(size = 8)) +
      theme(axis.text.x = element_text(
        angle = 90,
        hjust = 1,
        size = 8
      )) +
      theme(axis.text.y = element_text(size = 10)) +
      theme(axis.title.y = element_text(size = 10)) +
      theme(plot.title = element_text(size = 12)) +
      scale_fill_brewer(palette = "Set1")
      
    print(db)
    
    if (isFRACTION) {
      dc <- ggplot(evidence2fx,
                   aes(
                     x = bioreplicate,
                     y = Peptides,
                     fill = factor(fraction)
                   )) +
        geom_bar(stat = "identity", alpha = 0.7, na.rm = TRUE) +
        geom_text(
          aes(label = round(Peptides, digits = 0)),
          # hjust = 0.5,
          # vjust = 1.5,
          # size = 7,
          # position = position_stack()
          hjust = 1,
          size = 2.7,
          angle = 90,
          position = position_dodge(width = 1)
        ) +
        facet_wrap( ~ potential.contaminant,
                    ncol = 1,
                    scales = "free") +
        xlab("Experiment") + ylab("Counts") +
        labs(title = "Number of unique Peptides",
             fill = "Contaminants") +
        theme(legend.text = element_text(size = 10)) +
        theme(axis.text.x = element_text(
          angle = 90,
          hjust = 0,
          size = 10
        )) +
        theme(axis.text.y = element_text(size = 10)) +
        theme(axis.title.y = element_text(size = 10)) +
        theme(plot.title = element_text(size = 12)) +
        theme_linedraw()
        
      print(dc)
    }
    
    if(printPDF) garbage <- dev.off()
    if(verbose) message(" done ")
  }
  
  ### PEPTIDES OVERLAP
  if (plotPEPTOVERLAP) {
    if(verbose) message("--- Plot PEPTIDE OVERLAP", appendLF = FALSE)
    
    lst.pepExp <- list()
    for (i in unique(evidencekeys$bioreplicate)) {
      lst.pepExp[[i]] <-
        unique(subset(evidencekeys, ms.ms.count > 0 &
                        bioreplicate == i)[, c("sequence")])
    }
    
    lst.pepCond <- list()
    for (i in unique(evidencekeys$condition)) {
      lst.pepCond[[i]] <- unique(subset(evidencekeys,
                                        ms.ms.count > 0 &
                                          condition == i)[, c("sequence")])
    }
    
    uspep1 <- UpSetR::upset(
      fromList(lst.pepExp),
      nsets = length(unique(evidencekeys$bioreplicate)),
      nintersects = 50,
      order.by = "freq",
      text.scale = c(1, 1, 1, 1, 0.75, 0.75)
    )
    
    uspep2 <- UpSetR::upset(
      fromList(lst.pepCond),
      nsets = length(unique(evidencekeys$condition)),
      nintersects = 50,
      order.by = "freq",
      text.scale = c(1, 1, 1, 1, 0.75, 0.75)
    )
    
    if(printPDF) pdf(
      paste0(output_dir, "/", output_name,'.qcplot.PeptidesOverlap.pdf'),
      width = 10,
      height = 6,
      onefile = TRUE
    )
    
    print(uspep1)
    print(uspep2)
    
    if(printPDF) garbage <- dev.off()
    if(verbose) message(" done ")
  }
  
  
  ## PROTEINS
  if (plotPROTEINS) {
    if(verbose) message("--- Plot PROTEINS", appendLF = FALSE)
    
    if(printPDF) pdf(
      paste0(output_dir, "/", output_name,'.qcplot.Proteins.pdf'),
      width = 10, #nsamples * 3
      height = 6,
      onefile = TRUE
    )
    
    ea <- ggplot(evidence2, aes(
      x = bioreplicate,
      y = Proteins,
      fill = factor(condition))) +
      geom_bar(stat = "identity",
               position = position_dodge(width = 1),
               alpha = 0.7, na.rm = TRUE) +
      geom_text(
        aes(label = round(Proteins, digits = 0)),
        hjust = 0.5,
        vjust = -0.5,
        size = 2,
        position = position_dodge(width = 1),
        angle = 0
      ) +
      facet_wrap( ~ potential.contaminant, ncol = 1) +
      xlab("Experiment") + ylab("Counts") +
      labs(title = "Number of unique Protein Groups",
           fill = "") +
      theme_linedraw() +
      theme(legend.text = element_text(size = 8)) +
      theme(axis.text.x = element_text(
        angle = 90,
        hjust = 1,
        size = 8
      )) +
      theme(axis.text.y = element_text(size = 10)) +
      theme(axis.title.y = element_text(size = 10)) +
      theme(plot.title = element_text(size = 12)) +
      scale_fill_brewer(palette = "Spectral")
      
    print(ea)
    
    eb <- ggplot(evidence3,
                 aes(x = condition,
                     y = Proteins.mean,
                     fill = factor(potential.contaminant))) +
      geom_bar(stat = "identity",
               position = position_dodge(width = 1),
               alpha = 0.7, na.rm = TRUE) +
      geom_errorbar(
        aes(
          ymin = Proteins.mean - Proteins.sem,
          ymax = Proteins.mean + Proteins.sem
        ),
        width = .2,
        position = position_dodge(.9)
      ) +
      geom_text(
        aes(label = round(Proteins.mean, digits = 0)),
        hjust = 0.5,
        vjust = -0.5,
        size = 2,
        position = position_dodge(width = 1)
      ) +
      xlab("Condition") + ylab("Counts") +
      labs(title = "Mean number of unique Proteins",
           subtitle = "error bar = std error of the mean",
           fill = "Contaminants") +
      theme_linedraw() +
      theme(legend.text = element_text(size = 8)) +
      theme(axis.text.x = element_text(
        angle = 90,
        hjust = 1,
        size = 8
      )) +
      theme(axis.text.y = element_text(size = 10)) +
      theme(axis.title.y = element_text(size = 10)) +
      theme(plot.title = element_text(size = 12)) +
      scale_fill_brewer(palette = "Set1")
      
    print(eb)
    
    if(printPDF) garbage <- dev.off()
    
    if(verbose) message(" done ")
  }

  ### PEPTIDES OVERLAP
  if (plotPROTOVERLAP) {
    if(verbose) message("--- Plot PROTEIN OVERLAP", appendLF = FALSE)
    
    lst.protExp <- list()
    for (i in unique(evidencekeys$bioreplicate)) {
      lst.protExp[[i]] <-
        unique(subset(evidencekeys, ms.ms.count > 0 &
                        bioreplicate == i)[, c("proteins")])
    }
    
    lst.protCond <- list()
    for (i in unique(evidencekeys$condition)) {
      lst.protCond[[i]] <-
        unique(subset(evidencekeys, ms.ms.count > 0 &
                        condition == i)[, c("proteins")])
    }
    
    upprot1 <- UpSetR::upset(
      fromList(lst.protExp),
      nsets = length(unique(evidencekeys$bioreplicate)),
      nintersects = 50,
      order.by = "freq",
      text.scale = c(1, 1, 1, 1, 0.75, 0.75)
    )
    
    upprot2 <- UpSetR::upset(
      fromList(lst.protCond),
      nsets = length(unique(evidencekeys$condition)),
      nintersects = 50,
      order.by = "freq",
      text.scale = c(1, 1, 1, 1, 0.75, 0.75)
    )
    
    if(printPDF) pdf(
      paste0(output_dir, "/", output_name,'.qcplot.ProteinOverlap.pdf'),
      width = 10,
      height = 6,
      onefile = TRUE
    )
    
    print(upprot1)
    print(upprot2)
    
    if(printPDF) garbage <- dev.off()
    if(verbose) message(" done ")
  }
  
  
  # Peptide ion oversampling
  if (plotPIO) {
    if(verbose) message("--- Plot Plot Ion Oversampling", appendLF = FALSE)
    
    if(printPDF) pdf(
      paste0(output_dir, "/", output_name,'.qcplot.PepIonOversampling.pdf'),
      width = 10, #nsamples * 3
      height = 6,
      onefile = TRUE
    )
    fa <- ggplot(
      oversampling0[with(oversampling0, order(MSMS.counts)), ],
      aes(
        x = bioreplicate,
        y = FxOverSamp,
        fill = MSMS.counts,
        label = FxOverSamp
      )
    ) +
      geom_bar(stat = "identity", alpha = 0.7, na.rm = TRUE) +
      geom_text(
        aes(label = round(FxOverSamp, digits = 1)),
        vjust = 1 ,
        size = 2,
        position = position_stack()
      ) +
      xlab("Experiment") + ylab("Fraction (percentage)") +
      labs(title = "Peptide ion oversampling",
           subtitle = "Based on all the peptides reported by MaxQuant") +
      theme_linedraw() +
      theme(legend.text = element_text(size = 8)) +
      theme(axis.text.x = element_text(
        angle = 90,
        hjust = 1,
        size = 8
      )) +
      theme(axis.text.y = element_text(size = 10)) +
      theme(axis.title.y = element_text(size = 10)) +
      theme(plot.title = element_text(size = 12)) +
      scale_fill_brewer(palette = "Set1")
      
    print(fa)
    
    fb <- ggplot(oversampling1[with(oversampling1, order(MSMS.counts)), ],
                 aes(x = bioreplicate,
                     y = FxOverSamp,
                     fill = MSMS.counts,
                     label = FxOverSamp)) +
      geom_bar(stat = "identity", alpha = 0.7, na.rm = TRUE) +
      geom_text(
        aes(label = round(FxOverSamp, digits = 1)),
        vjust = 1 ,
        size = 2,
        position = position_stack()
      ) +
      xlab("Experiment") + ylab("Fraction (percentage)") +
      labs(title = "Peptide ion oversampling",
           subtitle = "Only peptides detected (MS1 AUC calculated)") +
      theme_linedraw() +
      theme(legend.text = element_text(size = 8)) +
      theme(axis.text.x = element_text(
        angle = 90,
        hjust = 1,
        size = 8
      )) +
      theme(axis.text.y = element_text(size = 10)) +
      theme(axis.title.y = element_text(size = 10)) +
      theme(plot.title = element_text(size = 12)) +
      scale_fill_brewer(palette = "Set1")
      
    print(fb)
    
    fc <- ggplot(oversampling2[with(oversampling2, order(MSMS.counts)), ],
                 aes(
                   x = bioreplicate,
                   y = FxOverSamp,
                   fill = MSMS.counts,
                   label = FxOverSamp)) +
      geom_bar(stat = "identity", alpha = 0.7, na.rm = TRUE) +
      geom_text(
        aes(label = round(FxOverSamp, digits = 1)),
        vjust = 1 ,
        size = 2,
        position = position_stack()
      ) +
      xlab("Experiment") + ylab("Fraction (percentage)") +
      labs(title = "Peptide ion oversampling",
           subtitle = "Only peptides detected (MS1 AUC calculated) and identified (confidence MS/MS)") +
      theme_linedraw() +
      theme(legend.text = element_text(size = 8)) +
      theme(axis.text.x = element_text(
        angle = 90,
        hjust = 1,
        size = 8
      )) +
      theme(axis.text.y = element_text(size = 10)) +
      theme(axis.title.y = element_text(size = 10)) +
      theme(plot.title = element_text(size = 12)) +
      scale_fill_brewer(palette = "Set1")
      
    print(fc)
    
    if(printPDF) garbage <- dev.off()
    
    if(verbose) message(" done ")
  }
  
  
  # Charge state distribution
  if (plotCS) {
    if(verbose) message("--- Plot Charge State", appendLF = FALSE)
    
    if(printPDF) pdf(
      paste0(output_dir, "/", output_name,'.qcplot.ChargeState.pdf'),
      width = 10, #nsamples * 3
      height = 6,
      onefile = TRUE)
    
    ga <- ggplot(chargeState[with(chargeState, order(charge)), ],
                 aes(x = bioreplicate, y = FxOverSamp, fill = charge)) +
      geom_bar(stat = "identity", alpha = 0.7, na.rm = TRUE) +
      theme_linedraw() +
      ggrepel::geom_text_repel(
        aes(label = round(FxOverSamp, digits = 1)),
        vjust = -0.5,
        size = 2,
        position = position_stack()
      ) +
      # geom_text(
      #   aes(label = round(FxOverSamp, digits = 1)),
      #   vjust = 1 ,
      #   size = 2,
      #   position = position_stack()
      # ) +
      xlab("Experiment") + ylab("Fraction (percentage)") +
      ggtitle("Precursor charge state distribution") +
      theme_linedraw() +
      theme(legend.text = element_text(size = 8)) +
      theme(axis.text.x = element_text(
        angle = 90,
        hjust = 1,
        size = 8
      )) +
      theme(axis.text.y = element_text(size = 10)) +
      theme(axis.title.y = element_text(size = 10)) +
      theme(plot.title = element_text(size = 12)) +
      scale_fill_brewer(palette = "Spectral")
    
    print(ga)
    
    if(printPDF) garbage <- dev.off()
    
    if(verbose) message(" done ")
  }
  
  # Mass error
  if (plotME & any(grepl("uncalibrated.mass.error..ppm.", colnames(evidencekeys))) ) {
    if(verbose) message("--- Plot Mass Error", appendLF = FALSE)
    if(printPDF) pdf(
      paste0(output_dir, "/", output_name,'.qcplot.MassError.pdf'),
      width = 10, #nsamples * 3
      height = 6,
      onefile = TRUE
    )
    fa <- ggplot(evidencekeys,
                 aes(x = bioreplicate, y = uncalibrated.mass.error..ppm.)) +
      geom_boxplot(varwidth = TRUE, 
                   aes(fill = factor(condition)), 
                   alpha = 0.7,
                   na.rm = TRUE) +
      geom_text(
        data = aggregate(
          uncalibrated.mass.error..ppm. ~ bioreplicate,
          evidencekeys,
          median
        ),
        aes(
          label = round(uncalibrated.mass.error..ppm., digits = 1),
          y = max(evidencekeys$uncalibrated.mass.error..ppm., na.rm = TRUE) + 2
        ),
        size = 2
      ) +
      xlab("Experiment") + ylab("mass error") +
      labs(title = "Precursor mass error (in ppm) distribution",
           caption = "Global median mass error on the top",
           fill = "") +
      theme_linedraw() +
      theme(legend.text = element_text(size = 8)) +
      theme(axis.text.x = element_text(angle = 90,
                                       hjust = 1,
                                       size = 8)) +
      theme(axis.text.y = element_text(size = 10)) +
      theme(axis.title.y = element_text(size = 10)) +
      theme(plot.title = element_text(size = 12)) +
      scale_fill_brewer(palette = "Spectral")
    print(fa)
    if(printPDF) garbage <- dev.off()
    if(verbose) message(" done ")
  }
  
  # mass-over-charge distribution
  if (plotMOCD) {
    if(verbose) message("--- Plot Mass-over-Charge distribution", appendLF = FALSE)
    
    if(printPDF) pdf(
      paste0(output_dir, "/", output_name,'.qcplot.MZ.pdf'),
      width = 10, #nsamples * 3
      height = 6,
      onefile = TRUE
    )
    ga <- ggplot(evidencekeys, aes(x = bioreplicate, y = m.z)) +
      geom_boxplot(varwidth = TRUE, aes(fill = factor(condition)), alpha = 0.7, na.rm = TRUE) +
      geom_text(
        data = aggregate(m.z ~ bioreplicate, evidencekeys, median),
        aes(label = round(m.z, digits = 1),
            y = max(evidencekeys$m.z, na.rm = TRUE) + 30),
        size = 2,
        angle = 0) +
      xlab("Experiment") + ylab("m/z") +
      labs(title = "Precursor mass-over-charge distribution",
           caption = "Global median m/z on the top",
           fill = "") +
      theme_linedraw() +
      theme(legend.text = element_text(size = 8)) +
      theme(axis.text.x = element_text(
        angle = 90,
        hjust = 1,
        size = 8
      )) +
      theme(axis.text.y = element_text(size = 10)) +
      theme(axis.title.y = element_text(size = 10)) +
      theme(plot.title = element_text(size = 12)) +
      scale_fill_brewer(palette = "Spectral")
    print(ga)
    
    if(printPDF) garbage <- dev.off()
    if(verbose) message(" done ")
  }
  
  
  #m Peptide Intensity CV
  if (plotPEPICV) {
    if(verbose) message("--- Plot Peptide Intensity CV", appendLF = FALSE)
    if(printPDF) pdf(
      paste0(output_dir, "/", output_name,'.qcplot.PeptideIntensityCV.pdf'),
      width = 10, #nsamples * 3
      height = 6,
      onefile = TRUE
    )
    peptCV <- data.table::data.table(subset(evidencekeys,!is.na(intensity)))
    peptCV$intensity <- as.numeric(peptCV$intensity)
  
    peptCV <- peptCV[, list(intensity = sum(intensity, na.rm = TRUE)),
                     by = list(condition, bioreplicate, modified.sequence)]
      
    peptCV <- peptCV[, list(pCV = 100 * (sd(intensity) / mean(intensity)),
                            sumInt = sum(intensity),
                            pDX = length(unique(bioreplicate))),
                     by = list(condition, modified.sequence)]
    peptCV <- peptCV[, bin.all := .artms_qcut(sumInt, 4)]
    peptCV <- peptCV[, bin.condition := .artms_qcut(sumInt, 4),
                     by = condition]
      
    
    ha <- ggplot(subset(peptCV,!is.na(pCV)), aes(condition, pCV)) +
      geom_boxplot(varwidth = TRUE, aes(fill = factor(condition)), alpha = 0.7, na.rm = TRUE) +
      geom_text(
        data = aggregate(pCV ~ condition, subset(peptCV,!is.na(pCV)), median),
        aes(label = round(pCV, digits = 1),
            y = max(peptCV$pCV, na.rm = TRUE) + 1),
        size = 2.5) +
      geom_text(
        data = aggregate(pCV ~ condition, subset(peptCV,!is.na(pCV)), length),
        aes(label = round(pCV, digits = 1), y = 0),
        size = 2) +
      xlab("Condition") + ylab("Coefficient of variance (%)") +
      labs(title = "Distribution of peptide feature intensity CV",
           fill = "") +
      theme_linedraw() +
      theme(legend.text = element_text(size = 8)) +
      theme(axis.text.x = element_text(angle = 90, 
                                       size = 8)) +
      theme(axis.text.y = element_text(size = 10)) +
      theme(axis.title.y = element_text(size = 10)) +
      theme(plot.title = element_text(size = 12)) +
      scale_fill_brewer(palette = "Spectral")
      
    print(ha)
    
    hb <- ggplot(subset(peptCV,!is.na(pCV)), aes(interaction(condition, bin.condition), pCV)) +
      geom_boxplot(varwidth = TRUE, aes(fill = factor(condition)), alpha = 0.7, na.rm = TRUE) +
      geom_text(
        data = aggregate(
          pCV ~ condition + bin.condition,
          subset(peptCV,!is.na(pCV)),
          median
        ),
        aes(
          label = round(pCV, digits = 1),
          y = max(peptCV$pCV, na.rm = TRUE) + 1
        ),
        size = 2,
        angle = 0
      ) +
      geom_text(
        data = aggregate(
          pCV ~ condition + bin.condition,
          subset(peptCV,!is.na(pCV)),
          length
        ),
        aes(label = round(pCV, digits = 1), y = 0),
        size = 2.5,
        angle = 0
      ) +
      xlab("Condition") + ylab("Coefficient of variance (%)") +
      labs(title = "Distribution of peptide feature intensity CV",
           caption = "For each condition, peptides were ranked by summed intensity 
           and the CV for each peptide was calculated. 
           Each condition shows 4 distribution (box) for low (1) to high (4) intensity peptides. 
           Overall median CV within each bin/condition is shown on the top and 
           number of features used to calculate CV is given on the bottom",
           fill = "") +
      theme_linedraw() +
      theme(legend.text = element_text(size = 8)) +
      theme(axis.text.x = element_text(angle = 90, 
                                       size = 8)) +
      theme(axis.text.y = element_text(size = 10)) +
      theme(axis.title.y = element_text(size = 10)) +
      theme(plot.title = element_text(size = 12, hjust = 0.5)) +
      
      scale_fill_brewer(palette = "Spectral")
      
    print(hb)
    
    if(printPDF) garbage <- dev.off()
    if(verbose) message(" done ")
  }
  
  # peptide detection (using modified.sequence)
  if (plotPEPDETECT) {
    if(verbose) message("--- Plot Peptide Detection (using modified.sequence)", appendLF = FALSE)
    if(printPDF) pdf(
      paste0(output_dir, "/", output_name,'.qcplot.PeptideDetection.pdf'),
      width = 10, #nsamples * 3
      height = 6,
      onefile = TRUE
    )
    peptDX <- peptCV[, .N, by = list(condition, pDX)]
    peptDXT <- peptDX[, list(N.total = sum(N)), by = list(condition)]
    peptDX <- merge(peptDX, peptDXT, by = "condition")
    peptDX$fxDx <- 100 * (peptDX$N / peptDX$N.total)
    
    ia <- ggplot(peptDX, aes(x = condition,
                             y = fxDx,
                             fill = factor(pDX))) +
      geom_bar(stat = "identity", alpha = 0.7, na.rm = TRUE) +
      ggrepel::geom_text_repel(
        aes(label = round(fxDx, digits = 1)),
        vjust = 1,
        size = 2,
        position = position_stack()
      ) +
      xlab("Condition") + ylab("Counts") +
      # ggtitle("Frequency of peptides detection") +
      labs(title = "Frequency of peptides detection",
           fill = "n") +
      theme_linedraw() +
      theme(legend.text = element_text(size = 8)) +
      theme(axis.text.x = element_text(angle = 90, 
                                       size = 8)) +
      theme(axis.text.y = element_text(size = 10)) +
      theme(plot.title = element_text(size = 12)) +
      scale_fill_brewer(palette = "Set1")
      
    print(ia)
    
    if(printPDF) garbage <- dev.off()
    if(verbose) message(" done ")
  }
  
  
  #m Protein Intensity CV
  if (plotPROTICV) {
    if(verbose) message("--- Plot Protein Intensity CV", appendLF = FALSE)
    if(printPDF) pdf(
      paste0(output_dir, "/", output_name,'.qcplot.ProteinIntensityCV.pdf'),
      width = 10, #nsamples * 3
      height = 6,
      onefile = TRUE
    )
    protCV <- data.table::data.table(subset(evidencekeys,!is.na(intensity)))
    protCV$intensity <- as.numeric(protCV$intensity)
    protCV <- protCV[, list(intensity = sum(intensity, na.rm = TRUE)), by = list(condition, bioreplicate, proteins)]
    protCV <- protCV[, list(pCV = 100 * (sd(intensity) / mean(intensity)),
                            sumInt = sum(intensity),
                            pDX = length(unique(bioreplicate))), 
                     by = list(condition, proteins)]
      
      
    protCV <- protCV[, bin.all := .artms_qcut(sumInt, 4)]
    protCV <- protCV[, bin.condition := .artms_qcut(sumInt, 4), by = condition]
    
    ja <- ggplot(subset(protCV,!is.na(pCV)), aes(condition, pCV)) +
      geom_boxplot(varwidth = TRUE, aes(fill = factor(condition)), alpha = 0.7, na.rm = TRUE) +
      geom_text(
        data = aggregate(pCV ~ condition, subset(protCV,!is.na(pCV)), median),
        aes(label = round(pCV, digits = 1),
            y = max(protCV$pCV, na.rm = TRUE) + 1),
        size = 2) +
      xlab("Condition") + ylab("Coefficient of variance (%)") +
      labs(title = "Distribution of Protein intensity CV",
           fill = "") +
      theme_linedraw() +
      theme(legend.text = element_text(size = 8)) +
      theme(axis.text.x = element_text(angle = 90, 
                                       size = 8)) +
      theme(axis.text.y = element_text(size = 10)) +
      theme(axis.title.y = element_text(size = 10)) +
      theme(plot.title = element_text(size = 12)) +
      scale_fill_brewer(palette = "Spectral")
      
    print(ja)
    
    jb <- ggplot(subset(protCV,!is.na(pCV)), aes(interaction(condition, bin.condition), pCV)) +
      geom_boxplot(varwidth = TRUE, aes(fill = factor(condition)), alpha = 0.7, na.rm = TRUE) +
      geom_text(
        data = aggregate(
          pCV ~ condition + bin.condition,
          subset(protCV,!is.na(pCV)),
          median
        ),
        aes(
          label = round(pCV, digits = 1),
          y = max(protCV$pCV, na.rm = TRUE) + 1
        ),
        size = 2,
        angle = 0
      ) +
      geom_text(
        data = aggregate(
          pCV ~ condition + bin.condition,
          subset(protCV,!is.na(pCV)),
          length
        ),
        aes(label = round(pCV, digits = 1), y = 0),
        size = 2,
        angle = 0
      ) +
      xlab("Condition") + ylab("Coefficient of variance (%)") +
      labs(title = "Distribution of Protein intensity CV",
           caption = "Proteins ranked by summed intensity and CV calculated for each protein.
           Each condition shows 4 distribution (box) for low (1) to high (4) intensity proteins.
           Top: overall median CV within each condition
           Bottom: number of protein groups used to calculate CV",
           fill = "") +
      theme_linedraw() +
      theme(legend.text = element_text(size = 8)) +
      theme(axis.text.x = element_text(angle = 90, 
                                       size = 8)) +
      theme(axis.text.y = element_text(size = 10)) +
      theme(axis.title.y = element_text(size = 10)) +
      theme(plot.title = element_text(size = 12)) +
      scale_fill_brewer(palette = "Spectral")
      
    print(jb)
    
    if(printPDF) garbage <- dev.off()
    if(verbose) message(" done ")
  }
  
  # Protein detection
  if (plotPROTDETECT) {
    if(verbose) message("--- Plot Protein Detection", appendLF = FALSE)
    if(printPDF) pdf(
      paste0(output_dir, "/", output_name,'.qcplot.ProteinDetection.pdf'),
      width = 10, #nsamples * 3
      height = 6,
      onefile = TRUE
    )
    protDX <- protCV[, .N, by = list(condition, pDX)]
    protDXT <- protDX[, list(N.total = sum(N)), by = list(condition)]
    protDX <- merge(protDX, protDXT, by = "condition")
    protDX$fxDx <- 100 * (protDX$N / protDX$N.total)
    
    ka <- ggplot(protDX, aes(
      x = condition,
      y = fxDx,
      fill = factor(pDX)
    )) +
      geom_bar(stat = "identity", alpha = 0.7, na.rm = TRUE) +
      ggrepel::geom_text_repel(
        aes(label = round(fxDx, digits = 1)),
        vjust = 1,
        size = 2,
        position = position_stack()
      ) +
      xlab("Condition") + ylab("Counts") +
      labs(title = "Protein Detection Frequency across bioreplicates by condition",
           fill = "n") +
      theme_linedraw() +
      theme(legend.text = element_text(size = 8)) +
      theme(axis.text.x = element_text(angle = 90, 
                                       size = 8)) +
      theme(axis.text.y = element_text(size = 10)) +
      scale_fill_brewer(palette = "Set1")
      
    print(ka)
    
    kb <- ggplot(subset(evidencekeys,!is.na(intensity)), aes(bioreplicate, log2(intensity))) +
      geom_boxplot(varwidth = TRUE,
                   aes(fill = potential.contaminant),
                   alpha = 0.7, na.rm = TRUE) +
      #ggrepel::geom_text_repel(data = aggregate(intensity ~ bioreplicate + potential.contaminant, subset(evidencekeys, !is.na(intensity)), median), aes(label = round(log2(intensity), digits=1), y = log2(max(evidencekeys$intensity, na.rm= TRUE))+0.5 ), size = 15) +
      xlab("Experiment") + 
      ylab("Log2 Intensity") +
      labs(title = "Peptide feature intensity distribution",
           fill = "Contaminants") +
      theme_linedraw() +
      theme(legend.text = element_text(size = 8)) +
      theme(axis.text.x = element_text(angle = 90, 
                                       size = 8)) +
      theme(axis.text.y = element_text(size = 10)) +
      theme(axis.title.y = element_text(size = 10)) +
      theme(plot.title = element_text(size = 12)) +
      scale_fill_brewer(palette = "Set1")
      
    print(kb)
    
    if (isFRACTION) {
      kc <-
        ggplot(subset(evidencekeys,!is.na(intensity)), aes(bioreplicate, log2(intensity))) +
        geom_boxplot(varwidth = TRUE,
                     aes(fill = potential.contaminant),
                     alpha = 0.7, na.rm = TRUE) +
        #ggrepel::geom_text_repel(data = aggregate(intensity ~ bioreplicate + potential.contaminant, subset(evidencekeys, !is.na(intensity)), median), aes(label = round(log2(intensity), digits=1), y = log2(max(evidencekeys$intensity, na.rm= TRUE))+0.5 ), size = 15) +
        xlab("Experiment") + ylab("Log2 Intensity") +
        facet_wrap( ~ fraction, ncol = 5) +
        ggtitle("Peptide feature intensity distribution by Fraction") +
        theme(legend.text = element_text(size = 8)) +
        theme(axis.text.x = element_text(angle = 90, 
                                         size = 10)) +
        theme(axis.text.y = element_text(size = 10)) +
        theme(axis.title.y = element_text(size = 10)) +
        theme(plot.title = element_text(size = 12)) +
        scale_fill_brewer(palette = "Set1")
      print(kc)
      
    }
    
    kd <- ggplot(evidencekeys, aes(x = log2(intensity))) +
      geom_density(alpha = .5, aes(fill = type), na.rm = TRUE) +
      labs(title = "Peptide feature intensity distribution by ID Type") +
      facet_wrap( ~ bioreplicate, ncol = 5) +
      theme_linedraw() +
      theme(legend.text = element_text(size = 8)) +
      theme(axis.text.x = element_text(size = 8)) +
      theme(axis.text.y = element_text(size = 10)) +
      theme(axis.title.y = element_text(size = 10)) +
      theme(plot.title = element_text(size = 12)) +
      scale_fill_brewer(palette = "Set1")
    
    print(kd)
    
    if(printPDF) garbage <- dev.off()
    if(verbose) message(" done ")
  }
  
  
  if (plotIDoverlap) {
    if(verbose) message("--- Plot ID overlap", appendLF = FALSE)
    if(printPDF) pdf(
      paste0(output_dir, "/", output_name,'.qcplot.ID-Overlap.pdf'),
      width = 20, #nsamples * 3
      height = 20,
      onefile = TRUE
    )
    ovlSeq <- subset(
      evidencekeys,!is.na(intensity) & !is.na(ms.ms.count),
      select = c("bioreplicate", "sequence")
    )
    ovlSeq <- unique(ovlSeq)
    ovlSeq <- .artms_sampleOverlap(ovlSeq,
                                   sampleID = 'bioreplicate',
                                   referenceID = 'sequence')
    ovlSeqM <- ovlSeq$M
    ovlSeqM[ovlSeqM == 1] <- NA
    # coordinates to move the keys
    lmat = rbind(c(0,3),c(2,1),c(0,4))
    lwid = c(1.5,4)
    lhei = c(1.5,4,1)
    gplots::heatmap.2(
      ovlSeqM * 100,
      col = hmcol,
      Rowv = FALSE,
      Colv = "Rowv",
      main = "Pairwise peptide identification overlap",
      cexRow = 1,
      cexCol = 1,
      margins = c(20, 20),
      cellnote = round(ovlSeqM * 100, 1),
      notecex = 1,
      notecol = "black",
      trace = "none",
      key = FALSE,
      keysize = 1,
      scale = "none",
      symm = TRUE,
      colsep = c(seq_len(length(
        colnames(ovlSeqM)
      ))),
      rowsep = c(seq_len(length(
        rownames(ovlSeqM)
      ))),
      sepcolor = "white",
      sepwidth = c(0.01, 0.01),
      dendrogram = "none",
      density.info = "none"
    )
    
    if (isFRACTION) {
      ovlSeqFx <- subset(
        evidencekeys,!is.na(intensity) & !is.na(ms.ms.count),
        select = c("bioreplicate", "fraction", "sequence")
      )
      ovlSeqFx <- unique(ovlSeqFx)
      ovlSeqFx$bioreplicate <- paste(ovlSeqFx$fraction,
                                     ovlSeqFx$bioreplicate,
                                     sep = ".")
      ovlSeqFx <- .artms_sampleOverlap(ovlSeqFx,
                                       sampleID = 'bioreplicate',
                                       referenceID = 'sequence')
      ovlSeqFxM <- ovlSeqFx$M
      ovlSeqFxM[ovlSeqFxM == 1] <- NA
      gplots::heatmap.2(
        ovlSeqFxM * 100,
        col = hmcol,
        Rowv = FALSE,
        Colv = "Rowv",
        cexRow = 0.5,
        cexCol = 0.5,
        margins = c(40, 40),
        #cellnote=round(ovlSeqFxM*100,1),
        #notecex=3,
        main = "Pairwise peptide identification overlap by Fraction",
        #notecol="black",
        trace = "none",
        key = TRUE,
        keysize = 1,
        scale = "none",
        symm = TRUE,
        colsep = c(seq_len(length(
          colnames(ovlSeqFxM)
        ))),
        rowsep = c(seq_len(length(
          rownames(ovlSeqFxM)
        ))),
        sepcolor = "white",
        sepwidth = c(0.0001, 0.0001),
        dendrogram = "none"
      )
    }
    
    
    ovlProt <- subset(
      evidencekeys,!is.na(intensity) & !is.na(ms.ms.count),
      select = c("bioreplicate", "proteins")
    )
    ovlProt <- unique(ovlProt)
    ovlProt <- .artms_sampleOverlap(ovlProt,
                                    sampleID = 'bioreplicate',
                                    referenceID = 'proteins')
    ovlProtM <- ovlProt$M
    ovlProtM[ovlProtM == 1] <- NA
    gplots::heatmap.2(
      ovlProtM * 100,
      col = hmcol,
      Rowv = FALSE,
      Colv = "Rowv",
      cexRow = 1,
      cexCol = 1,
      margins = c(40, 40),
      cellnote = round(ovlProtM * 100, 1),
      notecex = 1,
      main = "Pairwise protein identification overlap",
      notecol = "black",
      trace = "none",
      key = FALSE,
      keysize = 1,
      scale = "none",
      symm = TRUE,
      colsep = c(seq_len(length(
        colnames(ovlProtM)
      ))),
      rowsep = c(seq_len(length(
        rownames(ovlProtM)
      ))),
      sepcolor = "white",
      sepwidth = c(0.01, 0.01),
      dendrogram = "none"
    )
    if(printPDF) garbage <- dev.off()
    if(verbose) message(" done ")
  }
  
  
  if (plotPCA) {
    if(verbose) message("--- Plot PCA and Inter-Correlation (WARNING: it might take a long time. Please, be patient)")
    if(printPDF) pdf(
      paste0(output_dir, "/", output_name,'.qcplot.PCA.pdf'),
      width = 12, #nsamples * 3
      height = 10,
      onefile = TRUE
    )
    
    peptintmtx <- subset(evidencekeys,
                         !is.na(intensity),
                         select = c("bioreplicate", "sequence", "intensity"))

    peptintmtx$intensity <- as.integer64(peptintmtx$intensity)
  
    ##LEGACY
    # peptintmtx <- data.table::dcast(peptintmtx, 
    #                                 sequence ~ bioreplicate, 
    #                                 sum, 
    #                                 value.var = "intensity")
    
    peptintmtx <- peptintmtx %>% 
      tidyr::pivot_wider(id_cols = sequence, 
                         names_from = bioreplicate, 
                         values_from = intensity, 
                         values_fn = list(intensity = sum))
    
    
    peptintmtx <- as.matrix(peptintmtx[, -1])
    peptintmtx <- log2(peptintmtx)
    peptintmtx[!is.finite(peptintmtx)] <- NA
    
    if(nsamples < 20){
      pairs(peptintmtx,
            upper.panel = .artms_panelCor,
            diag.panel = .artms_panelHist,
            lower.panel = panel.smooth,
            pch = ".")
    }else{
      message("\t(-) Skip peptide-based correlation matrix (too many samples)")
    }
    
    mpept <- peptintmtx
    mpept[is.na(mpept)] <- 0
    pept.pca <- prcomp(t(mpept))
    pept.pcax <- as.data.frame(pept.pca$x)
    pept.pcax <- merge(unique(evidencekeys[, c("bioreplicate", "condition")]),
            pept.pcax,
            by.x = "bioreplicate",
            by.y = "row.names")
    
    ma <- ggplot(pept.pcax, aes(PC1, PC2, fill = condition)) + #, color=condition, shape=condition
      geom_point(alpha = .8, size = 5, na.rm = TRUE) +
      geom_label_repel(
        aes(label = bioreplicate),
        # label.size = 1,
        box.padding   = 0.8,
        point.padding = 0.2
        # segment.color = 'grey50'
      ) +
      labs(title = "Principal Component Analysis (Peptide Intensity)",
           fill = "") +
      theme_linedraw()

    print(ma)
    
    protintmtx <- subset(
      evidencekeys,
      !is.na(intensity),
      select = c("bioreplicate", "proteins", "intensity")
    )
    
    protintmtx$intensity <- as.integer64(protintmtx$intensity)
    ##LEGACY
    # protintmtx <- data.table::dcast(protintmtx, 
    #                                 proteins ~ bioreplicate, 
    #                                 sum, 
    #                                 value.var = "intensity")

    protintmtx <- protintmtx %>% 
      tidyr::pivot_wider(id_cols = proteins, 
                         names_from = bioreplicate, 
                         values_from = intensity, 
                         values_fn = list(intensity = sum ))
    
    
    protintmtx <- as.matrix(protintmtx[, -1])
    protintmtx <- log2(protintmtx)
    protintmtx[!is.finite(protintmtx)] <- NA
    
    if(nsamples < 20){
      pairs(
        protintmtx,
        upper.panel = .artms_panelCor,
        diag.panel = .artms_panelHist,
        lower.panel = panel.smooth,
        pch = "."
      )
    }else{
      message("\t(-) Skip Protein-based correlation matrix (too many samples)")
    }

    mprot <- protintmtx
    mprot[is.na(mprot)] <- 0
    prot.pca <- prcomp(t(mprot))
    prot.pcax <- as.data.frame(prot.pca$x)
    prot.pcax <- merge(unique(evidencekeys[, c("bioreplicate", "condition")]),
                       prot.pcax,
                       by.x = "bioreplicate",
                       by.y = "row.names")
      
    
    mb <- ggplot(prot.pcax, aes(PC1, PC2, fill = condition)) + #, color=condition, shape=condition
      geom_point(alpha = .8, size = 5, na.rm = TRUE) +
      geom_label_repel(
        aes(label = bioreplicate),
        #label.size = 2,
        box.padding   = 0.8,
        point.padding = 0.1,
        segment.color = 'grey50'
      ) +
      labs(title = "Principal Component Analysis (Protein Intensity)",
           fill = "") +
      theme_linedraw()
    
    print(mb)
    
    if(printPDF) garbage <- dev.off()
  }
  
  
  if (plotSP) {

    if(verbose) message("--- Plot Sample Preparation", appendLF = FALSE)
    if(printPDF) pdf(
      paste0(output_dir, "/", output_name,'.qcplot.SamplePrep.pdf'),
      width = 10, #nsamples * 3
      height = 6,
      onefile = TRUE
    )
    
    if("missed.cleavages" %in% colnames(evidencekeys)){
      evidence.misscleavages <- data.table(subset(evidencekeys, ms.ms.count > 0))
      misscleavages.tot <- evidence.misscleavages[, .N, by = list(bioreplicate)]
      misscleavages.dt <- evidence.misscleavages[, .N,
                                                 by = list(bioreplicate, missed.cleavages)]
      misscleavages.dt <- merge(misscleavages.dt,
                                misscleavages.tot,
                                by = "bioreplicate",
                                all = TRUE)
      misscleavages.dt$mc <- as.numeric(format(100 * (misscleavages.dt$N.x / misscleavages.dt$N.y),
                                               digits = 5))
      naplot <- ggplot(misscleavages.dt, aes(x = bioreplicate,
                                         y = mc,
                                         fill = as.factor(missed.cleavages),
                                         label = mc)) +
        geom_bar(stat = "identity", alpha = 0.7, na.rm = TRUE) +
        geom_text(
          aes(label = round(mc, digits = 1)),
          vjust = 0,
          size = 2,
          position = position_stack()
        ) +
        xlab("Experiment") + ylab("Fraction (percentage)") +
        labs(title = "Missing cleavages",
             fill = "") +
        theme_linedraw() +
        theme(legend.text = element_text(size = 8)) +
        theme(axis.text.x = element_text(
          angle = 90,
          hjust = 1,
          size = 8
        )) +
        theme(axis.text.y = element_text(size = 10)) +
        theme(axis.title.y = element_text(size = 10)) +
        theme(plot.title = element_text(size = 12)) +
        scale_fill_brewer(palette = "Set1")
        
      print(naplot)
    }


    oxMet.df <-
      plyr::ddply(
        evidencekeys,
        c("bioreplicate", "condition"),
        plyr::summarise,
        pct.OxM = (length(oxidation..m.[oxidation..m. > 0]) / length(sequence)) * 100)
    
    nb <- ggplot(oxMet.df,
                 aes(
                   x = bioreplicate,
                   y = pct.OxM,
                   fill = condition,
                   label = pct.OxM
                 )) +
      geom_bar(stat = "identity", alpha = 0.7, na.rm = TRUE) +
      geom_text(
        aes(label = round(pct.OxM,
                          digits = 1)),
        hjust = 1,
        size = 2.7,
        angle = 0,
        position = position_dodge(width = 1)
      ) +
      xlab("Experiment") +
      ylab("Fraction (percentage)") +
      ggtitle("Percentage of peptides with at least 1 Methionine oxidized") +
      theme_linedraw() +
      theme(legend.text = element_text(size = 8)) +
      theme(axis.text.x = element_text(
        angle = 90,
        hjust = 1,
        size = 8
      )) +
      theme(axis.text.y = element_text(size = 10)) +
      theme(axis.title.y = element_text(size = 10)) +
      theme(plot.title = element_text(size = 12)) +
      scale_fill_brewer(palette = "Spectral")
    
    print(nb)
    
    if(printPDF) garbage <- dev.off()
    if(verbose) message("... done")
  }
  
  if(verbose) message(">> QC extended completed")

} # END OF artmsQualityControlEvidenceExtended  


# @title Quantiles
#
# @description Quantiles
# @param x (datatype) Describe
# @param n (datatype) Describe
# @return What does it return?
# @keywords internal, stats
.artms_qcut <- function(x, n) {
  quantiles <- seq(0, 1, length.out = n + 1)
  cutpoints <- unname(quantile(x, quantiles, na.rm = TRUE))
  as.character(cut(
    x,
    cutpoints,
    labels = seq_len(n),
    include.lowest = TRUE
  ))
}

# @title Sample overlap
#
# @description Describe
# @param data (datatype) Describe
# @param sampleID (datatype) Describe
# @param referenceID (datatype) Describe
# @return What does it return?
# @keywords internal, stats
.artms_sampleOverlap <- function(data,
                                 sampleID = 'bioreplicate',
                                 referenceID = 'Sequence') {
  pept <- by(data,
             data[, sampleID],
             function(x)
               unique(as.character(x[, referenceID])))
  pepmatch <- lapply(pept,
                     function(x, y) {
                       lapply(y, function(y)
                         length(intersect(x, y)) / length(union(x, y)))
                     }, pept)
  m <- matrix(unlist(pepmatch),
              nrow = length(pept),
              ncol = length(pept),
              byrow = TRUE)
    
  colnames(m) <- rownames(m) <- names(pept)
  ans <- list(Peptides = pept, M = m)
  return(ans)
}

# @title Panel Cor
#
# @description Describe
# @param x (datatype) Describe
# @param y (datatype) Describe
# @param digits A number representing...
# @param prefix A prefix for the...
# @param cex.cor A Cex cor....
# @param ... No idea what this does
# @return What does it return?
# @keywords internal, stats
.artms_panelCor <- function(x,
                            y,
                            digits = 2,
                            prefix = "",
                            cex.cor,
                            ...) {
  usr <- par("usr")
  on.exit(par(usr))
  par(usr = c(0, 1, 0, 1))
  r <- abs(cor(x, y, use = "pairwise.complete.obs", method = "pearson"))
  txt <- format(c(r, 0.123456789), digits = digits)[1]
  txt <- paste(prefix, txt, sep = "")
  if (missing(cex.cor))
    cex.cor <- 0.8 / strwidth(txt)
  text(0.5, 0.5, txt, cex = cex.cor * r)
}
  

# @title Panel Histogram
# @description Describe
# @param x (datatype) Describe
# @param ... No idea what this does
# @return What does it return?
# @keywords internal, stats
.artms_panelHist <- function(x, ...) {
  usr <- par("usr")
  on.exit(par(usr))
  par(usr = c(usr[seq_len(2)], 0, 1.5))
  h <- hist(x, plot = FALSE)
  breaks <- h$breaks
  nB <- length(breaks)
  y <- h$counts
  y <- y / max(y)
  rect(breaks[-nB], 0, breaks[-1], y, col = "cyan", ...)
}

# ------------------------------------------------------------------------------
# @title Generate MSstats format object and file
#
# @description Takes as input a reduced version of the Evidence file and
# generates the input data.frame required by MSstats.
# It processes fractionated data as well.
# @param data_f (data.frame) of the filtered Maxquant evidence file.
# @param output_name (char) Output file name (to generate the output files). 
# '.txt' extension required
# @param data_object (logical) if TRUE the output_name cannot be the evidence file
# since it would be an data_object
# @param printTables (logical) `TRUE` (default) prints results tables
# @param verbose (logical) `TRUE` (default) shows function messages
# @return (data.frame) MSstats compatible format
# @keywords internal, MSstats, format, input, fractions
.artms_getMSstatsFormat <- function(data_f, 
                                    output_name, 
                                    data_object = FALSE,
                                    printTables = TRUE,
                                    verbose = TRUE) {
  
  Run = PeptideSequence = Condition_BioReplicate_Run = NULL
  
  if(verbose) message(">> CONVERTING THE DATA TO MSSTATS FORMAT ")

  if(any(missing(data_f) | 
         missing(output_name)))
    stop("Missed (one or many) required argument(s)
         Please, check the help of this function to find out more")
    
  if(data_object){
    output_name <- "artms_evidence.txt"
  }else{
    if (!grepl(".txt", output_name)) {
      stop("Argument <output_file> must have the extension '.txt'")
    }
  }
  
  data_f <- artmsChangeColumnName(data_f, 
                                  "Modified.sequence", 
                                  "PeptideSequence")
  
  data_f$PeptideSequence <- gsub("_", "", data_f$PeptideSequence)
  
  if(verbose) 
    message("-- Selecting Sequence Type: MaxQuant 'Modified.sequence' column")
  
  # DEAL WITH FRACTIONS FIRST 
  if( !("Fraction" %in% colnames(data_f)) ){
    if(verbose) message("\t(+) <Fraction> column added (with value 1, MSstats requirement)")
    data_f$Fraction <- 1
  }
  
  predmss <- data_f[, c("Proteins",
                        "PeptideSequence",
                        "Charge",
                        "IsotopeLabelType",
                        "Condition",
                        "BioReplicate",
                        "Fraction",
                        "Run",
                        "Intensity")]
  
  # step required by MSstats to add 'NA' intensity values for those
  # features not found in certain bioreplicates/runs
  # If this is not done, MSstats will still works,
  # but it will generate a gigantic warning.
  if(verbose)
    message("-- Adding NA values for missing values (required by MSstats) ")
  
  ##LEGACY
  # predmss_dc <- data.table::dcast(data = setDT(predmss),
  #                                 Proteins+PeptideSequence+Charge+IsotopeLabelType~Condition+
  #                                   BioReplicate + Run,
  #                                 value.var = "Intensity",
  #                                 fun.aggregate = sum,
  #                                 sep = "___")
  predmss_dc <- predmss %>% 
    dplyr::mutate(Condition_BioReplicate_Run = paste(Condition, BioReplicate, Run, Fraction, sep = "___") ) %>%
    tidyr::pivot_wider(id_cols = c(Proteins, PeptideSequence, Charge, IsotopeLabelType), 
                       names_from = Condition_BioReplicate_Run, 
                       values_from = Intensity, 
                       values_fn = list(Intensity = sum), values_fill = list(Intensity = NA))
                    
  
  ##LEGACY
  # predmss_melt <- data.table::melt(data = predmss_dc,
  #                                  id.vars = c('Proteins', 
  #                                              'PeptideSequence', 
  #                                              'Charge', 
  #                                              'IsotopeLabelType'),
  #                                  value.name = "Intensity")
  predmss_melt <- predmss_dc %>%
    tidyr::pivot_longer(cols = -c(Proteins, PeptideSequence, Charge, IsotopeLabelType), 
                        names_to = "variable", 
                        values_to = "Intensity")
  
  # And put back the condition, bioreplicate and run columns
  predmss_melt$Condition <- gsub("(.*)(___)(.*)(___)(.*)(___)(.*)", "\\1", predmss_melt$variable)
  predmss_melt$BioReplicate <- gsub("(.*)(___)(.*)(___)(.*)(___)(.*)", "\\3", predmss_melt$variable)
  predmss_melt$Run <- gsub("(.*)(___)(.*)(___)(.*)(___)(.*)", "\\5", predmss_melt$variable)
  predmss_melt$Fraction <- gsub("(.*)(___)(.*)(___)(.*)(___)(.*)", "\\7", predmss_melt$variable)
  
  # After the data has been aggregated, then we add the columns
  predmss_melt$ProductCharge <- NA
  predmss_melt$FragmentIon <- NA
  
  # Names required by MSstats
  predmss_melt <- artmsChangeColumnName(predmss_melt, "Proteins", "ProteinName")
  predmss_melt <- artmsChangeColumnName(predmss_melt, "Charge", "PrecursorCharge")

  # And re-sort it as msstats likes it
  dmss <- predmss_melt[, c("ProteinName",
                           "PeptideSequence",
                           "PrecursorCharge",
                           "FragmentIon",
                           "ProductCharge",
                           "IsotopeLabelType",
                           "Condition",
                           "BioReplicate",
                           "Run",
                           "Fraction",
                           "Intensity")]
  
  ## sanity check for zero's
  if ( nrow(dmss[!is.na(dmss$Intensity) & dmss$Intensity == 0, ]) > 0) {
    dmss[!is.na(dmss$Intensity) & dmss$Intensity == 0, ]$Intensity = NA
  }
  
  dmss <- as.data.frame(dmss)
  if(printTables){
    if(verbose) message("-- Write out the MSstats input file (-mss.txt) ")
    write.table(dmss,
                file = gsub('.txt', '-mss.txt', output_name),
                eol = "\n",
                sep = "\t",
                quote = FALSE,
                row.names = FALSE,
                col.names = TRUE)
  }
  return(dmss)
}

# ------------------------------------------------------------------------------
#' @title Quality Control of the MaxQuant summary.txt file
#'
#' @description Performs quality control based on the information available in
#' the MaxQuant summary.txt file.
#' @param summary_file (char or data.frame) The evidence file path and name, or
#' data.frame
#' @param keys_file (char or data.frame) The keys file path and name or
#' data.frame
#' @param output_dir (char) Name for the folder to output the results plots. 
#' Default is "qc_summary".
#' @param output_name (char) prefix output name (no extension).
#' Default: "qcExtended_summary"
#' @param isFractions (logical) `TRUE` if it is a 2D experiment (fractions).
#' Default: `FALSE`
#' @param plotMS1SCANS (logical) `TRUE` generates MS1 scan counts plot: 
#' Page 1 shows the number of MS1 scans in each BioReplicate. 
#' If replicates are present, Page 2 shows the mean number of MS1 scans 
#' per condition with error bar showing the standard error of the mean. 
#' If isFractions `TRUE`, each fraction is a stack on the individual bar graphs. 
#' @param plotMS2 (logical) `TRUE` generates MS2 scan counts plot: 
#' Page 1 shows the number of MSs scans in each BioReplicate. 
#' If replicates are present, Page 2 shows the mean number of MS1 scans per 
#' condition with error bar showing the standard error of the mean. 
#' If isFractions `TRUE`, each fraction is a stack on the individual bar graphs. 
#' @param plotMSMS (logical) `TRUE` generates MS2 identification rate (%) plot: 
#' Page 1 shows the fraction of MS2 scans confidently identified in each 
#' BioReplicate. If replicates are present, Page 2 shows the mean rate of MS2 
#' scans confidently identified per condition with error bar showing the 
#' standard error of the mean. 
#' If isFractions `TRUE`, each fraction is a stack on the individual bar graphs.
#' @param plotISOTOPE (logical) `TRUE` generates Isotope Pattern counts plot: 
#' Page 1 shows the number of Isotope Patterns with charge greater than 1 in 
#' each BioReplicate. If replicates are present, Page 2 shows the mean number 
#' of Isotope Patterns with charge greater than 1 per condition with error bar 
#' showing the standard error of the mean. 
#' If isFractions `TRUE`, each fraction is a stack on the individual bar graphs.
#' @param printPDF If `TRUE` (default) prints out the pdfs. Warning: plot
#' objects are not returned due to the large number of them. 
#' @param verbose (logical) `TRUE` (default) shows function messages
#' @return A number of plots from the summary file
#' @keywords qc, summary, keys
#' @examples
#' # Testing warning if files are not submitted
#' test <- artmsQualityControlSummaryExtended(summary_file = NULL,
#' keys_file = NULL)
#' @export
artmsQualityControlSummaryExtended <- function(summary_file,
                                               keys_file,
                                               output_dir = "qc_summary",
                                               output_name = "qcExtended_summary",
                                               isFractions = FALSE,
                                               plotMS1SCANS = TRUE,
                                               plotMS2 = TRUE,
                                               plotMSMS = TRUE,
                                               plotISOTOPE = TRUE,
                                               printPDF = TRUE,
                                               verbose = TRUE) {
  if(verbose){
    message("---------------------------------------------")
    message("artMS: EXTENDED QUALITY CONTROL (-summary.txt based)")
    message("---------------------------------------------")
  }
  
  if (is.null(summary_file) & is.null(keys_file)) {
    return("You need to provide both evidence and keys")
  }
  
  if(is.null(output_dir)){
    return("The output_dir argument cannot be NULL")
  }
  
  if(any(missing(summary_file) | missing(keys_file)))
    stop("Missed (one or many) required argument(s)
         Please, check the help of this function to find out more")
  
  # Getting data ready
  summarykeys <- artmsMergeEvidenceAndKeys(summary_file, keys_file,
                                           isSummary = TRUE,
                                           verbose = verbose)
    
  colnames(summarykeys) <- tolower(colnames(summarykeys))
  
  if ("fraction" %in% colnames(summarykeys)) {
    summary2fx <- summarykeys
  }
  
  if ("fraction" %in% colnames(summarykeys)) {
    summarykeys <- data.table(summarykeys[, c("condition",
                                              "bioreplicate",
                                              "ms",
                                              "ms.ms",
                                              "ms.ms.identified....",
                                              "isotope.patterns",
                                              "isotope.patterns.sequenced..z.1.")])
    
    summarykeys <- summarykeys[, list(ms = sum(ms),
                                      ms.ms = sum(ms.ms),
                                      ms.ms.identified.... = mean(ms.ms.identified....),
                                      isotope.patterns = sum(isotope.patterns),
                                      isotope.patterns.sequenced..z.1. = sum(isotope.patterns.sequenced..z.1.)),
                               by = list(condition, bioreplicate)]
  } else{
    summarykeys <-
      data.table(summarykeys[, c(
        "condition",
        "bioreplicate",
        "ms",
        "ms.ms",
        "ms.ms.identified....",
        "isotope.patterns",
        "isotope.patterns.sequenced..z.1."
      )])
  }
  
  summary2 <- plyr::ddply(
    summarykeys,
    c("condition"),
    plyr::summarise,
    num.MS1.mean = mean(ms),
    num.MS1.max = max(ms),
    num.MS1.min = min(ms),
    num.MS1.sem = sd(ms) / sqrt(length(ms)),
    num.MS2.mean = mean(ms.ms),
    num.MS2.max = max(ms.ms),
    num.MS2.min = min(ms.ms),
    num.MS2.sem = sd(ms.ms) / sqrt(length(ms.ms)),
    num.IsotopePatterns.mean = mean(isotope.patterns),
    num.IsotopePatterns.max = max(isotope.patterns),
    num.IsotopePatterns.min = min(isotope.patterns),
    num.IsotopePatterns.sem = sd(isotope.patterns) / sqrt(length(isotope.patterns)),
    num.IsotopePatternsSeq.mean = mean(isotope.patterns.sequenced..z.1.),
    num.IsotopePatternsSeq.max = max(isotope.patterns.sequenced..z.1.),
    num.IsotopePatternsSeq.min = min(isotope.patterns.sequenced..z.1.),
    num.IsotopePatternsSeq.sem = sd(isotope.patterns.sequenced..z.1.) /
      sqrt(length(isotope.patterns.sequenced..z.1.)),
    pct.MS2Id.mean = mean(ms.ms.identified....),
    pct.MS2Id.max = max(ms.ms.identified....),
    pct.MS2Id.min = min(ms.ms.identified....),
    pct.MS2Id.sem = sd(ms.ms.identified....) / sqrt(length(ms.ms.identified....))
  )
  
  # PLOTS-----
  if(verbose) message(">> GENERATING QC PLOTS ")
  
  # create output directory if it doesn't exist
  if(printPDF){
    if (!dir.exists(output_dir)) {
      if(verbose) message("-- Output folder created: ", output_dir)
      dir.create(output_dir, recursive = TRUE)
    }
  }
  
  ## NUMBER OF MS1 SCANS
  if (plotMS1SCANS) {
    if(verbose) message("--- Plot Number of MS1 scans", appendLF = FALSE)
    if(printPDF){
      pdf(paste0(output_dir, "/", output_name,'.qcplot.MS1scans.pdf'),
          width = 10, #nsamples * 3
          height = 6,
          onefile = TRUE)
    } 
    aa <- ggplot(summarykeys, aes(x = bioreplicate, y = ms, fill = condition)) +
      geom_bar(stat = "identity", alpha = 0.7, na.rm = TRUE) +
      geom_text(aes(label = round(ms, digits = 0)), vjust = 1 , size = 2) +
      xlab("Experiment") + ylab("Counts") +
      ggtitle("Number of MS1 scans") +
      theme_linedraw() +
      theme(legend.text = element_text(size = 8)) +
      theme(axis.text.x = element_text(
        angle = 90,
        hjust = 1,
        size = 10
      )) +
      theme(axis.text.y = element_text(size = 10)) +
            theme(axis.text.x = element_text(angle = 90,
                                             size = 8)) +
      theme(axis.title.y = element_text(size = 10)) +
      theme(plot.title = element_text(size = 12)) +
      scale_fill_brewer(palette = "Spectral")
      
    print(aa)
    
    ab <- ggplot(summary2, aes(
      x = condition,
      y = num.MS1.mean,
      fill = factor(condition)
    )) +
      geom_bar(stat = "identity",
               position = position_dodge(width = 1),
               alpha = 0.7, na.rm = TRUE) +
      geom_errorbar(
        aes(
          ymin = num.MS1.mean - num.MS1.sem,
          ymax = num.MS1.mean + num.MS1.sem
        ),
        width = .2,
        position = position_dodge(.9)
      ) +
      geom_text(
        aes(label = round(num.MS1.mean, digits = 0)),
        hjust = 0.5,
        vjust = -0.5,
        size = 2,
        position = position_dodge(width = 1)
      ) +
      xlab("Condition") + ylab("Counts") +
      ggtitle("Mean number of MS1 scans per condition, error bar= std error of the mean") +
      theme_linedraw() +
      theme(legend.text = element_text(size = 8)) +
      theme(axis.text.x = element_text(
        angle = 0,
        hjust = 1,
        size = 10
      )) +
      theme(axis.text.y = element_text(size = 10)) +
            theme(axis.text.x = element_text(angle = 90,
                                             size = 8)) +
      theme(axis.title.y = element_text(size = 10)) +
      theme(plot.title = element_text(size = 12)) +
      scale_fill_brewer(palette = "Spectral")
      
    print(ab)
    
    if (isFractions) {
      ac <- ggplot(summary2fx, aes(
        x = bioreplicate,
        y = ms,
        fill = factor(fraction)
      )) +
        geom_bar(stat = "identity", alpha = 0.7, na.rm = TRUE) +
        geom_text(
          aes(label = round(ms, digits = 0)),
          hjust = 0.5,
          vjust = 1.5,
          size = 2,
          position = position_stack()
        ) +
        xlab("Experiment") + ylab("Counts") +
        ggtitle("Number of MS1 scans per Fraction") +
        theme_linedraw() +
        theme(legend.text = element_text(size = 8)) +
        theme(axis.text.x = element_text(
          angle = 90,
          hjust = 0,
          size = 10
        )) +
        theme(axis.text.y = element_text(size = 10)) +
              theme(axis.text.x = element_text(angle = 90,
                                               size = 8)) +
        theme(axis.title.y = element_text(size = 10)) +
        theme(plot.title = element_text(size = 12))
        
      print(ac)
    }
    if(printPDF) garbage <- dev.off()
    if(verbose) message(" done ")
  }
  
  
  ## Number of MS2 scans
  if (plotMS2) {
    if(verbose) message("--- Plot Number of MS2 scans", appendLF = FALSE)
    
    if(printPDF){
      pdf(paste0(output_dir, "/", output_name,'.qcplot.MS2scans.pdf'),
          width = 10, #nsamples * 3
          height = 6,
          onefile = TRUE)
    } 
    
    ba <- ggplot(summarykeys, aes(x = bioreplicate, y = ms.ms, fill = condition)) +
      geom_bar(stat = "identity", alpha = 0.7, na.rm = TRUE) +
      geom_text(aes(label = round(ms.ms, digits = 0)), vjust = 1 , size = 2) +
      xlab("Experiment") + ylab("Counts") +
      ggtitle("Number of MS2 scans") +
      theme_linedraw() +
      theme(legend.text = element_text(size = 8)) +
      theme(axis.text.x = element_text(
        angle = 90,
        hjust = 1,
        size = 2
      )) +
      theme(axis.text.y = element_text(size = 10)) +
            theme(axis.text.x = element_text(angle = 90,
                                             size = 8)) +
      theme(axis.title.y = element_text(size = 10)) +
      theme(plot.title = element_text(size = 12)) +
      scale_fill_brewer(palette = "Spectral")
      
    print(ba)
    
    bb <- ggplot(summary2, aes(
      x = condition,
      y = num.MS2.mean,
      fill = factor(condition)
    )) +
      geom_bar(stat = "identity",
               position = position_dodge(width = 1),
               alpha = 0.7, na.rm = TRUE) +
      geom_errorbar(
        aes(
          ymin = num.MS2.mean - num.MS2.sem,
          ymax = num.MS2.mean + num.MS2.sem
        ),
        width = .2,
        position = position_dodge(.9)
      ) +
      geom_text(
        aes(label = round(num.MS2.mean, digits = 0)),
        hjust = 0.5,
        vjust = -0.5,
        size = 2,
        position = position_dodge(width = 1)
      ) +
      xlab("Condition") + ylab("Counts") +
      ggtitle("Mean number of MS2 scans per condition, error bar= std error of the mean") +
      theme_linedraw() +
      theme(legend.text = element_text(size = 8)) +
      theme(axis.text.x = element_text(
        angle = 0,
        hjust = 1,
        size = 10
      )) +
      theme(axis.text.y = element_text(size = 10)) +
            theme(axis.text.x = element_text(angle = 90,
                                             size = 8)) +
      theme(axis.title.y = element_text(size = 10)) +
      theme(plot.title = element_text(size = 12)) +
      scale_fill_brewer(palette = "Spectral")
      
    print(bb)
    
    if (isFractions) {
      bc <- ggplot(summary2fx, aes(
        x = bioreplicate,
        y = ms.ms,
        fill = factor(fraction)
      )) +
        geom_bar(stat = "identity", alpha = 0.7, na.rm = TRUE) +
        geom_text(
          aes(label = round(ms.ms, digits = 0)),
          hjust = 0.5,
          vjust = 1.5,
          size = 2,
          position = position_stack()
        ) +
        xlab("Experiment") + ylab("Counts") +
        ggtitle("Number of MS2 per Fraction") +
        theme_linedraw() +
        theme(legend.text = element_text(size = 8)) +
        theme(axis.text.x = element_text(
          angle = 90,
          hjust = 0,
          size = 10
        )) +
        theme(axis.text.y = element_text(size = 10)) +
              theme(axis.text.x = element_text(angle = 90,
                                               size = 8)) +
        theme(axis.title.y = element_text(size = 10)) +
        theme(plot.title = element_text(size = 12))
        
      print(bc)
    }
    
    summarykeys.reduced <- summarykeys[, 1:4]
    
    ## LEGACY
    # summarykeys.scans1 <- data.table::melt(summarykeys.reduced, 
    #                                       id.vars = seq_len(2))
    
    summarykeys.scans <- summarykeys.reduced %>%
      tidyr::pivot_longer(cols = -c(condition, bioreplicate), 
                          names_to = "variable", values_to = "value")
    
    bd <-ggplot(summarykeys.scans,
                aes(
                  x = interaction(variable, bioreplicate),
                  y = value,
                  fill = condition
                )) +
      geom_bar(stat = "identity", alpha = 0.7, na.rm = TRUE) +
      geom_text(
        aes(label = round(value, digits = 0)),
        angle = 90,
        vjust = 0.5 ,
        size = 2
      ) +
      xlab("Experiment") + ylab("Counts") +
      ggtitle("Number of MS1 and MS2 scans") +
      theme_linedraw() +
      theme(legend.text = element_text(size = 8)) +
      theme(axis.text.x = element_text(
        angle = 90,
        hjust = 1,
        size = 10
      )) +
      theme(axis.text.y = element_text(size = 10)) +
            theme(axis.text.x = element_text(angle = 90,
                                             size = 8)) +
      theme(axis.title.y = element_text(size = 10)) +
      theme(plot.title = element_text(size = 12)) +
      scale_fill_brewer(palette = "Spectral")
    print(bd)
    if(printPDF) garbage <- dev.off()
    if(verbose) message(" done ")
  }
  
  
  # Number of msms.identification rate
  if (plotMSMS) {
    if(verbose) message("--- Plot Number of msms.identification rate", appendLF = FALSE)
    
    if(printPDF){
      pdf(paste0(output_dir, "/", output_name,'.qcplot.MSMS.pdf'),
          width = 10, #nsamples * 3
          height = 6,
          onefile = TRUE)
    } 

    ca <- ggplot(summarykeys,
                 aes(x = bioreplicate, y = ms.ms.identified...., fill = condition)) +
      geom_bar(stat = "identity", alpha = 0.7, na.rm = TRUE) +
      geom_text(aes(label = round(ms.ms.identified...., digits = 2)), vjust = 1 , size = 2) +
      xlab("Experiment") + ylab("Rate") +
      ggtitle("MS2 Identification rate") +
      theme_linedraw() +
      theme(legend.text = element_text(size = 8)) +
      theme(axis.text.x = element_text(
        angle = 90,
        hjust = 1,
        size = 10
      )) +
      theme(axis.text.y = element_text(size = 10)) +
            theme(axis.text.x = element_text(angle = 90,
                                             size = 8)) +
      theme(axis.title.y = element_text(size = 10)) +
      theme(plot.title = element_text(size = 12)) +
      scale_fill_brewer(palette = "Spectral")
      
    print(ca)
    
    cb <- ggplot(summary2,
                 aes(
                   x = condition,
                   y = pct.MS2Id.mean,
                   fill = factor(condition)
                 )) +
      geom_bar(stat = "identity",
               position = position_dodge(width = 1),
               alpha = 0.7, na.rm = TRUE) +
      geom_errorbar(
        aes(
          ymin = pct.MS2Id.mean - pct.MS2Id.sem,
          ymax = pct.MS2Id.mean + pct.MS2Id.sem
        ),
        width = .2,
        position = position_dodge(.9)
      ) +
      geom_text(
        aes(label = round(pct.MS2Id.mean, digits = 2)),
        hjust = 0.5,
        vjust = -0.5,
        size = 2,
        position = position_dodge(width = 1)
      ) +
      xlab("Condition") + ylab("Rate") +
      ggtitle("Mean MS2 Identification rate across bioreplicates and fractions") +
      theme_linedraw() +
      theme(legend.text = element_text(size = 8)) +
      theme(axis.text.x = element_text(
        angle = 0,
        hjust = 1,
        size = 10
      )) +
      theme(axis.text.y = element_text(size = 10)) +
            theme(axis.text.x = element_text(angle = 90,
                                             size = 8)) +
      theme(axis.title.y = element_text(size = 10)) +
      theme(plot.title = element_text(size = 12)) +
      scale_fill_brewer(palette = "Spectral")
      
    print(cb)
    
    if (isFractions) {
      cc <- ggplot(summary2fx,
                   aes(
                     x = bioreplicate,
                     y = ms.ms.identified....,
                     fill = factor(fraction)
                   )) +
        geom_bar(stat = "identity", alpha = 0.7, na.rm = TRUE) +
        geom_text(
          aes(label = round(ms.ms.identified...., digits = 1)),
          hjust = 0.5,
          vjust = 1.5,
          size = 2,
          position = position_stack()
        ) +
        xlab("Experiment") + ylab("Counts") +
        ggtitle("MS2 Identification rate per Fraction") +
        theme_linedraw() +
        theme(legend.text = element_text(size = 8)) +
        theme(axis.text.x = element_text(
          angle = 90,
          hjust = 0,
          size = 10
        )) +
        theme(axis.text.y = element_text(size = 10)) +
              theme(axis.text.x = element_text(angle = 90,
                                               size = 8)) +
        theme(axis.title.y = element_text(size = 10)) +
        theme(plot.title = element_text(size = 12))
        
      print(cc)
    }
    if(printPDF) garbage <- dev.off()
    if(verbose) message(" done ")
  }
  
  
  # Number of isotope patterns
  if (plotISOTOPE) {
    if(verbose) message("--- Plot Number of isotope patterns", appendLF = FALSE)
    
    if(printPDF){
      pdf(paste0(output_dir, "/", output_name,'.qcplot.Isotope.pdf'),
          width = 10, #nsamples * 3
          height = 6,
          onefile = TRUE)
    } 
    
    da <- ggplot(summarykeys,
                 aes(x = bioreplicate, y = isotope.patterns, fill = condition)) +
      geom_bar(stat = "identity", alpha = 0.7, na.rm = TRUE) +
      geom_text(aes(label = round(isotope.patterns, digits = 0)), vjust = 1 , size = 2) +
      xlab("Experiment") + ylab("Counts") +
      ggtitle("Number of detected Isotope Patterns") +
      theme_linedraw() +
      theme(legend.text = element_text(size = 8)) +
      theme(axis.text.x = element_text(
        angle = 90,
        hjust = 1,
        size = 10
      )) +
      theme(axis.text.y = element_text(size = 10)) +
            theme(axis.text.x = element_text(angle = 90,
                                             size = 8)) +
      theme(axis.title.y = element_text(size = 10)) +
      theme(plot.title = element_text(size = 12)) +
      scale_fill_brewer(palette = "Spectral")
      
    print(da)
    
    db <- ggplot(summary2,
                 aes(
                   x = condition,
                   y = num.IsotopePatterns.mean,
                   fill = factor(condition)
                 )) +
      geom_bar(stat = "identity",
               position = position_dodge(width = 1),
               alpha = 0.7, na.rm = TRUE) +
      geom_errorbar(
        aes(
          ymin = num.IsotopePatterns.mean - num.IsotopePatterns.sem,
          ymax = num.IsotopePatterns.mean + num.IsotopePatterns.sem
        ),
        width = .2,
        position = position_dodge(.9)
      ) +
      geom_text(
        aes(label = round(num.IsotopePatterns.mean, digits = 0)),
        hjust = 0.5,
        vjust = -0.5,
        size = 2,
        position = position_dodge(width = 1)
      ) +
      xlab("Condition") + ylab("Counts") +
      ggtitle("Mean number of detected Isotope Patterns") +
      theme_linedraw() +
      theme(legend.text = element_text(size = 8)) +
      theme(axis.text.x = element_text(
        angle = 0,
        hjust = 1,
        size = 10
      )) +
      theme(axis.text.y = element_text(size = 10)) +
            theme(axis.text.x = element_text(angle = 90,
                                             size = 8)) +
      theme(axis.title.y = element_text(size = 10)) +
      theme(plot.title = element_text(size = 12)) +
      scale_fill_brewer(palette = "Spectral")
      
    print(db)
    
    if (isFractions) {
      dc <- ggplot(summary2fx,
                   aes(
                     x = bioreplicate,
                     y = isotope.patterns,
                     fill = factor(fraction)
                   )) +
        geom_bar(stat = "identity", alpha = 0.7, na.rm = TRUE) +
        geom_text(
          aes(label = round(isotope.patterns, digits = 0)),
          hjust = 0.5,
          vjust = 1.5,
          size = 2,
          position = position_stack()
        ) +
        xlab("Experiment") + ylab("Counts") +
        ggtitle("Number of detected Isotope Patterns per Fraction") +
        theme_linedraw() +
        theme(legend.text = element_text(size = 8)) +
        theme(axis.text.x = element_text(
          angle = 90,
          hjust = 0,
          size = 10
        )) +
        theme(axis.text.y = element_text(size = 10)) +
              theme(axis.text.x = element_text(angle = 90,
                                               size = 8)) +
        theme(axis.title.y = element_text(size = 10)) +
        theme(plot.title = element_text(size = 12))
        
      print(dc)
    }
    
    # Number of sequenced isotope patterns with charge = 2 or more
    dd <- ggplot(
      summarykeys,
      aes(x = bioreplicate, y = isotope.patterns.sequenced..z.1., 
          fill = condition)
    ) +
      geom_bar(stat = "identity", alpha = 0.7, na.rm = TRUE) +
      geom_text(aes(label = round(
        isotope.patterns.sequenced..z.1., digits = 0
      )), vjust = 1 , size = 2) +
      xlab("Experiment") + ylab("Counts") +
      ggtitle("Number of sequenced Isotope Patterns with 
              charge state greater than 1") +
      theme_linedraw() +
      theme(legend.text = element_text(size = 8)) +
      theme(axis.text.x = element_text(
        angle = 90,
        hjust = 1,
        size = 10
      )) +
      theme(axis.text.y = element_text(size = 10)) +
            theme(axis.text.x = element_text(angle = 90,
                                             size = 8)) +
      theme(axis.title.y = element_text(size = 10)) +
      theme(plot.title = element_text(size = 12)) +
      scale_fill_brewer(palette = "Spectral")
      
    print(dd)
    
    de <- ggplot(summary2,
                 aes(
                   x = condition,
                   y = num.IsotopePatternsSeq.mean,
                   fill = factor(condition)
                 )) +
      geom_bar(stat = "identity",
               position = position_dodge(width = 1),
               alpha = 0.7, na.rm = TRUE) +
      geom_errorbar(
        aes(
          ymin = num.IsotopePatternsSeq.mean - num.IsotopePatternsSeq.sem,
          ymax = num.IsotopePatternsSeq.mean + num.IsotopePatternsSeq.sem
        ),
        width = .2,
        position = position_dodge(.9)
      ) +
      geom_text_repel(
        aes(label = round(
          num.IsotopePatternsSeq.mean, digits = 0
        )),
        hjust = 0.5,
        vjust = -0.5,
        size = 2,
        position = position_dodge(width = 1)
      ) +
      xlab("Condition") + ylab("Counts") +
      ggtitle("Mean number of sequenced Isotope 
              Patterns with charge state greater than 1") +
      theme_linedraw() +
      theme(legend.text = element_text(size = 8)) +
      theme(axis.text.x = element_text(
        angle = 0,
        hjust = 1,
        size = 10
      )) +
      theme(axis.text.y = element_text(size = 10)) +
            theme(axis.text.x = element_text(angle = 90,
                                             size = 8)) +
      theme(axis.title.y = element_text(size = 10)) +
      theme(plot.title = element_text(size = 12)) +
      scale_fill_brewer(palette = "Spectral")
      
    print(de)
    
    if (isFractions) {
      df <- ggplot(
        summary2fx,
        aes(
          x = bioreplicate,
          y = isotope.patterns.sequenced..z.1.,
          fill = factor(fraction)
        )
      ) +
        geom_bar(stat = "identity", alpha = 0.7, na.rm = TRUE) +
        geom_text(
          aes(label = round(
            isotope.patterns.sequenced..z.1., digits = 0
          )),
          hjust = 0.5,
          vjust = 1.5,
          size = 2,
          position = position_stack()
        ) +
        xlab("Experiment") + ylab("Counts") +
        ggtitle("Number of sequenced Isotope Patterns  with charge state greater than 1 per Fraction") +
        theme_linedraw() +
        theme(legend.text = element_text(size = 8)) +
        theme(axis.text.x = element_text(
          angle = 90,
          hjust = 0,
          size = 10
        )) +
        theme(axis.text.y = element_text(size = 10)) +
        theme(axis.title.x = element_text(size = 10)) +
        theme(axis.title.y = element_text(size = 10)) +
        theme(plot.title = element_text(size = 12))
        
      print(df)
    }
    if(printPDF) garbage <- dev.off()
    if(verbose) message(" done ")
  }
  
} # END OF SUMMARY

# ==============================================================================
#' @title Summarize average intensity and retention time per protein
#'
#' @description Input an evidence file from MaxQuant and a file
#' containing a list of proteins of interest (optional).
#' The function will summarize from the evidence file and report back the
#' average intensity, average retention time, and the average caliberated
#' retention time. If a list of proteins is provided, then only those proteins
#' will be summarized and returned.
#' @param evidence_file (char) The filepath to the MaxQuant searched data
#' (evidence) file (txt tab delimited file).
#' @param protein_file (char) The file path to a file or vector containing
#' a list of proteins of interest.
#' @param output_file (char) The file name for the results
#' (must have the extension `.txt`). If empty, then the
#' results will be returned as an R object.
#' @param verbose (logical) `TRUE` (default) shows function messages
#' @return An R object with the results and a file with the results (if the
#' output_file argument is provided). It contains averages of Intensity,
#' Retention Time, Caliberated Retention Time
#' @keywords MaxQuant, evidence, summary, intensity, retention time, caliberated
#' @examples
#' ave_int <- artmsAvgIntensityRT(evidence_file = artms_data_ph_evidence)
#' @export
artmsAvgIntensityRT <- function(evidence_file,
                                protein_file = NULL,
                                output_file = FALSE,
                                verbose = TRUE) {
                                   
  
  if(any(missing(evidence_file)))
    stop("Missed evidence_file required argument.
         Please, check the help of this function to find out more")
  
  # read in data
  if(verbose) message(">> READING IN FILES... ")
  dat <- .artms_checkIfFile(evidence_file, is.evidence = TRUE)
  
  if (!is.null(protein_file)) {
    if(verbose) message(">> FILTERING OUT UNWANTED PROTEINS... ")
    proteins <- .artms_checkIfFile(protein_file)
    
    # pull out only cases where the proteins appear
    query <- paste(t(proteins), collapse = "|")
    idx <- grep(query, dat$Proteins)
    dat <- dat[idx,]
  }
  
  if(verbose) message(">> COMPUTING AVERAGES... ")
  # Compute the average Intensity
  dat.avg <-
    aggregate(data = dat[, c("Proteins",
                             "Modified.sequence",
                             "Charge",
                             "Intensity")], 
              Intensity ~ .,
              mean, na.rm = TRUE)
  # Compute the average Retention Time
  dat.ret <-
    aggregate(data = dat[, c("Proteins",
                             "Modified.sequence",
                             "Charge",
                             "Retention.time")], 
              Retention.time ~., 
              mean, na.rm = TRUE)
  # Compute the average Calibrated retention time
  dat.cal <-
    aggregate(data = dat[, c("Proteins",
                             "Modified.sequence",
                             "Charge",
                             "Calibrated.retention.time")],
              Calibrated.retention.time ~ .,
              mean,
              na.rm = TRUE)
  
  if(verbose) message(">> MERGING RESULTS... ")
  results <-
    merge(
      dat.avg,
      dat.ret,
      by = c("Proteins", "Modified.sequence", "Charge"),
      all.y = TRUE
    )
  results <-
    merge(
      results,
      dat.cal,
      by = c("Proteins", "Modified.sequence", "Charge"),
      all.y = TRUE
    )
  # add 'Avg' to names
  names(results)[4:6] = paste0("Avg_", names(results)[5:7])
  
  if(verbose) message(">> SUMMARIZATION COMPLETE!! ")
  
  if (output_file) {
    # write out results
    if(verbose) message("--- WRITING OUT RESULTS TO ", output_file, " ")
    write.table(
      results,
      output_file,
      quote = FALSE,
      row.names = FALSE,
      sep = "\t"
    )
    return(results)
  } else{
    return(results)
  }
}

# Everything about enrichments (including plots)


# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# @title Enrich for Protein Complexes using CORUM
#
# @description Enrich for Protein Complexes using CORUM
# @param df (data.frame) Data.frame with columns: `Protein` and `Conditions`
# @param backgroundNumber (int) Background number of genes
# @return (data.frame) A text delimited data.frame with protein complex 
# enrichment results
# @keywords internal, enrichment, protein, complexes
.artms_enrichForComplexes <- function(df, 
                                      backgroundNumber) {
  
  if(any(missing(df) | missing(backgroundNumber)))
    stop("Missed (one or many) required argument(s)
         Please, check the help of this function to find out more")
  
  listOfConditions <- unique(df$Comparisons)
  
  complexEnrichmentConditions <- data.frame()
  
  for (i in seq_len(length(listOfConditions))) {
    condi <- listOfConditions[i]
    tmp <- unique(df$Protein[which(df$Comparisons == condi)])
    tmpEnrich <-
      .artms_foldComplexEnrichment(tmp, 
                                   artms_data_corum_mito_database, 
                                   backgroundNumber)
    # Check point
    checkpc <- dim(tmpEnrich)[1]
    if (checkpc > 0) {
      tmpEnrich$Comparisons <- condi
      complexEnrichmentConditions <-
        rbind(complexEnrichmentConditions, tmpEnrich)
    }
  }
  return(complexEnrichmentConditions)
}


# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' @title Enrichment of changes in protein abundance or PTMs
#'
#' @description Enrichment analysis of the selected proteins
#' @param dataset (data.frame) with a `Gene` and `Comparison or Label` (with
#' the name of the comparisons specified in the contrast file) columns
#' @param species (char) Specie, only supported "human" or "mouse"
#' @param background (vector) Background genes for the enrichment analysis.
#' @param heatmaps (logical) if `TRUE` generates heatmaps (pdf),
#' `FALSE` (default) otherwise.
#' @param output_name (char) Name of the annotation files, which will be used
#' as well for the heatmaps (if `heatmaps` is selected)
#' Default `output_name = "enrichment.txt"`
#' @param verbose (logical) `TRUE` (default) shows function messages
#' @return (data.frame) Results from the enrichment analysis using Gprofiler
#' and heatmaps (if selected)
#' @keywords enrichment
#' @examples \dontrun{
#' # The data must be annotated (Protein and Gene columns)
#' data_annotated <- artmsAnnotationUniprot(
#'                       x = artms_data_ph_msstats_results,
#'                       columnid = "Protein",
#'                       species = "human")
#' # And then the enrichment
#' enrich_set <- artmsEnrichLog2fc(
#'                    dataset = data_annotated,
#'                    species = "human",
#'                    background = unique(data_annotated$Gene))
#'}
#' @export
artmsEnrichLog2fc <- function(dataset,
                              species,
                              background,
                              heatmaps = FALSE,
                              output_name = "enrichment.txt",
                              verbose = TRUE) {
                               
  if( !("gProfileR" %in% installed.packages()) ){
    stop("Package <gProfileR> required to run this function. Please, install and run it again")
  }
  
  if(any(missing(dataset) | 
         missing(species) |
         missing(background)))
    stop("Missed (one or many) required argument(s)
         Please, check the help of this function to find out more")

  if (heatmaps) {
    if (!grepl("\\.txt", output_name)) {
      stop("The <output_name> argument does not have extension '.txt'")
    }
  }
  
  # First check the "comparisons" column. if it is a msstats results file
  # should have instead the "Label" column.
  if (!any(grepl("Comparisons", colnames(dataset)))) {
    if (any(grepl("Label", colnames(dataset)))) {
      dataset <- artmsChangeColumnName(dataset, "Label", "Comparisons")
    } else{
      stop("This dataset does not have a <Label> or <Comparisons> column")
    }
  }
  
  # Selecting unique genes in each comparison
  pretmp <- dataset[c('Gene', 'Comparisons')]
  pretmp <- unique(pretmp)
  
  tmp = split(pretmp$Gene, pretmp$Comparisons, drop = TRUE)
  
  if (species == "human") {
    enrichgenes <- artmsEnrichProfiler(x = tmp,
                                       categorySource = c(
                                         'GO:BP',
                                         'GO:MF',
                                         'GO:CC',
                                         'KEGG',
                                         'REAC',
                                         'CORUM',
                                         'HPA',
                                         'OMIM'
                                       ),
                                       species = 'hsapiens',
                                       background = background,
                                       verbose = verbose) # 'HP'
  } else if (species == "mouse") {
    enrichgenes <-
      artmsEnrichProfiler(
        tmp,
        categorySource = c('GO:BP', 'GO:MF', 'GO:CC', 'KEGG', 'REAC', 'CORUM'),
        species = 'mmusculus',
        background, verbose = verbose
      )
  } else{
    stop(" The species (",
         species,
         ") not supported for the enrichment analysis!! ")
  }
  
  if (dim(enrichgenes)[1] == 0) {
    message("--- No significant results from the enrichment analysis ")
  } else{
    if (is.null(heatmaps)) {
      enrichgenes2plot <- enrichgenes[which(enrichgenes$term.size < 500), ]
      for (i in unique(enrichgenes2plot$domain)) {
        if(verbose) message("\t--- Plotting '", i, "' annotations... ")
        tmp <-
          enrichgenes2plot[which(enrichgenes2plot$domain == i), ]
        outfile <- gsub(".txt", paste0("_", i, ".txt"), output_name)
        .artms_EnrichmentPlotHeatmaps(tmp, outfile)
        if(verbose) message("out! ")
      }
    }
  }
  return(enrichgenes)
}


# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# @title Plot Corum Enrichment results
#
# @description Heatmap of significant enrichment
# @param x (data.frame) output from `artms_enrichForComplexes`
# @param outfile (char) output file name (must have the extenstion `.pdf`)
# @param theTitle (char) Plot's title
# @return (pdf) heatmap of the significantly enriched protein complexes
# @keywords internal, plot, heatmap, enrichment
.artms_plotCorumEnrichment <- function(x, 
                                       outfile, 
                                       theTitle) {
  Comparisons = p_value = var = NULL

  if(any(missing(x) | 
         missing(outfile) |
         missing(theTitle)))
    stop("Missed (one or many) required argument(s)
        Please, check the help of this function to find out more")
  
  checkPoint <- length(unique(x$Comparisons))
  
  if (checkPoint >= 1) {
    # Dealing with the smallest pvalues (pvalue = 0)
    dftemp <- x[-which(x$pvalue == 0), ]
    if (dim(dftemp)[1] > 0) {
      theMinimal <- min(dftemp$pvalue) / 10
      # Make the value replacement
      x$p_value <- x$pvalue
      x$p_value[x$p_value == 0] <- theMinimal
    } else{
      x$p_value <- x$pvalue
    }
    
    x$p_value <- -log10(x$p_value)
    
    ##LEGACY
    # toplot <- data.table::dcast(data = x,
    #                             ComplexName ~ Comparisons,
    #                             value.var = "p_value",
    #                             fun.aggregate = sum,
    #                             fill = 0)
    
    toplot <- x %>% 
      tidyr::pivot_wider(id_cols = ComplexName, 
                         names_from = Comparisons, 
                         values_from = p_value, 
                         values_fn = list(p_value = sum), 
                         values_fill = list(p_value = 0))

    toplot <- as.data.frame(toplot)
    
    rownames(toplot) <- toplot$ComplexName
    toplotmatrix <- subset(toplot, select = -c(ComplexName))

    xmatrix <- data.matrix(toplotmatrix)
    
    # HEATMAP
    # palette.breaks <- seq(1, 4, 0.1)
    # color.palette  <- grDevices::colorRampPalette(c("white", "steelblue"))(length(palette.breaks))
    
    if(var(xmatrix[,1]) == 0){
      message("----Heatmap of Corum enrichment is not possible")
    }else{
      pheatmap::pheatmap(
        xmatrix,
        filename = outfile,
        cluster_rows = TRUE,
        cluster_cols = FALSE,
        cellheight = 10,
        cellwidth = 25,
        main = theTitle,
        fontsize = 6,
        fontsize_row = 8,
        fontsize_col = 12,
        border_color = 'black',
        fontfamily = "Helvetica",
        treeheight_row = FALSE,
        treeheight_col = FALSE,
        # color = color.palette
      )
    }

  } else{
    message("---(-) Not enough enriched comparisons to plot the heatmap ")
  }
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' @title Enrichment analysis using GprofileR
#'
#' @description This function simplifies the enrichment analysis performed by
#' the excellent tool GprofileR.
#' @param x (list, data.frame) List of protein ids. It can be anything:
#' either a list of ids, or you could also send a data.frame and it will find
#' the columns with the IDs. Is not cool? Multiple list can be also sent
#' simultaneously, as for example running:
#' `tmp <- split(enrichment$Gene, enrichment$cl_number, drop= TRUE)`
#' @param categorySource (vector) Resources providing the terms on which
#' the enrichment will be performed. The supported resources by gprofiler are:
#' - GO (GO:BP, GO:MF, GO:CC): Gene Ontology (see more below)
#' - KEGG: Biological pathways
#' - REAC: Biological pathways (Reactome)
#' - TF: Regulatory motifs in DNA (TRANSFAC TFBS)
#' - MI: Regulatory motifs in DNA (miRBase microRNAs)
#' - CORUM: protein complexes database
#' - HP: Human Phenotype Ontology
#' - HPA: Protein databases (Human Protein Atlas)
#' - OMIM: Online Mendelian Inheritance in Man annotations:
#' - BIOGRID: BioGRID protein-protein interactions
#' The type of annotations for Gene Ontology:
#' - Inferred from experiment (IDA, IPI, IMP, IGI, IEP)
#' - Direct assay (IDA) / Mutant phenotype (IMP]
#' - Genetic interaction (IGI) / Physical interaction (IPI)
#' - Traceable author (TAS) / Non-traceable author (NAS) /
#' Inferred by curator (IC)
#' - Expression pattern (IEP) / Sequence or structural similarity (ISS)
#' / Genomic context (IGC)
#' - Biological aspect of ancestor (IBA) / Rapid divergence (IRD)
#' - Reviewed computational analysis (RCA) / Electronic annotation (IEA)
#' - No biological data (ND) / Not annotated or not in background (NA)
#' @param species (char) Specie code: Organism names are constructed by
#' concatenating the first letter of the name and the family name.
#' Example: human - ’hsapiens’, mouse - ’mmusculus’. Check gProfileR to find out
#' more about supported species.
#' @param background (vector) gene list to use as background for the enrichment
#' analysis. Default: `NA`
#' @details This function uses the following `gprofiler` arguments as default:
#' - ordered_query = FALSE
#' - significant = TRUE
#' - exclude_iea = TRUE
#' - underrep = FALSE
#' - evcodes = FALSE
#' - region_query = FALSE
#' - max_p_value = 0.05
#' - min_set_size = 0
#' - max_set_size = 0
#' - min_isect_size = 0
#' - correction_method = "analytical" #Options: "gSCS", "fdr", "bonferroni"
#' - hier_filtering = "none"
#' - domain_size = "known" # annotated or known
#' - numeric_ns = ""
#' - png_fn = NULL
#' - include_graph = TRUE
#' @param verbose (logical) `TRUE` (default) shows function messages
#' @return The enrichment results as provided by gprofiler
#' @keywords enrichment
#' @examples \dontrun{
#' # annotate the MSstats results to get the Gene name
#' data_annotated <- artmsAnnotationUniprot(
#'                                      x = artms_data_ph_msstats_results,
#'                                      columnid = "Protein",
#'                                      species = "human")
#'
#' # Filter the list of genes with a log2fc > 2
#' filtered_data <- 
#' unique(data_annotated$Gene[which(data_annotated$log2FC > 2)])
#'
#' # And perform enrichment analysis
#' data_annotated_enrich <- artmsEnrichProfiler(
#'                                    x = filtered_data,
#'                                    categorySource = c('KEGG'),
#'                                    species = "hsapiens",
#'                                    background = unique(data_annotated$Gene))
#'}
#' @export
artmsEnrichProfiler <- function(x,
                                categorySource = c('GO'),
                                species,
                                background = NA,
                                verbose = TRUE) {
  gprofiler = NULL
  
  if( !("gProfileR" %in% installed.packages()) ){
    stop("Package <gProfileR> required to run this function. Please, install and run it again")
  }
                                 
  if(any(missing(x) | missing(species)))
    stop("Missed (one or many) required argument(s)
         Please, check the help of this function to find out more")
  
  suppressWarnings(gProfileR::set_base_url("http://biit.cs.ut.ee/gprofiler"))
  if(verbose) message("---+ Enrichment analysis using gProfiler...", appendLF = FALSE) 
  suppressWarnings(
    enrichData <- gprofiler(x,
                            organism = species,
                            # "scerevisiae","hsapiens", "mmusculus"
                            ordered_query = FALSE,
                            significant = TRUE,
                            exclude_iea = TRUE,
                            # do you want to exclude electronic annotations (IEA)?
                            underrep = FALSE,
                            evcodes = FALSE,
                            region_query = FALSE,
                            max_p_value = 0.05,
                            min_set_size = 0,
                            max_set_size = 0,
                            min_isect_size = 0,
                            correction_method = "analytical",
                            #Options: "gSCS", "fdr", "bonferroni"
                            hier_filtering = "none",
                            domain_size = "known",
                            # annotated or known
                            custom_bg = background,
                            numeric_ns = "",
                            png_fn = NULL,
                            include_graph = TRUE,
                            src_filter = categorySource)
  )
    
  if(verbose) message("done! ")
  return(enrichData)
}

# Little function to
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# @title Simplify the gProfiler output
#
# @description Simplify the output from `artmsEnrichProfiler` resulted from
# running `gProfileR`
# @param gp (data.frame) with the results
# @return (data.frame) with the following columns:
#       'query.number', 'domain', 'p.value', 'query.size', 'overlap.size',
#       'term.size', 'recall', 'precision', 'term.id', 'term.name',
#       'intersection'
# @keywords internal, cleaning
.artms_cleanGPROFILER <- function(gp) {
  sendBack <- gp[c(
    'query.number',
    'domain',
    'p.value',
    'query.size',
    'overlap.size',
    'term.size',
    'recall',
    'precision',
    'term.id',
    'term.name',
    'intersection'
  )]
  return(sendBack)
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# @title Plot and save heatmaps of the significant enrichment results
#
# @description plot and save heatmaps of the significant enrichment results
# @param x (data.frame) output from gprofiler
# @param out_file (char) output file name (must have `.txt` extension)
# @return (pdf) A heatmap of the most significant enrichments
# @keywords internal, plot, heatmap, enrichments
.artms_EnrichmentPlotHeatmaps <- function(x, 
                                          out_file) {
  
  term.name = query.number = p.value = NULL
  
  ##LEGACY
  # x <- dcast(x,
  #            term.name ~ query.number,
  #            value.var = 'p.value',
  #            max,
  #            fill = 1)
  
  x <- x %>% tidyr::pivot_wider(id_cols = term.name, 
                                names_from = query.number, 
                                values_from = p.value,
                                values_fn = list(p.value = max),
                                values_fill = list(p.value = 1))
  
  # Let's stop this thing if there is not enough terms (we need at least 2)
  if (dim(x)[1] < 2) {
    return(message(" Not enough terms in this domain "))
  } else{
    row.names(x) = x$term.name
    x$term.name = c()
    # At least one comparison ready
    if (dim(x)[2] > 0) {
      idx <- apply(x, 1, function(y) {
        return(min(y) <= .05)
      })
      x <- x[which(idx), ]
      x <- -log10(x)
      
      # setting up breaks for colors of -log10 p-values
      HEAT_STYLE = "Q" ##HEAT_STYLE="SCORE"
      P_T = 0.05
      BREAKS = c(0, -log10(c(P_T, P_T / 10, P_T / 100, P_T / 1000)))
      LABELS = c(0, "  5E-2", "  5E-3", "< 5E-4", "")
      
      ## format stuff for heatmap
      colors = c(colorRampPalette(
        brewer.pal(n = 7, name = "Purples"))(length(BREAKS) - 1))
      LOWER_T = BREAKS[length(BREAKS)]
      term_groups_selected_w_display = x
      term_groups_selected_w_display[term_groups_selected_w_display > LOWER_T] =
        LOWER_T
      if (length(x) > 1) {
        #Do you need a main title: 
        #main=paste( gsub(".txt","",basename(out_file)) , 
        #"(color: -log10 p-value )"),
        pheatmap(
          term_groups_selected_w_display,
          cluster_cols = FALSE,
          cellheight = 10,
          cellwidth = 10,
          scale = "none",
          filename = gsub('.txt', '_heatmap.pdf', out_file),
          fontsize = 6,
          fontsize_row = 8,
          fontsize_col = 8,
          border_color = NA,
          color = colors,
          breaks = BREAKS,
          legend_breaks = BREAKS,
          legend_labels = LABELS,
          fontfamily = "Helvetica"
        )
      } else{
        message(" [artMS currently doesn't support heatmaps of a single set] ")
        #pheatmap(term_groups_selected_w_display, 
        #cluster_cols= FALSE,cluster_rows= FALSE, 
        #cellheight=10, cellwidth=10, scale="none", 
        #filename=gsub('.txt','_heatmap.pdf',out_file), 
        #fontsize=6, fontsize_row=8, fontsize_col=8,
        # border_color=NA, color = colors, 
        # fontfamily="Helvetica")
      }
    } else{
      message(" [Not enough significant terms for this domain] ")
    }
  }
  
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# @title Enrichment analysis of Protein Complexes (based on CORUM database)
#
# @description Enrichment analysis of Protein Complexes
# (based on CORUM database)
# @param mylist (vector) of protein ids
# @param corum (data.frame) The corum database (with the corum format
# and labels)
# @param background (int) Total number of proteins (number) to use as
# background
# @return (data.frame) with the list of protein complexes with a fold change
# larger than 1
# @keywords internal, enrichment, protein, complexes
# .artms_foldComplexEnrichment()
.artms_foldComplexEnrichment <- function(mylist, 
                                         corum, 
                                         background) {
  corum$Num.Uniprot.IDs <-
    vapply(corum$subunits.UniProt.IDs, function(x)
      length(unlist(strsplit(
        as.character(x), ";"
      ))), FUN.VALUE = 0)
  
  # Matches between my list and the protein complexes
  matchThis <- function(cor, mylist) {
    corv <- unlist(strsplit(as.character(cor), ";"))
    length(table(mylist[mylist %in% corv]))
  }
  corum$MyListOverlap <-
    vapply(corum$subunits.UniProt.IDs, function(x)
      matchThis(x, mylist), FUN.VALUE = 0)
  
  # Fold Complex Enrichment of the proteins observed in the list of proteins
  # over the expected. If it is greater than 1, it indicates that the category
  # is overrepresented in the experiment.
  # Conversely, the category is underrepresented if it is less than 1.
  
  # For the human complex enrichment we used this number for the background
  # numberHumanProteosome <- 8522
  # background <- numberHumanProteosome
  # This number is based on this reference:
  # Mol Cell Proteomics. 2012 Mar;11(3):M111.014050.
  # doi: 10.1074/mcp.M111.014050. Epub 2012 Jan 25.
  # Comparative proteomic analysis of eleven common cell lines reveals
  # ubiquitous but varying expression of most proteins.
  # Geiger T1, Wehner A, Schaab C, Cox J, Mann M.
  
  querysize <- length(unique(mylist)) #uploaded proteins
  corum$querysize <- querysize
  corum$expected <- (corum$Num.Uniprot.IDs / background) * querysize
  corum$FCEnrichment <- corum$MyListOverlap / corum$expected
  corum$pvalue <-
    phyper(
      corum$MyListOverlap,
      corum$Num.Uniprot.IDs,
      (background - corum$Num.Uniprot.IDs),
      querysize,
      lower.tail = FALSE
    )
  # Return only complexes with a FC > 1
  toreturn <-
    corum[which(corum$FCEnrichment >= 1 & corum$pvalue < 0.05), ]
  return(toreturn)
}

# CREATE SAMPLE DATA FILES FOR THE PACKAGE----

setwd('~/github/biodavidjm/artMS/')


# CONFIGURATION FILE----

# library(yaml)
artms_config <- yaml.load_file("~/github/biodavidjm/artMS/inst/extdata/artms_config.yaml")
save(artms_config, file = '~/github/biodavidjm/artMS/data/artms_config.RData', compress = TRUE)

# GENERATE RANDOM FILE----
artms_data_randomDF <- data.frame(replicate(10, sample(0:1, 100, rep = TRUE)))
save(artms_data_randomDF, file = 'data/artms_data_randomDF.RData', 
     compress = 'xz')


## CREATE THE OFFICIAL PHGLOBAL COMING WITH THE PACKAGE-----
setwd("~/sourcecode/artms/ph_full/")

evidence_file <- 'evidence.txt'
keys_file <- 'keys.txt'
contrast_file <- 'contrast.txt'

edf <- read.delim(evidence_file,
                  stringsAsFactors = FALSE)

kdf <- read.delim(keys_file,
                  stringsAsFactors = FALSE,
                  check.names = FALSE)


# Select 2 biological replicates
selectedBR <- c("qx006145", "qx006148", "qx006151", "qx006152")
edfnew <- edf[which(edf$Raw.file %in% selectedBR), ]

# Select columns
edfnew <- edfnew[c("Sequence",
                   "Length",
                   "Modifications",
                   "Modified.sequence",
                   "Oxidation..M..Probabilities",
                   "Phospho..STY..Probabilities",
                   "Oxidation..M..Score.Diffs",
                   "Phospho..STY..Score.Diffs",
                   "Oxidation..M.",
                   "Phospho..STY.",
                   "Missed.cleavages",
                   "Proteins",
                   "Leading.proteins",
                   "Leading.razor.protein",
                   "Type",
                   "Raw.file",
                   "MS.MS.m.z",
                   "Charge",
                   "m.z",
                   "Mass",
                   "Resolution",
                   "Mass.error..ppm.",
                   "Mass.error..Da.",
                   "Uncalibrated.mass.error..ppm.",
                   "Uncalibrated.mass.error..Da.",
                   "Calibrated.retention.time",
                   "Retention.time",
                   "Retention.length",
                   "PEP",
                   "MS.MS.count",
                   "MS.MS.scan.number",
                   "Score",
                   "Delta.score",
                   "Intensity",
                   "Reverse",
                   "Potential.contaminant")]

artms_data_ph_keys <- kdf[which(kdf$RawFile %in% selectedBR),]

# And random sampling lines
n <- round(dim(edfnew)[1] / 25)
artms_data_ph_evidence <- edfnew[sample(nrow(edfnew), n),]

# # print out evidence & keys
# write.table(
#   artms_data_ph_evidence,
#   file = "~/sourcecode/artms/ph/artms_data_ph_evidence.txt",
#   quote = FALSE,
#   sep = "\t",
#   row.names = FALSE,
#   col.names = TRUE
# )
# write.table(
#   artms_data_ph_keys,
#   file = "~/sourcecode/artms/ph/artms_data_ph_keys.txt",
#   quote = FALSE,
#   sep = "\t",
#   row.names = FALSE,
#   col.names = TRUE
# )

# artms_data_ph_evidence----
# artms_data_ph_evidence <- read.delim("~/sourcecode/artms/ph/artms_data_ph_evidence.txt", stringsAsFactors = FALSE)
save(artms_data_ph_evidence, 
     file = '~/github/biodavidjm/artMS/data/artms_data_ph_evidence.RData', 
     compress = TRUE)

# artms_data_ph_keys----
# artms_data_ph_keys <- read.delim("~/sourcecode/artms/extdata/artms_data_ph_keys.txt",
#                                  stringsAsFactors = FALSE)
  
save(artms_data_ph_keys, 
     file = '~/github/biodavidjm/artMS/data/artms_data_ph_keys.RData', 
     compress = TRUE )

# artms_data_ph_contrast -----
contrast_file <- "~/sourcecode/artms/ph_full/contrast.txt"
artms_data_ph_contrast <- readLines(contrast_file, warn = FALSE)
save(artms_data_ph_contrast, 
     file = "~/github/biodavidjm/artMS/data/artms_data_ph_contrast.RData")

# artms_data_ph_config ----
artms_data_ph_config <- artms_config

artms_data_ph_config$files$evidence <- ""
artms_data_ph_config$files$keys <- ""
artms_data_ph_config$files$contrasts <- ""
artms_data_ph_config$files$summary <- ""
artms_data_ph_config$files$output <- "quant-test/results.txt"
artms_data_ph_config$qc$basic <- 0
artms_data_ph_config$qc$extended <- 0
artms_data_ph_config$qc$extendedSummary <- 0
artms_data_ph_config$data$filters$modifications <- "PH"

save(artms_data_ph_config, 
     file = "~/github/biodavidjm/artMS/data/artms_data_ph_config.RData", 
     compress = TRUE)

artms_data_ph_config$files$evidence <- artms_data_ph_evidence
artms_data_ph_config$files$keys <- artms_data_ph_keys
artms_data_ph_config$files$contrasts <- artms_data_ph_contrast
artms_data_ph_config$output_extras <- 0
artms_data_ph_config$msstats$profilePlots <- "before, after"

msresults <- artmsQuantification(yaml_config_file = artms_data_ph_config, 
                                 data_object = TRUE,  
                                 display_msstats = FALSE, 
                                 verbose = TRUE, 
                                 printPDF = FALSE, 
                                 printTables = FALSE)
                            
                    

# Results -----

# Run MSstats on the full version (4 biological replicates)-----
setwd("~/sourcecode/artms/ph_full/")
artmsWriteConfigYamlFile(config_file_name = "artms_full_phglobal.yaml")
artmsQuantification(yaml_config_file = "artms_full_phglobal.yaml")

# Load and write out data objects for artMS
artms_data_ph_msstats_modelqc <- read.delim("~/sourcecode/artms/ph_full/phglobal/new2021/results_ModelQC.txt", stringsAsFactors = FALSE)
save(artms_data_ph_msstats_modelqc, 
     file = "~/github/biodavidjm/artMS/data/artms_data_ph_msstats_modelqc.RData",
     compress = TRUE)

artms_data_ph_msstats_results <-read.delim("~/sourcecode/artms/ph_full/phglobal/new2021/results.txt", stringsAsFactors = FALSE)
save(artms_data_ph_msstats_results, 
     file = "~/github/biodavidjm/artMS/data/artms_data_ph_msstats_results.RData",
     compress = TRUE)


# Using the full version to generate the results file

# To generate results first run:
artms_data_ph_config$files$evidence <- artms_data_ph_evidence
artms_data_ph_config$files$keys <- artms_data_ph_keys
artms_data_ph_config$files$contrasts <- artms_data_ph_contrast

artms_data_ph_quantifications <- artmsQuantification(yaml_config_file = artms_data_ph_config, 
                                                     data_object = TRUE,
                                                     printPDF = TRUE,
                                                     display_msstats = TRUE,
                                                     verbose = TRUE)

artms_data_ph_msstats_results <- as.data.frame(artms_data_ph_quantifications$ComparisonResult)
artms_data_ph_msstats_results$Protein <- as.character(artms_data_ph_msstats_results$Protein)
artms_data_ph_msstats_results$Label <- as.character(artms_data_ph_msstats_results$Label)
artms_data_ph_msstats_results$issue <- as.character(artms_data_ph_msstats_results$issue)

artms_data_ph_msstats_modelqc <- as.data.frame(artms_data_ph_quantifications$ModelQC)
artms_data_ph_msstats_modelqc$RUN <- as.integer(artms_data_ph_msstats_modelqc$RUN)
artms_data_ph_msstats_modelqc$Protein <- as.character(artms_data_ph_msstats_modelqc$Protein)
artms_data_ph_msstats_modelqc$originalRUN <- as.integer(artms_data_ph_msstats_modelqc$originalRUN)
artms_data_ph_msstats_modelqc$GROUP <- as.character(artms_data_ph_msstats_modelqc$GROUP)
artms_data_ph_msstats_modelqc$SUBJECT <- as.character(artms_data_ph_msstats_modelqc$SUBJECT)

save(artms_data_ph_msstats_results, 
     file = "~/github/biodavidjm/artMS/data/artms_data_ph_msstats_results2.RData",
     compress = TRUE)

save(artms_data_ph_msstats_modelqc, 
     file = "~/github/biodavidjm/artMS/data/artms_data_ph_msstats_modelqc2.RData",
     compress = TRUE)



# CORUM dataset----
artms_data_corum_mito_database <- read.delim("~/github/biodavidjm/artMS/inst/extdata/20170801_corum_mitoT.txt", 
                                             stringsAsFactors = FALSE)
  
save(artms_data_corum_mito_database,
     file = 'data/artms_data_corum_mito_database.RData',
     compress = TRUE)





# PATHOGENS----
message("--- PATHOGEN IN SAMPLES: TB\n")
artms_data_pathogen_TB <- read.delim('~/Box Sync/db/uniprot/uniprot-tr-myctb_tuberculosis_ATCC35801_TMC10-onlyEntryID.fasta',
                                     header = FALSE,
                                     sep = "\t",
                                     quote = "",
                                     stringsAsFactors = FALSE) # pathogen.ids$Entry, "TB",
  
names(artms_data_pathogen_TB) <- c('Entry')
save(artms_data_pathogen_TB, 
     file = '~/github/biodavidjm/artMS/data/artms_data_pathogen_TB.RData', 
     compress = 'xz')

message("--- PATHOGEN IN SAMPLES: LEGIONELLA PNEUMOPHILA")
artms_data_pathogen_LPN <-
  read.delim(
    '~/Box Sync/db/uniprot/uniprot-legionella-proteome_UP000000609.txt',
    header = TRUE,
    sep = "\t",
    quote = "",
    stringsAsFactors = FALSE
  ) # pathogen.ids$Entry, "Lpn",
artms_data_pathogen_LPN <- artms_data_pathogen_LPN[c('Entry')]

save(artms_data_pathogen_LPN,
     file = '~/github/biodavidjm/artMS/data/artms_data_pathogen_LPN.RData',
     compress = 'xz')


# VIGNETTES
artmsQualityControlEvidenceBasic(evidence_file = artms_data_ph_evidence,
                                 keys_file = artms_data_ph_keys,
                                 prot_exp = "PH")

# ------------------------------------------------------------------------------
# @title Check if an input is a file or a data object
# @description This function is used in order to make it so a user can submit
#  either a path to a data file or a data object in data.frame or data.table
#  form.
# @param input_file (object or data.frame) The filepath/object to be checked.
# @param dont_check_names if TRUE (default) names are not checked, otherwise
# names will be checked
# @param is.evidence (logical) Whether or not the file to be read in is an
# evidence file. This will assign proper classes to the evidence file
# when being read in.
# @return An R data object
# @keywords internal, file, evidence, input
.artms_checkIfFile <- function(input_file, 
                               dont_check_names = TRUE,
                               is.evidence = FALSE) {
  # check if already a data.frame or data.table
  if (is.data.table(input_file)) {
    x <- data.table(input_file)
  } else if (is.data.frame(input_file)) {
    x <- data.frame(input_file)
  } else if (is.vector(input_file)){
    if(length(input_file) == 1){
      if(!file.exists(input_file)){
        stop("The file ", input_file, " does not exist! ")
      }else{
        if(dont_check_names){
          x <- read.delim(input_file, 
                          stringsAsFactors = FALSE)
        }else{
          x <- read.delim(input_file, 
                          stringsAsFactors = FALSE,
                          check.names = FALSE)
        }
        
      }
    }else{
      stop("The input object is not valid")
    }
  }else{
    stop("The input object is not valid")
  }
  
  if(is.evidence){
    message("--- Evidence file ready")
    return(x)
  }else{
    return(x)
  }
  
}

#' @title Check if a given evidencee file was generated by a new version of
#' MaxQuant (v>1)
#' @description MaxQuant introduced changes in the column names and number
#' of columns for the evidence file in version 1 (we think).
#' This function check whether the evidence comes from the latest version
#' of MaxQuant.
#' @param evidence_file the evidence file name
#' @return (logical) `TRUE` if it is a newer version of MaxQuant,
#' `FALSE` otherwise
#' @keywords file, evidence, input, check version
#' @examples
#' artmsIsEvidenceNewVersion(evidence_file = artms_data_ph_evidence)
#' @export
artmsIsEvidenceNewVersion <- function(evidence_file) {
  rawEvidence <- .artms_checkIfFile(input_file = evidence_file, 
                                    is.evidence = TRUE)
  
  if (any(grepl("^Leading.proteins$", names(rawEvidence))) &
      any(grepl("^Leading.razor.protein$", names(rawEvidence)))) {
    return(TRUE)
  } else{
    return(FALSE)
  }
}


# ------------------------------------------------------------------------------
# @title Read the Evidence File
#
# @description Read in a MaxQuant searched Evidence file using data.table.
# This function properly classes each column and so fread doesn't have
# to guess.
# @param evidence_file (char)The filepath to the MaxQuant searched data
# (evidence) file (txt tab delimited file).
# @param verbose (logical) `TRUE` (default) shows function messages
# @return (data.frame) with the evidence file with defining classes
# @keywords internal, MaxQuant, evidence
.artms_read_evidence_file <- function(evidence_file,
                                      verbose = TRUE) {
  if(verbose) message("--- Reading in evidence file... ")
  # read in the first line to get the header names
  cols <- readLines(evidence_file, 1)
  cols <- data.frame(V1 = unlist(strsplit(cols, "\t")), 
                     stringsAsFactors = FALSE)
  
  cols$idx <- seq_len(dim(cols)[1])
  
  # get data frame of pre-recorded column names and their respective classes
  col.classes <-
    as.data.frame(matrix(
      c(
        "Sequence",
        "character",
        "Length",
        "integer",
        "Modifications",
        "character",
        "Modified sequence",
        "character",
        "Oxidation (M) Probabilities",
        "character",
        "Oxidation (M) Score Diffs",
        "character",
        "Acetyl (Protein N-term)",
        "integer",
        "Oxidation (M)",
        "integer",
        "Missed cleavages",
        "integer",
        "Proteins",
        "character",
        "Leading proteins",
        "character",
        "Leading Proteins",
        "character",
        "Leading razor protein",
        "character",
        "Leading Razor Protein",
        "character",
        "Gene names",
        "character",
        "Gene Names",
        "character",
        "Protein names",
        "character",
        "Protein Names",
        "character",
        "Type",
        "character",
        "Raw file",
        "character",
        "Experiment",
        "character",
        "MS/MS m/z",
        "numeric",
        "Charge",
        "integer",
        "m/z",
        "numeric",
        "Mass",
        "numeric",
        "Resolution",
        "numeric",
        "Uncalibrated - Calibrated m/z [ppm]",
        "numeric",
        "Uncalibrated - Calibrated m/z [Da]",
        "numeric",
        "Mass Error [ppm]",
        "numeric",
        "Mass error [ppm]",
        "numeric",
        "Mass Error [Da]",
        "numeric",
        "Mass error [Da]",
        "numeric",
        "Uncalibrated Mass Error [ppm]",
        "numeric",
        "Uncalibrated mass error [ppm]",
        "numeric",
        "Uncalibrated Mass Error [Da]",
        "numeric",
        "Uncalibrated mass error [Da]",
        "numeric",
        "Max intensity m/z 0",
        "numeric",
        "Retention time",
        "numeric",
        "Retention length",
        "numeric",
        "Calibrated retention time",
        "numeric",
        "Calibrated retention time start",
        "numeric",
        "Calibrated retention time finish",
        "numeric",
        "Retention time calibration",
        "numeric",
        "Match time difference",
        "numeric",
        "Match m/z difference",
        "numeric",
        "Match q-value",
        "numeric",
        "Match score",
        "numeric",
        "Number of data points",
        "integer",
        "Number of scans",
        "integer",
        "Number of isotopic peaks",
        "integer",
        "PIF",
        "numeric",
        "Fraction of total spectrum",
        "numeric",
        "Base peak fraction",
        "numeric",
        "PEP",
        "numeric",
        "MS/MS Count",
        "integer",
        "MS/MS count",
        "integer",
        "MS/MS Scan Number",
        "integer",
        "MS/MS scan number",
        "integer",
        "Score",
        "numeric",
        "Delta score",
        "numeric",
        "Combinatorics",
        "integer",
        "Intensity",
        "numeric",
        "Reverse",
        "character",
        "Potential contaminant",
        "character",
        "id",
        "integer",
        "Protein group IDs",
        "character",
        "Peptide ID",
        "integer",
        "Mod. peptide ID",
        "integer",
        "MS/MS IDs",
        "character",
        "Best MS/MS",
        "integer",
        "AIF MS/MS IDs",
        "logical",
        "Oxidation (M) site IDs",
        "character",
        "Acetyl (K) Probabilities",
        "character",
        "GlyGly (K) Probabilities",
        "character",
        "Phospho (STY) Probabilities",
        "Character",
        "Acetyl (K) Score Diffs",
        "character",
        "GlyGly (K) Score Diffs",
        "character",
        "Phospho (STY) Score Diffs",
        "character",
        "Acetyl (K)",
        "integer",
        "GlyGly (K)",
        "integer",
        "Phospho (STY)",
        "integer",
        "Acetyl (K) site IDs",
        "character",
        "GlyGly (K) site IDs",
        "character",
        "Phospho (STY) site IDs",
        "character",
        "Contaminant",
        "character",
        "Fraction",
        "integer"
      ),
      ncol = 2,
      byrow = TRUE
    ), stringsAsFactors = FALSE)
  # merge the classes to the columns
  cols.matched = merge(cols, col.classes, by = "V1", all.x = TRUE)
  # re-order things to match the initial order
  cols.matched <- cols.matched[order(cols.matched$idx),]
  
  # Stop if there is an issue
  if (length(which(is.na(cols.matched$V2))) > 0) {
    stop("The evidence file contains columns that are not recognize.
        If they are new columns, add them to 'col.classes' of
        function <.artms_read_evidence_file>",
        paste(cols.matched$V1[which(is.na(cols.matched$V2))],"\n")
    )
  }
  
  # read in the evidence file with their classes
  x <- fread(evidence_file, 
             integer64 = "double",
             colClasses = cols.matched$V2)
  # make sure all the column names are the same as supporting analytical
  # functions are looking for
  x <- .artms_unifyColumnNames(x)
  return(x)
}

# ------------------------------------------------------------------------------
# @title Unify column names
#
# @description The newer versions of MaxQuant change the letter case in the
# column names. This is to help keep things unified still.
# @param x (data.frame) Loaded evidence file.
# @return (data.frame) with the evidence file with defining classes and
# unified columns
# @keywords internal, MaxQuant, evidence
.artms_unifyColumnNames <- function(x) {
  names(x) <-
    gsub("^Mass error \\[ppm\\]$", "Mass Error \\[ppm\\]", names(x))
  names(x) <-
    gsub("^Mass error \\[Da\\]$", "Mass Error \\[Da\\]", names(x))
  names(x) <- gsub("^Protein Names$", "Protein names", names(x))
  names(x) <-
    gsub("^Leading Razor Protein$", "Leading razor protein", names(x))
  names(x) <-
    gsub("^Leading Proteins$", "Leading proteins", names(x))
  names(x) <- gsub("^Gene Names$", "Gene names", names(x))
  names(x) <- gsub("^MS/MS count$", "MS/MS Count", names(x))
  names(x) <-
    gsub("^MS/MS scan number$", "MS/MS Scan Number", names(x))
  names(x) <-
    gsub("^Uncalibrated mass error \\[Da\\]$",
         "Uncalibrated Mass Error \\[Da\\]",
         names(x))
  names(x) <-
    gsub("^Uncalibrated mass error \\[ppm\\]$",
         "Uncalibrated Mass Error \\[ppm\\]",
         names(x))
  
  return(x)
}

context("Check Evidence file")

test_that("checkIfFile",{
  # Data.frame to test
  L3 <- LETTERS[1:3]
  fac <- sample(L3, 10, replace = TRUE)
  d <- data.frame(x = 1, y = 1:10, fac = fac)
  
  expect_true(is.data.frame(.artms_checkIfFile(d)))
  expect_error(.artms_checkIfFile(L3))
})

test_that("Check artmsQuantification() output", {
  # Get data ready
  artms_data_ph_config$files$evidence <- artms_data_ph_evidence
  artms_data_ph_config$files$keys <- artms_data_ph_keys
  artms_data_ph_config$files$contrasts <- artms_data_ph_contrast
  artms_data_ph_config$output_extras <- 0
  msresults <- artmsQuantification(yaml_config_file = artms_data_ph_config, 
                                   data_object = TRUE,  
                                   display_msstats = FALSE, 
                                   verbose = TRUE, 
                                   printPDF = FALSE, 
                                   printTables = FALSE)
  # Test
  expect_equal( length(msresults), 5 )
  expect_equal(dim(msresults$ComparisonResult)[1], 2)
  expect_equal(dim(msresults$ComparisonResult)[2], 11)
  expect_equal(dim(msresults$ModelQC)[1], 5)
  expect_equal(dim(msresults$ModelQC)[2], 13)
  expect_equal(length(msresults$FittedModel[[1]]), 13)
})

#' @rawNamespace import(AnnotationDbi, except = c(head))
#' @import bit64
#' @import circlize
#' @importFrom cluster pam
#' @importFrom corrplot corrplot
#' @importFrom dplyr mutate desc count arrange desc everything one_of
#' @import data.table
#' @import getopt
#' @import ggdendro
#' @import ggplot2
#' @importFrom gplots heatmap.2
#' @import ggrepel
#' @importFrom graphics pairs plot barplot hist lines par panel.smooth rect strwidth text
#' @importFrom grDevices colorRampPalette dev.off pdf
#' @import grid
#' @import limma
#' @import MSstats
#' @import openxlsx
#' @import org.Hs.eg.db
#' @import pheatmap
#' @rawNamespace import(plotly, except = c(last_plot, mutate, arrange, 
#' rename, summarise, select, add_heatmap))
#' @importFrom plyr ddply summarise rename
#' @import RColorBrewer
#' @rawNamespace import(seqinr, except = c(zscore, count, a))
#' @importFrom stats aggregate as.dendrogram cor dist fisher.test hclust prcomp quantile sd
#' kmeans median order.dendrogram phyper as.dist complete.cases qt IQR
#' @import stringr
#' @importFrom tidyr unnest pivot_wider pivot_longer
#' @import UpSetR
#' @importFrom utils combn read.delim sessionInfo write.table setTxtProgressBar 
#' txtProgressBar head globalVariables capture.output
#' @import VennDiagram
#' @import yaml

utils::globalVariables(
  c(
    "...",
    "..count..",
    "AbMean",
    "Abundance",
    "ABUNDANCE",
    "adj.pvalue",
    "artms_config",
    "artms_data_corum_mito_database",
    "artms_data_pathogen_LPN",
    "artms_data_pathogen_TB",
    "Bait",
    "bin.all",
    "bin.condition",
    "BiorepCount",
    "BioReplicate",
    "bioreplicate",
    "category",
    "charge",
    "cluster",
    "Comparison",
    "ComplexName",
    "CondCount",
    "condition",
    "Condition",
    "Contaminant",
    "ENTREZID",
    "EntrezID",
    "evidencekeys",
    "experiment",
    "Experiment",
    "FEATURE",
    "fraction",
    "Fraction",
    "fxDx",
    "FxOverSamp",
    "Gene",
    "GENENAME",
    "GROUP",
    "iLog2FC",
    "imputedDFext",
    "installed.packages",
    "IntDetection",
    "intensity",
    "Intensity",
    "Ions",
    "Ions.mean",
    "Ions.sem",
    "iPvalue",
    "isotope.patterns",
    "isotope.patterns.sequenced..z.1.",
    "IsotopeLabelType",
    "isPtm",
    "Label",
    "log2FC",
    "log2fc_file",
    "m.z",
    "mc",
    "missed.cleavages",
    "MODIFICATION",
    "Modifications",
    "modified.sequence",
    "ms",
    "ms.ms",
    "ms.ms.count",
    "ms.ms.identified....",
    "MSMS.counts",
    "N",
    "nsamples",
    "num.IsotopePatterns.mean",
    "num.IsotopePatterns.sem",
    "num.IsotopePatternsSeq.mean",
    "num.IsotopePatternsSeq.sem",
    "num.MS1.mean",
    "num.MS1.sem",
    "num.MS2.mean",
    "num.MS2.sem",
    "output_dir",
    "oxidation..m.",
    "pathogen.ids",
    "PC1",
    "PC2",
    "pct.MS2Id.mean",
    "pct.MS2Id.sem",
    "pct.OxM",
    "pCV",
    "pDX",
    "PeakWidth.mean",
    "pearson",
    "Peptides",
    "Peptides.mean",
    "Peptides.sem",
    "potential.contaminant",
    "Prey",
    "prot_names",
    "Protein",
    "PROTEIN",
    "ProteinName",
    "proteins",
    "Proteins",
    "Proteins.mean",
    "Proteins.sem",
    "PSMs",
    "PSMs.mean",
    "PSMs.sem",
    "ptm_site",
    "PTMone",
    "PTMsite",
    "pvalue",
    "quantile",
    "RawFile",
    "rect",
    "res_index",
    "retention.length",
    "reverse",
    "sample_name",
    "Species",
    "species",
    "strwidth",
    "SUBJECT",
    "sumInt",
    "SYMBOL",
    "text",
    "TR",
    "tr1",
    "tr2",
    "type",
    "uncalibrated.mass.error..ppm.",
    "uniprot_ac",
    "uniprot_id",
    "value",
    "variable",
    "ymax",
    "ymin"
  )
)


# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' @title Relative quantification using MSstats
#'
#' @description Relative quantification using MSstats including:
#' - plots
#' - quantifications (log2fc, pvalues, etc)
#' - normalized abundance values
#' @param yaml_config_file (char, required) The yaml file name and location
#' @param data_object (logical) flag to indicate whether the configuration file
#' is a string to a file that should be opened or config object (yaml). 
#' Default is `FALSE`. Choose `TRUE` if `yaml_config_file` is a yaml object
#' @param printPDF (logical) if `TRUE` (default), prints out pdf
#' @param printTables (logical) `TRUE` (default) print results tables
#' @param display_msstats (logical) if `TRUE`, prints MSstats outputs (default is `FALSE`)
#' @param return_results_object (logical) Default is `FALSE`. 
#' If `TRUE`, it returns a list of data frames with MSstats results, including:
#' - `comparisonResult`: comparison results
#' - `ModelQC`
#' - `FittedModel`: fit model details
#' - `power`: power calculations
#' - `sample_size`: sample size estimations
#' @param verbose (logical) `TRUE` (default) shows function messages
#' @return The relative quantification of the conditions and comparisons
#' specified in the keys/contrast file resulting from running MSstats, in
#' addition to quality control plots (if selected)
#' @keywords main, driver, function
#' @examples
#' # Recommended
#' # artmsQuantification(yaml_config_file = "your-config-file.yaml")
#' 
#' # Example to test this function using the example dataset available in artMS
#' # Step 1: Add evidence, keys, and contrast to configuration object
#' artms_data_ph_config$files$evidence <- artms_data_ph_evidence
#' artms_data_ph_config$files$keys <- artms_data_ph_keys
#' artms_data_ph_config$files$contrasts <- artms_data_ph_contrast
#' 
#' # Step 2: Run the quantification step
#' quant_results <- artmsQuantification(yaml_config_file = artms_data_ph_config, 
#'                                      data_object = TRUE, 
#'                                      display_msstats = FALSE,
#'                                      printPDF = FALSE,
#'                                      printTables = FALSE)
#' # Check the list of data frames "quant_results". Nothing should be printed out. 
#' @export
artmsQuantification <- function(yaml_config_file,
                                data_object = FALSE,
                                printPDF = TRUE,
                                printTables = TRUE,
                                display_msstats = FALSE,
                                return_results_object = FALSE,
                                verbose = TRUE) {
  
  # Check if the yaml file is already open first
  if(data_object){
    config <- yaml_config_file
  }else{
    config <- yaml.load_file(yaml_config_file)
  }
  
  # CHECK POINT: it's a file or an object
  if(!data_object){
    if(!file.exists(config$files$contrasts)){
      stop("the file ", config$files$contrasts, " does not exist! ")
    }
    
    if(!file.exists(config$files$evidence)){
      stop("the file ", config$files$evidence, " does not exist! ")
    }
    
    if(!file.exists(config$files$keys)){
      stop("the file ", config$files$keys, " does not exist! ")
    }
  }

  if(!(grepl("\\.txt$", config$files$output))){
    stop("the file ", config$files$output, " must have extension .txt (tab delimited)" )
  }
  
  # LET'S HELP THE DISTRACTED USER
  if (!(is.null(config$data$filters$modification))) {
    config$data$filters$modification <- toupper(config$data$filters$modification)
  }
  
  if(isFALSE(printTables) & isFALSE(printPDF) & isFALSE(return_results_object)){
    return_results_object = TRUE
  }
  
  if(printTables){
    session <- sessionInfo()
    sink("artms_sessionInfo_quantification.log")
    print(session)
    sink()
  }
  
  # Quality Control
  if (config$qc$basic) {
    artmsQualityControlEvidenceBasic(
      evidence_file = config$files$evidence,
      keys_file = config$files$keys,
      prot_exp = toupper(config$data$filters$modifications),
      isSILAC = config$data$silac$enabled)
  }
  
  if (config$qc$extended) {
    artmsQualityControlEvidenceExtended(evidence_file = config$files$evidence,
                                        keys_file = config$files$keys,
                                        isSILAC = config$data$silac$enabled)
  }
  
  if(!is.null(config$qc$extendedSummary)){
    if (config$qc$extendedSummary) {
      if(!is.null(config$files$summary)){
        if(file.exists(config$files$summary)){
          artmsQualityControlSummaryExtended(summary_file = config$files$summary,
                                              keys_file = config$files$keys)
        }else{
          message("\n\tWARNING:  The file ", config$files$summary, " does not exist! ")
          message("\tQC report based on summary.txt won't be performed\n")
        }
      }else{
        message("\n\tWARNING: QC Summary-based selected but file path/name not provided. Skipped\n")
      }
    }
  }
  
  if(verbose){
    message("--------------------------------------------")
    message("artMS: Relative Quantification using MSstats")
    message("--------------------------------------------")
    message(">> Reading the configuration file")
  }
  
  # Create output folder
  output_full_path <- config$files$output
  output_dir <- dirname(output_full_path)
  
  # message("output_full_path: ", output_full_path)
  # message("output_dir: ", output_dir) 
  
  # create output directory if it doesn't exist-----
  if(printTables | printPDF){
    if (!dir.exists(output_dir)) {
      if(verbose) message("-- Folder: [", output_dir, "] created" )
      dir.create(output_dir, recursive = TRUE)
    }
  }

  # process MaxQuant data, link with keys, and convert for MSStats format-----
  if ( config$data$enabled ) {
    if(verbose) message(">> LOADING DATA ")
    ## Found more bugs in fread (issue submitted to data.table on github by
    ## JVD but it was closed with the excuse that 'is was not reproducible'
    ## although he provided examples)
    ## Not worth the compromise in data integrity just to save time
    ## reading in data
    
    # CHECKING FOR SILAC EXPERIMENT
    if (!is.null(config$data$silac$enabled)) {
      if (config$data$silac$enabled) {
        output <- gsub(".txt", "-silac.txt", config$files$evidence)
        x <- artmsSILACtoLong(config$files$evidence,
                                  output,
                                  verbose = verbose)
      } else{
        x <- .artms_checkIfFile(config$files$evidence)
        x <- .artms_checkRawFileColumnName(x)
      }
    } else{
      x <- .artms_checkIfFile(config$files$evidence)
      x <- .artms_checkRawFileColumnName(x)
    }
    
    x <- data.table(x)
    
    keys <- .artms_checkIfFile(config$files$keys)
    keys <- .artms_checkRawFileColumnName(keys)
    
    keys <- data.table(keys)
    
    # Let's make sure that the contrast file is right
    if (config$msstats$enabled) {
      # Read in contrast file
      contrasts <- .artms_writeContrast(
        contrast_file = config$files$contrasts, 
        all_conditions = unique(as.character(keys$Condition)))
    }
    
    if (!'IsotopeLabelType' %in% colnames(x)) {
      x[, IsotopeLabelType := 'L']
    }
    
    # HACK FOR SILAC DATA
    # 1. Make a function of this part
    # 2. Use it in Quality controls.
    if (!is.null(config$data$silac$enabled)) {
      if (config$data$silac$enabled) {
        x <- .artmsMergeSilacEvidenceKeys(evisilac = x,
                                          keysilac = keys)
      } else{
        x <- artmsMergeEvidenceAndKeys(x, 
                                       keys, 
                                       by = c('RawFile', 'IsotopeLabelType'),
                                       verbose = verbose)
      }
    } else{
      x <- artmsMergeEvidenceAndKeys(x, 
                                     keys, 
                                     by = c('RawFile', 'IsotopeLabelType'),
                                     verbose = verbose)
    }
    
    ## fix for weird converted values from fread
    if(verbose) message(">> CONVERT Intensity values < 1 to NA")
    x[Intensity < 1, ]$Intensity <- NA
    
    ## FILTERING : handles Protein Groups and Modifications-----
    if (config$data$filters$enabled){
      data_f <- .artms_filterData(x = x, 
                                  config = config, 
                                  verbose = verbose)
    }else{
      data_f <- x
    }
    
    ## FORMATTING IN WIDE FORMAT TO CREATE HEATMAPS
    
    data_w <- .artms_castMaxQToWidePTM(data_f)
    
    ## HEATMAPS
    if ( all(!is.null(config$data$sample_plots) & config$data$sample_plots & printPDF) ) {
      keys_in_data <- keys[keys$RawFile %in% unique(x$RawFile), ]
      .artms_sampleCorrelationHeatmap(data_w = data_w,
                                      keys = keys_in_data,
                                      config = config)
      .artms_samplePeptideBarplot(data_f, config)
    }
    
  }else{
    if(verbose) 
      message("-- Data not selected (will require to load evidence-mss.txt file)")
  }
  
  ## MSSTATS
  if (config$msstats$enabled) {
    
    # Load the keys files
    keys <- .artms_checkIfFile(config$files$keys)
    keys <- .artms_checkRawFileColumnName(keys)
    # keys <- data.table(keys)

    # Read in contrast file
    contrasts <- .artms_writeContrast(contrast_file = config$files$contrasts, 
                                      all_conditions= unique(as.character(keys$Condition)))
      
    selectedConditions <- as.character(colnames(contrasts))
    
    if ( is.null(config$msstats$msstats_input) ) {
      # Check point to prevent MSstats crashed in the number of conditions
      # in the comparisons is not the same than the one in the keys file
      if(!all(unique(data_f$Condition) %in% selectedConditions)){
        if(verbose) message(" (!) WARNING! the number of unique conditions in evidence file are different from the conditions in contrast!
                                  All the extra conditions will be deleted from the evidence file")
        data_f <- data_f[data_f$Condition %in% selectedConditions,]
      }
      dmss <- .artms_getMSstatsFormat(data_f = data_f,
                                      output_name = config$files$evidence,
                                      data_object = data_object,
                                      printTables = printTables,
                                      verbose = verbose)
    } else {
      if(verbose) message(sprintf("\t+ READING PREPROCESSED FILE: %s ",
                                  config$msstats$msstats_input))
      dmss <- read.delim(config$msstats$msstats_input,
                         stringsAsFactors = FALSE,
                         sep = '\t')
    }
    
    if(verbose) message(">> RUNNING MSstats (it usually takes a 'long' time: please, be patient)")

    if(display_msstats){
      
      suppressWarnings(results <- .artms_runMSstats(dmss = dmss, 
                                                    contrasts = contrasts, 
                                                    config = config,
                                                    verbose = verbose))
    }else{
      if(verbose) message("\t(MSstats messages are turned off. Select <display_msstats = TRUE> to activate MSstats outputs)")
      capture.output( 
        suppressMessages(
          suppressWarnings(results <- .artms_runMSstats(dmss = dmss,
                                                        contrasts = contrasts, 
                                                        config = config,
                                                        printTables = printTables,
                                                        printPDF = printPDF,
                                                        verbose = verbose)
                           )
          )
        )
    }
    
    if(verbose) message(">> MSstats done")
    
    if(data_object){
      return(results)
    }else{
      if(return_results_object){
        return(results)
      }
    }
    
  } else{
    if(verbose) message("\t+ MSstats not selected")
  }
  
  ## ANNOTATING RESULT FILE
  if ( all(config$output_extras$enabled & printPDF) ) {
    if (!config$msstats$enabled){
      message("-- MSstats was not enabled: <output_extras> cannot be done!")
    }else{
      .artms_writeExtras(results = results$ComparisonResult, 
                         config = config)
    }
  }
  if(verbose) message(">> ANALYSIS COMPLETED")
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' @title Write out a template file of the artMS configuration file (yaml)
#' 
#' @description Creates a template file of the artMS configuration file, which
#' is required to run `artmsQuantification`. Check `?artms_config` and the 
#' vignettes to find out more about the details of the structure of the file
#' and how to fill it up
#' @param config_file_name (char) The name for the configuration file. It must
#' have a `.yaml` extension. If `NULL`, it returns the config as a yaml object
#' @param overwrite (logical) Default FALSE
#' @param verbose (logical) `TRUE` (default) shows function messages
#' @return A file (or yaml data object) of the artMS configuration file
#' @keywords config, yaml
#' @examples 
#' config_empty <- artmsWriteConfigYamlFile(config_file_name = NULL)
#' @export
artmsWriteConfigYamlFile <- function(config_file_name = "artms_config_file.yaml",
                                     overwrite = FALSE, 
                                     verbose = TRUE){
  
  if(!is.null(config_file_name)){
    if(grepl("\\.yaml", config_file_name)){
      if(overwrite){
        if(file.exists(config_file_name)){
          if(verbose) message("- Overwriting the configuration file")
          write_yaml(x = artms_config, file = config_file_name )
        }
      }else{
        if(file.exists(config_file_name)){
          stop("File ", config_file_name, " already exits. 
               Are you sure you want to overwrite it? 
               if so, select `overwrite = TRUE` ")
        }else{
          write_yaml(x = artms_config, file = config_file_name )
          if(verbose) message(">> File ", config_file_name," is out ")
        }
      }
    }else{
      stop("The <config_file_name> must have the extension .yaml")
    }
  }else{
    return(artms_config)
  }
}

# ------------------------------------------------------------------------------
# @title Run MSstats
# @description Run MSstats giving a processed evidence file (MSstats format)
# and a contrast file. It also generates a series of summary plots before,
# and after normalization.
# @param dmss (data.frame) Formatted and filtered evidence file 
# (MSstats format)
# @param contrasts (data.frame) The contrast data.frame in MSstats format
# @param config (yaml object) the configation (imported yaml) object
# @param printPDF (logical) if `TRUE` (default) prints pdf files
# @param printTables (logical) `TRUE` (default) prints results tables
# @param verbose (logical) `TRUE` (default) shows function messages
# @return It generates several output files
# - If selected `before` and/or `after`, the `ProfilePlot` and `QCPlot` plots
# by the MSstats `dataProcessPlots` function are generated
# (in `.pdf` format)
# - Text file output of `quantification()` (`-mss-sampleQuant.txt`)
# - Text file output of `quantification(type="Group")` (`-mss-groupQuant.txt`)
# - MSstats `ProcessedData` normalized results (`-mss-normalized.txt`)
# - MSstats `ComparisonResult` results
# - MSstats `ModelQC` results
# - MSstats `designSampleSize` sample size
# - MSstats `designSampleSize` power experiment
# @keywords internal, run, MSstats, contrast, intensity, plots, QC
.artms_runMSstats <- function(dmss, 
                              contrasts, 
                              config,
                              printPDF = TRUE,
                              printTables = TRUE,
                              verbose = TRUE) {
  
  # Check configuration parameters for msstats-----
  config <- .artms_provide_msstats_config_miss_parameters(config = config, 
                                                          verbose = verbose)

  # plot the data BEFORE normalization-----
  if(printPDF){
    if (grepl('before', config$msstats$profilePlots)) {
      if(verbose) message("-- QC PLOT: before")
      mssquant <- dataProcess(
        raw = dmss,
        logTrans = config$msstats$logTrans,
        normalization = FALSE,
        nameStandards = NULL,
        featureSubset = config$msstats$feature_subset,
        remove_uninformative_feature_outlier = config$msstats$remove_uninformative_feature_outlier,
        min_feature_count = config$msstats$min_feature_count,
        n_top_feature = config$msstats$n_top_feature,
        summaryMethod = config$msstats$summaryMethod,
        equalFeatureVar = config$msstats$equalFeatureVar,
        censoredInt = config$msstats$censoredInt,
        MBimpute = config$msstats$MBimpute,
        remove50missing = config$msstats$remove50missing,
        fix_missing = unlist(config$msstats$fix_missing),
        maxQuantileforCensored = config$msstats$maxQuantileforCensored,
        use_log_file = config$msstats$use_log_file,
        append = config$msstats$append,
        verbose = verbose,
        log_file_path = unlist(config$msstats$log_file_path)
      )
      dataProcessPlots(
        data = mssquant,
        type = "ProfilePlot",
        featureName = "Peptide",
        address = gsub('.txt', '-before', config$files$output)
      )
      dataProcessPlots(
        data = mssquant,
        type = "QCPlot",
        address = gsub('.txt', '-before', config$files$output)
      )
    }
  } #if print pdf
  
  # Normalization------
  
  if (!is.null(config$msstats$normalization_reference) &
      config$msstats$normalization_method == 'globalStandards') {
    # if globalStandars is selected, must have a reference protein(s)
    normalization_refs = unlist(lapply(
      strsplit(config$msstats$normalization_reference, split = ','),
      FUN = .artms_trim
    ))
    mssquant <- dataProcess(
      raw = dmss,
      logTrans = config$msstats$logTrans,
      normalization = config$msstats$normalization_method,
      nameStandards = normalization_refs,
      featureSubset = config$msstats$feature_subset,
      remove_uninformative_feature_outlier = config$msstats$remove_uninformative_feature_outlier,
      min_feature_count = config$msstats$min_feature_count,
      n_top_feature = config$msstats$n_top_feature,
      summaryMethod = config$msstats$summaryMethod,
      equalFeatureVar = config$msstats$equalFeatureVar,
      censoredInt = config$msstats$censoredInt,
      MBimpute = config$msstats$MBimpute,
      remove50missing = config$msstats$remove50missing,
      fix_missing = unlist(config$msstats$fix_missing),
      maxQuantileforCensored = config$msstats$maxQuantileforCensored,
      use_log_file = config$msstats$use_log_file,
      append = config$msstats$append,
      verbose = verbose,
      log_file_path = unlist(config$msstats$log_file_path)
    )
  } else{
    if(verbose) message(
      sprintf(
        '-- Normalization method: %s',
        config$msstats$normalization_method
      )
    )
  
    mssquant = dataProcess(
      raw = dmss,
      logTrans = config$msstats$logTrans,
      normalization = config$msstats$normalization_method,
      nameStandards = NULL,
      featureSubset = config$msstats$feature_subset,
      remove_uninformative_feature_outlier = config$msstats$remove_uninformative_feature_outlier,
      min_feature_count = config$msstats$min_feature_count,
      n_top_feature = config$msstats$n_top_feature,
      summaryMethod = config$msstats$summaryMethod,
      equalFeatureVar = config$msstats$equalFeatureVar,
      censoredInt = config$msstats$censoredInt,
      MBimpute = config$msstats$MBimpute,
      remove50missing = config$msstats$remove50missing,
      fix_missing = unlist(config$msstats$fix_missing),
      maxQuantileforCensored = config$msstats$maxQuantileforCensored,
      use_log_file = config$msstats$use_log_file,
      append = config$msstats$append,
      verbose = verbose,
      log_file_path = unlist(config$msstats$log_file_path)
    )
  }
  
  # plot the data AFTER normalization-----
  if(printPDF){
    if (grepl('after', config$msstats$profilePlots)) {
      if(verbose) message("-- QC PLOT: after")
      dataProcessPlots(
        data = mssquant,
        type = "ProfilePlot",
        featureName = "Peptide",
        address = gsub('.txt', '-after', config$files$output)
      )
      dataProcessPlots(
        data = mssquant,
        type = "QCPlot",
        address = gsub('.txt', '-after', config$files$output)
      )
    }
  }
  
  if (!all(levels(mssquant$ProteinLevelData$GROUP) == colnames(contrasts))) {
    stop(
      sprintf(
        '\tERROR IN CONTRAST COMPARISON: GROUP LEVELS DIFFERENT FROM 
        CONTRASTS FILE \tGROUP LEVELS\t%s\n\tCONTRASTS FILE\t%s ',
        paste(levels(mssquant$GROUP_ORIGINAL), collapse = ','),
        paste(colnames(contrasts), collapse = ',')
      )
    )
  }
  
  # protein sample/group quantification----
  if(printTables){
    write.table(
      quantification(mssquant, type = "Sample"),
      file = gsub('.txt', '-mss-sampleQuant.txt', config$files$output),
      eol = "\n",
      sep = "\t",
      quote = FALSE,
      row.names = FALSE,
      col.names = TRUE
    )
    write.table(
      quantification(mssquant, type = "Group"),
      file = gsub('.txt', '-mss-groupQuant.txt', config$files$output),
      eol = "\n",
      sep = "\t",
      quote = FALSE,
      row.names = FALSE,
      col.names = TRUE
    )
    
    if(verbose) message(sprintf('-- FITTING CONTRASTS:\t%s\n',paste(rownames(contrasts), collapse = ',')))
    
    write.table(
      mssquant$FeatureLevelData,
      file = gsub('.txt', '-mss-FeatureLevelData.txt', config$files$output),
      eol = "\n",
      sep = "\t",
      quote = FALSE,
      row.names = FALSE,
      col.names = TRUE
    )
    
    write.table(
      mssquant$ProteinLevelData,
      file = gsub('.txt', '-mss-ProteinLevelData.txt', config$files$output),
      eol = "\n",
      sep = "\t",
      quote = FALSE,
      row.names = FALSE,
      col.names = TRUE
    )
  }
  
  if(verbose) message("-- GROUP COMPARISON")
  results <- groupComparison(data = mssquant, 
                             contrast.matrix = contrasts, 
                             use_log_file = FALSE)
  
  if(printTables){
    write.table(
      results$ComparisonResult,
      file = config$files$output,
      eol = "\n",
      sep = "\t",
      quote = FALSE,
      row.names = FALSE,
      col.names = TRUE
    )
    write.table(
      results$ModelQC,
      file = gsub(".txt", "_ModelQC.txt", config$files$output),
      eol = "\n",
      sep = "\t",
      quote = FALSE,
      row.names = FALSE,
      col.names = TRUE
    )
  }

  if(verbose) message("-- MSstats FINISHED! ")
  
  #(1) Minimal number of biological replicates per condition-----
  if(verbose) message(">> CALCULATING SAMPLE SIZE FOR FUTURE EXPERIMENTS ")
  results.ss1 <- designSampleSize(data = results$FittedModel,
                                  numSample = TRUE,
                                  desiredFC = c(0.58, 2),
                                  FDR = 0.05,
                                  power = 0.95, 
                                  use_log_file = FALSE)
    
    
  results.ss2 <- designSampleSize(data = results$FittedModel,
                                  numSample = TRUE,
                                  desiredFC = c(0.58, 2),
                                  FDR = 0.05,
                                  power = 0.9, 
                                  use_log_file = FALSE)
  
  results.ss3 <- designSampleSize(data = results$FittedModel,
                                  numSample = TRUE,
                                  desiredFC = c(0.58, 2),
                                  FDR = 0.05,
                                  power = 0.8, 
                                  use_log_file = FALSE)
    
    
  results.ss <- rbind(results.ss1, results.ss2, results.ss3)

  #(2) Power calculation-----
  if(verbose) message(">> CALCULATING POWER OF EXPERIMENT")
  results.power1 <- designSampleSize(data = results$FittedModel,
                                     numSample = 2,
                                     desiredFC = c(0.58, 2),
                                     FDR = 0.05,
                                     power = TRUE, 
                                     use_log_file = FALSE)
  results.power2 <- designSampleSize(data = results$FittedModel,
                                     numSample = 3,
                                     desiredFC = c(0.58, 2),
                                     FDR = 0.05,
                                     power = TRUE, 
                                     use_log_file = FALSE)
  results.power3 <- designSampleSize(data = results$FittedModel,
                                     numSample = 4,
                                     desiredFC = c(0.58, 2),
                                     FDR = 0.05,
                                     power = TRUE, 
                                     use_log_file = FALSE)
  results.power4 <- designSampleSize(data = results$FittedModel,
                                     numSample = 5,
                                     desiredFC = c(0.58, 2),
                                     FDR = 0.05,
                                     power = TRUE, 
                                     use_log_file = FALSE)

  results.power <- rbind(results.power1, results.power2, results.power3, results.power4)
  
  if(printTables){
    write.table(
      results.ss,
      file = gsub(".txt", "_sampleSize.txt", config$files$output),
      eol = "\n",
      sep = "\t",
      quote = FALSE,
      row.names = FALSE,
      col.names = TRUE
    )
    write.table(
      results.power,
      file = gsub(".txt", "_experimentPower.txt", config$files$output),
      eol = "\n",
      sep = "\t",
      quote = FALSE,
      row.names = FALSE,
      col.names = TRUE
    )
  }

  results$power <- results.power
  results$sample_size <- results.ss
  
  return(results)
}

#' @title Converts the Protein ID column of the evidence 
#' file selected by the user to mod-site-specific notation: 
#' `ProteinID` to `ProteinID_AAnumber` notation
#'
#' @description It enables the modified-peptide specific quantification by
#' converting the Protein column of the evidence file selected by the user  
#' to an `ProteinID_AAnumber`notation. 
#' In this way, each of the modified peptides can be quantified
#' independently across conditions. 
#' 
#' !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
#' 
#' WARNING: we have detected a version of MaxQuant (>1.6.3.0) outputs a`
#' "Modified sequence" column of the evidence file that has two important 
#' changes for the annotation of phosphorylation:
#' - Uses `p` instead of `(ph)`
#' - The modified residue (i.e. `STY`) is the residue on the right of the `p`, 
#' instead of the residue to the left of `(ph)`, as usual.
#' We have introduced a modification to detect and address this issue, but
#' we advice the user to double check both the new evidence file with the
#' introduce new notation and the `-mapping.txt` file and check that there
#' are no NA values for the notation of phophopeptides.
#' 
#' !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
#' 
#' @param evidence_file (char) The evidence file name and location
#' @param column_name (char) The Protein Column Name to map. Options:
#' - `Leadind razor protein` (default)
#' - `Leading protein`
#' - `Proteins`
#' It only supports Uniprot Entry IDs and RefSeq, but it might work for
#' other database IDs
#' @param ref_proteome_file (char) The reference proteome used as database
#' to search the `evidence.txt` file with MaxQuant. It will be used to map the
#' modified peptide to the protein sequence and find the site location.
#' Therefore, it does not use the MaxQuant's `Phospho (STY)Sites.txt`
#' @param output_file (char) Output file name 
#' (`ptmsites-evidence.txt` recommended)
#' @param overwrite_evidence (logical) if <output_file> is the same 
#' as <evidence_file>, `overwrite_evidence = FALSE` (default) doesn't allow to
#' overwrite the evidence file. Otherwise, `overwrite_evidence = TRUE` allows
#' to overwrite the evidence_file (this option might be activated if the user
#' allows to use the same `ptm-sites-evidence.txt` file to re-annotate all
#' the Protein IDs columns)
#' @param mod_type (char) The posttranslational modification. Options:
#' - `UB`: Protein Ubiquitination
#' - `PH`: Protein Phosphorylation
#' - `AC`: Protein Acetylation
#' - `PTM:XXX:yy` : User defined PTM. Replace XXX with 1 or more 1-letter amino
#' acid codes on which to find modifications (all uppercase).  Replace yy with 
#' modification name used within the evidence file (require lowercase characters).
#' Example: `PTM:STY:ph` will find modifications on aa S,T,Y with this 
#' format `_AAGGAPS(ph)PPPPVR_`. This would be equivalent to `mod_type = PH` 
#' @param verbose (logical) `TRUE` (default) shows function messages
#' @return (file) Return a new evidence file with the specified Protein id 
#' column modified by adding the sequence site location(s) + postranslational
#' modification(s) to the uniprot entry / refseq id.
#' 
#' Output ID examples: `A34890_ph3`; `Q64890_ph24_ph456`; 
#' `Q64890_ub34_ub129_ub234`; `Q64890_ac35`.
#' @keywords evidence, convert, ptm, ph, ub, ac
#' @examples
#' # Testing warning if files are not submitted. 
#' artmsProtein2SiteConversion(evidence_file = NULL, ref_proteome_file = NULL, 
#' output_file = NULL)
#' @export
artmsProtein2SiteConversion <- function (evidence_file,
                                         ref_proteome_file,
                                         column_name = c('Leading razor protein', 
                                                         'Leading proteins', 
                                                         'Proteins'),
                                         output_file,
                                         mod_type,
                                         overwrite_evidence = FALSE,
                                         verbose = TRUE) {
  
  if(is.null(evidence_file) & 
     is.null(ref_proteome_file) & 
     is.null(output_file)){
    return("Files must not be NULL")
  }
  
  if(any(missing(evidence_file) | 
         missing(ref_proteome_file) |
         missing(output_file) | 
         missing(mod_type)))
    stop("Missed (one or many) required argument(s)
         Please, check the help of this function to find out more")
  
  # expect format like PTM:STY:ph where STY and ph can be set to any set of amino acids and any case-sensitive name used in evidence file
  
  if (substr(mod_type, 1, 4) == "PTM:"){
    parsed <- .parseFlexibleModFormat(mod_type)
    if (is.null(parsed))
      stop("Error: unexpected format for specifying a user-defined modification type", mod_type)
    mod_type <- parsed[1]
    maxq_mod_residue <- parsed[2]
    mod_residue <- parsed[3]
  }else{
    mod_type <- toupper(mod_type)
    if(!mod_type %in% c("PH", "UB", "AC"))
      stop("the mod_type ", mod_type, " is not supported")
  }
  # CHECK PROTEIN COLUMN
  column_name <- match.arg(column_name)
  
  # When the evidence file is not a data.frame, check that it does not have
  # the same as the output file
  if(is.vector(evidence_file)){
    if(!overwrite_evidence){
      if(evidence_file == output_file) 
        stop("<output_file> cannot be the same as <evidence_file>. 
              If you are confident about overwritting the evidence file, 
              then make the argument 'overwrite_evidence = TRUE'")
    }
  }
  
  if(verbose){
    message("-------------------------------------------------------------")
    message("artMS: Annotate Protein ID column to ProteinID_Site notation ")
    message("-------------------------------------------------------------")
  }
  
  if(verbose)
    message(">> FILTERING MODIFIED PEPTIDES")
  
  if (mod_type == 'UB') {
    if(verbose) message('--- SELECTING << UB >> MODIFIED PEPTIDES ')
    maxq_mod_residue <- 'K\\(gl\\)'
    mod_residue <- 'K'
  } else if (mod_type == 'PH') {
    if(verbose) message('--- SELECTING << PH >> MODIFIED PEPTIDES ')
    maxq_mod_residue <- '(S|T|Y)\\(ph\\)'
    mod_residue <- 'S|T|Y'
  } else if (mod_type == 'AC') {
    if(verbose) message('--- SELECTING << AC >> MODIFIED PEPTIDES ')
    maxq_mod_residue <- 'K\\(ac\\)'
    mod_residue <- 'K'
  } else if (mod_type == "PTM"){
    if(verbose) message ("--- SELECTING PEPTIDES MODIFIED WITH USER DEFINED PTM, with this regex: ", maxq_mod_residue)
    
  } else{
    stop(
      mod_type, " is not supported. 
      Check help to get the list of supported PTMs"
    )
  }
  
  ## map mod sites in data to index
  if(verbose) message("--- OPENING EVIDENCE FILE ")
  ## read in maxq. data
  maxq_data <- .artms_checkIfFile(evidence_file, dont_check_names = FALSE)
  maxq_data <- as.data.table(maxq_data)
  
  # Check maxquant version:
  if("Leading Razor Protein" %in% colnames(maxq_data))
    setnames(maxq_data, 'Leading Razor Protein', 'Leading razor protein')
  
  if("Leading Proteins" %in% colnames(maxq_data))
    setnames(maxq_data, 'Leading Proteins', 'Leading proteins')
  
  # remove contaminants, keep unique sequences, fix names
  maxq_data <- maxq_data[grep("CON__|REV__", maxq_data$Proteins, invert = TRUE), ]
  maxq_data <- maxq_data[grep("CON__|REV__", maxq_data$`Leading proteins`, invert = TRUE), ]
  
  # In case the evidence file comes as data.frame
  if("Modified sequence" %in% colnames(maxq_data)){
    if(verbose) message("---(+) 'Modified sequence' column available") 
  }else if("Modified.sequence" %in% colnames(maxq_data)){
    setnames(maxq_data, 'Modified.sequence', 'Modified sequence')  
  }else{
    stop("Problem with evidence file: <Modified sequence> column not found")
  }
  
  # Fix PROTEIN id when coming with the full uniprot id
  
  if(any(grepl("(^sp\\|)(.*)(\\|.*)", maxq_data$Proteins)))
    maxq_data$Proteins <- gsub("(sp\\|)(.*)(\\|.*)", "\\2", maxq_data$Proteins)
  
  if(any(grepl("(^sp\\|)(.*)(\\|.*)", maxq_data$`Leading proteins`)))
    maxq_data$`Leading proteins` <- gsub("(sp\\|)(.*)(\\|.*)", "\\2", maxq_data$`Leading proteins`)
  
  if(any(grepl("(^sp\\|)(.*)(\\|.*)", maxq_data$`Leading razor protein`)))
    maxq_data$`Leading razor protein` <- gsub("(sp\\|)(.*)(\\|.*)", "\\2", maxq_data$`Leading razor protein`)
  
  if(any(grepl("(^tr\\|)(.*)(\\|.*)", maxq_data$Proteins)))
    maxq_data$Proteins <- gsub("(tr\\|)(.*)(\\|.*)", "\\2", maxq_data$Proteins)
  
  if(any(grepl("(^tr\\|)(.*)(\\|.*)", maxq_data$`Leading proteins`)))
    maxq_data$`Leading proteins` <- gsub("(tr\\|)(.*)(\\|.*)", "\\2", maxq_data$`Leading proteins`)
  
  if(any(grepl("(^tr\\|)(.*)(\\|.*)", maxq_data$`Leading razor protein`)))
    maxq_data$`Leading razor protein` <- gsub("(tr\\|)(.*)(\\|.*)", "\\2", maxq_data$`Leading razor protein`)
  
  # PH: Check if the protein has already been annotated;
  if( length( grep("_S\\d", maxq_data[[column_name]]) ) > 10 ) 
    stop("The protein column <", column_name, "> seems to be already converted 
to ptm-site/peptide specific notation.
Otherwise, notice that this function only support Uniprot Entry Id or Refseq")
  
  # UB: check annotated already
  if( length( grep("_K\\d", maxq_data[[column_name]]) ) > 10 ) 
    stop("The protein column <", column_name, "> seems to be already converted 
to ptm-site/peptide specific notation.
Otherwise, notice that this function only support Uniprot Entry Id or Refseq.
If the proteins are still Uniprot Entry IDs and the file has not been converted already, please, contact artMS developers")
  
  
  if (mod_type == 'PH') {
    if(any(grepl("pS", maxq_data$`Modified sequence`)) |
       any(grepl("pT", maxq_data$`Modified sequence`)) |
       any(grepl("pY", maxq_data$`Modified sequence`)) ){
      message("\n____________________________________________________________")
      message("| WARNING: ")
      message("| The version of MaxQuant that generated this evidence file")
      message("| introduced changes in the <Modified Sequence> column that requires some changes.")
      message("| Basically, the phosphorylation site is indicated as 'pS' instead of '(ph)' as in other versions")
      message("| Next: artMS will apply changes to return to the previous notation")
      message("____________________________________________________________\n")
      maxq_data$`Modified sequence` <- gsub("pS", "S(ph)", maxq_data$`Modified sequence`)
      maxq_data$`Modified sequence` <- gsub("pT", "T(ph)", maxq_data$`Modified sequence`)
      maxq_data$`Modified sequence` <- gsub("pY", "Y(ph)", maxq_data$`Modified sequence`)
    }
  }
  
  if( any(grepl("Phospho", maxq_data$`Modified sequence`)) | 
      any(grepl("GlyGly", maxq_data$`Modified sequence`)) |
      any(grepl("Carbamidomethyl", maxq_data$`Modified sequence`)) |
      any(grepl("Oxidation", maxq_data$`Modified sequence`)) |
      any(grepl("Acetyl", maxq_data$`Modified sequence`))
      ){
    if(verbose) message("--- Long notation detected in <Modified sequence> column: converting to short notation")
    maxq_data$ModifiedSequenceLong <- maxq_data$`Modified sequence`
    maxq_data$`Modified sequence` <- .convertLong2ShortPTMFormat(specModSequence = maxq_data$`Modified sequence`)
  }

  
  if(verbose) message("--- READING REFERENCE PROTEOME ")
  ## read in reference proteome
  ref_proteome <- read.fasta(file = ref_proteome_file,
                             seqtype = "AA",
                             as.string = TRUE,
                             set.attributes = TRUE,
                             legacy.mode = TRUE,
                             seqonly = FALSE,
                             strip.desc = FALSE)
    
  
  ## make mod-site index
  p_seqs <- c()
  p_names <- c()
  p_annots <- c()
  
  for (e in ref_proteome) {
    p_seqs <- c(p_seqs, e[1])
    p_names <- c(p_names, attr(e, 'name'))
    p_annots <- c(p_annots, attr(e, 'Annot'))
  }
  
  ref_table <- data.table(names = p_names,
                          annots = p_annots,
                          seqs = p_seqs)
    
  
  # REFSEQ database
  ref_table[, uniprot_ac := gsub('([a-z,0-9,A-Z,\\.,\\_]+)', '\\1', names)]
  
  # UNIPROT
  ref_table[, uniprot_ac := gsub('([a-z,0-9,A-Z]+\\|{1})([A-Z,0-9,\\_, \\-]+)(\\|[A-Z,a-z,0-9,_]+)',
                                 '\\2', names)]
            
  
  # get all indicies/locations of the mod_residue S|T|Y
  indices <- lapply(ref_table$seqs, function(x)
    as.vector(str_locate_all(x, pattern = mod_residue)[[1]][, 1]))
    
  # list which residue it actually finds at each location
  ptm_sites <- lapply(ref_table$seqs, function(x)
    as.vector(str_match_all(x, pattern = mod_residue)))
    
  # get the number of residue locations per string
  lengths <- unlist(lapply(indices, FUN = length))
  # repeate the uniprot_ac as many times as a residue (S|T|Y) is found
  keys <- rep(ref_table$uniprot_ac, lengths)
  # combine the list of the exploded proteins (above) and add in which sites
  # are found as well as the location the residue was found in
  protein_indices <-   data.table(uniprot_ac = keys,
                                  ptm_site = unlist(ptm_sites),
                                  res_index = unlist(indices))
    
  
  
  # EXTRACT PTM POSITIONS FROM MODIFIED PEPTIDES IN EVIDENCE FILE
  
  
  unique_peptides_in_data <- unique(maxq_data[, c(column_name, 'Modified sequence'), with = FALSE])
    
  setnames(unique_peptides_in_data, 'Modified sequence', 'sequence') 
  
  mod_sites <- c()
  mod_seqs <- c()
  
  # keep a count of peptides with mod found in file
  num_mod_peptides <- 0 
  
  if(verbose) message("--- EXTRACTING PTM POSITIONS FROM THE MODIFIED PEPTIDES 
   (This might take a long time depending on the size of the fasta/evidence file) ")
  for (i in seq_len(nrow(unique_peptides_in_data))) {
    entry <- unique_peptides_in_data[i, ]
    peptide_seq <- entry$sequence
    ## cleanup the sequence (removing all modifications) for matching the 
    ## protein sequence
    peptide_seq_clean <- gsub('[a-z,0-9,\\(,\\),_]', '', peptide_seq)
    mod_sites_in_peptide <-
      str_locate_all(string = peptide_seq, pattern = maxq_mod_residue)[[1]][, 1]
    
    if (length(mod_sites_in_peptide) > 0) {
      num_mod_peptides <- num_mod_peptides + 1
      uniprot_acs <- entry[[column_name]]
      # separates the ambiguous cases (;) and appends the site info to all 
      # the proteins
      uniprot_acs <- str_split(string = uniprot_acs, pattern = ';')[[1]]
      
      for (uac in uniprot_acs) {
        # find the protein's full sequence based on the uniprot_ac
        protein_seq <- ref_table[uniprot_ac == uac, ]$seqs
        if (length(protein_seq) > 0) {
          ## get the position of the peptide in the protein sequence
          peptide_index_in_protein <- str_locate(protein_seq, peptide_seq_clean)[[1]][1]

          for (m in seq_len(length(mod_sites_in_peptide))) {
            mod_site <- mod_sites_in_peptide[m]
            peptide_seq_before_site <-
              str_sub(peptide_seq, 1, mod_site - 1)
            ## count all AA (not counting all modifications) before the
            ## modification to get the relative position of the modification
            ## in the peptide sequence
            residues_before_site <-
              str_count(string = peptide_seq_before_site, 
                        pattern = 'A|C|D|E|F|G|H|I|K|L|M|N|P|Q|R|S|T|V|W|Y')
            mod_site_index_in_protein <-
              peptide_index_in_protein + residues_before_site
            protein_mod_sites <- protein_indices[uniprot_ac == uac, ]
            if (!is.na(mod_site_index_in_protein)) {
              mod_res <-
                protein_mod_sites[res_index == mod_site_index_in_protein, 
                                  ptm_site]
              mod_site_id <-
                sprintf(
                  '%s_%s%s',
                  uac,
                  str_sub(
                    protein_seq,
                    mod_site_index_in_protein,
                    mod_site_index_in_protein
                  ),
                  mod_site_index_in_protein
                )
              mod_sites <- c(mod_sites, mod_site_id)
              mod_seqs <- c(mod_seqs, peptide_seq)
              # message("mod_site_id ", mod_site_id, " peptide_seq ",peptide_seq)
              # stopifnot(length(mod_sites) == length(mod_seqs))
            } else{
              message(
                sprintf(
                  'MISMATCH\t%s\n\tPEPTIDE_SEQ\t%s\n\tMOD_SITE\t%s\n\t
                  PEPTIDE_IDX_IN_PROTEIN\t%s\n\tRESIDUES_BEFORE_SITE
                  \t%s\n\tPROTEIN_SEQ\t%s\n',
                  mod_site_id,
                  peptide_seq,
                  mod_site,
                  peptide_index_in_protein,
                  residues_before_site,
                  protein_seq
                )
              )
            }
          }
        }else{
          message("- Protein <", uac, "> not in the sequence database")
        }
      }
    }
  }
  
  # CHECK that the mapping went well
  if (num_mod_peptides == 0)
    stop("No peptides were found in the evidence file with the selected modification")
  
  if(is.null(mod_seqs))
    stop("Protein IDs from evidence file and sequence database do not match. 
         Are you sure that you are using the right sequence database?")
  
  if(is.null(mod_sites))
    stop("Protein IDs from evidence file and sequence database do not match. 
         Are you sure that you are using the right sequence database?")
  
  mod_site_mapping <- data.table(mod_sites, mod_seqs)
  mod_site_mapping_agg <- aggregate(mod_sites ~ mod_seqs,
                                    mod_site_mapping,
                                    FUN = function(x) paste(x, collapse = ';'))
    
  setnames(maxq_data, 'Modified sequence', 'mod_seqs')
  unmapped_mod_seqs <- maxq_data[!(mod_seqs %in% mod_site_mapping_agg$mod_seqs) &
                                   grepl('(gl)', mod_seqs) & !grepl('REV__|CON__', eval(column_name)), ]
    
  unmapped_mod_seqs <- unique(unmapped_mod_seqs[, c('mod_seqs', column_name), with = FALSE])
  
  if (dim(unmapped_mod_seqs)[1] > 0) {
    if(verbose) message('>> UNABLE TO MAP \t')
    if(verbose) print(unmapped_mod_seqs)
  } else{
    if(verbose) message("--- ALL SEQUENCES MAPPED ")
  }
  
  final_data <- merge(maxq_data, mod_site_mapping_agg, by = 'mod_seqs')
  ref_protein_column <- paste0(column_name, " Ref")
  setnames(
    final_data,
    c(column_name, 'mod_sites', 'mod_seqs'),
    c(ref_protein_column, column_name, 'Modified sequence')
  )
  
  final_data <- as.data.frame(final_data)
  final_data <- final_data %>% dplyr::select(column_name, everything())
  
  write.table(
    final_data,
    file = output_file,
    eol = '\n',
    sep = '\t',
    quote = FALSE,
    row.names = FALSE,
    col.names = TRUE
  )
  
  ## write a mapping table
  protein_seq_mapping <-
    unique(maxq_data[, c(column_name, 'mod_seqs'), with = FALSE])
  # setnames(protein_seq_mapping, eval(column_name), 'Protein')
  mapping_table <- merge(protein_seq_mapping,
                         mod_site_mapping_agg,
                         by = 'mod_seqs',
                         all = TRUE)
  write.table(
    mapping_table,
    file = gsub('.txt', '-mapping.txt', output_file),
    eol = '\n',
    sep = '\t',
    quote = FALSE,
    row.names = FALSE,
    col.names = TRUE
  )
  
  if(verbose) 
    message(
      ">> FILES OUT:\n",
      "--- New evidence-site file: ",
      output_file,
      "\n",
      "--- Details of the Mappings: ",
      gsub('.txt', '-mapping.txt', output_file)
    )
  if(verbose) message(">> CONVERSION COMPLETED")
}


# @title Convert PTM long notation to short
#
# @description One version of MaxQuant annotates the peptides in a strange way 
# XXXXS(Phospho (STY))XXXX. This function converts them to the old notation
# @param specModSequence (vector) 
# @param mods (char) the PTM modification: PH, UB, CAM, MOX, NAC
# @keywords PTM
.convertLong2ShortPTMFormat <- function(specModSequence, 
                                      mods=c("PH", "UB", "CAM", "MOX", "NAC", "AC")){
  result <- specModSequence
  specFormats <- list (PH = '([STY])[[(]Phospho \\(STY\\)[])]',
                       UB = '(K)[[(]GlyGly \\(K\\)[])]',
                       CAM = '([C])[[(]Carbamidomethyl \\(C\\)[])]',
                       MOX = '([M])[[(]Oxidation \\(M\\)[])]',
                       NAC = '([A-Z_])[[(]Acetyl \\(Protein N-term\\)[])]',
                       AC = '([A-Z_])[[(]Acetyl \\(K\\)[])]')
  artmsFormats <- list (PH ='\\1\\(ph\\)',
                        UB ='\\1\\(gl\\)',
                        CAM = '\\1\\(cam\\)',
                        MOX = '\\1\\(ox\\)',
                        NAC = '\\1\\(ac\\)',
                        AC = '\\1\\(ac\\)')
  stopifnot(names(specFormats)==names(artmsFormats))
  for (mod in mods){
    if (mod %in% names(specFormats)){
      result <- gsub(specFormats[[mod]], artmsFormats[[mod]], result)
    }else{
      (stop("I don't know how to deal with requested mod: ", mod))
    }
  }
  return (result)
}


# @title Parse Flexible PTM format
#
# @description One version of MaxQuant annotates the peptides in a strange way 
# XXXXS(Phospho (STY))XXXX. This function converts them to the old notation
# @inputStr input string. It should be
# @param mods (char) the PTM modification: PH, UB, CAM, MOX, NAC
# @keywords PTM
.parseFlexibleModFormat <- function(inputStr){
  parts <- unlist(strsplit(inputStr, split = ":"))
  if (length(parts) != 3){
    message("PTM format requires exactly three parts separated by colons (:); example PTM:STY:ph")
    return (NULL)
  }
  if (grepl("[^A-Z]", parts[2])){
    message ("The second position in ", inputStr," must be all uppercase characters specifying amino acids by their 1-letter code")
    return (NULL)
  }
  if (grepl("[^a-z]", parts[3])){
    message ("The PTM name used in the evidence file, and the third position in ", inputStr, " must be all lowercase characters")
    # possibly numeric characters and "_" could be used as currently coded, but to be safe... 
    return (NULL)
  }
  
  # take something like STY, split then paste to S|T|Y
  mod_residue <- paste(unlist(strsplit(parts[2], split = "")), collapse = "|")
  maxq_mod_residue <- paste0("(", mod_residue, ")", "\\(", parts[3], "\\)", collapse = "")
  return (c(parts[1], maxq_mod_residue, mod_residue))
}

# ------------------------------------------------------------------------------
#' @title Quality Control analysis of the evidence-like metabolomics dataset
#'
#' @description Quality Control analysis of the evidence-like metabolomics 
#' dataset
#' @param evidence_file (char or data.frame) The evidence file path and name, or
#' data.frame
#' @param keys_file (char or data.frame) The keys file path and name or 
#' data.frame
#' @param output_name (char) prefix output name (no extension).
#' Default: "qcPlots_metab"
#' @param met_exp (char) Metabolomics experiment. Only one option available
#' (so far):
#' - `MV`: Markview output
#' @param plotINTDIST if `TRUE` (default) plots both *Box-dot plot* 
#' and *Jitter plot* of biological replicates based on MS (raw) 
#' intensity values.
#' @param plotCORMAT if `TRUE` (default) generates up to 3 pdf files for 
#' technical replicates, biological replicates, and conditions. Each pdf file 
#' contains: 
#' - *Correlation matrix* for all the biological replicates using 
#' MS Intensity values, 
#' - *Clustering matrix* of the MS Intensities and correlation distribution 
#' - *histogram* of the distribution of correlations
#' @param plotINTMISC if `TRUE` (default) plots several pages, including 
#' bar plots of *Total Sum of Intensities in BioReplicates*, 
#' *Total Sum of Intensities in Conditions*, 
#' *Total Feature Counts in BioReplicates*, 
#' *Total Feature Counts in conditions* separated by categories 
#' (INT: has a intensity value NOINT: no intensity value ) 
#' *Box plots* of MS Intensity values per 
#' biological replicates and conditions; *bar plots* of total intensity 
#' by bioreplicates and conditions; Barplots of 
#' *total feature counts* by bioreplicates and conditions.
#' @param printPDF If `TRUE` (default) prints out the pdfs. Warning: plot
#' objects are not returned due to the large number of them. 
#' @param verbose (logical) `TRUE` (default) shows function messages
#' @return Quality control files and plots for metabolomics
#' @keywords QC, quality, control, evidence, metabolomics
#' @examples
#' # Testing that input arguments cannot be null
#' artmsQualityControlMetabolomics(evidence_file = NULL,
#'                  keys_file = NULL,
#'                  met_exp = "MV")
#' @export
artmsQualityControlMetabolomics <- function(evidence_file,
                             keys_file,
                             met_exp = c('MV'),
                             output_name = "qcPlots_metab",
                             plotINTDIST = FALSE,
                             plotCORMAT = TRUE,
                             plotINTMISC = TRUE,
                             printPDF = TRUE,
                             verbose = TRUE) {
  
  Run = Feature = BioReplicate_Run = NULL
  
  if(any(missing(evidence_file) | missing(keys_file)))
    stop("Missed (one or many) required argument(s)
         Please, check the help of this function to find out more")
  
  if( is.null(evidence_file) & is.null(keys_file) ){
    return("Both <evidence_file> and <keys_file> cannot be NULL")
  }

  met_exp <- toupper(met_exp)
  met_exp <- match.arg(met_exp)
  
  if(verbose) message(" QUALITY CONTROL -------------------\n>> LOADING FILES ")
  
  # EVIDENCE:
  evidencekeys <- artmsMergeEvidenceAndKeys(evidence_file, 
                                             keys_file,
                                             verbose = verbose)
  
  # remove 
  
  ekselecta <- aggregate(Intensity ~ Proteins + Condition + BioReplicate + Run,
                         data = evidencekeys,
                         FUN = sum)
    
  ekselectaBioreplica <- aggregate(Intensity ~ Proteins + Condition + BioReplicate,
                                   data = ekselecta,
                                   FUN = sum)
    
  
  # Checking the overall distribution of intensities before anything else
  # Based on Intensity
  ekselectaBioreplica$Abundance <- log2(ekselectaBioreplica$Intensity)
  
  if (plotINTDIST){
    if(verbose) message(">> GENERATING THE INTENSITY DISTRIBUTION PLOTS ")
    intDistribution <- paste0(output_name, ".qcplot.plotINTDIST.pdf")
  
    j <- ggplot2::ggplot(ekselectaBioreplica, aes(BioReplicate, Intensity))
    j <- j + geom_jitter(width = 0.3, size = 0.5)
    j <- j + theme_minimal()
    j <-
      j + theme(axis.text.x = element_text(
        angle = 90,
        hjust = 1,
        vjust = 0.5
      ))
    # Based on Abundance
    k <- ggplot2::ggplot(ekselectaBioreplica, aes(BioReplicate, Abundance))
    k <- k + geom_jitter(width = 0.3, size = 0.5)
    k <- k + theme_minimal()
    k <-
      k + theme(axis.text.x = element_text(
        angle = 90,
        hjust = 1,
        vjust = 0.5
      ))

    if(printPDF) pdf(intDistribution)
    plot(j)
    plot(k)
    if(printPDF) garbage <- dev.off()
    
  }
  
  # Feature generation: Combine Sequence and Charge.
  evidencekeys$Feature <- paste0(evidencekeys$Modified.sequence, "_", evidencekeys$Charge)
  
  # ============================================================================
  # GENERAL QUALITY CONTROL: CHECK PROPORTION OF IntDetectionS
  # sum of total intensities, label them as IntDetections and non-IntDetections
  # and plot intensities for each group

  evigeneral <- evidencekeys[c('Feature',
                               'Proteins',
                               'Intensity',
                               'Condition',
                               'BioReplicate',
                               'Run')]
  
  # Now let's classify the proteins as INT and INT
  evigeneral$IntDetection <- ifelse( evigeneral$Intensity > 0, "INT", "NOINT")
  
  # AGGREGATE ALL THE INTENSITIES PER PROTEIN, summing everything up
  evigeneral$Intensity <- as.numeric(evigeneral$Intensity)
  
  evisummary <- aggregate(
      Intensity ~ Condition + BioReplicate + IntDetection + Run,
      data = evigeneral,
      FUN = sum
    )
  
  # CLEANING THE EVIDENCE OF NOINT
  if( any( dim(evidencekeys[which(evidencekeys$Intensity == 0),])[1] > 0 ) ){
    evidencekeysclean <- evidencekeys[-which(evidencekeys$Intensity == 0),]
  }else{
    evidencekeysclean <- evidencekeys
  }
    

  # Create matrix of reproducibility TECHNICAL REPLICAS
  data2matrix <- evidencekeysclean
  # Make sure that the Intensity is numeric
  data2matrix$Intensity <- as.numeric(data2matrix$Intensity)
  
  # Check the number of TECHNICAL REPLICAS by 
  # checking the first technical replica
  technicalReplicas <- unique(data2matrix$Run[which(data2matrix$BioReplicate == data2matrix$BioReplicate[1])])
  palette.breaks <- seq(1, 3, 0.1)
  color.palette  <- colorRampPalette(c("white", "steelblue"))(length(palette.breaks))
  
  if(plotCORMAT){
    if(verbose) message(">> GENERATING CORRELATION MATRICES ")
    if (length(technicalReplicas) > 1) {
      # First aggregate at the protein level by summing up everything
      biorepliaggregated <- aggregate(Intensity ~ Feature + Proteins + Condition + BioReplicate + Run,
                                      data = data2matrix,
                                      FUN = sum)
        
        
      biorepliaggregated$Intensity <- log2(biorepliaggregated$Intensity)
      
      ##LEGACY
      # evidencekeyscleanDCASTbioreplicas <- data.table::dcast(data = biorepliaggregated,
      #                                                        Proteins + Feature ~ BioReplicate + Run,
      #                                                        value.var = "Intensity")
      
      evidencekeyscleanDCASTbioreplicas <- biorepliaggregated %>% 
        dplyr::mutate(BioReplicate_Run = paste(BioReplicate, Run, sep = "_")) %>%
        tidyr::pivot_wider(id_cols = c(Proteins, Feature), 
                           names_from = BioReplicate_Run, 
                           values_from = Intensity)
        
      precordfBioreplicas <- evidencekeyscleanDCASTbioreplicas[, 3:dim(evidencekeyscleanDCASTbioreplicas)[2]]
      Mtechnicalrep <- cor(precordfBioreplicas, use = "pairwise.complete.obs")
      
      # And now for clustering
      if(verbose) message("--- By Technical replicates ")
      matrixCorrelationBioreplicas <-
        paste0(output_name, ".qcplot.plotCORMAT.pdf")
      
      if(printPDF) pdf(matrixCorrelationBioreplicas, width = 20, height = 20) #
      corrplot::corrplot(
        Mtechnicalrep,
        method = "square",
        addCoef.col = "white",
        number.cex = 1,
        tl.cex = 1.5,
        tl.col = "black",
        title = "Matrix Correlation based on Feature Intensities"
      )
      pheatmap::pheatmap(
        Mtechnicalrep,
        cluster_rows = TRUE,
        cluster_cols = TRUE,
        cellheight = 10,
        cellwidth = 25,
        main = "Clustering Technical Replicates",
        fontsize = 6,
        fontsize_row = 8,
        fontsize_col = 12,
        border_color = 'black',
        fontfamily = "Helvetica",
        treeheight_row = FALSE,
        treeheight_col = FALSE,
        color = color.palette
      )

      if(printPDF) garbage <- dev.off()
    } else{
      if(verbose) message("--- NO Technical Replicates detected ")
    }
    
    # biological replicates
    biorepliaggregated <- NULL
    
    # First deal with the technical replica
    if (length(technicalReplicas) > 1) {
      # Aggregate at the protein level by summing up everything
      biorepliaggregated <-
        aggregate(
          Intensity ~ Feature + Proteins + Condition + BioReplicate + Run,
          data = data2matrix,
          FUN = sum
        )
      biorepliaggregated <-
        aggregate(
          Intensity ~ Feature + Proteins + Condition + BioReplicate,
          data = biorepliaggregated,
          FUN = mean
        )
    } else{
      biorepliaggregated <-
        aggregate(
          Intensity ~ Feature + Proteins + Condition + BioReplicate,
          data = data2matrix,
          FUN = sum
        )
    }
    
    biorepliaggregated$Intensity <- log2(biorepliaggregated$Intensity)
    evidencekeyscleanDCASTbioreplicas <-
      data.table::dcast(data = biorepliaggregated,
                        Proteins + Feature ~ BioReplicate,
                        value.var = "Intensity")
    precordfBioreplicas <-
      evidencekeyscleanDCASTbioreplicas[, 3:dim(evidencekeyscleanDCASTbioreplicas)[2]]
    Mbioreplicas <-
      cor(precordfBioreplicas, use = "pairwise.complete.obs")
    
    if(verbose) message("--- By Biological replicates ")
    matrixCorrelationBioreplicas <-
      paste0(output_name, ".qcplot.correlationMatrixBR.pdf")
    if(printPDF) pdf(matrixCorrelationBioreplicas,
        width = 20,
        height = 20) #, width = 20, height = 20
    corrplot::corrplot(
      Mbioreplicas,
      method = "square",
      addCoef.col = "white",
      number.cex = 0.9,
      tl.cex = 1.5,
      tl.col = "black",
      title = "Matrix Correlation based on Feature Intensities"
    )
    pheatmap::pheatmap(
      Mbioreplicas,
      cluster_rows = TRUE,
      cluster_cols = TRUE,
      cellheight = 10,
      cellwidth = 25,
      main = "Clustering Biological Replicates",
      fontsize = 6,
      fontsize_row = 8,
      fontsize_col = 12,
      border_color = 'black',
      fontfamily = "Helvetica",
      treeheight_row = FALSE,
      treeheight_col = FALSE,
      color = color.palette
    )
    
    if(printPDF) garbage <- dev.off()
    
    # Create matrix of reproducibility CONDITIONS
    biorepliaggregated <- NULL
    
    # biological replicas
    # First deal with the technical replica
    if (length(technicalReplicas) > 1) {
      biorepliaggregated <-
        aggregate(
          Intensity ~ Feature + Proteins + Condition + BioReplicate + Run,
          data = data2matrix,
          FUN = sum
        )
      biorepliaggregated <-
        aggregate(
          Intensity ~ Feature + Proteins + Condition + BioReplicate,
          data = biorepliaggregated,
          FUN = mean
        )
    } else{
      biorepliaggregated <-
        aggregate(
          Intensity ~ Feature + Proteins + Condition + BioReplicate,
          data = data2matrix,
          FUN = sum
        )
    }
    # Now let's sum up based on conditions
    biorepliaggregated <-
      aggregate(Intensity ~ Feature + Proteins + Condition,
                data = biorepliaggregated,
                FUN = median)
    biorepliaggregated$Intensity <- log2(biorepliaggregated$Intensity)
    evidencekeyscleanDCASTconditions <-
      data.table::dcast(data = biorepliaggregated,
                        Proteins + Feature ~ Condition,
                        value.var = "Intensity")
    precordfConditions <-
      evidencekeyscleanDCASTconditions[, 3:dim(evidencekeyscleanDCASTconditions)[2]]
    Mcond <- cor(precordfConditions, use = "pairwise.complete.obs")

    
    if(verbose) message("--- By Conditions ")
    matrixCorrelationCond <-
      paste0(output_name, ".qcplot.correlationMatrixConditions.pdf")
    if(printPDF) pdf(matrixCorrelationCond)
    corrplot(
      Mcond,
      method = "square",
      addCoef.col = "white",
      number.cex = 0.6,
      tl.col = "black",
      title = "Matrix Correlation based on Feature Intensities"
    )
    pheatmap(
      Mcond,
      cluster_rows = TRUE,
      cluster_cols = TRUE,
      cellheight = 10,
      cellwidth = 25,
      main = "Clustering Conditions",
      fontsize = 6,
      fontsize_row = 8,
      fontsize_col = 12,
      border_color = 'black',
      fontfamily = "Helvetica",
      treeheight_row = FALSE,
      treeheight_col = FALSE,
      color = color.palette
    )
    
    if(printPDF) garbage <- dev.off()
  } # END OF plotCORMAT

  
  if(plotINTMISC){
    # DETAILS
    if(verbose) message(">> GENERATING INTENSITY STATS PLOTS ")
    if (met_exp == "MV") {
      ekselect <- evidencekeysclean[c('Feature',
                                      'Proteins',
                                      'Intensity',
                                      'Condition',
                                      'BioReplicate',
                                      'Run')]
      # Aggregate the technical replicas
      ekselecta <-
        aggregate(
          Intensity ~ Proteins + Condition + BioReplicate + Run,
          data = ekselect,
          FUN = sum
        )
      ekselectaBioreplica <-
        aggregate(
          Intensity ~ Proteins + Condition + BioReplicate + Run,
          data = ekselecta,
          FUN = sum
        )
      # Select Unique number of proteins per Condition
      ac <- ekselecta[c('Proteins', 'Condition')]
      b <- unique(ac)
      # Select Unique number of proteins per Biological Replica
      bc <- ekselecta[c('Proteins', 'Condition', 'BioReplicate')]
      bb <- unique(bc)
      # Select unique number of proteins in Technical Replicates
      cc <- ekselecta[c('Proteins', 'Condition', 'BioReplicate', 'Run')]
      cc$TR <- paste0(ekselecta$BioReplicate, "_", ekselecta$Run)
      ccc <- cc[c('Proteins', 'Condition', 'TR')]
      cd <- unique(ccc)
    } else {
      stop("METABOLOMICS experiment not recognized ")
    }
    
    if(verbose) message("--- ", met_exp, " PROCESSED ")
    reproName <- paste0(output_name, ".qcplot.plotINTMISC.pdf")
    
    #QC: SUM of intensities per biological replica (peptides vs IntDetection)
    pisa <-
      ggplot2::ggplot(evisummary,
                      aes(x = BioReplicate, y = Intensity, fill = IntDetection)) +
      geom_bar(stat = "identity",
               position = position_dodge(width = 0.7),
               width = 0.7) +
      theme_minimal() +
      theme(axis.text.x = element_text(
        angle = 90,
        hjust = 1,
        vjust = 0.5
      ),
      legend.title = element_blank()) +
      ggtitle("QC: Total Sum of Intensities in BioReplicates") +
      scale_fill_brewer(palette = "Paired")
    
    #QC: SUM of intensities per condition (peptides vs IntDetection)
    pisb <-
      ggplot2::ggplot(evisummary, aes(x = Condition, y = Intensity, 
                                      fill = IntDetection)) +
      geom_bar(stat = "identity",
               position = position_dodge(width = 0.7),
               width = 0.7) +
      theme_minimal() +
      theme(axis.text.x = element_text(
        angle = 90,
        hjust = 1,
        vjust = 0.5
      ),
      legend.title = element_blank()) +
      ggtitle("QC: Total Sum of Intensities in Conditions") +
      scale_fill_brewer(palette = "Paired")
    
    #QC: TOTAL COUNT OF PEPTIDES IN EACH BIOLOGICAL REPLICA
    pisc <-
      ggplot2::ggplot(evigeneral, aes(x = BioReplicate, fill = IntDetection)) +
      geom_bar(stat = "count",
               position = position_dodge(width = 0.7),
               width = 0.7) +
      theme_minimal() +
      theme(axis.text.x = element_text(
        angle = 90,
        hjust = 1,
        vjust = 0.5
      ),
      legend.title = element_blank()) +
      ggtitle("QC: Total Feature Counts in BioReplicates")
    
    #QC: TOTAL COUNT OF PEPTIDES IN EACH BIOLOGICAL REPLICA
    pisd <-
      ggplot2::ggplot(evigeneral, aes(x = Condition, fill = IntDetection)) +
      geom_bar(stat = "count",
               position = position_dodge(width = 0.7),
               width = 0.7) +
      theme_minimal() +
      theme(axis.text.x = element_text(
        angle = 90,
        hjust = 1,
        vjust = 0.5
      ),
      legend.title = element_blank()) +
      ggtitle("QC: Feature Counts in Conditions")
    
    #QC: MAX INTENSITY IN REDUNDANT PEPTIDES, AND AGGREGATE FOR EACH PROTEIN 
    #THE SUM OF INTENSITY
    ekselectaBioreplica2plot <- ekselectaBioreplica
    ekselectaBioreplica2plot$Intensity <-
      log2(ekselectaBioreplica2plot$Intensity)
    pise <-
      ggplot2::ggplot(ekselectaBioreplica2plot,
                      aes(
                        x = as.factor(BioReplicate),
                        y = Intensity,
                        fill = Intensity
                      )) +
      geom_boxplot(aes(fill = BioReplicate)) +
      # scale_y_log10() + coord_trans(y = "log10") +
      theme_linedraw() +
      theme(
        axis.text.x = element_text(
          angle = 90,
          hjust = 1,
          vjust = 0.5
        ),
        legend.position = "none"
      ) +
      labs(x = "BioReplicate", y = "log2(Intensity)") +
      ggtitle("Feature Intensity in BioReplicates Excluding IntDetections. 
              Max intensity of TR")
    
    pisf <-
      ggplot2::ggplot(ekselectaBioreplica2plot,
                      aes(x = Condition, y = Intensity, fill = Intensity)) +
      geom_boxplot(aes(fill = Condition)) +
      # scale_y_log10() + coord_trans(y = "log10") +
      theme_linedraw() +
      theme(
        axis.text.x = element_text(
          angle = 90,
          hjust = 1,
          vjust = 0.5
        ),
        legend.position = "none"
      ) +
      labs(x = "Condition", y = "log2(Intensity)") +
      ggtitle("Feature Intensity in Conditions Excluding IntDetections. 
              Max intensity of TR")
    
    pisg <- ggplot2::ggplot(ekselectaBioreplica) +
      theme(axis.text.x = element_text(
        angle = 90,
        hjust = 1,
        vjust = 0.5
      )) +
      geom_bar(
        aes(BioReplicate, Intensity),
        position = "dodge",
        stat = "summary",
        fun.y = "mean",
        fill = "black",
        colour = "orange"
      ) +
      ggtitle("Total Intensity in Biological Replicas Excluding IntDetections. 
              Max intensity of TR")
    
    pish <- ggplot2::ggplot(ekselectaBioreplica) +
      theme(axis.text.x = element_text(
        angle = 90,
        hjust = 1,
        vjust = 0.5
      )) +
      geom_bar(
        aes(Condition, Intensity),
        position = "dodge",
        stat = "summary",
        fun.y = "mean",
        fill = "black",
        colour = "green"
      ) +
      ggtitle("Total Intensity in Conditions Excluding IntDetections. 
              Max intensity of TR")
    
    pisi <- ggplot2::ggplot(cd, aes(x = TR, fill = Condition)) +
      geom_bar(stat = "count") +
      theme(
        axis.text.x = element_text(
          angle = 90,
          hjust = 1,
          vjust = 0.9
        ),
        legend.position = "none"
      ) +
      geom_text(
        stat = 'count',
        aes(label = ..count..),
        vjust = -0.5,
        size = 2.7
      ) +
      ggtitle("Unique Features in Technical Replicas")
    
    pisj <- ggplot2::ggplot(bb, aes(x = BioReplicate, fill = Condition)) +
      geom_bar(stat = "count") +
      theme(
        axis.text.x = element_text(
          angle = 90,
          hjust = 1,
          vjust = 0.5
        ),
        legend.position = "none"
      ) +
      geom_text(
        stat = 'count',
        aes(label = ..count..),
        vjust = -0.5,
        size = 2.7
      ) +
      ggtitle("Unique Features in Biological Replicas")
    
    pisk <- ggplot2::ggplot(b, aes(x = Condition, fill = Condition)) +
      geom_bar(stat = "count") +
      theme(
        axis.text.x = element_text(
          angle = 90,
          hjust = 1,
          vjust = 0.5
        ),
        legend.position = "none"
      ) +
      geom_text(
        stat = 'count',
        aes(label = ..count..),
        vjust = -0.5,
        size = 2.7
      ) +
      ggtitle("Unique Features in Condition")
    
    if(printPDF) pdf(reproName)
    print(pisa)
    print(pisb)
    print(pisc)
    print(pisd)
    print(pise)
    print(pisf)
    print(pisg)
    print(pish)
    print(pisi)
    print(pisj)
    print(pisk)
    if(printPDF) garbage <- dev.off()
  } #END OF plotINTMISC
  
  if(verbose) message(">> BASIC QUALITY CONTROL ANALYSIS COMPLETED!  ")
}

utils::globalVariables(
  c("AbMean",
    "Charge",
    "Intensity.H",
    "Intensity.L",
    "Label_variable",
    "Modified.sequence",
    "RawFile_IsotopeLabelType"))

# @title Long to Wide format selecting the `Modified.sequence` column of the
# evidence file
#
# @description Facilitates applying the pivot_wider function, i.e., takes long-format
# data and casts it into wide-format data.
# @param d_long (data.frame) in long format
# @return (data.frame) Evidence file reshaped by rawfile and IsotopeLabelType
# @keywords internal, data.frame, pivot_wider, ptm
.artms_castMaxQToWidePTM <- function(d_long) {
  # Old data.table approach
  # data_w <- data.table::dcast(
  #   Proteins + Modified.sequence + Charge ~ RawFile + IsotopeLabelType,
  #   data = d_long,
  #   value.var = 'Intensity',
  #   fun.aggregate = sum,
  #   fill = NA
  # )

  data_w <- d_long %>%
      dplyr::mutate(RawFile_IsotopeLabelType = paste(RawFile, IsotopeLabelType, sep = "_")) %>%
      dplyr::select(Proteins, Modified.sequence, Charge, RawFile_IsotopeLabelType, Intensity) %>%
      tidyr::pivot_wider(names_from = RawFile_IsotopeLabelType,
                         values_from = Intensity,
                         values_fn = list(Intensity = sum)) %>%
                
      dplyr::arrange(Proteins)
  
  #artmsChangeColumnName is faster than dplyr rename
  data_w <- artmsChangeColumnName(data_w, 
                                  oldname = "Modified.sequence", 
                                  newname = "Sequence")

  return(data_w)
}


# @title Check the `MS/MS Count` column name on the evidence data.table
#
# @description Address case issue with the MS/MS Count column name
# @param (data.frame) keys or evidence files
# @return (data.frame) with the `MS/MS Count` column name
# @keywords internal msmscount, columname
.artms_checkMSMSColumnName <- function(df) {
  if (!('MS/MS Count' %in% colnames(df))) {
    if ("MS/MS count" %in% colnames(df)) {
      df <- artmsChangeColumnName(df, 'MS/MS count', 'MS/MS Count')
    } else{
      stop("cannot find the <MS/MS Count> column")
    }
  }
  return(df)
}



# @title Check the `Raw file` column name on the evidence or keys data.frame
#
# @description Depending on how the data is loaded, the `Raw file` column
# might have different format. This function check to ensure consistency in
# both the evidence and keys data.frames
# @param (data.frame) keys or evidence files
# @return (data.frame) with the `RawFile` column name
# @keywords internal rawfile, columname
.artms_checkRawFileColumnName <- function(df) {
  if (!('RawFile' %in% colnames(df))) {
    if ("Raw.file" %in% colnames(df)) {
      df <- artmsChangeColumnName(df, 'Raw.file', 'RawFile')
    } else if ("Raw file" %in% colnames(df)) {
      df <- artmsChangeColumnName(df, 'Raw file', 'RawFile')
    } else{
      stop("cannot find the <Raw.file> column")
    }
  }
  return(df)
}



#' @title Change a specific column name in a given data.frame
#'
#' @description Making easier to change a column name in any data.frame
#' @param dataset (data.frame) with the column name you want to change
#' @param oldname (char) the old column name
#' @param newname (char) the new name for that column
#' @return (data.frame) with the new specified column name
#' @keywords rename, data.frame, columns
#' @examples
#' artms_data_ph_evidence <- artmsChangeColumnName(
#'                                dataset = artms_data_ph_evidence,
#'                                oldname = "Phospho..STY.",
#'                                newname = "PH_STY")
#' @export
artmsChangeColumnName <- function(dataset, oldname, newname) {
  if (!(oldname %in% colnames(dataset))) {
    stop(" The Column name provided <",
         oldname,
         "> was not found in the object provided ")
  }
  colnames(dataset)[grep(paste0('^', oldname, '$'), colnames(dataset))] <-
    newname
  return(dataset)
}


# @title Filtering data
#
# @description Apply the filtering options, i.e., remove protein groups and/or
# contaminants, and/or, select posttranslational modification (if any)
# @param x (data.frame) Evidence file
# @param config (yaml.object) Configuration object (opened yaml file)
# @param verbose (logical) `TRUE` (default) shows function messages
# @return (data.frame) filtered according to the options selected
# @keywords internal, filtering, remove, proteingroups, ptms
.artms_filterData <- function(x, 
                              config,
                              verbose = TRUE) {
  if(verbose) message(">> FILTERING ")
  
  if (config$data$filters$contaminants) {
    x <- artmsFilterEvidenceContaminants(x, verbose = verbose)
  }
  
  if (config$data$filters$protein_groups == 'remove') {
    if(verbose) message("-- Removing protein groups")
    # SELECT FIRST THE LEADING RAZOR PROTEIN AS PROTEINS, DEPENDING ON THE 
    # MAXQUANT VERSION
    
    # Address the old version of maxquant
    if ( "Leading.Razor.Protein" %in% colnames(x) ) {
      x <- artmsChangeColumnName(x, "Leading.Razor.Protein", "Leading.razor.protein")
    }
    
    x <- artmsLeaveOnlyUniprotEntryID(x, "Proteins")
    x <- artmsLeaveOnlyUniprotEntryID(x, "Leading.razor.protein")
    
    # Check: if neither old version nor new version of leading razor protein
    # is found... stop it
    if ( "Leading.razor.protein" %in% colnames(x) ) {
      x$Proteins <- NULL
      data_f <- artmsChangeColumnName(x, "Leading.razor.protein", "Proteins")
      if(verbose) message("-- Use <Leading.razor.protein> as Protein ID")
    }else{
      stop("<Leading razor protein> column not found. Proteins groups cannot be removed")
    }
    
    # This is not necessary for now: the leading razor protein is unique
    # data_f <- .artms_removeMaxQProteinGroups(x)

  } else if (config$data$filters$protein_groups == 'keep') {
    if(verbose) message("-- Protein groups are kept")
    data_f <- x
  } else{
    stop(
      "\nfiltering option for <protein_groups> not valid 
      (options available: keep or remove)"
    )
  }

  
  # DEAL WITH OLD CONFIGURATION FILES WHEN config$data$filters$modification 
  # COULD BE EMPTY
  if (is.null(config$data$filters$modification)) {
    if(verbose) message("--- NO config$data$filters$modification provided. 
        Using 'AB' as default ")
  } else if (config$data$filters$modification == 'AB' |
             config$data$filters$modification == 'APMS') {
    if(verbose) message(sprintf("-- PROCESSING %s",
                                config$data$filters$modification))
  } else if (config$data$filters$modification == 'UB') {
    data_f = data_f[Modifications %like% 'GlyGly']
  } else if (config$data$filters$modification == 'PH') {
    data_f = data_f[Modifications %like% 'Phospho']
  } else if (config$data$filters$modification == 'AC') {
    data_f = data_f[Modifications %like% 'Acetyl']
  } else{
    stop("The config > data > filters > modification ",
         config$data$filters$modification," is not valid option")
  }
  return(data_f)
}


#' @title Remove contaminants and empty proteins from the MaxQuant evidence file
#'
#' @description Remove contaminants and erronously identified 'reverse'
#' sequences by MaxQuant, in addition to empty protein ids
#' @param x (data.frame) of the Evidence file
#' @param verbose (logical) `TRUE` (default) shows function messages
#' @return (data.frame) without REV__ and CON__ Protein ids
#' @keywords cleanup, contaminants
#' @examples
#' ef <- artmsFilterEvidenceContaminants(x = artms_data_ph_evidence)
#' @export
artmsFilterEvidenceContaminants <- function(x,
                                            verbose = TRUE) {
  # Remove contaminants and reversed sequences (labeled by MaxQuant)
  
  x <- .artms_checkIfFile(x)
  x <- .artms_checkRawFileColumnName(x)
  
  data_selected <- x[grep("CON__|REV__", x$Proteins, invert = TRUE), ]
  
  # Remove empty proteins names
  blank.idx <- which(data_selected$Proteins == "")
  if (length(blank.idx) > 0)
    data_selected = data_selected[-blank.idx, ]
  
  if(verbose) message("-- Contaminants CON__|REV__ removed")
  return(data_selected)
}


#' @title Merge evidence.txt (or summary.txt) with keys.txt files 
#' @description Merge the evidence and keys files on the given columns
#' @param x (data.frame or char) The evidence data, either as data.frame or
#' the file name (and path). It also works for the summary.txt file
#' @param keys The keys data, either as a data.frame or file name (and path)
#' @param by (vector) specifying the columns use to merge the evidence and keys.
#' Default: `by=c('RawFile')`
#' @param isSummary (logical) TRUE or FALSE (default)
#' @param verbose (logical) `TRUE` (default) shows function messages
#' @return (data.frame) with the evidence and keys merged
#' @keywords merge, evidence, summary, keys
#' @examples
#' evidenceKeys <- artmsMergeEvidenceAndKeys(x = artms_data_ph_evidence,
#'                                            keys = artms_data_ph_keys)
#' @export
artmsMergeEvidenceAndKeys <- function(x, 
                                     keys, 
                                     by = c('RawFile'),
                                     isSummary = FALSE,
                                     verbose = TRUE) {

  if(verbose){
    message(">> MERGING FILES ")
  }

  x <- .artms_checkIfFile(x)
  keys <- .artms_checkIfFile(keys)
  
  x <- .artms_checkRawFileColumnName(x)
  keys <- .artms_checkRawFileColumnName(keys)
  
  # Make sure that the Intensity column is not empty
  if(!isSummary){
    if(all(is.na(x$Intensity))){
      stop("The <Intensity> column of the evidence file is empty. artMS cannot continue")
    }  
  }
  
  if(any(grepl("Experiment", colnames(keys)))){
    keys <- artmsChangeColumnName(keys, "Experiment", "ExperimentKeys")
  }
  
  if(isSummary){
    if(any(grepl("Experiment", colnames(x)))){
      x <- subset(x, Experiment != "") 
    }
  }
  
  # Processing FRACTIONS
  # Helping users: processing old requirement
  if("FractionKey" %in% colnames(keys)){
    keys <- artmsChangeColumnName(keys, "FractionKey", "Fraction")
    if(verbose) message("-- (!!) WARNING: column name <FractionKey> deprecated. Please, use <Fraction> instead")
  }
  
  if("Fraction" %in% colnames(keys)){
    # The Fraction column will be the one to use: delete from evidence
    if("Fraction" %in% colnames(x)){
      x <- subset(x, select = -c(Fraction))
    }
  }
  
  requiredColumns <- c('RawFile', 
                       'IsotopeLabelType',
                       'Condition',
                       'BioReplicate', 
                       'Run')
                   
  # Check that the keys file is correct
  if (any(!requiredColumns %in% colnames(keys))) {
    stop('Column names in keys not conform to schema. Required columns:\n', 
           sprintf('\t%s ', requiredColumns))
  }
  
  # Check if the number of RawFiles is the same.
  unique_data <- sort(unique(x$RawFile))
  unique_keys <- sort(unique(keys$RawFile))
  
  keys_not_found <- setdiff(unique_keys, unique_data)
  data_not_found <- setdiff(unique_data, unique_keys)
  
  if(length(keys_not_found) > 0 | length(data_not_found) > 0){
    if(length(keys_not_found) != 0){
      message(
        sprintf(
          "--(-) Raw.files in keys not found in evidence file:\n %s\n",
          paste(keys_not_found, collapse = ';'))
      ) 
    }
    if(length(data_not_found) != 0){
      if (!any(grepl("Total", data_not_found))){
        message(
          sprintf(
            "--(-) Raw.files in evidence not found in keys file:\n %s\n",
            paste(data_not_found, collapse = ';')
          )
        )
      }
    }
  }
  
  x <- merge(x, keys, by = by)
  
  # Make the 0 values, NA values
  if(any(x$Intensity == 0, na.rm = TRUE)){
    zero_values <- length(x$Intensity[(x$Intensity == 0)])
    total_values <- length(x$Intensity)
    x$Intensity[(x$Intensity == 0)] <- NA
    message("---- (r) ", zero_values, " peptides with Intensity = 0 out of ", total_values, " replaced by NAs")
  }
  
  return(x)
}



#' @title Convert the SILAC evidence file to MSstats format
#'
#' @description Converting the evidence file from a SILAC search to a format
#' compatible with MSstats. It basically modifies the Raw.files adding the
#' Heavy and Light label
#' @param evidence_file (char) Text filepath to the evidence file
#' @param output (char) Text filepath of the output name. If NULL it does not
#' write the output
#' @param verbose (logical) `TRUE` (default) shows function messages
#' @return (data.frame) with SILAC data processed for MSstats (and output file)
#' @keywords convert, silac, evidence
#' @examples \dontrun{
#' evidence2silac <- artmsSILACtoLong(evidence_file = "silac.evicence.txt",
#'                                    output = "silac-evidence.txt")
#' }
#' @export
artmsSILACtoLong <- function(evidence_file, 
                             output = NULL,
                             verbose = TRUE) {
  
  file <- Sys.glob(evidence_file)
  if(verbose) message(">> PROCESSING SILAC EVIDENCE FILE")

  # LEGACY reshape the data and split the heavy and light data
  # tmp <- fread(file, integer64 = 'double')
  # tmp_long <- data.table::melt(tmp, 
  #                               measure.vars = c("Intensity L", "Intensity H"))
  # tmp_long[, Intensity := NULL]
  # setnames(tmp_long, 'value', 'Intensity')
  # setnames(tmp_long, 'variable', 'IsotopeLabelType')
  # setnames(tmp_long, 'Raw file', 'Raw.file')
  # levels(tmp_long$IsotopeLabelType) = c('L', 'H')
  # tmp_long[!(is.na(tmp_long$Intensity) && tmp_long$Intensity < 1), ]$Intensity = NA
  
  tmp <- read.delim(file, stringsAsFactors = FALSE)
  tmp <- dplyr::select (tmp,-c(Intensity))
  
  tmp_long <- tmp %>% 
    tidyr::pivot_longer(cols = c(`Intensity.L`, `Intensity.H`), 
                        names_to = "IsotopeLabelType", 
                        values_to = "Intensity")
  
  tmp_long$IsotopeLabelType <- gsub("Intensity.L", "L", tmp_long$IsotopeLabelType)
  tmp_long$IsotopeLabelType <- gsub("Intensity.H", "H", tmp_long$IsotopeLabelType)
  
  if(!is.null(output)){
    write.table(
      tmp_long,
      file = output,
      sep = '\t',
      quote = FALSE,
      row.names = FALSE,
      col.names = TRUE
    )
    if(verbose) message("--- File ", output, " is ready ")
  }
  # colnames(tmp_long) <- gsub(" ", ".", colnames(tmp_long))
  # colnames(tmp_long) <- gsub("/", ".", colnames(tmp_long))
  # colnames(tmp_long) <- gsub("\\(", ".", colnames(tmp_long))
  # colnames(tmp_long) <- gsub("\\)", ".", colnames(tmp_long))
  tmp_long <- as.data.frame(tmp_long)
  return(tmp_long)
}

# @title Provide missing MSstats configuration parameters 
#
# @description MSstats version > 4 introduced a number of major changes 
# affecting the `dataProcess` function. As consequence, the artMS configuration 
# file was updated. The user can now provide any parameter required by 
# dataProcess. But if the user is still using old artMS configuration files,
# this function provides the new extra paramenters not available in previous
# versions
# @param d_long (data.frame) in long format
# @return (data.frame) Evidence file reshaped by rawfile and IsotopeLabelType
# @keywords msstats, parameters
.artms_provide_msstats_config_miss_parameters <- function(config, 
                                                          verbose = TRUE){
  
  Fraction = NULL
  
  if(any(grepl("^msstats$", names(config)))){
    
    cmmp = 0 #count missing msstats parameters
    
    if( !any(grepl("^logTrans$", names(config$msstats))) ){
      config$msstats$logTrans = 2
      cmmp = cmmp + 1
    }
    if( !any(grepl("^normalization_method$", names(config$msstats))) ){
      config$msstats$normalization_method = "equalizeMedians"
      cmmp = cmmp + 1
    }
    if( !any(grepl("^normalization_reference$", names(config$msstats))) ){
      config$msstats$normalization_reference = list(NULL)
      cmmp = cmmp + 1
    }
    if( !any(grepl("^feature_subset$", names(config$msstats))) ){
      config$msstats$feature_subset = "all"
      cmmp = cmmp + 1
    }
    if( !any(grepl("^remove_uninformative_feature_outlier$", names(config$msstats))) ){
      config$msstats$remove_uninformative_feature_outlier = FALSE
      cmmp = cmmp + 1
    }
    if( !any(grepl("^min_feature_count$", names(config$msstats))) ){
      config$msstats$min_feature_count = 2
      cmmp = cmmp + 1
    }
    if( !any(grepl("^n_top_feature$", names(config$msstats))) ){
      config$msstats$n_top_feature = 3
      cmmp = cmmp + 1
    }
    if( !any(grepl("^summaryMethod$", names(config$msstats))) ){
      config$msstats$summaryMethod = "TMP"
      cmmp = cmmp + 1
    }
    if( !any(grepl("^equalFeatureVar$", names(config$msstats))) ){
      config$msstats$equalFeatureVar = TRUE
      cmmp = cmmp + 1
    }
    if( !any(grepl("^censoredInt$", names(config$msstats))) ){
      config$msstats$censoredInt = "NA"
      cmmp = cmmp + 1
    }
    if( !any(grepl("^MBimpute$", names(config$msstats))) ){
      config$msstats$MBimpute = TRUE
      cmmp = cmmp + 1
    }
    if( !any(grepl("^remove50missing$", names(config$msstats))) ){
      config$msstats$remove50missing = FALSE
      cmmp = cmmp + 1
    }
    if( !any(grepl("^fix_missing$", names(config$msstats))) ){
      config$msstats$fix_missing = list(NULL)
      cmmp = cmmp + 1
    }
    if(!any(grepl("^maxQuantileforCensored$", names(config$msstats)))){
      config$msstats$maxQuantileforCensored = 0.999
      cmmp = cmmp + 1
    }
    if(!any(grepl("^use_log_file$", names(config$msstats)))){
      config$msstats$use_log_file = FALSE
      cmmp = cmmp + 1
    }
    if(!any(grepl("^append$", names(config$msstats)))){
      config$msstats$append = FALSE
      cmmp = cmmp + 1
    }
    if(!any(grepl("^log_file_path$", names(config$msstats)))){
      config$msstats$log_file_path = list(NULL)
    }
    
    if(cmmp > 0){
      message(paste("(!) WARNING:", cmmp," <msstats> parameter(s) missing. 
                Default paramenter(s) will be provided. 
                Please, check the artMS configuration file: you might be using an old version. 
                To get a new version use artmsWriteConfigYamlFile()"))
    }
    return(config)
  }
}


# @title Merge keys and Evidence from SILAC experiments
#
# @description Merge keys and Evidence from SILAC experiments
# @param evisilac (char) Output from artmsSILACtoLong
# @param keysilac (char) keys files with SILAC details
# @return df with both evidence and keys from silac merge
# @keywords internal, silac, merge
.artmsMergeSilacEvidenceKeys <- function(evisilac, 
                                         keysilac){
  
  
  evisilac <- .artms_checkIfFile(evisilac)
  evisilac <- .artms_checkRawFileColumnName(evisilac)
  
  keys <- .artms_checkIfFile(keysilac)
  keys <- .artms_checkRawFileColumnName(keys)
  
  # Check the labels from the keys file
  hlvalues <- unique(keys$IsotopeLabelType)
  hl2find <- c("L","H")
  
  if(length(setdiff(hlvalues, hl2find)) > 0){
    stop("The IsotopeLabelType available in the keys file are not from a 
         SILAC experiment: they must be H and L")
  }
  
  evisilac$RawFile = paste(evisilac$RawFile, 
                           evisilac$IsotopeLabelType, 
                           sep = '')
  keysilac$RawFile = paste(keysilac$RawFile, 
                       keysilac$IsotopeLabelType, 
                       sep = '')
  keysilac$Run = paste(keysilac$IsotopeLabelType, 
                   keysilac$Run , 
                   sep = '')
  
  evisilac$IsotopeLabelType = 'L'
  keysilac$IsotopeLabelType = 'L'
  
  df <- artmsMergeEvidenceAndKeys(x = evisilac, 
                                  keys =  keysilac,
                                  by = c('RawFile', 'IsotopeLabelType'),
                                  verbose = FALSE)
  return(df)
}
  


# @title Remove protein groups
#
# @description Remove the group of proteins ids separated by separated by `;`
# @param x (data.frame) with a `Proteins` column.
# @return (data.frame) with the protein groups removed
# @keywords maxquant, remove, proteingroups
.artms_removeMaxQProteinGroups <- function(x) {
  data_selected = x[grep(";", x$Proteins, invert = TRUE), ]
  return(data_selected)
}



#' @title Reshape the MSstats results file from long to wide format
#'
#' @description Converts the normal MSStats results.txt file into "wide" format
#' where each row represents a unique protein's results, and each column
#' represents the comparison made by MSStats. The fold change and p-value
#' of each comparison will be its own column.
#' @param results_msstats (char) Input file name and location
#' (MSstats `results.txt` file)
#' @param output_file (char) Output file name and location
#' (e.g. `results-wide.txt`). If `NULL` (default) returns an
#' R object (data.frame)
#' @param select_pvalues (char) Either
#' - `pvalue` or
#' - `adjpvalue` (default)
#' @param species (char) Specie name for annotation purposes.
#' Check `?artmsMapUniprot2Entrez` to find out more about the
#' supported species (e.g `species = "human"`)
#' @param verbose (logical) `TRUE` (default) shows function messages
#' @return (output file tab delimited) reshaped file with unique protein ids
#' and as many columns log2fc and adj.pvalues as comparisons available
#' @keywords msstats, results, wide, reshape
#' @examples
#' ph_results_wide <- artmsResultsWide(
#'                          results_msstats = artms_data_ph_msstats_results,
#'                          output_file = NULL,
#'                          species = "human")
#' @export
artmsResultsWide <- function(results_msstats,
                             output_file = NULL,
                             select_pvalues = c("adjpvalue", "pvalue"),
                             species,
                             verbose = TRUE) {
  
  # Debug
  # results_msstats = artms_data_ph_msstats_results
  # output_file = NULL
  # select_pvalues = "pvalue"
  # species = "human"
  # verbose = TRUE
  
  if(any(missing(results_msstats) | missing(species)))
    stop("Missed (one or many) required argument(s)
         Please, check the help of this function to find out more")
  
  if(verbose) message(">> RESHAPING MSSTATS RESULTS TO wide FORMAT ")
  results_msstats <- .artms_checkIfFile(results_msstats)
  
  select_pvalues <- match.arg(select_pvalues)
  pvals <- if(select_pvalues == "adjpvalue") "adj.pvalue" else "pvalue"
  selectedColumns <- c('Protein', 'Label', 'log2FC', pvals)
  
  # input_l <- data.table::melt(data = results_msstats[,selectedColumns], 
  #                             id.vars = c('Protein', 'Label'))
  
  input_l <- results_msstats %>%
    dplyr::select(one_of(selectedColumns)) %>%
    tidyr::pivot_longer(
      cols = -c(Protein, Label), 
      names_to = "variable", 
      values_to = "value")
  
  
  # ## then cast to get combinations of LFCV/PVAl and Label as columns
  # input_w <- data.table::dcast(Protein ~ Label + variable,
  #                              data = input_l,
  #                              value.var = c('value'))
  
  input_w <- input_l %>% 
    dplyr::mutate(Label_variable = paste(Label, variable, sep = "_")) %>%
    dplyr::select(Protein, Label_variable, value) %>%
    tidyr::pivot_wider(names_from = Label_variable,
                       values_from = value)
  
  suppressMessages(
    input_w <- artmsAnnotationUniprot(input_w, 
                                      "Protein", 
                                      species)
    )
                                                      

  if (!is.null(output_file)) {
    write.table(
      input_w,
      file = output_file,
      eol = '\n',
      sep = '\t',
      quote = FALSE,
      row.names = FALSE,
      col.names = TRUE
    )
    if(verbose) message("--- Results wide are out! ")
  } else{
    return(input_w)
  }
}


#' @title Outputs the spectral counts from the MaxQuant evidence file.
#'
#' @description Outputs the spectral counts from the MaxQuant evidence file.
#' @param evidence_file (char) Maxquant evidence file or data object
#' @param keys_file (char) Keys file with the experimental design or data object
#' @param output_file (char) Output file name (add `.txt` extension).
#' If `NULL` (default) it returns a data.frame object
#' @param verbose (logical) `TRUE` (default) shows function messages
#' @return A txt file with biological replicates, protein id, and spectral
#' count columns
#' @keywords spectral_counts, evidence
#' @examples
#' summary_spectral_counts <- artmsSpectralCounts(
#'                                  evidence_file = artms_data_ph_evidence,
#'                                  keys_file = artms_data_ph_keys)
#' @export
artmsSpectralCounts <- function(evidence_file,
                                 keys_file,
                                 output_file = NULL,
                                 verbose = TRUE) {
  if(verbose) message(">> EXTRACTING SPECTRAL COUNTS FROM THE EVIDENCE FILE ")
  
  x <- .artms_checkIfFile(evidence_file)
  keys <- .artms_checkIfFile(keys_file)
  
  x <- .artms_checkRawFileColumnName(x)
  keys <- .artms_checkRawFileColumnName(keys)
  
  
  x <- artmsMergeEvidenceAndKeys(x, 
                                 keys, 
                                 by = c('RawFile'),
                                 verbose = verbose)
  data_sel <-
    x[, c('Proteins',
             'Condition',
             'BioReplicate',
             'Run',
             'MS.MS.count')]
  data_sel <-
    artmsChangeColumnName(data_sel, 'MS.MS.count', 'spectral_counts')
  data_sel <-
    aggregate(
      spectral_counts ~ Proteins + Condition + BioReplicate + Run,
      data = data_sel,
      FUN = sum
    )
  data_sel <-
    data.frame(
      data_sel,
      AllCondition = paste(
        data_sel$Condition,
        data_sel$BioReplicate,
        data_sel$Run,
        sep = '_'
      )
    )
  
  if (!is.null(output_file)) {
    write.table(
      data_sel[, c('AllCondition', 'Proteins', 'spectral_counts')],
      file = output_file,
      eol = '\n',
      sep = '\t',
      quote = FALSE,
      row.names = FALSE,
      col.names = TRUE
    )
    if(verbose) message(">> OUTPUT FILE <", output_file, "> is ready ")
  } else{
    return(data_sel)
  }
}


# @title Remove white spaces
#
# @description Remove white spaces
# @param x (vector) A string
# @return (vector) with no white spaces
# @keywords internal, remove, whitespace
.artms_trim <- function (x) {
  gsub("^\\s+|\\s+$", "", x)
}


# @title Generate the contrast matrix required by MSstats from a txt file
# @description It simplifies the process of creating the contrast file
# @param contrast_file The text filepath of contrasts
# @param all_conditions a vector with all the conditions in the keys file
# @return (data.frame) with the contrast file in the format required by
# MSstats
# @author Tom Nguyen, David Jimenez-Morales
# @keywords check, contrast
.artms_writeContrast <- function(contrast_file, 
                                 all_conditions = NULL) {

  if(file.exists(contrast_file)){
    input_contrasts <- readLines(contrast_file, warn = FALSE)
  }else{
    # It assumes it is an already opened file
    input_contrasts <- contrast_file
    if(!is.character(input_contrasts)) stop("CONTRAST file/data is NULL")
    if(!grepl("-", input_contrasts)) stop("CONTRAST file/data not according guidelines")
  }

  #remove empty lines
  input_contrasts <- input_contrasts[vapply(input_contrasts, nchar, FUN.VALUE = 0) > 0]
  
  # check if contrast_file is old-format (i.e the contrast_file is a matrix)
  headers <- unlist(strsplit(input_contrasts[1], split = "\t"))
  if (length(headers) > 1) {
    newinput_contrasts <- c()
    for (i in 2:length(input_contrasts)) {
      newinput_contrasts <-
        c(newinput_contrasts, unlist(strsplit(input_contrasts[i], 
                                              split = "\t"))[1])
    }
    input_contrasts <- newinput_contrasts
  }
  
  # validate the input
  input_contrasts <- trimws(input_contrasts)
  valid <- TRUE
  accepted_chars <- c(LETTERS, letters, 0:9, '-', '_')
  for (x in input_contrasts) {
    if (x != "") {
      characs <- unlist(strsplit(x, split = ''))
      not_allowed_count <-
        length(which(!(characs %in% accepted_chars)))
      if (not_allowed_count > 0) {
        valid <- FALSE
        stop(paste(x, " is not a valid input"))
      }
      
      dash_count <- length(which(characs == '-'))
      if (dash_count != 1) {
        valid <- FALSE
        stop(paste(x, "needs to contain exactly 1 '-'"))
      }
    }
  }
  
  if (valid) {
    mat <- t(as.data.frame(strsplit(input_contrasts, split = '-')))
    rownames(mat) <- NULL
    conds <- sort(unique(c(mat[, 1], mat[, 2])))
    contrast_matrix <-
      matrix(0, nrow = nrow(mat), ncol = length(conds))
    colnames(contrast_matrix) <- conds
    rownames(contrast_matrix) <- input_contrasts
    
    for (i in seq_len(nrow(mat)) ) {
      cond1 <- mat[i, 1]
      cond2 <- mat[i, 2]
      contrast_matrix[i, cond1] <- 1
      contrast_matrix[i, cond2] <- -1
    }
    
    # check if conditions are all found in Evidence/Key
    if (!is.null(all_conditions)) {
      d <- setdiff(conds, all_conditions)
      if (length(d) > 0) {
        msg <-
          paste("These conditions are not found in the dataset:",
                paste(d, collapse = ","))
        stop(msg)
      }
    }
    return (contrast_matrix)
  } else{
    stop(
      'Something went wrong while generating the contrast file.
      Please, let the developers know at <artms.help@gmail.com>'
    )
  }
}

#' artMS configuration template
#' 
#' @description The configuration file in `yaml` format contains 
#' the configuration details required to run `artmsQuantification()`, which
#' includes quality control functions
#' 
#' @format The configuration (`yaml`) file contains the following sections:
#' \describe{
#' \item{files}{ 
#' - `evidence` : /path/to/the/evidence.txt
#' - `keys` : /path/to/the/keys.txt
#' - `contrasts` : /path/to/the/contrast.txt
#' - `summary` : /path/to/the/summary.txt
#' - `output` : /path/to/the/output/results/results.txt}
#' 
#' \item{qc}{
#' - basic: 1 # 1 = yes; 0 = no
#' - extended: 1 # 1 = yes; 0 = no
#' - extendedSummary: 0 # 1 = yes; 0 = no
#' }
#' 
#' \item{data}{
#' - enabled : 1 # 1 = yes; 0 = no
#' - silac: 
#'   - enabled : 0 # 1 for SILAC experiments
#' - filters: 
#'      - enabled : 1
#' - contaminants : 1
#' - protein_groups : remove #remove, keep
#' - modifications : ab # PH, UB, AB, APMS
#' - sample_plots : 1 # correlation plots }
#' 
#' \item{msstats}{
#' -  enabled : 1
#' -  msstats_input : # blank if not previous msstats input file is available
#' -  profilePlots : none # before, after, before-after, none
#' -  normalization_method : equalizeMedians # globalStandards 
#' (include a reference protein(s) ), equalizeMedians, quantile, 0
#' -  normalization_reference :  #should be a value in the Protein column
#' -  summaryMethod : TMP # "TMP"(default) means Tukey's median polish, 
#' which is robust estimation method. "linear" uses linear mixed model. 
#' "logOfSum" conducts log2 (sum of intensities) per run.
#' -  censoredInt : NA  # Missing values are censored or at random. 'NA' 
#' (default) assumes that all 'NA's in 'Intensity' column are censored. 
#' '0' uses zero intensities as censored intensity. In this case, 
#' NA intensities are missing at random. The output from Skyline 
#' should use '0'. Null assumes that all NA intensites are randomly missing.
#' -  MBimpute : 1 # only for summaryMethod="TMP" and censoredInt='NA' or '0'.
#'  TRUE (default) imputes 'NA' or '0' (depending on censoredInt option) by 
#'  Accelated failure model. FALSE uses the values assigned by cutoffCensored
#' - For all othe features, please check documentation for MSstats' dataProcess function
#' }
#' 
#' \item{output_extras}{
#' - output_extras :
#'   - enabled : 1 # if 0, it wont do anything in this section
#' - annotate :  
#'   - enabled: 1 # 1|0 whether to annotate the proteins in the results or not
#' - species : HUMAN  # Supported species: HUMAN, MOUSE, ANOPHELES, ARABIDOPSIS, 
#' BOVINE, WORM, CANINE, FLY, ZEBRAFISH, ECOLI_STRAIN_K12, ECOLI_STRAIN_SAKAI, 
#' CHICKEN, RHESUS, MALARIA, CHIMP, RAT, YEAST, PIG, XENOPUS
#' - plots:
#'   - volcano: 1
#'   - heatmap: 1
#'   - LFC : -1.5 1.5 # Range of minimal log2fc
#'   - FDR : 0.05
#'   - heatmap_cluster_cols : 0
#'   - heatmap_display : log2FC # log2FC or pvalue}
#' }
"artms_config"


#' CORUM Protein Complexes database use for complex enrichment analysis
#' 
#' @description The list of protein complexes has been enriched with
#' mitochondria proteins from mouse, as described in this paper:
#' 
#' 2018 - Ruchi Masand, Esther Paulo, Dongmei Wu , Yangmeng Wang, 
#' Danielle L. Swaney, David Jimenez-Morales, Nevan J. Krogan, and Biao Wang
#' Proteome Imbalance of Mitochondrial Electron Transport Chain in Brown 
#' Adipocytes Leads to Metabolic Benefits. 
#' Cell Metab. 2018 Mar 06; 27(3):616-629.e4 
#' @details LAST CORUM DOWNLOAD DATE: 2017-08-01 
#' @format Tab delimited file.
#' \describe{
#' To find out more about the format and columns available at CORUM, 
#' please visit this 
#' [link](http://mips.helmholtz-muenchen.de/corum/)
#' }
"artms_data_corum_mito_database"

#' TB PATHOGEN: Mycobacterium tuberculosis 
#' (strain ATCC 35801 / TMC 107 / Erdman) UNIPROTS IDS
#'
#' @format A data.frame of Entry IDs
"artms_data_pathogen_TB"


#' LPN PATHOGEN: Legionella pneumophila subsp. pneumophila 
#' (strain Philadelphia 1 / ATCC 33152 / DSM 7513) UNIPROT IDS
#'
#' @format A data.frame of Entry IDs
"artms_data_pathogen_LPN"

#' Evidence file example
#'
#' @description Evidence file from a PH experiment consisting of two 
#' head and neck cancer cell lines ("Conditions" `"Cal33"` and `"HSC6"`). 
#' 
#' Unfortunately, the number of lines was reduced to 1/20 due 
#' to bioconductor limitations on data size, but it should be enough to test the 
#' qc and quantification functions. The number of total columns 
#' from the original evidence file was also reduced to 36 
#' (out of the original 66 columns). 
#' Check `colnames(artms_data_ph_evidence)` for details
#'
#' @format A data frame with all the columns available in an evidence file
#' generated with MaxQuant version 1.6.2.3
"artms_data_ph_evidence"


#' Keys File Example
#' 
#' @description the `artMS` keys file provides the details of the experimental 
#' design for any given proteomics experiment. 
#' 
#' This particular example belongs to a PH experiment consisting of two 
#' head and neck cancer cell lines ("Conditions" `"Cal33"` and `"HSC6"`), 
#' with 2 biological replicates each (in this reduced version)
#' 
#' @format Tab delimited file with the following columns:
#' \describe{
#' \item{Raw.file}{Raw file processed. Each one should be a unique
#' biological (or technical) replicate}
#' 
#' \item{IsotopeLabelType}{Type of labeling. `L` is used for label free 
#' experiments}
#' 
#' \item{Condition}{Label for conditions. VERY IMPORTANT: Only alpha-numeric 
#' characters and `underscore (_)` are allowed}
#' 
#' \item{BioReplicate}{Label for the Biological replicates. VERY IMPORTANT:
#' Use the same labeling for bioreplicate as the Condition, but adding a 
#' `dash (-)` corresponding to the number of biological replicate. 
#' For example, for `Condition` `"Cal"`, use `Cal-1`, `Cal-2`, `Cal-3`, etc 
#' for the bioreplicates}
#' 
#' \item{Run}{The MS run number}
#' }
"artms_data_ph_keys"

#' Contrast example for the PH dataset
#'
#' @description Contrast file with the relative quantification to be performed
#' for the two conditions available in the example dataset: "Cal33-HSC6". 
#' See vignette for more details on how to prepare the contrast file.
#'
#' @format list with one comparison: "Cal33-HSC6"
"artms_data_ph_contrast"


#' MSstats results example
#'
#' @description Relative quantification results obtained running MSstats
#' on the available PH datasets (global analysis). 
#' Changes in protein phosphorylation were quantified between two conditions
#' (check `artms_data_ph_contrast`)
#'
#' @format A data frame resulting from running the latest version of MSstats
"artms_data_ph_msstats_results"

#' MSstats modelQC example
#'
#' @description Normalized data obtained from the `artmsQuantification()` step
#' of the PH dataset (global analysis)
#'
#' @format A data frame resulting from running the latest version of
#' `MSstats::groupComparison` function required as input for 
#' artmsAnalysisQuantifications()
"artms_data_ph_msstats_modelqc"


#' Random data set
#'
#' Dataset randomly generated for testing purposes
#'
#' @format A data frame with 100 rows and 10 variables:
#' \describe{
#' Dataset generated using this code
#' 
#' `data.frame(replicate(10,sample(0:1,100,rep=TRUE)))`
#' }
"artms_data_randomDF"


#' artMS configuration for the available PH dataset
#' 
#' @description The configuration file with default options to run the
#' available PH dataset with `artmsQuantification()``
#' 
#' @format The configuration (`yaml`) file contains the following sections:
#' \describe{
#' \item{files}{ 
#' - `evidence` : Empty. To test an example, run `artms_data_ph_config$files$evidence <- artms_data_ph_evidence`
#' - `keys` : Empty To test an example datasets run `artms_data_ph_config$files$keys <- artms_data_ph_keys`
#' - `contrasts` : Empty. To test the example datasets, run `artms_data_ph_config$files$contrasts <- artms_data_ph_contrast`
#' - `summary` : 
#' - `output` : "results.txt"
#' }
#' 
#' \item{qc}{
#' - basic: 0
#' - extended: 0
#' - extendedSummary: 0
#' }
#' 
#' \item{data}{
#' - enabled : 1 
#' - silac: 
#'   - enabled : 0 
#' - filters: 
#'      - enabled : 1
#' - contaminants : 1
#' - protein_groups : remove 
#' - modifications : PH 
#' - sample_plots : 1 
#' }
#' 
#' \item{msstats}{
#' -  enabled : 1
#' -  msstats_input : # blank if not previous msstats input file is available
#' -  profilePlots : none # before, after, before-after, none
#' -  normalization_method : equalizeMedians 
#' -  normalization_reference :  #should be a value in the Protein column
#' -  summaryMethod : TMP 
#' -  censoredInt : NA
#' -  cutoffCensored : minFeature 
#' -  MBimpute : 1 
#' -  feature_subset: all 
#' }
#' 
#' \item{output_extras}{
#' - output_extras :
#'   - enabled : 1 
#' - annotate :  
#'   - enabled: 1 
#' - species : HUMAN  
#' - plots:
#'   - volcano: 1
#'   - heatmap: 1
#'   - LFC : -1 1 
#'   - FDR : 0.05
#'   - heatmap_cluster_cols : 0
#'   - heatmap_display : log2FC}
#' }
"artms_data_ph_config"

# ------------------------------------------------------------------------------
# @title Write extras
# @description Extras after MSstats, as annotations, volcano plots, heatmaps
# @param results MSstats results
# @param config The configuration object (yaml)
# 
# @return Extras as selected in the yaml file, including:
# - volcano plot (pdf)
# - Adding annotations (gene symbol based on uniprot)
# @keywords extras, annotations, volcano
.artms_writeExtras <- function(results, 
                               config,
                               verbose = TRUE) {
  
  if (length(results) == 0 | !exists('results')) {
    stop("No results found to annotate!")
  }
  
  # Annotation
  if (config$output_extras$annotate$enabled) {
    message(">> ANNOTATING THE RESULTS (adding gene symbols and protein names)")
    if(!is.null(config$output_extras$annotate$species)){
      results_ann <- artmsAnnotationUniprot(x = results, 
                                            columnid = 'Protein', 
                                            species = config$output_extras$annotate$species)
      output_annotated_file <- gsub(".txt", "-annotated.txt", config$files$output)
      write.table(results_ann, output_annotated_file, quote = FALSE, 
                  row.names = FALSE, 
                  col.names = TRUE, 
                  sep = "\t")
    } else {
      if(verbose) message("--- Cannot find species name in the file name 
(Add specie to field 'output_extras > annotate > species' in the config file)")
      results_ann <- results
    }
  }else{
    results_ann <- results
  }
  
  lfc_lower <- as.numeric(unlist(strsplit(config$output_extras$plots$LFC, split = " "))[1])
  lfc_upper <- as.numeric(unlist(strsplit(config$output_extras$plots$LFC, split = " "))[2])
  
  ## This option was originally available in the config file but
  ## it was removed
  config$output_extras$plots$comparisons <- "all"
  ## ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
  
  selected_labels <- config$output_extras$plots$comparisons
  if (is.null(selected_labels) || selected_labels == 'all') 
    selected_labels = '*'
  
  # remove the Inf, -Inf log2FC hits.
  results_ann <- results_ann[!is.infinite(results_ann$log2FC), ]
  
  ## select data points  by LFC & FDR criterium in single condition and
  ## adding corresponding data points from the other conditions
  sign_hits <- .artms_significantHits(results_ann,
                                      labels = selected_labels,
                                      LFC = c(lfc_lower, lfc_upper),
                                      FDR = config$output_extras$plots$FDR)

  if (dim(sign_hits)[1] == 0){
    message("--(-) No significant hits detected in this experiment. aborting plots. ")
  }else{
    sign_labels <- unique(sign_hits$Label)
    if(verbose) message( sprintf(
      "-- Selected hits for plots with LFC between %s and %s at %s FDR:%s ",
      lfc_lower,
      lfc_upper,
      config$output_extras$plots$FDR,
      nrow(sign_hits) / length(sign_labels)))
    
    ## REPRESENTING RESULTS AS HEATMAP, if enabled
    if (config$output_extras$plots$heatmap) {
      # Heatmap only for > 1 comparison
      if (dim(sign_hits)[1] > 1) {
        if(verbose) message(">> PLOTTING HEATMAP FOR SIGNIFICANT CHANGES ")
        heat_labels <- .artms_prettyPrintHeatmapLabels(uniprot_acs = sign_hits$Protein,
                                                       uniprot_ids = sign_hits$name,
                                                       gene_names = sign_hits$Gene.names)
          
        heat_data_w <- .artms_plotHeat(mss_F = sign_hits,
                                       out_file =  gsub('.txt', '-sign.pdf', config$files$output),
                                       names = heat_labels,
                                       cluster_cols = config$output_extras$plots$heatmap_cluster_cols,
                                       display = config$output_extras$plots$heatmap_display,
                                       verbose = verbose)
      }
    }
  }
  
  if (config$output_extras$plots$volcano) {
    if(verbose) message(">> PLOTTING VOLCANO PLOT ")
    file_name <- gsub('.txt', '-volcano.pdf', config$files$output)
    artmsVolcanoPlot(
      mss_results = results_ann[grep(selected_labels, results_ann$Label), ], 
      lfc_upper = lfc_upper, lfc_lower = lfc_lower, 
      FDR = config$output_extras$plots$FDR,
      output_name = file_name,
      verbose = verbose
    )
  }
}

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' @title Convert Markview Metabolomics file (alignment table) into 
#' a artMS compatible format
#' 
#' @description `artMS` enables the relative quantification of untargeted 
#' polar metabolites using the alignment table generated by Markview. 

#' MarkerView is an ABSciex software that supports the files 
#' generated by Analyst software (`.wiff`) used to run our specific mass 
#' spectrometer (ABSciex Triple TOF 5600+). 
#' It also supports `.t2d` files generated by the 
#' Applied Biosystems 4700/4800 MALDI-TOF. 

#' MarkerView software is used to align mass spectrometry data from several 
#' samples for comparison. Using the import feature in the software, `.wiff` 
#' files (also `.t2d` MALDI-TOF files and tab-delimited `.txt` mass spectra data 
#' in mass-intensity format) are loaded for retention time alignment. 
#' Once the data files are selected, a series of windows will appear wherein 
#' peak finding, alignment, and filtering options can be entered and selected. 
#' These options include minimum spectral peak width, minimum retention time 
#' peak width, retention time and mass tolerance, and the ability to filter 
#' out peaks that do not appear in more than a user selected number of samples.
#' 
#' `artmsConvertMetabolomics`` processes the markview file to enable 
#' QC analysis and relative quantification using the artMS functions
#' 
#' @param input_file (char) Markview input file
#' @param out_file (char) Output file name
#' @param id_file (char) KEGG database
#' @param verbose (logical) `TRUE` (default) shows function messages
#' @return (text file) Outputs the converted output name
#' @keywords metabolomics, convert
#' @examples
#' # Testing that the arguments cannot be null
#' artmsConvertMetabolomics(input_file = NULL, 
#'                          out_file = NULL)
#' @export
artmsConvertMetabolomics <- function(input_file, 
                                out_file, 
                                id_file = NULL,
                                verbose = TRUE){
  
  if(missing(input_file))
    stop("Input file name <input_file> is missed")
  if(missing(out_file))
    stop("<out_file> is missed")
  
  if(is.null(input_file) & is.null(out_file)){
    return("Both <input_file> and <out_file> are required")
  }
  
  if(verbose) message(">> Reading in data from: ", input_file," ")
  
  x <- fread(input_file)
  tmp <- data.table::melt(data=x, id=c(seq_len(7)), 
                          variable.name="RawFile", 
                          value.name="Intensity")
  
  tmp[,'Row'] = NULL
  tmp[,'Index'] = NULL
  setnames(tmp, c(1,3), c('Modified.sequence', 'Retention time'))
  tmp$Modified.sequence <- gsub(' \\([0-9]+\\)', '', tmp$Modified.sequence)
  tmp$Proteins <- tmp$Modified.sequence
  tmp$Charge <- 1
  
  tmp <- as.data.frame(tmp, stringsAsFactors = FALSE)
  
  # annotate with  known id's
  if(!is.null(id_file)){
    ids <- read.delim(id_file, stringsAsFactors = FALSE, sep='\t')
    ids$KEGG <- gsub('\\\xca','',ids$KEGG)
    #### This could be optimized!!!!!
    for( i in seq_len(length(ids$KEGG)) ){  
      idx <- tmp$'m/z' %in% ids$m.z[i]
      tmp$Proteins[idx] = ids$KEGG[i]
    }
  }
  
  if(verbose) message(">> Writing out data to:", out_file, "... ")
  write.table(as.data.frame(tmp, stringsAsFactors = FALSE), 
              out_file, 
              row.names = FALSE, 
              col.names = TRUE, 
              sep ='\t', 
              quote = FALSE)
  if(verbose) message('>> CONVERSION COMPLETE! ')
}

utils::globalVariables(c("Organism"))

#' @importFrom scales percent
#' @title Quality Control analysis of the MaxQuant evidence file
#'
#'
#' @description Quality Control analysis of the MaxQuant evidence file
#' @param evidence_file (char or data.frame) The evidence file path and name, or
#' data.frame
#' @param keys_file (char or data.frame) The keys file path and name or 
#' data.frame
#' @param output_dir (char) Name for the folder to output the results plots. 
#' Default is "qc_basic".
#' @param output_name (char) prefix output name (no extension).
#' Default: "qcBasic_evidence"
#' @param prot_exp (char) Proteomics experiment. 6 options available:
#' - `APMS`: affinity purification mass spectrometry
#' - `AB`: protein abundance
#' - `PH`: protein phosphorylation
#' - `UB`: protein ubiquitination (aka ubiquitylation)
#' - `AC`: protein acetylation
#' - `PTM:XXX:yy` : User defined PTM. Replace XXX with 1 or more 1-letter amino
#' acid codes on which to find modifications (all uppercase).  Replace yy with 
#' modification name used within the evidence file (require lowercase characters).
#' Example for phosphorylation: `PTM:STY:ph` will find modifications on 
#' aa S,T,Y with this example format `_AAGGAPS(ph)PPPPVR_`. This means that 
#' the user could select phosphorylation as `PH` or `PTM:STY:ph`
#' @param isSILAC if `TRUE` processes SILAC input files. Default is `FALSE`
#' @param plotINTDIST if `TRUE` plots both *Box-dot plot* 
#' and *Jitter plot* of biological replicates based on MS (raw) 
#' intensity values, otherwise `FALSE` (default) 
#' @param plotREPRO if `TRUE` plots a correlation dotplot for all the 
#' combinations of biological replicates of conditions, based on MS Intensity 
#' values using features (peptide+charge). Otherwise `FALSE` (default)
#' @param plotCORMAT if `TRUE` (default) plots a 
#' - *Correlation matrix* for all the biological replicates using 
#' MS Intensity values, 
#' - *Clustering matrix* of the MS Intensities
#' @param plotINTMISC if `TRUE` (default) plots several pages, including 
#' bar plots of *Total Sum of Intensities in BioReplicates*, 
#' *Total Sum of Intensities in Conditions*, 
#' *Total Peptide Counts in BioReplicates*, 
#' *Total Peptide Counts in conditions* separated by categories: 
#' `CON`: contaminants, `PROT` peptides, `REV` reversed sequences used by 
#' MaxQuant to estimate the FDR; *Box plots* of MS Intensity values per 
#' biological replicates and conditions; *bar plots* of total intensity 
#' (excluding contaminants) by bioreplicates and conditions; Barplots of 
#' *total feature counts* by bioreplicates and conditions.
#' @param plotPTMSTATS IF `TRUE` (default) plots stats related to the 
#' selected modification, including: 
#' *bar plot of peptide counts and intensities*, broken by `PTM/other` 
#' categories; bar plots of *total sum-up of MS intensity values* by 
#' other/PTM categories.
#' @param printPDF If `TRUE` (default) prints out the pdfs. Warning: plot
#' objects are not returned due to the large number of them. 
#' @param verbose (logical) `TRUE` (default) shows function messages
#' @return Quality control files and plots
#' @keywords QC, quality, control, evidence
#' @examples
#' artmsQualityControlEvidenceBasic(evidence_file = artms_data_ph_evidence,
#'                                  keys_file = artms_data_ph_keys, 
#'                                  prot_exp =  "PH", 
#'                                  isSILAC = FALSE,
#'                                  plotINTDIST = FALSE,
#'                                  plotREPRO = TRUE,
#'                                  plotCORMAT = FALSE,
#'                                  plotINTMISC = FALSE,
#'                                  plotPTMSTATS = FALSE,
#'                                  printPDF = FALSE,
#'                                  verbose = FALSE)
#' 
#' # But we recommend the following test:
#' # 1. Go to a working directory: 
#' # setwd("/path/to/your/working/directory/")
#' # 2. Run the following command to print out all the pdf files
#' # artmsQualityControlEvidenceBasic(evidence_file = artms_data_ph_evidence,
#' #                                  keys_file = artms_data_ph_keys, 
#' #                                  prot_exp =  "PH")
#' # 3. Check your working directory and you should find pdf files with 
#' # all the QC plots
#' @export
artmsQualityControlEvidenceBasic <- function(evidence_file,
                             keys_file,
                             prot_exp = c('AB', 'PH', 'UB', 'AC', 'APMS', 'PTM:XXX:yy'),
                             output_dir = "qc_basic",
                             output_name = "qcBasic_evidence",
                             isSILAC = FALSE,
                             plotINTDIST = FALSE,
                             plotREPRO = FALSE,
                             plotCORMAT = TRUE,
                             plotINTMISC = TRUE,
                             plotPTMSTATS = TRUE,
                             printPDF = TRUE,
                             verbose = TRUE) {
  
  # Define global variables
  TechReplica = ..prop.. = ..x.. = PTM = keysilac = Run = Feature = BioReplicate_Run = NULL

  if (is.null(evidence_file) & is.null(keys_file)) {
    return("Evidence and keys cannot be NULL")
  }
  
  if(is.null(output_dir)){
    return("The output_dir argument cannot be NULL")
  }
  
  if (substr(prot_exp, 1, 4) == "PTM:"){
    parsed <- .parseFlexibleModFormat(prot_exp)
    if (is.null(parsed))
      stop("Error: unexpected format for specifying a user-defined modification type", prot_exp)
    prot_exp <- parsed[1]
    maxq_mod_residue <- parsed[2]
    mod_residue <- parsed[3]
  }else{
    prot_exp <- toupper(prot_exp)
    if(!prot_exp %in% c('AB', 'PH', 'UB', 'AC', 'APMS')){
      stop("the prot_exp ", prot_exp, " is not supported")
    }else{
      prot_exp <- match.arg(prot_exp)
    }
  }
  
  if(verbose){
    message("---------------------------------------------------")
    message("artMS: BASIC QUALITY CONTROL (evidence.txt based)")
    message("---------------------------------------------------")
  }
  
  # open keys-----
  keys <- .artms_checkIfFile(keys_file)
  keys <- .artms_checkRawFileColumnName(keys)
  
  printSmall <- ifelse(length(unique(keys$BioReplicate))<5, TRUE, FALSE)
  
  # check silac-----
  if(isSILAC){
    evidence_silac  <- artmsSILACtoLong(evidence_file,
                                       output = NULL,
                                       verbose = verbose)
    
    evidencekeys <- .artmsMergeSilacEvidenceKeys(evisilac = evidence_silac,
                                                 keysilac = keys)
  }else{
    evidencekeys <- artmsMergeEvidenceAndKeys(x = evidence_file, 
                                              keys = keys,
                                              verbose = verbose)
  }

  # Data processing-----
  ekselecta <- aggregate(Intensity ~ Proteins + Condition + BioReplicate + Run,
                         data = evidencekeys,
                         FUN = sum)
    
  ekselectaBioreplica <- aggregate(Intensity ~ Proteins + Condition + BioReplicate,
                                   data = ekselecta,
                                   FUN = sum)
    
  
  # Checking the overall distribution of intensities before anything else
  # Based on Intensity
  ekselectaBioreplica$Abundance <- log2(ekselectaBioreplica$Intensity)
  
  # Feature generation: Combine Sequence and Charge.
  evidencekeys$Feature <- paste0(evidencekeys$Modified.sequence, 
                                 "_", 
                                 evidencekeys$Charge)
  
  # GENERAL QUALITY CONTROL: CHECK PROPORTION OF CONTAMINANTS 
  # sum of total intensities, label them as contaminants and non-contaminants
  # and plot intensities for each group
  
  # Careful with old versions of MaxQuant. Leading Proteins is also needed
  # because it might have proteins label as contaminant not available in the
  # Proteins column
  if (any(grep("Leading.Proteins", names(evidencekeys)))) {
    evidencekeys <- artmsChangeColumnName(evidencekeys, 
                                          "Leading.Proteins", 
                                          "Leading.proteins")
  }
  
  # Combine all the fractions if this is a fraction experiment by summing them up
  if ( "Fraction" %in% colnames(evidencekeys) ){
    # Sum up all the fractions first
    if(verbose) message("-- Processing Fractions (sum up intensities)")
    evidencekeys <- aggregate(Intensity~RawFile+Feature+Proteins+Leading.proteins+Condition+BioReplicate+Run,
                              data = evidencekeys, 
                              FUN = sum)
  }
  
  # Check the total number of modified and non-modified residues to plot
  if (prot_exp == "UB") {
    # Check the total number of modified and non-modified residues to plot
    evidencekeys$PTM <- ifelse(grepl("(gl)", evidencekeys$Modified.sequence),
                               "ub",
                               "other")
  } else if (prot_exp == "PH") {
    evidencekeys$PTM <- ifelse(grepl("(ph)", evidencekeys$Modified.sequence),
                               "ph",
                               "other")
  } else if (prot_exp == "AC") {
    evidencekeys$PTM <- ifelse(grepl("(ac)", evidencekeys$Modified.sequence),
                               "ac",
                               "other")
  } else if (prot_exp == "PTM") {
    evidencekeys$PTM <- ifelse(grepl(maxq_mod_residue, evidencekeys$Modified.sequence),
                               "ptm",
                               "other")
  }
  
  flag_organism <- FALSE
  if("Organism" %in% colnames(evidencekeys)){
    flag_organism <- TRUE
    columns_selected <- c('Feature',
                          'Proteins',
                          'Leading.proteins',
                          'Intensity',
                          'Condition',
                          'BioReplicate',
                          'Run',
                          'Organism')
  }else{
    columns_selected <- c('Feature',
                          'Proteins',
                          'Leading.proteins',
                          'Intensity',
                          'Condition',
                          'BioReplicate',
                          'Run')
  }
  
  evigeneral <- evidencekeys[columns_selected]
  
  evigeneral$TechReplica <- paste(evigeneral$BioReplicate, evigeneral$Run, sep = "-")
  
  # Now let's classify the proteins as contaminants and no contaminants
  evigeneral$Contaminant <- ifelse(
    grepl("CON_|REV_", evigeneral$Leading.proteins),
    ifelse(grepl("REV_", evigeneral$Leading.proteins), "REV", "CON"),
    "PROT"
  )
  
  
  # AGGREGATE ALL THE INTENSITIES PER PROTEIN, summing everything up
  evigeneral$Intensity <- as.numeric(evigeneral$Intensity)
  
  if(flag_organism){
    evisummary <- aggregate(Intensity ~ Condition + BioReplicate + Contaminant + Run + Organism,
                            data = evigeneral,
                            FUN = sum)
  } else {
    evisummary <- aggregate(Intensity ~ Condition + BioReplicate + Contaminant + Run,
                            data = evigeneral,
                            FUN = sum)
  }

  
  # Combine Bioreplicate+Run for plotting
  evisummary$TechReplica <- paste(evisummary$BioReplicate, 
                                  evisummary$Run, 
                                  sep = "-")
  
  # CLEANING THE EVIDENCE OF CONTAMINANTS
  evidencekeysclean <- artmsFilterEvidenceContaminants(x = evidencekeys, 
                                                       verbose = FALSE)
  
  if (prot_exp == "UB") {
    evidencekeysclean <- evidencekeysclean[c(
      'Feature',
      'Modified.sequence',
      'Proteins',
      'Intensity',
      'Condition',
      'BioReplicate',
      'Run'
    )]
    evidencekeysclean <- evidencekeysclean[grep("(gl)", evidencekeysclean$Modified.sequence), ]
  }else if (prot_exp == "PH") {
    evidencekeysclean <- evidencekeysclean[c(
      'Feature',
      'Modified.sequence',
      'Proteins',
      'Intensity',
      'Condition',
      'BioReplicate',
      'Run'
    )]
    evidencekeysclean <- evidencekeysclean[grep("(ph)", evidencekeysclean$Modified.sequence), ]
  }else if (prot_exp == "AC") {
    evidencekeysclean <- evidencekeysclean[c(
      'Feature',
      'Modified.sequence',
      'Proteins',
      'Intensity',
      'Condition',
      'BioReplicate',
      'Run'
    )]
    evidencekeysclean <- evidencekeysclean[grep("(ac)", evidencekeysclean$Modified.sequence), ]
  }else if (prot_exp == "PTM") {
    evidencekeysclean <- evidencekeysclean[c(
      'Feature',
      'Modified.sequence',
      'Proteins',
      'Intensity',
      'Condition',
      'BioReplicate',
      'Run'
    )]
    evidencekeysclean <- evidencekeysclean[grep(maxq_mod_residue, evidencekeysclean$Modified.sequence), ]
  }
  
  
  # Create matrix of reproducibility TECHNICAL REPLICAS
  data2matrix <- evidencekeysclean
  # Make sure that the Intensity is numeric
  data2matrix$Intensity <- as.numeric(data2matrix$Intensity)
  
  # Check the number of TECHNICAL REPLICAS by 
  # checking the first technical replica
  technicalReplicas <- FALSE
  if(length(unique(evisummary$BioReplicate)) < length(unique(evisummary$Run))){
    technicalReplicas <- TRUE
  }
  
  # PLOT TIME-----
  
  # Create output folder
  
  # create output directory if it doesn't exist
  if(printPDF){
    if (!dir.exists(output_dir)) {
      if(verbose) message("-- Output folder created: ", output_dir)
      dir.create(output_dir, recursive = TRUE)
    }
  }
  
  # plotINTDIST------
  if(plotINTDIST){
    if(verbose) message("-- Plot: intensity distribution")
      intDistribution <- paste0(output_dir, "/", output_name, ".qcplot.IntensityDistributions.pdf")
    
    j <- ggplot(ekselectaBioreplica, aes(BioReplicate, Intensity))
    j <- j + geom_jitter(width = 0.3, size = 0.5, na.rm = TRUE)
    j <- j + theme_minimal()
    j <- j + theme(axis.text.x = element_text(angle = 90,
                                              hjust = 1,
                                              vjust = 0.5))
    # Based on Abundance
    k <- ggplot(ekselectaBioreplica, aes(BioReplicate, Abundance))
    k <- k + geom_jitter(width = 0.3, size = 0.5, na.rm = TRUE)
    k <- k + theme_minimal()
    k <- k + theme(axis.text.x = element_text(angle = 90,
                                              hjust = 1,
                                              vjust = 0.5))
      
    if(printPDF) pdf(intDistribution)
      plot(j)
      plot(k)
    if(printPDF) garbage <- dev.off()
  }

  if(plotREPRO){
    if(verbose) message("-- Plot: Reproducibility scatter plots")
    seqReproName <- paste0(output_dir, "/", output_name, ".qcplot.BasicReproducibility.pdf")

    if(printPDF) pdf(seqReproName)
      .artms_plotReproducibilityEvidence(data =  evidencekeysclean, 
                                         verbose = verbose)
    if(printPDF) garbage <- dev.off()
  }

  palette.breaks <- seq(1, 3, 0.1)
  color.palette <- colorRampPalette(c("white", "steelblue"))(length(palette.breaks))
  
  if(plotCORMAT){
    if(verbose) message("-- Plot: correlation matrices")
    
    if (technicalReplicas) {
      # First aggregate at the protein level by summing up everything
      biorepliaggregated <- aggregate(
        Intensity ~ Feature + Proteins + Condition + BioReplicate + Run,
        data = data2matrix,
        FUN = sum
      )
        
      biorepliaggregated$Intensity <- log2(biorepliaggregated$Intensity)
      
      ##LEGACY
      # evidencekeyscleanDCASTbioreplicas <- data.table::dcast(data = biorepliaggregated,
      #                                                        Proteins + Feature ~ BioReplicate + Run,
      #                                                        value.var = "Intensity")
      
      evidencekeyscleanDCASTbioreplicas <- biorepliaggregated %>% 
        dplyr::mutate(BioReplicate_Run = paste(BioReplicate, Run, sep = "_")) %>%
        tidyr::pivot_wider(id_cols = c(Proteins, Feature), 
                           names_from = BioReplicate_Run,
                           values_from = Intensity)
        
      precordfBioreplicas <- evidencekeyscleanDCASTbioreplicas[, 3:dim(evidencekeyscleanDCASTbioreplicas)[2]]
      Mtechnicalrep <- cor(precordfBioreplicas, use = "pairwise.complete.obs")
      
      # And now for clustering
      if(verbose) message("---- by Technical replicates ")
    }
      
    
    # biological replicates
    biorepliaggregated <- NULL
    if(verbose) message("---- by Biological replicates ")
    
    # First deal with the technical replica
    if (technicalReplicas) {
      # Aggregate at the protein level by summing up everything
      biorepliaggregated <- aggregate(
        Intensity ~ Feature + Proteins + Condition + BioReplicate + Run,
        data = data2matrix,
        FUN = sum
      )
        
      biorepliaggregated <- aggregate(
        Intensity ~ Feature + Proteins + Condition + BioReplicate,
        data = biorepliaggregated,
        FUN = mean
      )
    } else{
      biorepliaggregated <- aggregate(
        Intensity ~ Feature + Proteins + Condition + BioReplicate,
        data = data2matrix,
        FUN = sum
      )
    }
    
    biorepliaggregated$Intensity <- log2(biorepliaggregated$Intensity)
    
    ##LEGACY
    # evidencekeyscleanDCASTbioreplicas <- data.table::dcast(data = biorepliaggregated,
    #                                                        Proteins + Feature ~ BioReplicate,
    #                                                        value.var = "Intensity")
    
    evidencekeyscleanDCASTbioreplicas <- biorepliaggregated %>%
      tidyr::pivot_wider(id_cols = c(Proteins, Feature), 
                         names_from = BioReplicate, 
                         values_from = Intensity)

    precordfBioreplicas <- evidencekeyscleanDCASTbioreplicas[, 3:dim(evidencekeyscleanDCASTbioreplicas)[2]]
    Mbioreplicas <- cor(precordfBioreplicas, use = "pairwise.complete.obs")
  
    
    # Create matrix of reproducibility CONDITIONS
    biorepliaggregated <- NULL
    
    # biological replicas
    # First deal with the technical replica
    if(verbose) message("---- by Conditions ")
    if (technicalReplicas) {
      biorepliaggregated <- aggregate(
        Intensity ~ Feature + Proteins + Condition + BioReplicate + Run,
        data = data2matrix,
        FUN = sum
      )
        
      biorepliaggregated <- aggregate(
        Intensity ~ Feature + Proteins + Condition + BioReplicate,
        data = biorepliaggregated,
        FUN = mean
      )
    } else{
      biorepliaggregated <- aggregate(
        Intensity ~ Feature + Proteins + Condition + BioReplicate,
        data = data2matrix,
        FUN = sum
      )
    }
    # Now let's sum up based on conditions
    biorepliaggregated <- aggregate(Intensity ~ Feature + Proteins + Condition,
                                    data = biorepliaggregated,
                                    FUN = median)
    biorepliaggregated$Intensity <- log2(biorepliaggregated$Intensity)
    
    ##LEGACY
    # evidencekeyscleanDCASTconditions <- data.table::dcast(data = biorepliaggregated,
    #                                                       Proteins + Feature ~ Condition,
    #                                                       value.var = "Intensity")
    
    evidencekeyscleanDCASTconditions <- biorepliaggregated %>%
      tidyr::pivot_wider(id_cols = c(Proteins, Feature), 
                         names_from = Condition, 
                         values_from = Intensity)
    evidencekeyscleanDCASTconditions <- as.data.frame(evidencekeyscleanDCASTconditions)

    precordfConditions <- evidencekeyscleanDCASTconditions[, 3:dim(evidencekeyscleanDCASTconditions)[2]]
    Mcond <- cor(precordfConditions, use = "pairwise.complete.obs")
    
    matrixCorrelationMatrix <- paste0(output_dir, "/", output_name, ".qcplot.CorrelationMatrix.pdf")
    if(printPDF){
      if(printSmall){
        pdf(matrixCorrelationMatrix)
      }else{
        pdf(matrixCorrelationMatrix, width = 20, height = 20)
      }
    } 
    
    if(technicalReplicas){
      
      corrplot(
        Mtechnicalrep,
        method = "square",
        type = "upper",
        addCoef.col = "white",
        number.cex = 0.9,
        tl.cex = 1.5,
        tl.col = "black",
        title = "Technical replicas: Matrix Correlation based on MS Intensities",
        mar=c(0,0,1,0)
      )
      
    }
    
    corrplot(
      Mbioreplicas,
      method = "square",
      type = "upper",
      addCoef.col = "white",
      number.cex = 0.9,
      tl.cex = 1.5,
      tl.col = "black",
      title = "BioReplicates: Matrix Correlation based on MS Intensities",
      mar=c(0,0,1,0)
    )
    
    corrplot(
      Mcond,
      method = "square",
      type = "upper",
      addCoef.col = "white",
      number.cex = 0.9,
      tl.cex = 1.5,
      tl.col = "black",
      title = "Conditions: Matrix Correlation based on MS Intensities",
      mar=c(0,0,1,0)
    )
    
    if(printPDF) garbage <- dev.off()
    
    
    matrixCorrelationMatrixCluster <- paste0(output_dir, "/", output_name, ".qcplot.CorrelationMatrixCluster.pdf")
    if(printPDF){
      if(printSmall){
        pdf(matrixCorrelationMatrixCluster)
      }else{
        pdf(matrixCorrelationMatrixCluster, width = 20, height = 20)
      }
    } 
    if(technicalReplicas){
      
      corrplot(
        Mtechnicalrep,
        method = "square",
        type = "full",
        addCoef.col = "white",
        number.cex = 0.9,
        tl.cex = 1.5,
        tl.col = "black",
        title = "Technical replicas: Matrix Correlation based on MS Intensities (clustered)",
        mar=c(0,0,1,0),
        order = "hclust",
        hclust.method = "complete"
      )
      
    }
    
    corrplot(
      Mbioreplicas,
      method = "square",
      type = "full",
      addCoef.col = "white",
      number.cex = 0.9,
      tl.cex = 1.5,
      tl.col = "black",
      title = "BioReplicates: Matrix Correlation based on MS Intensities (clustered)",
      mar=c(0,0,1,0),
      order = "hclust",
      hclust.method = "complete"
    )
    
    corrplot(
      Mcond,
      method = "square",
      type = "full",
      addCoef.col = "white",
      number.cex = 0.9,
      tl.cex = 1.5,
      tl.col = "black",
      title = "Conditions: Matrix Correlation based on MS Intensities (clustered)",
      mar=c(0,0,1,0),
      order = "hclust",
      hclust.method = "complete"
    )
      
      if(printPDF) garbage <- dev.off()
      
  } # plotCORMAT ends
  
  # plotINTMISC-----
  if(plotINTMISC){
    # DETAILS
    if(verbose) message("-- Plot: intensity stats")
    if (prot_exp == "APMS" | prot_exp == "AB") {
      ekselect <- evidencekeysclean[c('Feature',
                                      'Proteins',
                                      'Intensity',
                                      'Condition',
                                      'BioReplicate',
                                      'Run')]
      
      # Aggregate the technical replicas
      ekselecta <- aggregate(
        Intensity ~ Proteins + Condition + BioReplicate + Run,
        data = ekselect,
        FUN = sum
      )

      ekselectaBioreplica <- aggregate(
        Intensity ~ Proteins + Condition + BioReplicate + Run,
        data = ekselecta,
        FUN = sum
      )
        
      # Select Unique number of proteins per Condition
      ac <- ekselecta[c('Proteins', 'Condition')]
      b <- unique(ac)
      # Select Unique number of proteins per Biological Replica
      bc <- ekselecta[c('Proteins', 'Condition', 'BioReplicate')]
      bb <- unique(bc)
    
      
      # Select unique number of proteins in Technical Replicates
      cc <- ekselecta[c('Proteins', 'Condition', 'BioReplicate', 'Run')]
      cc$TR <- paste0(ekselecta$BioReplicate, "_", ekselecta$Run)
      ccc <- cc[c('Proteins', 'Condition', 'TR')]
      cd <- unique(ccc)
    } else if (prot_exp == "UB" | prot_exp == "PH" | prot_exp == "AC" | prot_exp == "PTM") {
      ekselectall <- evidencekeysclean[c('Feature',
                                         'Modified.sequence',
                                         'Proteins',
                                         'Intensity',
                                         'Condition',
                                         'BioReplicate',
                                         'Run')]
        
      
      if(prot_exp == "UB"){
        ekselectgly <- ekselectall[grep("(gl)", ekselectall$Modified.sequence), ]
      }else if(prot_exp == "PH"){
        ekselectgly <- ekselectall[grep("(ph)", ekselectall$Modified.sequence), ]
      }else if(prot_exp == "AC"){
        ekselectgly <- ekselectall[grep("(ac)", ekselectall$Modified.sequence), ]
      }else if(prot_exp == "PTM"){
        ekselectgly <- ekselectall[grep(maxq_mod_residue, ekselectall$Modified.sequence), ]
      }else{
        stop("Impossible error. Please report to developers (artms.help@gmail.com)")
      }
      
      # Select the REDUNDANT peptides with the meanimum intensity
      ekselectaBioreplica <- aggregate(
        Intensity ~ Proteins + Condition + BioReplicate + Run,
        data = ekselectgly,
        FUN = sum
      )
        
      # Select Unique number of proteins per Condition
      ac <- ekselectaBioreplica[c('Proteins', 'Condition')]
      b <- unique(ac)
      # Select Unique number of proteins per Biological Replica
      bc <- ekselectaBioreplica[c('Proteins', 'Condition', 'BioReplicate')]
      bb <- unique(bc)
      # Select unique number of proteins in Technical Replicates
      cc <- ekselectaBioreplica[c('Proteins', 'Condition', 'BioReplicate', 'Run')]
      cc$TR <- paste0(ekselectaBioreplica$BioReplicate,
                      "_",
                      ekselectaBioreplica$Run)
      ccc <- cc[c('Proteins', 'Condition', 'TR')]
      cd <- unique(ccc)
      
      # Check the total number of modified and non-modified residues to plot
      if(prot_exp == "UB"){
        evidencekeys$PTM <- ifelse(grepl("(gl)", evidencekeys$Modified.sequence),
                                   "ub",
                                   "other")
      }else if(prot_exp == "PH"){
        evidencekeys$PTM <- ifelse(grepl("(ph)", evidencekeys$Modified.sequence),
                                   "ph",
                                   "other")
      }else if(prot_exp == "AC"){
        evidencekeys$PTM <- ifelse(grepl("(ac)", evidencekeys$Modified.sequence),
                                   "ac",
                                   "other")
      }else if(prot_exp == "PTM"){
        evidencekeys$PTM <- ifelse(grepl(maxq_mod_residue, evidencekeys$Modified.sequence),
                                   "ptm",
                                   "other")
      }else{
        stop("Impossible error. Please report to developers (artms.help@gmail.com)")
      }
    }else {
      stop("Proteomics experiment not recognized ")
    }
    
    #QC: SUM of intensities per biological replica (peptides vs contaminant)
    pisa <- ggplot(evisummary,
                   aes(x = BioReplicate, y = Intensity, fill = Contaminant)) +
      geom_bar(stat = "identity",
               position = position_dodge(width = 0.7),
               width = 0.7,
               na.rm = TRUE) +
      theme_minimal() +
      theme(axis.text.x = element_text(
        angle = 90,
        hjust = 1,
        vjust = 0.5,
        size = 8
      ),
      legend.title = element_blank()) +
      ggtitle("QC: Total Sum of Intensities in BioReplicates") +
      scale_fill_brewer(palette = "Paired")
    
    #QC: SUM of intensities per technical replicate (peptides vs contaminant)
    if(technicalReplicas){
      pisatech <- ggplot(evisummary,
                         aes(x = TechReplica, y = Intensity, fill = Contaminant)) +
        geom_bar(stat = "identity",
                 position = position_dodge(width = 0.7),
                 width = 0.7,
                 na.rm = TRUE) +
        theme_minimal() +
        theme(axis.text.x = element_text(
          angle = 90,
          hjust = 1,
          vjust = 0.5,
          size = 4
        ),
        legend.title = element_blank()) +
        ggtitle("QC: Total Sum of Intensities in Technical Replicates") +
        scale_fill_brewer(palette = "Paired")
    }
    
    pisacont <- ggplot(evigeneral, aes(Contaminant, group = Condition, fill = Contaminant)) + 
      geom_bar(aes(y = ..prop.., fill = factor(..x..)), 
               stat="count", 
               na.rm = TRUE) +
      geom_text(aes( label = scales::percent(..prop..),
                     y= ..prop.. ), stat= "count", vjust = -.5, size = 2) +
      labs(y = "Percent") +
      facet_grid(~Condition) +
      scale_y_continuous(labels = scales::percent) +
      theme_linedraw() +
      theme(axis.text.x = element_text(
        angle = 90,
        hjust = 1,
        vjust = 0.5
      ), legend.position = "none") +
      ggtitle("QC: percent Contaminants") +
      scale_fill_brewer(palette = "Paired")
    
    pisasum <- ggplot2::ggplot(evisummary) +
      # geom_bar(stat = "identity",
      #          position = position_dodge(width = 0.7),
      #          width = 0.7) +
      geom_bar(
        aes(BioReplicate, Intensity, fill = Condition),
        position = "dodge",
        stat = "summary",
        fun = "sum",
        na.rm = TRUE
      ) +
      theme_minimal() +
      theme(axis.text.x = element_text(
        angle = 90,
        hjust = 1,
        vjust = 0.5,
        size = 8
      ),
      legend.title = element_blank()) +
      ggtitle("QC: Total Sum of Intensities in BioReplicates") +
      scale_fill_brewer(palette = "Paired")
    
    if(technicalReplicas){
      pisasumtech <- ggplot2::ggplot(evisummary) +
        # geom_bar(stat = "identity",
        #          position = position_dodge(width = 0.7),
        #          width = 0.7) +
        geom_bar(
          aes(TechReplica, Intensity, fill = Condition),
          position = "dodge",
          stat = "summary",
          fun = "sum",
          na.rm = TRUE,
          size = 4
        ) +
        theme_minimal() +
        theme(axis.text.x = element_text(
          angle = 90,
          hjust = 1,
          vjust = 0.5
        ),
        legend.title = element_blank()) +
        ggtitle("QC: Total Sum of Intensities in Technical Replicates") +
        scale_fill_brewer(palette = "Paired")
    }
    
    #QC: SUM of intensities per condition (peptides vs contaminant)
    pisb <- ggplot(evisummary, aes(x = Condition, y = Intensity, fill = Contaminant)) +
      geom_bar(stat = "identity",
               position = position_dodge(width = 0.7),
               width = 0.7,
               na.rm = TRUE) +
      theme_minimal() +
      theme(axis.text.x = element_text(
        angle = 90,
        hjust = 1,
        vjust = 0.5
      ),
      legend.title = element_blank()) +
      ggtitle("QC: Total Sum of Intensities in Conditions") +
      scale_fill_brewer(palette = "Paired")
      
    
    #QC: TOTAL COUNT OF PEPTIDES IN EACH BIOLOGICAL REPLICA
    pisc <- ggplot(evigeneral, aes(x = BioReplicate, fill = Contaminant)) +
      geom_bar(stat = "count",
               position = position_dodge(width = 0.7),
               width = 0.7,
               na.rm = TRUE) +
      theme_minimal() +
      theme(axis.text.x = element_text(
        angle = 90,
        hjust = 1,
        vjust = 0.5,
        size = 8
      ),
      legend.title = element_blank()) +
      ggtitle("QC: Total Peptide Counts in BioReplicates")
    
    if(technicalReplicas){
      pisctech <- ggplot(evigeneral, aes(x = TechReplica, fill = Contaminant)) +
        geom_bar(stat = "count",
                 position = position_dodge(width = 0.7),
                 width = 0.7,
                 na.rm = TRUE) +
        theme_minimal() +
        theme(axis.text.x = element_text(
          angle = 90,
          hjust = 1,
          vjust = 0.5,
          size = 8
        ),
        legend.title = element_blank()) +
        ggtitle("QC: Total Peptide Counts in Technical Replicates")
    }
    
    #QC: TOTAL COUNT OF PEPTIDES IN EACH BIOLOGICAL REPLICA
    pisd <- ggplot(evigeneral, aes(x = Condition, fill = Contaminant)) +
      geom_bar(stat = "count",
               position = position_dodge(width = 0.7),
               width = 0.7,
               na.rm = TRUE) +
      theme_minimal() +
      theme(axis.text.x = element_text(
        angle = 90,
        hjust = 1,
        vjust = 0.5
      ),
      legend.title = element_blank()) +
      ggtitle("QC: Peptide Counts in Conditions")
      
    
    #QC: MAX INTENSITY IN REDUNDANT PEPTIDES, AND AGGREGATE FOR EACH PROTEIN 
    #THE SUM OF INTENSITY
    
    ekselectaBioreplica$TechReplica <- paste(ekselectaBioreplica$BioReplicate, ekselectaBioreplica$Run, "-")
    pise <- ggplot(ekselectaBioreplica,
                   aes(
                     x = as.factor(BioReplicate),
                     y = log2(Intensity),
                     fill = Condition
                   )) +
      geom_boxplot(aes(fill = Condition),
                   na.rm = TRUE) +
      # scale_y_log10() + coord_trans(y = "log10") +
      theme_linedraw() +
      theme(
        axis.text.x = element_text(
          angle = 90,
          hjust = 1,
          vjust = 0.5,
          size = 8
        ),
        legend.position = "none"
      ) +
      labs(x = "BioReplicate", y = "log2(Intensity)") +
      ggtitle("Protein Intensity in BioReplicates (Excluding contaminants)")
    
    if(technicalReplicas){
      pisetech <- ggplot(ekselectaBioreplica,
                     aes(
                       x = as.factor(TechReplica),
                       y = log2(Intensity),
                       fill = Condition
                     )) +
        geom_boxplot(aes(fill = BioReplicate),
                     na.rm = TRUE) +
        # scale_y_log10() + coord_trans(y = "log10") +
        theme_linedraw() +
        theme(
          axis.text.x = element_text(
            angle = 90,
            hjust = 1,
            vjust = 0.5,
            size = 6
          ),
          legend.position = "none"
        ) +
        labs(x = "Technical Replicate", y = "log2(Intensity)") +
        ggtitle("Protein Intensity in Technical Replicates (Excluding contaminants)")
    }
    
    pisf <- ggplot(ekselectaBioreplica,
                   aes(x = Condition, y = log2(Intensity), fill = Intensity)) +
      geom_boxplot(aes(fill = Condition),
                   na.rm = TRUE) +
      # scale_y_log10() + coord_trans(y = "log10") +
      theme_linedraw() +
      theme(
        axis.text.x = element_text(
          angle = 90,
          hjust = 1,
          vjust = 0.5
        ),
        legend.position = "none"
      ) +
      labs(x = "Condition", y = "log2(Intensity)") +
      ggtitle("Protein Intensity in Conditions (Excluding contaminants)")
      
    
    pisg <- ggplot(ekselectaBioreplica) +
      theme_minimal() +
      theme(axis.text.x = element_text(
        angle = 90,
        hjust = 1,
        vjust = 0.5
      )) +
      geom_bar(
        aes(BioReplicate, Intensity),
        position = "dodge",
        stat = "summary",
        fun = "mean",
        fill = "black",
        colour = "orange",
        na.rm = TRUE
      ) +
      ggtitle("Total Intensity in Biological Replicas (Excluding contaminants)")
    
    pish <- ggplot(ekselectaBioreplica) +
      theme_linedraw() +
      theme(axis.text.x = element_text(
        angle = 90,
        hjust = 1,
        vjust = 0.5
      )) +
      geom_bar(
        aes(Condition, Intensity),
        position = "dodge",
        stat = "summary",
        fun = "mean",
        fill = "black",
        colour = "green",
        na.rm = TRUE
      ) +
      ggtitle("Total Intensity in Conditions (Excluding contaminants)")
    
    if(technicalReplicas){
      pisi <- ggplot(cd, aes(x = TR, fill = Condition)) +
        geom_bar(stat = "count",
                 na.rm = TRUE) +
        theme_linedraw() +
        theme(
          axis.text.x = element_text(
            angle = 90,
            hjust = 1,
            vjust = 0.9,
            size = 6
          ),
          legend.position = "none"
        ) +
        geom_text(
          stat = 'count',
          aes(label = ..count..),
          # vjust = -0.5,
          hjust = 1,
          size = 2.7,
          angle = 90
        ) +
        ggtitle("Unique IDs in Technical Replicas")
    }

    pisj <- ggplot(bb, aes(x = BioReplicate, fill = Condition)) +
      geom_bar(stat = "count",
               na.rm = TRUE) +
      theme_linedraw() +
      theme(
        axis.text.x = element_text(
          angle = 90,
          hjust = 1,
          vjust = 0.5,
          size = 8
        ),
        legend.position = "none"
      ) +
      geom_text(
        stat = 'count',
        aes(label = ..count..),
        # vjust = -0.5,
        hjust = 1,
        size = 2.7,
        angle = 90
      ) +
      ggtitle("Unique IDs in Biological Replicas")
    
    pisk <- ggplot(b, aes(x = Condition, fill = Condition)) +
      geom_bar(stat = "count",
               na.rm = TRUE) +
      theme_linedraw() +
      theme(
        axis.text.x = element_text(
          angle = 90,
          hjust = 1,
          vjust = 0.5
        ),
        legend.position = "none"
      ) +
      geom_text(
        stat = 'count',
        aes(label = ..count..),
        vjust = -0.5,
        size = 2.7
      ) +
      ggtitle("Unique IDs in Condition")
    
    if(flag_organism){
      #QC: SUM of intensities per biological replica (peptides vs contaminant)
      pisorganism <- ggplot(evisummary,
                            aes(x = BioReplicate, y = Intensity, fill = Organism)) +
        geom_bar(stat = "identity",
                 position = position_dodge(width = 0.7),
                 width = 0.7,
                 na.rm = TRUE) +
        theme_minimal() +
        theme(axis.text.x = element_text(angle = 90,
                                         hjust = 1,
                                         vjust = 0.5,
                                         size = 8), 
              legend.title = element_blank()) +
        ggtitle("QC: Total Sum of Intensities in BioReplicates") +
        scale_fill_brewer(palette = "Paired")
      
      #QC: SUM of intensities per technical replicate (peptides vs contaminant)
      if(technicalReplicas){
        pisorganismtech <- ggplot(evisummary,
                                  aes(x = TechReplica, y = Intensity, fill = Organism)) +
          geom_bar(stat = "identity",
                   position = position_dodge(width = 0.7),
                   width = 0.7,
                   na.rm = TRUE) +
          theme_minimal() +
          theme(axis.text.x = element_text(
            angle = 90,
            hjust = 1,
            vjust = 0.5,
            size = 4
          ),
          legend.title = element_blank()) +
          ggtitle("QC: Total Sum of Intensities in Technical Replicates") +
          scale_fill_brewer(palette = "Paired")
      }
      
      oo <- evigeneral[c('Proteins', 'Condition', 'BioReplicate', 'Organism')]
      oo <- unique(oo)
      oo <- oo[-which(is.na(oo$Organism)),]
      
      pisorganismcounts <- ggplot(oo, aes(x = BioReplicate, fill = Organism)) +
        geom_bar(stat = "count", na.rm = TRUE) +
        theme(axis.text.x = element_text(angle = 90,
                                         hjust = 1,
                                         vjust = 0.5,
                                         size = 8),
              legend.title = element_blank()) +
        geom_text(
          stat = 'count',
          aes(label = ..count..),
          # vjust = -0.5,
          hjust = 1,
          size = 2.7,
          angle = 90
        ) +
        ggtitle("Unique IDs in Biological Replicas by Organism")
    }
    
    if(verbose) message("---- ", prot_exp, " PROCESSED ")
    reproName <- paste0(output_dir, "/", output_name, ".qcplot.IntensityStats.pdf")

    if(printPDF){
      if(printSmall){
        pdf(reproName)
      }else{
        pdf(reproName, width = 10, height = 6)
      }
    } 
      print(pisa)
      if(technicalReplicas) print(pisatech)
      print(pisacont)
      print(pisb)
      print(pisasum)
      if(technicalReplicas) print(pisasumtech)
      print(pisc)
      if(technicalReplicas) print(pisctech)
      print(pisd)
      print(pise)
      print(pisf)
      print(pisg)
      print(pish)
      if(technicalReplicas) print(pisi)
      print(pisj)
      print(pisk)
      if(flag_organism){
        print(pisorganism)
        if(technicalReplicas) pisorganismtech
        print(pisorganismcounts)
      } 
    if(printPDF) garbage <- dev.off()
  }# ends plotINTMISC
  
  # plotPTMSTATS-----
  if(plotPTMSTATS){
    if (prot_exp == "PH" | prot_exp == "UB" | prot_exp == "AC" | prot_exp == "PTM") {
      if(verbose) message("-- Plot: PTM ", prot_exp, " stats")
      modName <- paste0(output_dir, "/", output_name, ".qcplot.PTMStats.pdf")
      
      x <- ggplot(evidencekeys, aes(x = BioReplicate, fill = PTM))
      x <- x + geom_bar(stat = "count",
                        position = position_dodge(width = 0.7),
                        width = 0.7, na.rm = TRUE)
        
      x <- x + theme_minimal()
      x <- x + theme(
        axis.text.x = element_text(
          angle = 90,
          hjust = 1,
          vjust = 0.5
        ),
        legend.title = element_blank()
      )
      x <- x + ggtitle("Peptide Count in Biological Replicas")
      
      xpercent1 <- ggplot(evidencekeys, aes(PTM, 
                                         group = Condition, 
                                         fill = PTM)) + 
        geom_bar(aes(y = ..prop.., fill = factor(..x..)), 
                 stat="count", 
                 na.rm = TRUE) +
        geom_text(aes( label = scales::percent(..prop..),
                       y= ..prop.. ), 
                  stat= "count", 
                  vjust = -.5,
                  size = 2) +
        labs(y = "Percent") +
        facet_grid(~Condition) +
        scale_y_continuous(labels = scales::percent) +
        theme_linedraw() +
        theme(axis.text.x = element_text(
          angle = 90,
          hjust = 1,
          vjust = 0.5
        ), legend.position = "none") +
        ggtitle("QC: PTM enrichment") +
        scale_fill_brewer(palette = "Paired")
      
      xpercent2 <- ggplot(evidencekeys, aes(PTM, 
                                            group = BioReplicate, 
                                            fill = PTM)) + 
        geom_bar(aes(y = ..prop.., fill = factor(..x..)), 
                 stat="count", 
                 na.rm = TRUE) +
        geom_text(aes( label = scales::percent(..prop..),
                       y= ..prop.. ), 
                  stat= "count", 
                  vjust = -.5,
                  size = 2) +
        labs(y = "Percent") +
        facet_grid(~BioReplicate) +
        scale_y_continuous(labels = scales::percent) +
        theme_linedraw() +
        theme(axis.text.x = element_text(
          angle = 90,
          hjust = 1,
          vjust = 0.5
        ), legend.position = "none",
        strip.text.x = element_text(angle = 90)) +
        ggtitle("QC: PTM enrichment") +
        scale_fill_brewer(palette = "Paired")
      
      y <- ggplot(evidencekeys, aes(x = Condition, fill = PTM))
      y <- y + geom_bar(stat = "count",
                        position = position_dodge(width = 0.7),
                        width = 0.7, na.rm = TRUE)
      y <- y + theme_minimal()
      y <- y + theme(
        axis.text.x = element_text(
          angle = 90,
          hjust = 1,
          vjust = 0.5
        ),
        legend.title = element_blank()
      )
        
      y <- y + ggtitle("Peptide Count in Conditions")
      
      u <- ggplot(evidencekeys,
                  aes(x = BioReplicate, y = Intensity, fill = PTM))
      u <- u + geom_bar(stat = "identity",
                        position = position_dodge(width = 0.7),
                        width = 0.7, na.rm = TRUE)
        
      u <- u + theme_minimal()
      u <- u + theme(
        axis.text.x = element_text(
          angle = 90,
          hjust = 1,
          vjust = 0.5
        ),
        legend.title = element_blank()
      )
        
      u <- u + ggtitle("Total Peptide Intensity in Biological Replicas")

      u <- u + scale_fill_brewer(palette = "Paired")
      
      z <- ggplot(evidencekeys,
                  aes(x = Condition, y = Intensity, fill = PTM))
        
      z <- z + geom_bar(stat = "identity",
                        position = position_dodge(width = 0.7),
                        width = 0.7, na.rm = TRUE)
        
      z <- z + theme_minimal()
      z <- z + theme(
        axis.text.x = element_text(
          angle = 90,
          hjust = 1,
          vjust = 0.5
        ),
        legend.title = element_blank()
      )
        
      z <- z + ggtitle("Total Peptide Intensity in Conditions")
      z <- z + scale_fill_brewer(palette = "Paired")
      
      #QC: Total Peptides
      if(printPDF) pdf(modName, width = 10, height = 6)
        print(x)
        print(xpercent1)
        print(xpercent2)
        print(y)
        print(u)
        print(z)
      if(printPDF) garbage <- dev.off()
    }
  } #plotPTMSTATS
  
  if(verbose) message("<< Basic quality control analysis completed!")
}

---
title: "artMS: Analytical R Tools for Mass Spectrometry"
author: "David Jimenez-Morales"
date: "`r Sys.Date()`"
output: 
  BiocStyle::html_document:
  # toc: true
  # # pdf_document: true
    toc_float: true
    toc_collapsed: true
    toc_depth: 3
    number_sections: true
    theme: lumen
  
vignette: >
    %\VignetteIndexEntry{Learn to use artMS}
    %\usepackage[utf8]{inputenc}
    %\VignetteEngine{knitr::rmarkdown}
---


```{r set-options, echo=FALSE, cache=FALSE}
options(width = 2000)
```

<!--
(---------------------------------------------------------------------)
-->

# OVERVIEW

[artMS](http://bioconductor.org/packages/release/bioc/html/artMS.html) is a [Bioconductor](http://bioconductor.org) package 
that provides a set of tools for the analysis and integration of 
large-scale proteomics (mass-spectrometry-based) datasets 
obtained using the popular proteomics software 
[MaxQuant](https://www.maxquant.org/). 

The functions available in artMS can be grouped into the 
following categories:

- Multiple quality control (QC) functions.
- Relative quantification using [MSstats](http://msstats.org/).
- Downstream analysis and integration of quantification (enrichment, 
clustering, PCA, summary plots, etc)
- Generation of input files for other tools, including 
[SAINTq](http://saint-apms.sourceforge.net/Main.html), [SAINTexpress](http://saint-apms.sourceforge.net/Main.html), 
[Photon](https://github.com/jdrudolph/photon), 
and [Phosfate](http://phosfate.com/)

[Click here for details about all the functions available in artMS](https://biodavidjm.github.io/artMS/reference/index.html).

For a graphical overview check 
[the slides presented at the 2021 online workshop of the Association of Biomolecular Resource Facilities (ABRF)](http://biodavidjm.github.io/slides/artMS_davidjm_2021-ABRF/index.html)

## What's new?

Check the repo [NEWS file](https://github.com/biodavidjm/artMS/blob/master/NEWS) to be up to date with the new features, improvements, bug fixes affecting the package.

## How to install

### Bioconductor

`artMS version >= 1.10.1` had many changes to adjust for changes in MSstats. This version requires:

- Install `R version >= 4.1.0` (check the R version running on your system by executing the function `getRversion()`)
- Bioconductor: `BiocManager::install("BiocVersion")`
- artMS: `BiocManager::install("artMS")`
- If you are planning to use the `artmsAnalysisQuantifications()` to perform a comprehensive downstream analysis of the quantitative results, then install the following packages:

```
# From bioconductor:
BiocManager::install(c("ComplexHeatmap", "org.Mm.eg.db"))

# From CRAN:
install.packages(c("factoextra", "FactoMineR", "gProfileR", "PerformanceAnalytics"))
```

Extra: Why Bioconductor? [Here you can find a nice summary of many good reasons](https://bioinformatics.stackexchange.com/questions/639/why-bioconductor)).

### Development version from Github (unstable)

Assuming that you have an `R (>= 4.1)` version running on your system, 
follow these steps:

```
install.packages("devtools")
library(devtools)
install_github("biodavidjm/artMS")
```

Once installed, the package can be loaded and attached to your current 
workspace as follows:

```{r, eval=TRUE}
library(artMS)
```


## Input files

`r Biocpkg("artMS")` performs the different analyses taking as input the 
following files:

- **evidence.txt** file: output of the quantitative proteomics software 
package **MaxQuant**. 
- **keys.txt** (tab-delimited) txt file generated **by the user** describing 
the experimental design.
- **contrast.txt** (tab-delimited) txt file generated **by the user** with the 
comparisons between conditions to be quantified.

Check [below](#inputfiles) to find out more about generating the input files.


## Configuration file

[//]: # (Having a large number of arguments in an R function or pipeline 
can make the task of providing them a cumbersome endeavor.)
 
`artmsQuantification()` requires a large number of arguments, specially those
related to the statistical package `r Biocpkg("MSstats")`.
To facilite the task of providing all those arguments, the function 
`artmsQuantification()` takes a config file (in `yaml` format) for the 
customization of the parameters for quantification (using `MSstats`) 
and other operations, including QC analyses, charts, and annotations. 

A configuration file template can be generated by running 
`artmsWriteConfigYamlFile()`

Check [below](#configfile) to learn the details of the configuration file.


## Basic workflows

### Proteomics

- **Generate the input files**: Check the [input files](#inputfiles) 
section for details

- **Quality Control**: if you are interested in performing only quality 
control analysis, run the following functions:

    - `artmsQualityControlEvidenceBasic()`: QC based on the `evidence.txt` file
    - `artmsQualityControlEvidenceExtended()`: based on the `evidence.txt` file
    - `artmsQualityControlSummaryExtended()`: based on the `summary.txt` file

- **Relative Quantification**: fill up the configuration file and run the 
following function:

  - `artmsQuantification(yaml_config_file = "config.yaml")` 
  ([here the details](#relquan))

- **Analysis of Quantifications**: performs annotations, clustering analysis,
PCA analysis, enrichment analysis by running the function 

  - `artmsAnalysisQuantifications()` ([here the details](#analysisquan))

- **Miscellaneous functions**: Check [below](#misc) to discover more useful functions
provided by the `artMS` package.


### Metabolomics (unstable)

`r Biocpkg("artMS")` also enables the relative quantification of untargeted 
polar metabolites using the alignment table generated by [MarkerView](https://sciex.com/products/software/markerview-software).
This means that the metabolites do not need to have an `ID`, as the 
`m/z` and `retention time` will be used as identifiers. Typical workflow:

- Run QC on the metabolomics dataset: `artmsQualityControlMetabolomics()`

- Relative quantification: `artmsQuantification()` (notice that a few options
must be changed in the config file before running the function)

Please, keep in mind that most of the functions available in artMS 
don't work for metabolomics data due to annotation issues 
(protein/gene ids are the primary ids for most 
of the functions). Check the [metabolomics section](#metabolomics) to find out
more.


<!--
(-----------------------------------------------------------------------------)
-->


# REQUIRED INPUT FILES

<a name="inputfiles" />

***IMPORTANT***

Before you begin, please, set the working directory. artMS will work from 
that working directory. For example:

```
setwd("/path/to/my/working_directory/project_proteomics/")
```

Most of the artMS functions will create sub-folders to output files assuming that this working directory has been set and will try to find all the required files in this working directory.


## Input files

Three basic (tab-delimited) files are required to perform the full pack of 
operations:

### `evidence.txt`

The output of the quantitative proteomics software package 
[MaxQuant](http://www.biochem.mpg.de/5111795/maxquant). It combines all the 
information about the identified peptides.

### `keys.txt`

Tab delimited file generated by the user. It summarizes the experimental 
design of the evidence file. `artMS` merges the `keys.txt` and `evidence.txt` 
by the "RawFile" column. Each RawFile corresponds to a unique individual 
experimental technical replicate / biological replicate / Condition / Run. 

For any basic label-free proteomics experiment, the keys file **must contain** 
the following columns and rules:

- **RawFile**: The name of the RAW-file for which the mass spectral data was 
derived from.
- **IsotopeLabelType**: `'L'` for label free experiments (`'H'` will be used 
for SILAC experiments, [see below](#silac))
- **Condition**: The conditions names must follow these rules:
    - Use only letters (A - Z, both uppercase and 
    lowercase) and numbers (0 - 9). The only special character allowed 
    is underscore (`_`).
    - **Very important: A condition name cannot begin with a number 
    (R limitation)**.
- **BioReplicate**: biological replicate number. It is based on the condition 
name. Use as prefix the corresponding  `Condition` name, and add as suffix 
`dash (-)` plus the biological replicate number. 
For example, if condition `H1N1_06H` has too biological replicates, 
name them `H1N1_06H-1` and `H1N1_06H-2`
- **Run**: a **unique number** for all the MS runs (from 1 to the total number 
of raw files). It will be especially useful when having technical replicates. 
For example, in the table below, there are 2 technical replicates of the same 
biological replicate (Cal_33-1, technical replicates 1 and 2). 
A special case is SILAC experiments (H and L label are run simultaneously. 
See [below](#silac) to find out more)

**RawFile**|**IsotopeLabelType**|**Condition**|**BioReplicate**|**Run**
-----|-----|-----|-----|-----
qx006145|L|Cal_33|Cal_33-1|1
qx006146|L|Cal_33|Cal_33-2|2
qx006151|L|HSC6|HSC6-1|3
qx006152|L|HSC6|HSC6-2|4

For more examples, check the artMS data object `artms_data_ph_keys`

*Tip*: it is recommended to use Microsoft Excel (OpenOffice Cal / or similar) 
to generate the keys file. *Do not forget* to choose the 
*format = Tab Delimited Text (.txt)* when saving the file 
(use *save as* option)

### `contrast.txt`

The comparisons between conditions that the user wants to quantify.

- Example #1: the comparison for the keys described above would be:

```
HSC6-Cal_33
```

- Example #2: let's quantify changes in protein abundance between wild type 
(`WT_A549`) relative to two additional experimental conditions with drugs 
(`WT_DRUG_A` and `WT_DRUG_B`), but also changes in protein abundance between
`DRUG_A` and `DRUG_B`, the contrast file would look like this:

```
WT_DRUG_A-WT_A549
WT_DRUG_B-WT_A549
WT_DRUG_A-WT_DRUG_B
```

**Requirements**: 

- The two conditions to be compared must be separated by a dash symbol (`-`),
and only one dash symbol is allowed, i.e., only one comparison per line.

As a result of the quantification, the condition on the left will take the 
positive log2FC sign -if the protein is more abundant in condition 
on the left (numerator), and the condition on the right the negative log2FC -if a 
protein is more abundant in condition on the right term (denominator).


**Example of wrong comparisons**

Only condition names are allowed. Individual Bioreplicates cannot be compared.
For example, this is wrong:

```
# WRONG:
HSC6-Cal_33-1
```







<a name="configfile" />

## The artMS configuration file

---

***IMPORTANT***

Before you begin, please, set the working directory. artMS will work from 
that working directory. For example:

```
setwd("/path/to/my/working_directory/project_proteomics/")
```

Most of the artMS functions will create sub-folders to output files assuming that this working directory has been set and will try to find all the required files in this working directory.

---

The configuration file (in `yaml` format) contains a variety of options 
available for the QC, quantification, and annotations performed by `artMS`.

To generate a sample configuration file, go to the project folder 
(`setwd(/path/to/my/working_directory/project_proteomics/)`) and execute:

```{r, eval = FALSE}
library(artMS)
artmsWriteConfigYamlFile(config_file_name = "my_config.yaml")
```

Open the `my_config.yaml` file with your favorite editor (RStudio for example). 

***Note: Although the configuration file might look complex, 
the default options work very well***.

The configuration (`yaml`) file contains the following sections:

### Section: `files`

Assuming that your working directory (e.g. `setwd(/path/to/my/working_directory/project_proteomics/)`) has the following structure:

```
`-- data
    |-- projectx-contrasts.txt
    |-- projectx-evidence.txt
    `-- projectx-keys.txt
```

The `files` section of the configuration file should look like this:

```
files :
  evidence : data/projectx-evidence.txt
  keys : data/projectx-keys.txt
  contrasts : data/projectx-contrast.txt
  summary: data/projectx-summary.txt # Optional
  output : results_folder/projectx--results.txt # this will be created
```

Notice that in this example all the input files are located in the `data/` folder, however, for the results file, an extra folder has been added in the `output` section of the configuration file example (`results_folder`): artMS will create that folder structure (no need to create it before hand) and will save the results files (and all the extra outputs) in that folder. This means that you could write another completely different folder for the output (e.g. "`results_folder2/other-name-results.txt`") and artMS will create the folder for you.

---

### Section: `qc`

```
qc:
  basic: 1 # 1 = yes; 0 = no
  extended: 1 # 1 = yes; 0 = no
  extendedSummary: 0 # 1 = yes; 0 = no
```

Select to perform both 'basic' and 'extended' quality control based on the
`evidence.txt` file or 'extendedSummary' based on the `summary.txt` file.
[Read below](#qualitycontrol) 
to find out more about the details of each type of analysis.


### Section: `data`

```
data:
  enabled : 1 # 1 = yes; 0 = no
  silac: 
    enabled : 0 # 1 for SILAC experiments
  filters: 
    enabled : 1
    contaminants : 1
    protein_groups : remove # remove, keep
    modifications : AB # PH, UB, AC, AB, APMS
  sample_plots : 1 # correlation plots
```

Let's break it down `data`:

- `enabled : 1`: to pre-process the data provided in the *files* section. 
`0`: won't process the data (and a pre-generated MSstats file will be expected)

- `silac`:
    - `enabled : 1`: check if the files belong to a SILAC experiment. 
    See **Special case: SILAC** [below](#silac) for details
    - `enabled : 0`: no silac experiment (default)

- `filters`:
    - `enabled : 1` Enables filtering (this section)
    - `contaminants : 1` Removes contaminants (`CON__` and `REV__` 
    labeled by MaxQuant). To keep contaminats: `0`
    - `protein_groups : remove` choose whether `remove` or `keep` 
    protein groups
    -  `modifications : ` select if a PTM proteomics experiment, i.e.:
       + `AB`: Global protein abundant (default), i.e, no-modification
       + `PH`, `UB`, or `AC`: shortcuts for the most frequent posttranslational modifications
       + `PTM:XXX:yy`: User defined PTM, i.e. this is another way to select any PTM supported by MaxQuant, including the most frequent PTM. Replace `XXX` with 1 or more 1-letter amino acid codes on which to find modifications (all uppercase). Replace yy with modification name used within the evidence file (require lowercase characters). Example for phosphorylation: `PTM:STY:ph` will find modifications on aa S,T,Y with this example format `_AAGGAPS(ph)PPPPVR_`. This means that you could use the shortcut `modifications: PH` or `modifications: PTM:STY:ph` and both would analyze phosphorylation peptides
       + `APMS`: affinity purification mass-spectrometry  
       
- `sample_plots` 
    - `1` Generate correlation plots
    - `0` otherwise

---

### Section: `msstats`

Section updated in artMS version > 1.10.1. It allows the user to customize all the arguments of the MSstats [dataProcess](https://rdrr.io/bioc/MSstats/man/dataProcess.html) function for running the quantification. This new version of the `msstats` section is fully compatible with the previous version. However, we recommend to use the latest version of the configuration file.

```
msstats :
  enabled: 1 
  msstats_input:
  profilePlots: none 
  normalization_method: equalizeMedians  
  normalization_reference:
  summaryMethod: TMP 
  MBimpute: 1 
  feature_subset: all 
  n_top_feature: 3 
  logTrans: 2 
  remove_uninformative_feature_outlier: FALSE 
  min_feature_count: 2
  equalFeatureVar: TRUE
  censoredInt: NA
  remove50missing: FALSE
  fix_missing: NULL
  maxQuantileforCensored: 0.999
  use_log_file: TRUE
  append: FALSE
  log_file_path: NULL
```

Let's break down the most important arguments:

- `enabled : ` Choose `1` to run MSstats, `0` otherwise.
- `msstats_input :` leave it blank if MSstats will be run 
(previous `enabled : 1`). But if MSstats was already run and the 
`evidence-mss.txt` file is available, then choose `enabled : 0` and provide here the `evidence-mss.txt` file path/name
- `profilePlots :` Choose one of the following options:
    * `before` plot before normalization
    * `after` plot after normalization
    * `before-after`: recommended, although computational expensive
    * `none` no normalization plots
- `normalization_method :` available options:
    - `equalizeMedians` (recommended)
    - `quantile`
    - `FALSE`: no normalization (not recommended)
    - `globalStandards` if selected, specified the reference protein in 
    `normalization_reference` (next SECTION)
- `normalization_reference :` provide a protein id if and only if `normalization_method: globalStandards` is selected 
as the `normalization_method` (above), otherwise leave blank. If multiple protein IDs are used for normalization, then provide them comma separated (for example `normalization_reference: Q86U42, O75822`)
- `summaryMethod :` `TMP`(default) means Tukey's median polish, which is robust estimation method. `linear` uses linear mixed model. `logOfSum` conducts log2 (sum of intensities) per run.
- `censoredInt :` 
    - `NA` (default) Missing values are censored or at random. 'NA' assumes 
    that all 'NA's in 'Intensity' column are censored. 
    - `0` uses zero intensities as censored intensity. In this case, 
    NA intensities are missing at random. The output from Skyline should use 
    `0`. Null assumes that all `NA` intensities are randomly missing.
- `MBimpute :` 
    - `TRUE` only for `summaryMethod="TMP"` and `censoredInt='NA'` or `0`. 
    TRUE (default) imputes 'NA' or '0' (depending on censoredInt option) by 
    Accelerated failure model. 
    - `FALSE` uses the values assigned by cutoffCensored.

For all the other parameters, please, check the documentation for the [dataProcess](https://rdrr.io/bioc/MSstats/man/dataProcess.html) function of MSstats.

---

### Section: `output_extras`

```
  enabled : 1 # if 0, won't process anything on this section
  annotate :  
    enabled: 1 
    species : HUMAN
  plots:
    volcano: 1
    heatmap: 1
    LFC : -0.58 0.58 # Range of minimal log2fc
    FDR : 0.05 # adjusted p-value, false discovery rate
    heatmap_cluster_cols : 0
    heatmap_display : log2FC # log2FC or pvalue
```

Extra actions to perform based on the MSstats results, including 
*annotations* and *plots* (heatmaps and volcano plots). 
Let's break it down:

- `enabled : ` 1 (default) enables this section, 0 turns it off
- `annotate :` 
    * `enabled`: 1 (default), will generate a `-results-annotated.txt` 
file that includes `Gene` and `Protein.Name` (only for supported species)
    * `species`: The supported species are: HUMAN, MOUSE, ANOPHELES, 
    ARABIDOPSIS, BOVINE, WORM, CANINE, FLY, ZEBRAFISH, ECOLI_STRAIN_K12, 
    ECOLI_STRAIN_SAKAI, CHICKEN, RHESUS, MALARIA, CHIMP, RAT, YEAST, PIG, 
    XENOPUS
- `plots :` options for additional plots 
    * `volcano :` 1
    * `LFC :` log2 fold change cutoff (minimal negative and positive value)
    * `FDR :` false discovery rate cutoff for significance (recommended: 0.05)
    * `heatmap :` correlation plots
    * `heatmap_cluster_cols :` 1 perfoms clustering of columns, 
    0 (default) doesn't
    * `heatmap_display :` choose to display either `log2FC` or `pvalue`


<a name="fractions" />

## Special case: Protein fractionation

To handle protein fractionation experiments, one additional column "`Fraction`" must be added to the `keys.txt` file with the information about fractions. For example:

**Raw.file**|**IsotopeLabelType**|**Condition**|**BioReplicate**|**Run**|**Fraction**
:-----:|:-----:|:-----:|:-----:|:-----:|:-----:
S9524\_Fx1|L|AB|AB-1|1|1
S9524\_Fx2|L|AB|AB-1|1|2
S9524\_Fx3|L|AB|AB-1|1|3
S9524\_Fx4|L|AB|AB-1|1|4
S9524\_Fx5|L|AB|AB-1|1|5
S9524\_Fx6|L|AB|AB-1|1|6
S9524\_Fx7|L|AB|AB-1|1|7
S9524\_Fx8|L|AB|AB-1|1|8
S9524\_Fx9|L|AB|AB-1|1|9
S9524\_Fx10|L|AB|AB-1|1|10
S9525\_Fx1|L|AB|AB-2|2|1
S9525\_Fx2|L|AB|AB-2|2|2
S9525\_Fx3|L|AB|AB-2|2|3
S9525\_Fx4|L|AB|AB-2|2|4
S9525\_Fx5|L|AB|AB-2|2|5
S9525\_Fx6|L|AB|AB-2|2|6
S9525\_Fx7|L|AB|AB-2|2|7
S9525\_Fx8|L|AB|AB-2|2|8
S9525\_Fx9|L|AB|AB-2|2|9
S9525\_Fx10|L|AB|AB-2|2|10
S9526\_Fx1|L|AB|AB-3|3|1
S9526\_Fx2|L|AB|AB-3|3|2
S9526\_Fx3|L|AB|AB-3|3|3
S9526\_Fx4|L|AB|AB-3|3|4
S9526\_Fx5|L|AB|AB-3|3|5
S9526\_Fx6|L|AB|AB-3|3|6
S9526\_Fx7|L|AB|AB-3|3|7
S9526\_Fx8|L|AB|AB-3|3|8
S9526\_Fx9|L|AB|AB-3|3|9
S9526\_Fx10|L|AB|AB-3|3|10

Deprecated: In previous versions of artMS (v <= 1.9), the `config.yaml` file contained an additional *fractions* section that had to be activated as follow:

```
fractions: 
  enabled : 1 # 1 for protein fractions, 0 otherwise
```

This option is not longer required, as artMS will use the "`Fraction`" of the keys file to detect that multiple fractions are available.

<a name="silac" />

## Special case: SILAC

One of the most widely used techniques that enable relative protein 
quantification is *stable isotope labeling by amino acids in cell culture* 
(SILAC). The `keys.txt` file can capture the typical SILAC experiment. 
The following example shows a SILAC experiment with two conditions, 
two biological replicates, and two technical replicates:

**RawFile**|**IsotopeLabelType**|**Condition**|**BioReplicate**|**Run**
:-----:|:-----:|:-----:|:-----:|:-----:
QE20140321-01|H|iso|iso-1|1
QE20140321-02|H|iso|iso-1|2
QE20140321-04|L|iso|iso-2|3
QE20140321-05|L|iso|iso-2|4
QE20140321-01|L|iso\_M|iso\_M-1|1
QE20140321-02|L|iso\_M|iso\_M-1|2
QE20140321-04|H|iso\_M|iso\_M-2|3
QE20140321-05|H|iso\_M|iso\_M-2|4

It is also required to activate the *silac* option in the yaml file as follows:

```
silac: 
  enabled : 1 # 1 for SILAC experiments
```


<a name="qualitycontrol" />

<!--
Quality Control
-------------------------------------------------------------------------------
-->

# QUALITY CONTROL

`artMS` provides 3 functions to perform QC analyses.


<a name="qcbasicevidence" />

## Basic QC (`evidence.txt`-based)

The basic quality control analysis takes as input both the `evidence.txt`
and [keys.txt](#inputfiles) files
and generates several QC plots exploring different aspects of
the MS data. Run it as follows:

```{r, eval=FALSE}
artmsQualityControlEvidenceBasic(
  evidence_file = artms_data_ph_evidence,
  keys_file = artms_data_ph_keys,
  prot_exp = "PH")
```

The following `pdf` can be generated:

- **plotCORMAT (qcBasic_evidence.qcplot.CorrelationMatrix.pdf)** (default):  *Correlation matrix* for *technical replicates* (if available),  *biological replicates*, and conditions based on MS Intensity values
- **plotCORMAT (qcBasic_evidence.qcplot.CorrelationMatrixCluster.pdf** (default):  Clustered version of the *Correlation matrices* as for *.CorrelationMatrix.pdf*
- **plotINTMISC (qcBasic_evidence.qcplot.IntensityStats.pdf)**: several pages, 
including bar plots of 
*Total Sum of Intensities in BioReplicates*, 
*Total Sum of Intensities in Conditions*, 
*Total Peptide Counts in BioReplicates*,  
*Total Peptide Counts in conditions* 
grouped by categories (`CON`: contaminants, `PROT` peptides, `REV` reversed 
sequences used by MaxQuant to estimate the FDR); 
*Box plots* 
of MS Intensity values per biological replicates and conditions; 
*bar plots* 
of total intensity (excluding contaminants) by bioreplicates and conditions; 
Bar plots of *total feature counts* by bioreplicates and conditions.
- **plotPTMSTATS (qcBasic_evidence.qcplot.PTMStats.pdf)**: If any PTM is 
selected (`PH`, `UB`, `AC`, `PTM:##) an extra pdf 
file will be generated with stats related to the selected modification, 
including: *bar plot of peptide counts and intensities*, broken by 
`PTM/other` categories; bar plots of *total sum-up of MS intensity values* by 
other/PTM categories.
- **plotREPRO (qcBasic_evidence.qcplot.BasicReproducibility)**: correlation dot plot for 
all the combinations of biological replicates of every condition, based on MS Intensity values of features (peptide+charge)
- **plotINTDIST (qcBasic_evidence.qcplot.IntensityDistributions.pdf)**: 2 pages. *Box-dot plot* and *Jitter plot* of MS (raw) intensity values for each biological replicate.

Check `?artmsQualityControlEvidenceBasic()` to find out more options. Remember: by default, all the plots are printed to a `pdf` file by running:

```{r, eval=TRUE}
artmsQualityControlEvidenceBasic( 
    evidence_file = artms_data_ph_evidence,
    keys_file = artms_data_ph_keys,
    prot_exp = "PH", 
    plotPTMSTATS = TRUE,
    plotINTDIST = FALSE, plotREPRO = FALSE,
    plotCORMAT = FALSE, plotINTMISC = FALSE,
    printPDF = FALSE, verbose = FALSE)
```


<a name="qcextendedevidence" />

## Extended QC (`evidence.txt`-based)

It takes as [input](#inputfiles) the `evidence.txt` and `keys.txt`
files as follows:

```{r, eval=FALSE}
artmsQualityControlEvidenceExtended(
   evidence_file = artms_data_ph_evidence,
   keys_file = artms_data_ph_keys)
```

and generates the following QC plots:

- **plotCS (qcExtended_evidence.qcplot.ChargeState)**: charge state 
distribution of PSMs confidently identified in each BioReplicate.

- **plotIC (qcExtended_evidence.qcplot.PCA.pdf)**: pairwise intensity correlation and Principal Component Analysis (PCA)
   + Page 1 and 3: pairwise peptide and protein intensity correlation and 
scatter plot between any 2 BioReplicates, respectively. 
   + Page 2 and 4: principal component analysis at the intensity level 
   for both peptide and proteins, respectively. 

- **plotIONS (qcExtended_evidence.qcplot.Ions.pdf)**:
A peptide ion is defined in the context of m/z, in other words, a unique 
peptide sequence may give rise to multiple ions with different charge state 
and/or amino acid modification. 
   + Page 1: number of ions confidently identified in each BioReplicate 
   (top panel: non-contaminants, bottom: contaminants)
   + Page 2: mean number of peptide ions per condition with error bar 
   showing the standard error of the mean 
   (categories: non-contaminants / contaminants)
 
- **plotME (qcExtended_evidence.qcplot.MassError.pdf)**: 
Distribution of precursor error for all PSMs confidently 
identified in each BioReplicate.

- **plotMOCD (qcExtended_evidence.qcplot.MZ.pdf)**: 
Distribution of precursor mass-over-charge for all PSMs 
confidently identified in each BioReplicate.

- **plotPIO (qcExtended_evidence.qcplot.PepIonOversampling.pdf)**: 
   + Page 1: proportion of all peptide ions (including peptides 
matched across runs) fragmented once, twice and thrice or more. 
   + Page 2: proportion of peptide ions (with intensity detected) 
fragmented once, twice and thrice or more. 
   + Page 3: proportion of peptide ions (with intensity detected 
   and MS/MS identification) fragmented once, twice, and 3 or more

- **plotPEPDETECT (qcExtended_evidence.qcplot.PeptideDetection.pdf)**: 
frequency of peptide detection across BioReplicates by condition, 
showing the percentage of peptides detected once, twice, thrice,
and so on (based on the number of bioreplicates for each condition).

- **plotPEPICV (qcExtended_evidence.qcplot.PeptideIntensity.pdf)**: 
Peptide intensity coefficient of variance (CV) plot.  
The CV is calculated for each feature (peptide ion) identified in more 
than one replicate. 
   + Page 1 shows the distribution of CV's for each condition, while 
   + Page 2 shows the distribution of CV's within 4 bins of intensity 
   (i.e., 4 quantiles of average intensity).

- **plotPEPTIDES (qcExtended_evidence.qcplot.Peptides.pdf)**: 
peptide statistics plot.
   + Page 1: number of unique peptide sequences (disregards the charge 
state or amino acid modifications) confidently identified in each 
BioReplicate (top panel: non-contaminants, bottom: contaminants)
   + Page 2: mean number of peptides per condition with error bar showing the 
   standard error of the mean.

- **plotPEPTOVERLAP (qcExtended_evidence..qcplot.PeptidesOverlap.pdf)**: 
peptide overlaps across bioreplicates (page 1) and conditions (page 2)
   
- **plotPROTDETECT (qcExtended_evidence.qcplot.ProteinDetection.pdf)**: 
Protein detection frequency plot:
   + Page 1: protein group frequency of detection across BioReplicates for 
   each condition, showing the percentage of proteins detected per bioreplicates
   + Page 2: feature (peptide ion) intensity distribution within each 
   BioReplicate (potential contaminant proteins are plot separately). 
   + Page 3: density of feature intensity for different feature types 
   (i.e., MULTI-MSMS, MULTI-SECPEP).
   
- **plotPROTICV (qcExtended_evidence.qcplot.ProteinIntensityCV.pdf)**: 
protein intensity coefficient of variance (CV) plot. 
The CV is calculated for each protein (after summing the peptide intensities) 
identified in more than one replicate. 
   + Page 1: istribution of CV's for each condition
   + Page 2: distribution of CV's within 4 bins of 
   intensity (i.e., 4 quantiles of average intensity).

- **plotPROTEINS (qcExtended_evidence.qcplot.Proteins.pdf)**: protein statistics,
   + Page 1: number of protein groups confidently identified in each BioReplicate. 
   + Page 2: mean number of protein groups per condition with error bar showing 
   the standard error of the mean (categories: non-contaminants / contaminants)
   
- **plotPROTOVERLAP (qcExtended_evidence..qcplot.ProteinOverlap.pdf)**: 
Protein overlap across bioreplicates (page 1) and conditions (page 2) 

- **plotPSM (qcExtended_evidence.qcplot.PSM.pdf)** : Peptide-spectrum-matches (PSMs).
   + Page 1: number of PSMs confidently identified in each BioReplicate. 
   + Page 2: mean number of PSMs per condition with error bar showing the 
   standard error of the mean (categories: non-contaminants / contaminants)

- **plotIDoverlap (qcExtended_evidence.qcplot.ID-Overlap.pdf)**: heatmap of pairwise 
identification overlap: 
   + Page 1: peptide identification overlap, only peptides detected and 
   identified between any 2 BioReplicates
   + Page 2: protein identification overlap, only proteins with at least 1 
   peptide detected and identified between any 2 BioReplicates

- **plotSP (qcExtended_evidence.qcplot.SamplePrep.pdf)**: sample quality metrics.
   + Page 1: missing cleavage distribution of all peptides confidently 
   identified in each BioReplicate. 
   + Page 2: fraction of peptides with at least one methionine 
   oxidized in each BioReplicate. 

- **plotTYPE (qcExtended_evidence.qcplot.Type.pdf)**: identification type. 
MaxQuant classifies each peptide identification into different categories 
(e.g., MSMS, MULTI-MSMS, MULTI-SECPEP). This plot shows the distribution of 
identification type in each BioReplicate

Examples: printing `plotTYPE`, `plotPEPICV`, and `plotPCA` plots only:

```{r, eval = TRUE}
artmsQualityControlEvidenceExtended(
  evidence_file = artms_data_ph_evidence,
  keys_file = artms_data_ph_keys,
  plotPCA = TRUE,
  plotTYPE = TRUE,
  plotPEPTIDES = TRUE,
  plotPSM = FALSE,
  plotIONS = FALSE,
  plotPEPTOVERLAP = FALSE,
  plotPROTEINS = FALSE,
  plotPROTOVERLAP = FALSE,
  plotPIO = FALSE,
  plotCS = FALSE,
  plotME = FALSE,
  plotMOCD = FALSE,
  plotPEPICV = FALSE,
  plotPEPDETECT = FALSE,
  plotPROTICV = FALSE,
  plotPROTDETECT = FALSE,
  plotIDoverlap = FALSE,
  plotSP = FALSE,
  printPDF = FALSE,
  verbose = FALSE)
```


<a name="qcextendedsummary" />

## Extended QC (`summary.txt` based)

It requires two files:

- [keys.txt](#inputfiles) file
- MaxQuant `summary.txt` file. As described by MaxQuant's `table.pdf`, the 
summary file contains summary information for all the raw files processed 
with a single MaxQuant run, including statistics on the peak detection. 
The QC analysis of this file  gathers a quick overview on the 
quality of every RawFile based on this `summary.txt`. Run it as follows:

```{r, eval = FALSE}
artmsQualityControlSummaryExtended(summary_file = "summary.txt",
                                    keys_file = artms_data_ph_keys)
```

It generates the following `pdf` plots:

- **plotMS1SCANS (.qcplot.MS1scans.pdf)**: generates MS1 scan counts plot: 
Page 1 shows the number of MS1 scans in each BioReplicate. 
If replicates are present, Page 2 shows the mean number of MS1 scans 
per condition with error bar showing the standard error of the mean. 
If `isFractions` is `TRUE`, each fraction is a stack on the individual 
bar graphs. 

- **plotMS2 (.qcplot.MS2scans.pdf)**: generates MS2 scan counts plot: 
Page 1 shows the number of MSs scans in each BioReplicate. 
If replicates are present, Page 2 shows the mean number of MS1 scans per 
condition with error bar showing the standard error of the mean. 
If `isFractions` is `TRUE`, each fraction is a stack on the individual bar graphs. 

- **plotMSMS (.qcplot.MSMS.pdf)**: generates MS2 identification rate (%) plot: 
Page 1 shows the fraction of MS2 scans confidently identified in each 
BioReplicate. If replicates are present, Page 2 shows the mean rate of MS2 
scans confidently identified per condition with error bar showing the 
standard error of the mean. 
If `isFractions` `TRUE`, each fraction is a stack on the individual bar graphs.

- **plotISOTOPE (.qcplot.Isotope.pdf)**: generates Isotope Pattern counts plot: 
Page 1 shows the number of Isotope Patterns with charge greater than 1 in 
each BioReplicate. If replicates are present, Page 2 shows the mean number 
of Isotope Patterns with charge greater than 1 per condition with error bar 
showing the standard error of the mean. 
If `isFractions` `TRUE`, each fraction is a stack on the individual bar graphs.


<!--
RELATIVE QUANTIFICATION
-------------------------------------------------------------------------------
-->

<a name="relquan" />

# RELATIVE QUANTIFICATION

The relative quantification is a fundamental step in the analysis of MS data. 
`artMS` facilitates and simplifies the analysis using 
[MSstats](http://msstats.org/), a fantastic statistical package for the relative
quantification of Mass-Spectrometry based proteomics.

All the options and parameters required to run a relative quantification 
analysis using `MSstats` (in addition to other options) are summarized in 
`artMS` through a configuration file in `.yaml` format. Check the
[input-files](#inputfiles) section to find out more about each of the options.

Different types of proteomics experiments can be quantified including changes 
in global protein abundance (AB), affinity purification mass spectrometry 
(APMS), and different type of posttranslational modifications, 
including phosphorylation (PH), ubiquitination (UB), and acetylation (AC). 

`artMS` also enables the relative quantification of untargeted polar metabolites 
using the alignment table generated by 
[MarkerView](https://sciex.com/products/software/markerview-software).
This means that `artMS` does not require an ID for the metabolites,
as the m/z and retention time will be combined and used as identifiers.


<a name="globalab" />

## Quantification of Changes in Global Protein Abundance

The quantification of changes in protein abundance between different conditions
requires to fill up the following sections of the
[config file](#configfile):

```
files:
  evidence : /path/to/the/evidence.txt
  keys : /path/to/the/keys.txt
  contrasts : /path/to/the/contrast.txt
  output : /path/to/the/output/results_ptm_global/results.txt
  .
  .
  .
data:
  .
  .
  .
  filters:
    modifications : AB 
```

The remaining options can be left unmodified (and run the default parameters). 
Then run the following `artMS` function:

```
artmsQuantification(
  yaml_config_file = '/path/to/config/file/artms_ab_config.yaml')
```


<a name="globalptm" />

## Quantification of Changes in Global Phosphorylation, Ubiquitination, Acetylation (or any PTM)

***Warning: This quantification is only possible for experiments that have
used methods to enrich for the modified peptides (e.g. phosphorylation) prior to 
the mass spectrometry analysis.***

The **global PTM** quantification analysis 
calculates changes of the PTM at the *protein level*. 
This means that all the **modified** peptides for every protein are used to quantify changes in protein phosphorylation, ubiquitination, or acetylation between different conditions. 
The **site-specific** analysis ([explained next](#siteptm)) 
would quantify changes at the *site level*, i.e., each modified peptide for every PTM site
is / are quantified independently between the different conditions (one or more different peptides could be detected for the same PTM)

Only two sections need to be filled up using the **default** 
`r Biocpkg("artMS", vignette="input-files.html", label="configuration")` 
file:

```
files:
  evidence : /path/to/the/evidence.txt
  keys : /path/to/the/keys.txt
  contrasts : /path/to/the/contrast.txt
  output : /path/to/the/output/results_ptm_global/results.txt
  .
  .
  .
data:
  .
  .
  .
  filters:
    modifications : PH
```

The remaining options can be left unmodified. 

Once the configuration `yaml` file is ready, run the following command:

```
artmsQuantification(
  yaml_config_file = '/path/to/config/file/artms_phglobal_config.yaml')
```


<a name="siteptm" />

## PTM-Site/Peptide-specific Quantification of Changes (PH, UB, AC)

***Warning: This quantification is only possible for experiments that have
used methods to enrich phosphopeptides or ubiquitinated peptides prior to 
the mass spectrometry analysis.***

Abbreviations:

- `PH` = Protein phosphorylation
- `UB` = Protein Ubiquitination
- `AC` = Protein Acetylation
- `PTM:XXX:yy` : User defined PTM (any PTM supported by MaxQuant). 
Replace XXX with 1 or more 1-letter amino acid codes on which to find modifications (all uppercase).  Replace yy with modification name used within the evidence file (require lowercase characters). Example: `PTM:STY:ph` will find modifications on aa S,T,Y with this example format `_AAGGAPS(ph)PPPPVR_`

The `site-specific` analysis quantifies changes at the modified peptide level.
This means that changes in every modified (PH, UB, AC, or PTM) peptide of a given protein
will be quantified individually. The caveat is that the proportion of missing 
values should increase in general relative to a typical non-PTM **global** analysis (protein global abundance quantification). 
Both **sites** and **global** ptm analysis are highly correlated due to the usually only 
one or two peptides drive the overall changes in PTMs for every protein.

To run a site/peptide specific analysis follow these steps:

1. **Important pre-processing step on the 
evidence file to enable the ptm-site/peptide-specific analysis**. 
This step takes any of the Proteins id columns selected by the user
(either `Leading razor protein`, `Leading protein`, or `Proteins`) 
and re-annotates it to incorporate the ptm-site/peptide-specific information.
By default, this function converts the column `Leading razor protein`.
This step is *computational expensive*, which means that it might take several
minutes to finish (depending on the size of the fasta database, evidence file,
computer power, etc)

It also requires the same reference proteome (fasta sequence database) used 
for the MaxQuant search.

*For phosphorylation*:

```
artmsProtein2SiteConversion(
  evidence_file = "/path/to/the/evidence.txt", 
  ref_proteome_file = "/path/to/the/reference_proteome.fasta", 
  output_file = "/path/to/the/output/ph-sites-evidence.txt", 
  mod_type = "PH")
```

As a result, the IDs in the "Leading razor protein" column will contain 
site/peptide-specific notation. For example:

Before: `P12345` 
After: `P12345_S23_S45`

*For ubiquitination*:

```
artmsProtein2SiteConversion(
  evidence_file = "/path/to/the/evidence.txt", 
  ref_proteome_file = "/path/to/the/reference_proteome.fasta", 
  output_file = "/path/to/the/output/ub-sites-evidence.txt", 
  mod_type = "UB")
```

*For acetylation*:

```
artmsProtein2SiteConversion(
  evidence_file = "/path/to/the/evidence.txt", 
  ref_proteome_file = "/path/to/the/reference_proteome.fasta", 
  output_file = "/path/to/the/output/ac-sites-evidence.txt", 
  mod_type = "AC")
```

*For all PTMS supported by MaxQuant (in addition to PH, AC, UB)*:

Example with PH and UB:

```
# Phosphopeptide in evidence file: `_AAGGAPS(ph)PPPPVR_`
artmsProtein2SiteConversion(
  evidence_file = "/path/to/the/evidence.txt", 
  ref_proteome_file = "/path/to/the/reference_proteome.fasta", 
  output_file = "/path/to/the/output/ac-sites-evidence.txt", 
  mod_type = "PTM:STY:(ph)")

# Ubiquitinated peptide: `_AAASK(gl)LGEFAK_`
artmsProtein2SiteConversion(
  evidence_file = "/path/to/the/evidence.txt", 
  ref_proteome_file = "/path/to/the/reference_proteome.fasta", 
  output_file = "/path/to/the/output/ac-sites-evidence.txt", 
  mod_type = "PTM:K:(gl)")
```

**Tip: How to re-annotate all the Protein columns on the same file**.

By default, `artmsProtein2SiteConversion` doesn't allow to overwrite the 
`evidence.txt` file for security reasons (you don't want to lose the 
evidence file if something goes wrong). To overwrite the evidence file the
argument `overwrite_evidence` must be turned on (`overwrite_evidence = TRUE`).

If the `column_name` argument is not used, `artmsProtein2SiteConversion`
converts the `Leading razor protein` column, which is used in the
quantification step when `protein_groups : remove` is selected (default). However,
if `protein_groups : keep` is used, `artMS` will use the `Proteins` column.
To convert the `Proteins` column to the site/peptide-specific notation, then 
add the argument `column_name = "Proteins"`.

To annotate both columns of the same file, first generate the 
"site-evidence.txt" file, and then use this same output file as the 
`evidence_file` and activate `overwrite.evidence = TRUE`. 

In summary, to annotate both the "Leading razor protein" and `Proteins` columns
follow these steps:

```
# Convert 'Leading razor protein' evidence's file column
artmsProtein2SiteConversion(
  evidence_file = "/path/to/the/evidence.txt", # ORIGINAL
  column_name = "Leading razor protein",
  ref_proteome_file = "/path/to/the/reference_proteome.fasta", 
  output_file = "/path/to/the/phsites-evidence.txt", # SITES VERSION
  mod_type = "PH")

# Convert 'Proteins' evidence's file column
artmsProtein2SiteConversion(
  evidence_file = "/path/to/the/phsites-evidence.txt", # <- USE SITES VERSION
  column_name = "Proteins",
  overwrite_evidence = TRUE, # <--- TURN ON
  ref_proteome_file = "/path/to/the/reference_proteome.fasta", 
  output_file = "/path/to/the/phsites-evidence.txt", # <- SITES VERSION
  mod_type = "PH")
```


2. Generate a new configuration file (`phsites_config.yaml` or 
`ubsites_config.yaml`) as [explained above](#configfile), but using the "new" 
`sites-evidence.txt` file instead of the original
`evidence.txt` file:

```
files:
  evidence : /path/to/the/evidence-site.txt
  keys : /path/to/the/keys.txt
  contrasts : /path/to/the/contrast.txt
  output : /path/to/the/output/results_ptmSITES/sites-results.txt # <- this one
  .
  .
  .
data:
  .
  .
  .
  filters:
    modifications : PH # <- Don't forget this one.
```

Once the new `yaml` file has been created, execute:

```
artmsQuantification(
  yaml_config_file = '/path/to/config/file/phsites_config.yaml')
```

## Output files

The files generated after succesfully running `artmsQuantification` are 
(based on
[MSstats documentation](http://msstats.org/wp-content/uploads/2017/01/MSstats_v3.7.3_manual.pdf)):

### TXT (tab delimited) files 

- **results.txt**: the results of the quantification in long format, i.e., each
line is a protein for each comparison. It includes the following columns:
    * `Protein`: Protein ID
    * `Label`: comparison (from contrast.txt)
    * `log2FC`: log2 fold change
    * `SE`: standard error
    * `Tvalue`: test statistic of the Student test
    * `DF`: degree of freedom of the Student test
    * `pvalue`: raw p-values
    * `adj.pvalue`: p-values adjusted among all the proteins in the specific 
    comparison using the approach by Benjamini and Hochberg 
    * `issue`: shows if there is any issue for inference in corresponding 
    protein and comparison, for example, OneConditionMissing or CompleteMissing.
    * `MissingPercentage`: percentage of random and censored missing 
    in the corresponding run and protein out of the total number of 
    feature in the corresponding protein.
    * `ImputationPercentage`: percentage of imputation
    * Note:  If a protein is completely missed in one of the conditions of 
    a given comparison, it would get the flag `OneConditionMission` with 
    `adj.pvalue=0` and `log2FC=Inf` or `-Inf` even though `pvalue=NA`. 
    For example, if for the comparison `Condition A - Condition B` 
    one protein is completely missed for condition B, then 
    `log2FC = Inf` and `adj.pvalue = 0`. 
    `SE`, `Tvalue`, and `pvalue` will all be `NA`.

- **results-annotated.txt**: same as `results.txt` but 3 more columns of
annotations, i.e., `Gene`, `ProteinName`, and `EntrezID`
- Different outputs from the MSstats `dataProcess` and `quantification` step (check 
[MSstats documentation](http://msstats.org/msstats-2/) to find out more), 
including:
    * `results_ModelQC.txt`
    * `results-mss-sampleQuant.txt`
    * `results-mss-groupQuant.txt`
    * `results-mss-FeatureLevelData.txt`
    * `results-mss-ProteinLevelData.txt`
    
- Sample size and power calculations:
    * `results_sampleSize.txt`
    * `results_experimentPower.txt`

### Plots (pdf)

- `results-heatmap.pdf`
- `results-peptidecounts-perBait.pdf`
- `results-peptidecounts.pdf`
- `results-sign.pdf`
- `results-volcano.pdf`


<a name="analysisquan" />

# ANALYSIS OF QUANTIFICATIONS

Before running this function, the following packages must be installed on your system:

- From bioconductor: 

```
BiocManager::install(c("ComplexHeatmap", "org.Mm.eg.db"))
```

- From CRAN:
```
install.packages(c("factoextra", "FactoMineR", "gProfileR", "PerformanceAnalytics"))
```

`artmsAnalysisQuantifications()` performs a comprehensive analysis of the 
quantifications outputs obtained from the function 
[artmsQuantification()](#relquan). It includes:

- Annotations
- Summary files in different formats (xls, txt) and shapes (long, wide)
- Basic Imputation (see *Notes on Imputation* and input option **`mnbr`** below)
- Numerous summary plots
- Enrichment analysis using 
[Gprofiler](https://cran.r-project.org/web/packages/gProfileR/index.html)
- PCA of protein abundance
- PCA of quantifications
- Clustering analysis

## Inputs

It takes as input two files generated from the previous quantification step 
([artmsQuantification()](#relquan))

- `-results.txt` : MSstats quantification results
- `-results_ModelQC.txt` : MSstats abundance values. It will be used to extract details about reproducibility.

To run this analysis:

1. Set as the working directory the folder with the results obtained from 
`artmsQuantification()`.

```
setwd('~/path/to/the/results_quantification/')
```

And then run the following function (e.g., for a protein abundance 
"AB" experiment)

```
artmsAnalysisQuantifications(log2fc_file = "ab-results.txt",
                              modelqc_file = "ab-results_ModelQC.txt",
                              species = "human",
                              output_dir = "AnalysisQuantifications")
```

A few comments on the available options for `artmsAnalysisQuantifications`:

- `isPTM`: two options:
   + `"noptm"`: use for protein abundance (`AB`), Affinity Purification-Mass 
Spectrometry (`APMS`), and global analysis of posttranslational modifications 
(`PH`, `UB`, `AC`) use the option .
   + `"ptmsites"`: use for site specific PTM analysis.
- `species`: this downstream analysis supports (for now) `"human"` and 
`"mouse"` only.
- `outliers`: outliers can be kept (default) or could be removed from the abundance data. Options:
   + `keep`: keeps the outliers 
   + `iqr`: removes any outlier outside +/- 6 x interquartile range from the mean (recommended) 
   + `std`: it removes any outliers outside +/- 6 x the standard deviation from the mean
- `enrich`: If `TRUE`, it will perform enrichment analysis using `gProfileR`
- `isBackground`. If `enrich = TRUE`, the user can provide a background 
gene list (add the file path as well)
- `mnbr`: **PARAMETER FOR NAIVE IMPUTATION**. 
Minimal number of biological replicates for "naive 
imputation" and filtering. Default: `mnbr = 2`. 
*Details*:
Intensity values for proteins/PTMs that are completely missed in one 
of the two conditions compared ("condition A"), but are found in 
at least 2 biological replicates (`mnbr = 2`) of the other "condition B", 
are imputed (values artificially assigned) and the log2FC values calculated. 
The goal is to keep those proteins/PTMs that are consistently found in 
one of the two conditions (in this case "condition B") and facilitate the 
inclusion in downstream analysis (if wished). The imputed intensity values 
are sampled from the lowest intensity values detected in the experiment, 
and (**WARNING**) the p-values are just randomly assigned between 
0.05 and 0.01 for illustration purposes (when generating a volcano 
plot with the output of `artmsAnalysisQuantifications`) or to include 
them when making a cutoff of `p-value < 0.05` for enrichment analysis or similar.
**CAUTION**: `mnbr` would also add the constraint that any protein 
must be identified in at least `nmbr` biological replicates of the 
**same** condition or it will be filtered out. That is,
if `mnbr = 2`, a protein found in two conditions but only in one
biological replicate in each of them, it would be removed.

## Outputs

**Summary file (`summary.xlsx`)**

*Reminder*: for any given relative quantification, as for example WT-Mutant:

- Proteins with positive log2fc values (i.e. `log2fc > 0`) are more abundant
in the condition on the left / numerator (WT)
- Proteins with negative log2fc values (i.e. `log2fc < 0`) are more abundant
in the condition on the right / denominator (Mutant)

The summary excel file (`results-summary.xlsx`) gathers several tabs:

- `log2fcImputed`: includes quantitative results. 
   +  Self-defining columns: *Protein*, *Gene*, *ProteinName*, *EntrezID*, *Comparison*, 
*log2FC*, *pvalue*, *adj.pvalue*. 
   + Not self-defining columns: 
      - *imputed* (`yes/no`) indicates whether the iLog2FC value has been imputed according to the `nmbr` criteria (see above)
      - *iLog2FC*, *iPvalue*: in addition to the model based log2fc and pvalues, 
these columns include imputed log2fc and pvalues (when completely missed 
in one of the conditions), followed by as many columns as the number of 
conditions, indicating the number of biological replicates in which each 
protein/ptmsite was identified.
      - *CMA* stands for "Condition Most Abundant", i.e. condition in which each protein was found more abundantly (based on the log2fc).
- `wide_iLog2fc`: log2fc values (including imputed values) in wide format, i.e., 
each row is a unique protein/ptmsite. The columns shows the values for each 
of the comparisons.
- `wide_iPvalue`: same as before, but for pvalues (including imputed)
- `enrichALL`: enrichment analysis using GProfileR for all the proteins changing 
significantly in any direction (ab(log2fc) > 0 and pvalue < 0.05)
- `enrich-MACpos`: enrichment of only the positive significant changes 
(log2fc > 1, pvalue < 0.05)
- `enirch-MACneg`: enrichment of only the negative significant changes 
(log2fc < -1, pvalue < 0.05)
- `enMACallCorum, enMACposCorum, enMACnegCorum`: same as above but only for 
protein complex enrichment analysis (based on CORUM)


**Text files**

- `results-log2fc-long.txt`: same as the `log2fcImputed` tab from the 
summary file
- `results-log2fc-wide.txt`: wide version (i.e., each row is an individual 
protein) of pvalues and adj.pvalues for each comparison


**Gene Enrichment analysis**: enrichment analysis only supported for 
human and mouse. Check the 
[GprofileR documentation](https://cran.r-project.org/web/packages/gProfileR/index.html) 
to find out more about the details:

- `results-enrich-MAC-allsignificants.txt`: all significant changes 
(abs(log2fc) > 1 & pvalue < 0.05)
- `results-enrich-MAC-positives.txt`: only positive significant changes
(log2fc > 1 & pvalue < 0.05)
- `results-enrich-MAC-negatives.txt`: all significant changes (based on p-value
only)


**Protein Complex Enrichment analysis (based on CORUM)**

- `results-enrich-MAC-allsignificants-corum.txt`
- `results-enrich-MAC-positives-corum.txt`
- `results-enrich-MAC-negatives-corum.txt`

- `results-enrich-MAC-allsignificants-corum.pdf`
- `results-enrich-MAC-negatives-corum.pdf`
- `results-enrich-MAC-positives-corum.pdf`


**Clustering**

- `results.clustering.log2fc.all-overview.pdf`
- `results.clustering.log2fc.all-zoom.pdf`
- `results.clustering.log2fcSign.all-overview.pdf`
- `results.clustering.log2fcSign.all-zoom.pdf`


- `results.log2fc-clusterheatmap-enriched.txt`
- `results.log2fc-clusterheatmap.txt`
- `results.log2fc-clusterheatmap.pdf`
- `results.log2fc-clusters.pdf`


**Correlations**

- `results.correlationConditions.pdf`
- `results.reproducibilityAbundance.pdf`
- `results.correlationQuantifications.pdf`
- `results.log2fc-corr.pdf`

**Miscellaneous**

- `results.relativeABUNDANCE.pdf`

- `results.distributions.pdf`
- `results.distributionsFil.pdf`
- `results.imputation.pdf`
- `results.TotalQuantifications.pdf`



**PCA**

***Based on relative abundance***

- `results-pca-pca01.pdf`
- `results-pca-pca02.pdf`
- `results-pca-pca03.pdf`
- `results-pca-pca04.pdf`


***Based on significant changes***

- `results.log2fc-dendro.pdf`
- `results.log2fc-individuals-pca.pdf`


<!--
MISCELLANEOUS FUNCTIONS
-------------------------------------------------------------------------------
-->


<a name="misc" />

# MISCELLANEOUS FUNCTIONS

`artMS` also provides a number of very handy functions.


## Annotate data.frame with Gene Symbol, Name, ENTREZ based on Uniprot IDs

Takes the given `columnid` (of Uniprot IDs) from the input data.frame,
and map the gene symbol, name, and entre id 
(source: [bioconductor annotation packages](https://bioconductor.org/packages/3.8/data/annotation/))

```
# This example adds annotations to the evidence file available in
# artMS, based on the column 'Proteins'.

evidence_anno <- artmsAnnotationUniprot(x = artms_data_ph_evidence,
                                        columnid = 'Proteins',
                                        species = 'human')
```


## Average Intensity, RT, CR

Taking as input the evidence file, it will summarize and return back 
the average intensity, average retention time, and the average calibrated
retention time for each protein. If a list of proteins is provided, then only 
those proteins will be summarized and returned. Check `?artmsAvgIntensityRT()` 
to find out more options.


```
artmsAvgIntensityRT(evidence_file = '/path/to/the/evidence.txt)
```

## Change column name

Changes a given column name in the input data.frame

```{r, eval = FALSE}
artms_data_ph_evidence <- artmsChangeColumnName(
                               dataset = artms_data_ph_evidence,
                               oldname = "Phospho..STY.",
                               newname = "PH_STY")
```

## Individual abundance dot plots

Protein abundance dot plots for each unique uniprot id. It can take a long time

```
artmsDataPlots(input_file = "results/ab-results-mss-normalized.txt",
               output_file = "results/ab-results-mss-normalized.pdf")
```

## Enrichment analysis function

Enrichment analysis based on a data.frame with `Gene` and `Comparison`/`Label` 
protein (i.e, typical MSstats results)

```
# The data must be annotated (Protein and Gene columns)
data_annotated <- artmsAnnotationUniprot(
                      x = artms_data_ph_msstats_results,
                      columnid = "Protein",
                      species = "human")
# And then the enrichment
enrich_set <- artmsEnrichLog2fc(
                   dataset = data_annotated,
                   species = "human",
                   background = unique(data_annotated$Gene), 
                   verbose = FALSE)
```

## Enrichment analysis using gProfileR

Function that simplifies enrichment analysis using gProfileR

```
# annotate the MSstats results to get the Gene name
data_annotated <- artmsAnnotationUniprot(
                                     x = artms_data_ph_msstats_results,
                                     columnid = "Protein",
                                     species = "human")

# Filter the list of genes with a log2fc > 2
filtered_data <- 
unique(data_annotated$Gene[which(data_annotated$log2FC > 2)])

# And perform enrichment analysis
data_annotated_enrich <- artmsEnrichProfiler(
                                   x = filtered_data,
                                   categorySource = c('KEGG'),
                                   species = "hsapiens",
                                   background = unique(data_annotated$Gene))
```


## MaxQuant evidence file to SAINTexpress format

Converts the MaxQuant evidence file to the 3 required files by [SAINTexpress](http://saint-apms.sourceforge.net/Main.html). Choose one of the 
following quantitative MS metrics:

- MS spectral counts (use msspc)
- MS intensities (use msint)

```
artmsEvidenceToSaintExpress(evidence_file = "/path/to/evidence.txt", 
                            keys_file = "/path/to/keys.txt", 
                            ref_proteome_file = "/path/to/org.proteome.fasta")
```


## MaxQuant evidence file to SAINTq format

Converts the MaxQuant evidence file to the required files by [SAINTq](http://saint-apms.sourceforge.net/Main.html). The user can filter 
based on either peptides with spectral counts (use `msspc`) or all the peptides
(use `all`) for the analysis. The quantitative metric can be also chosen 
(either MS intensity or spectral counts)

```
artmsEvidenceToSAINTq(evidence_file = "/path/to/evidence.txt", 
                      keys_file = "/path/to/keys.txt", 
                      output_dir = "saintq_input_files")
```

## Generate Phosfate input file

It generates the Phosfate input file from the `imputedL2fcExtended.txt` file 
resulting from running the `artmsAnalysisQuantifications()` on a ph-site 
quantification (see above). Notice that the only species suported by PHOTON 
is humans.

```
artmsPhosfateOutput(inputFile = "your-imputedL2fcExtended.txt")
```

## Generate Photon input file

It generates the Photon input file from the `imputedL2fcExtended.txt` file 
resulting from running the `artmsAnalysisQuantifications()` on a ph-site 
quantification (see above). Please, notice that the only species suported by 
PHOTON is humans.

```
artmsPhotonOutput(inputFile = "your-imputedL2fcExtended.txt")
```


## Remove contaminants and empty proteins from the MaxQuant evidence file

Remove contaminants and erroneously identified 'reverse' sequences by MaxQuant,
in addition to empty protein ids

```
evidencefiltered <- artmsFilterEvidenceContaminants(x = artms_data_ph_evidence)
```


## Generate ph-site specific evidence file

Generate extended detailed ph-site file, where every line is a ph site instead 
of a peptide. Therefore, if one peptide has multiple ph sites it will be 
breaking down in multiple extra lines for each of the sites. 

```
artmsGeneratePhSiteExtended(df = dfobject, 
                            species = "mouse", 
                            ptmType = "ptmsites",
                            output_name = log2fc_file)
```



<!--
METABOLOMICS
-------------------------------------------------------------------------------
-->


<a name="metabolomics" />

# METABOLOMICS

`artMS` enables the relative quantification of untargeted polar metabolites 
using the alignment table generated by [MarkerView](https://sciex.com/products/software/markerview-software).
This means that the metabolites do not need to have an id in order to perform 
the quantification, as the m/z and retention time will be used as identifiers. 

[MarkerView](https://sciex.com/products/software/markerview-software) is an 
ABSciex software that supports the files generated by Analyst software 
(`.wiff`) used to run our specific mass 
spectrometer (ABSciex Triple TOF 5600+). 
It also supports `.t2d` files generated by the 
Applied Biosystems 4700/4800 MALDI-TOF. 

Markview is used to align mass spectrometry data from several 
samples for comparison. Using the import feature in the software, `.wiff` files 
(also `.t2d` MALDI-TOF files and tab-delimited `.txt` mass spectra data 
in mass-intensity format) are loaded for retention time alignment. 
Once the data files are selected, a series of windows will appear wherein 
peak finding, alignment, and filtering options can be entered and selected. 
These options include minimum spectral peak width, minimum retention time 
peak width, retention time and mass tolerance, and the ability to filter 
out peaks that do not appear in more than a user selected number of samples.

The alignment file is further processed and formatted to perform QC 
and relative quantification using the following `artMS` functions:

## Convert Metabolomics

Pre-process the markview `.txt` file to generate 
an "evidence-like" file by running:

```
artmsConvertMetabolomics(input_file = "markview-output.txt", 
                         out_file = "metabolomics-evidence.txt")
```

## QC Metabolomics

Perform quality control analysis on the metabolomics 
data by running: 


```
artmsQualityControlMetabolomics(evidence_file = "metabolomics-evidence.txt",
                                keys_file = "metabolomics-keys.txt")
```

It generates the following plots:

- `plotINTDIST.pdf` contains both *Box-dot plot* 
and *Jitter plot* of biological replicates based on MS (raw) 
intensity values.
- `plotREPRO.pdf` correlation dotplot for all the 
combinations of biological replicates of conditions, based on MS Intensity 
values using features (mz_rt+charge)
- `plotCORMAT.pdf`, includes up to 3 pdf files for 
technical replicates, biological replicates, and conditions. 
Each pdf file contains: 
    - *Correlation matrix* for all the biological replicates using 
MS Intensity values, 
    - *Clustering matrix* of the MS Intensities and correlation distribution 
    - *histogram* of the distribution of correlations
- `plotINTMISC.pdf` the pdf contains several pages, including 
bar plots of *Total Sum of Intensities in BioReplicates*, 
*Total Sum of Intensities in Conditions*, 
*Total Feature Counts in BioReplicates*, 
*Total Feature Counts in conditions* separated by categories 
(INT: has a intensity value NOINT: no intensity value ) 
*Box plots* of MS Intensity values per 
biological replicates and conditions; *bar plots* of total intensity 
by bioreplicates and conditions; Barplots of 
*total feature counts* by bioreplicates and conditions.


## Relative Quantification: 

The relative quantification is performed using 
`MSstats`. It requires a configuration file (`yaml` format, please check above).
A template can be generated by running:
`artmsWriteConfigYamlFile(config_file_name = "metab_config.yaml")`.
The relative quantification is performed by running:

```
artmsQuantification(yaml_config_file = "metabConfig.yaml")
```








<!--
TESTING FILES
-------------------------------------------------------------------------------
-->



# TESTING FILES

The artMS package provides the following testing datasets

**Phosphoproteomics dataset**: 
example dataset consisting of two head and neck cancer cell lines 
(conditions `"Cal33"` and `"HSC6"`), 2 biological 
replicates each). The number of peptides was reduced to 1/8 due to bioconductor 
limitations on data size.

- Evidence file: `artms_data_ph_evidence`
- Keys file: `artms_data_ph_keys`
- Results file: `artms_data_ph_msstats_results`: results after running 
`artmsQuantification()` on the reduced version

The full data set (2 conditions, 4 biological replicates) can be found at the 
following urls:

- `url_evidence <- 'http://kroganlab.ucsf.edu/artms/ph/evidence.txt'`
- `url_keys <- 'http://kroganlab.ucsf.edu/artms/ph/keys.txt'`

**Protein Complexes dataset**: downloaded (2017-08-01) from 
[CORUM](http://mips.helmholtz-muenchen.de/corum/) database  
and further enriched with annotations of mouse mitochondrial complexes 
not available at CORUM. Used for complex enrichment calculations.

- `artms_data_corum_mito_database`

**Pathogens Uniprot IDs**:

- `artms_data_pathogen_LPN`: *Legionella pneumophila philadelphia* 
(downloaded 2017-07-17)
- `artms_data_pathogen_TB`: *Mycobacterium tuberculosis* 
strain ATCC 35801 / TMC 107 / Erdman (downloaded 2018-04-01)

Check the individual help pages (e.g, `?artms_data_ph_evidence`) to find out 
more about them.


<!--
HELP
-------------------------------------------------------------------------------
-->


# HELP

Errors or warnings? try to update the package first (resinstall) just in case 
a newer version is already available fixing the issue. 

**Does the issue persist after reinstallation?** Then, please, 
<a href="https://github.com/biodavidjm/artMS/issues" target="_blank">submit your error as a new issue</a> at the official Github repository.

Any other inquiries: <artms.help@gmail.com>

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/annotations.R
\name{artmsMapUniprot2Entrez}
\alias{artmsMapUniprot2Entrez}
\title{Map GENE SYMBOL, NAME, AND ENTREZID to a vector of Uniprot IDS}
\usage{
artmsMapUniprot2Entrez(uniprotkb, species)
}
\arguments{
\item{uniprotkb}{(vector) Vector of UniprotKB IDs}

\item{species}{(char) The species name. Species currently supported
as part of artMS: check \code{?artmsIsSpeciesSupported()} to find out the
list of supported species`}
}
\value{
(data.frame) with ENTREZID and GENENAMES mapped on UniprotKB ids
}
\description{
Map GENE SYMBOL, NAME, AND ENTREZID to a vector of Uniprot IDS
}
\examples{
# Load an example with human proteins
exampleID <- c("Q6P996", "B1N8M6")
artmsMapUniprot2Entrez(uniprotkb = exampleID, 
                       species = "HUMAN")
}
\keyword{annotation,}
\keyword{ids}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MSstats_functions.R
\name{artmsSILACtoLong}
\alias{artmsSILACtoLong}
\title{Convert the SILAC evidence file to MSstats format}
\usage{
artmsSILACtoLong(evidence_file, output = NULL, verbose = TRUE)
}
\arguments{
\item{evidence_file}{(char) Text filepath to the evidence file}

\item{output}{(char) Text filepath of the output name. If NULL it does not
write the output}

\item{verbose}{(logical) \code{TRUE} (default) shows function messages}
}
\value{
(data.frame) with SILAC data processed for MSstats (and output file)
}
\description{
Converting the evidence file from a SILAC search to a format
compatible with MSstats. It basically modifies the Raw.files adding the
Heavy and Light label
}
\examples{
\dontrun{
evidence2silac <- artmsSILACtoLong(evidence_file = "silac.evicence.txt",
                                   output = "silac-evidence.txt")
}
}
\keyword{convert,}
\keyword{evidence}
\keyword{silac,}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{artms_data_corum_mito_database}
\alias{artms_data_corum_mito_database}
\title{CORUM Protein Complexes database use for complex enrichment analysis}
\format{
Tab delimited file.
\describe{
To find out more about the format and columns available at CORUM,
please visit this
\href{http://mips.helmholtz-muenchen.de/corum/}{link}
}
}
\usage{
artms_data_corum_mito_database
}
\description{
The list of protein complexes has been enriched with
mitochondria proteins from mouse, as described in this paper:

2018 - Ruchi Masand, Esther Paulo, Dongmei Wu , Yangmeng Wang,
Danielle L. Swaney, David Jimenez-Morales, Nevan J. Krogan, and Biao Wang
Proteome Imbalance of Mitochondrial Electron Transport Chain in Brown
Adipocytes Leads to Metabolic Benefits.
Cell Metab. 2018 Mar 06; 27(3):616-629.e4
}
\details{
LAST CORUM DOWNLOAD DATE: 2017-08-01
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/otherApps.R
\name{artmsPhosfateOutput}
\alias{artmsPhosfateOutput}
\title{Generate Phosfate Input file}
\usage{
artmsPhosfateOutput(inputFile, output_dir = ".", verbose = TRUE)
}
\arguments{
\item{inputFile}{(char) the \code{imputedL2fcExtended.txt} file name and location}

\item{output_dir}{(char) Name of the folder to output results
(Default: current directory. Recommended: \code{phosfate_input})}

\item{verbose}{(logical) \code{TRUE} (default) to show function messages}
}
\value{
Multiple output files (inputs of phosfate)
}
\description{
It takes as input the \code{imputedL2fcExtended.txt} results
generated by the \code{artmsAnalysisQuantifications()} function and generates
the \href{http://phosfate.com/}{Phosfate} input file (or data.frame)
Please, notice that the only species suported by Phosfate is humans.
}
\examples{
\dontrun{
artmsPhosfateOutput(inputFile)
}
}
\keyword{files}
\keyword{generate,}
\keyword{outputs,}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{artms_data_ph_contrast}
\alias{artms_data_ph_contrast}
\title{Contrast example for the PH dataset}
\format{
list with one comparison: "Cal33-HSC6"
}
\usage{
artms_data_ph_contrast
}
\description{
Contrast file with the relative quantification to be performed
for the two conditions available in the example dataset: "Cal33-HSC6".
See vignette for more details on how to prepare the contrast file.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/protein2SiteConversion.R
\name{artmsProtein2SiteConversion}
\alias{artmsProtein2SiteConversion}
\title{Converts the Protein ID column of the evidence
file selected by the user to mod-site-specific notation:
\code{ProteinID} to \code{ProteinID_AAnumber} notation}
\usage{
artmsProtein2SiteConversion(
  evidence_file,
  ref_proteome_file,
  column_name = c("Leading razor protein", "Leading proteins", "Proteins"),
  output_file,
  mod_type,
  overwrite_evidence = FALSE,
  verbose = TRUE
)
}
\arguments{
\item{evidence_file}{(char) The evidence file name and location}

\item{ref_proteome_file}{(char) The reference proteome used as database
to search the \code{evidence.txt} file with MaxQuant. It will be used to map the
modified peptide to the protein sequence and find the site location.
Therefore, it does not use the MaxQuant's \verb{Phospho (STY)Sites.txt}}

\item{column_name}{(char) The Protein Column Name to map. Options:
\itemize{
\item \verb{Leadind razor protein} (default)
\item \verb{Leading protein}
\item \code{Proteins}
It only supports Uniprot Entry IDs and RefSeq, but it might work for
other database IDs
}}

\item{output_file}{(char) Output file name
(\code{ptmsites-evidence.txt} recommended)}

\item{mod_type}{(char) The posttranslational modification. Options:
\itemize{
\item \code{UB}: Protein Ubiquitination
\item \code{PH}: Protein Phosphorylation
\item \code{AC}: Protein Acetylation
\item \code{PTM:XXX:yy} : User defined PTM. Replace XXX with 1 or more 1-letter amino
acid codes on which to find modifications (all uppercase).  Replace yy with
modification name used within the evidence file (require lowercase characters).
Example: \code{PTM:STY:ph} will find modifications on aa S,T,Y with this
format \verb{_AAGGAPS(ph)PPPPVR_}. This would be equivalent to \code{mod_type = PH}
}}

\item{overwrite_evidence}{(logical) if <output_file> is the same
as <evidence_file>, \code{overwrite_evidence = FALSE} (default) doesn't allow to
overwrite the evidence file. Otherwise, \code{overwrite_evidence = TRUE} allows
to overwrite the evidence_file (this option might be activated if the user
allows to use the same \code{ptm-sites-evidence.txt} file to re-annotate all
the Protein IDs columns)}

\item{verbose}{(logical) \code{TRUE} (default) shows function messages}
}
\value{
(file) Return a new evidence file with the specified Protein id
column modified by adding the sequence site location(s) + postranslational
modification(s) to the uniprot entry / refseq id.

Output ID examples: \code{A34890_ph3}; \code{Q64890_ph24_ph456};
\code{Q64890_ub34_ub129_ub234}; \code{Q64890_ac35}.
}
\description{
It enables the modified-peptide specific quantification by
converting the Protein column of the evidence file selected by the user
to an \code{ProteinID_AAnumber}notation.
In this way, each of the modified peptides can be quantified
independently across conditions.

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

WARNING: we have detected a version of MaxQuant (>1.6.3.0) outputs a`
"Modified sequence" column of the evidence file that has two important
changes for the annotation of phosphorylation:
\itemize{
\item Uses \code{p} instead of \code{(ph)}
\item The modified residue (i.e. \code{STY}) is the residue on the right of the \code{p},
instead of the residue to the left of \code{(ph)}, as usual.
We have introduced a modification to detect and address this issue, but
we advice the user to double check both the new evidence file with the
introduce new notation and the \code{-mapping.txt} file and check that there
are no NA values for the notation of phophopeptides.
}

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
}
\examples{
# Testing warning if files are not submitted. 
artmsProtein2SiteConversion(evidence_file = NULL, ref_proteome_file = NULL, 
output_file = NULL)
}
\keyword{ac}
\keyword{convert,}
\keyword{evidence,}
\keyword{ph,}
\keyword{ptm,}
\keyword{ub,}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/annotations.R
\name{artmsLeaveOnlyUniprotEntryID}
\alias{artmsLeaveOnlyUniprotEntryID}
\title{Leave only the Entry ID from a typical full Uniprot IDs in a
given column}
\usage{
artmsLeaveOnlyUniprotEntryID(x, columnid)
}
\arguments{
\item{x}{(data.frame) that contains the \code{columnid}}

\item{columnid}{(char) Column name with the full uniprot ids}
}
\value{
(data.frame) with only Entry IDs.
}
\description{
Downloading a Reference Uniprot fasta database includes several
Uniprot IDs for every protein. If the regular expression available in
Maxquant is not activated, the full id will be used in the Proteins,
Lead Protein, and Leading Razor Protein columns. This script leaves only the
Entry ID.

For example, values in a Protein column like this:

\verb{sp|P12345|Entry_name;sp|P54321|Entry_name2}

will be replace by

`P12345;P54321``
}
\examples{
# Example of data frame with full uniprot ids and sequences
p <- c("sp|A6NIE6|RN3P2_HUMAN;sp|Q9NYV6|RRN3_HUMAN", 
       "sp|A7E2V4|ZSWM8_HUMAN", 
       "sp|A5A6H4|ROA1_PANTR;sp|P09651|ROA1_HUMAN;sp|Q32P51|RA1L2_HUMAN", 
       "sp|A0FGR8|ESYT2_HUMAN")
s <- c("ALENDFFNSPPRK", "GWGSPGRPK", "SSGPYGGGGQYFAK", "VLVALASEELAK")
evidence <- data.frame(Proteins = p, Sequences = s, stringsAsFactors = FALSE)

# Replace the Proteins column with only Entry ids
evidence <- artmsLeaveOnlyUniprotEntryID(x = evidence, columnid = "Proteins")
}
\keyword{annotation,}
\keyword{ids}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MSstats_functions.R
\name{artmsSpectralCounts}
\alias{artmsSpectralCounts}
\title{Outputs the spectral counts from the MaxQuant evidence file.}
\usage{
artmsSpectralCounts(
  evidence_file,
  keys_file,
  output_file = NULL,
  verbose = TRUE
)
}
\arguments{
\item{evidence_file}{(char) Maxquant evidence file or data object}

\item{keys_file}{(char) Keys file with the experimental design or data object}

\item{output_file}{(char) Output file name (add \code{.txt} extension).
If \code{NULL} (default) it returns a data.frame object}

\item{verbose}{(logical) \code{TRUE} (default) shows function messages}
}
\value{
A txt file with biological replicates, protein id, and spectral
count columns
}
\description{
Outputs the spectral counts from the MaxQuant evidence file.
}
\examples{
summary_spectral_counts <- artmsSpectralCounts(
                                 evidence_file = artms_data_ph_evidence,
                                 keys_file = artms_data_ph_keys)
}
\keyword{evidence}
\keyword{spectral_counts,}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/qualityControlEvidenceBasic.R
\name{artmsQualityControlEvidenceBasic}
\alias{artmsQualityControlEvidenceBasic}
\title{Quality Control analysis of the MaxQuant evidence file}
\usage{
artmsQualityControlEvidenceBasic(
  evidence_file,
  keys_file,
  prot_exp = c("AB", "PH", "UB", "AC", "APMS", "PTM:XXX:yy"),
  output_dir = "qc_basic",
  output_name = "qcBasic_evidence",
  isSILAC = FALSE,
  plotINTDIST = FALSE,
  plotREPRO = FALSE,
  plotCORMAT = TRUE,
  plotINTMISC = TRUE,
  plotPTMSTATS = TRUE,
  printPDF = TRUE,
  verbose = TRUE
)
}
\arguments{
\item{evidence_file}{(char or data.frame) The evidence file path and name, or
data.frame}

\item{keys_file}{(char or data.frame) The keys file path and name or
data.frame}

\item{prot_exp}{(char) Proteomics experiment. 6 options available:
\itemize{
\item \code{APMS}: affinity purification mass spectrometry
\item \code{AB}: protein abundance
\item \code{PH}: protein phosphorylation
\item \code{UB}: protein ubiquitination (aka ubiquitylation)
\item \code{AC}: protein acetylation
\item \code{PTM:XXX:yy} : User defined PTM. Replace XXX with 1 or more 1-letter amino
acid codes on which to find modifications (all uppercase).  Replace yy with
modification name used within the evidence file (require lowercase characters).
Example for phosphorylation: \code{PTM:STY:ph} will find modifications on
aa S,T,Y with this example format \verb{_AAGGAPS(ph)PPPPVR_}. This means that
the user could select phosphorylation as \code{PH} or \code{PTM:STY:ph}
}}

\item{output_dir}{(char) Name for the folder to output the results plots.
Default is "qc_basic".}

\item{output_name}{(char) prefix output name (no extension).
Default: "qcBasic_evidence"}

\item{isSILAC}{if \code{TRUE} processes SILAC input files. Default is \code{FALSE}}

\item{plotINTDIST}{if \code{TRUE} plots both \emph{Box-dot plot}
and \emph{Jitter plot} of biological replicates based on MS (raw)
intensity values, otherwise \code{FALSE} (default)}

\item{plotREPRO}{if \code{TRUE} plots a correlation dotplot for all the
combinations of biological replicates of conditions, based on MS Intensity
values using features (peptide+charge). Otherwise \code{FALSE} (default)}

\item{plotCORMAT}{if \code{TRUE} (default) plots a
\itemize{
\item \emph{Correlation matrix} for all the biological replicates using
MS Intensity values,
\item \emph{Clustering matrix} of the MS Intensities
}}

\item{plotINTMISC}{if \code{TRUE} (default) plots several pages, including
bar plots of \emph{Total Sum of Intensities in BioReplicates},
\emph{Total Sum of Intensities in Conditions},
\emph{Total Peptide Counts in BioReplicates},
\emph{Total Peptide Counts in conditions} separated by categories:
\code{CON}: contaminants, \code{PROT} peptides, \code{REV} reversed sequences used by
MaxQuant to estimate the FDR; \emph{Box plots} of MS Intensity values per
biological replicates and conditions; \emph{bar plots} of total intensity
(excluding contaminants) by bioreplicates and conditions; Barplots of
\emph{total feature counts} by bioreplicates and conditions.}

\item{plotPTMSTATS}{IF \code{TRUE} (default) plots stats related to the
selected modification, including:
\emph{bar plot of peptide counts and intensities}, broken by \code{PTM/other}
categories; bar plots of \emph{total sum-up of MS intensity values} by
other/PTM categories.}

\item{printPDF}{If \code{TRUE} (default) prints out the pdfs. Warning: plot
objects are not returned due to the large number of them.}

\item{verbose}{(logical) \code{TRUE} (default) shows function messages}
}
\value{
Quality control files and plots
}
\description{
Quality Control analysis of the MaxQuant evidence file
}
\examples{
artmsQualityControlEvidenceBasic(evidence_file = artms_data_ph_evidence,
                                 keys_file = artms_data_ph_keys, 
                                 prot_exp =  "PH", 
                                 isSILAC = FALSE,
                                 plotINTDIST = FALSE,
                                 plotREPRO = TRUE,
                                 plotCORMAT = FALSE,
                                 plotINTMISC = FALSE,
                                 plotPTMSTATS = FALSE,
                                 printPDF = FALSE,
                                 verbose = FALSE)

# But we recommend the following test:
# 1. Go to a working directory: 
# setwd("/path/to/your/working/directory/")
# 2. Run the following command to print out all the pdf files
# artmsQualityControlEvidenceBasic(evidence_file = artms_data_ph_evidence,
#                                  keys_file = artms_data_ph_keys, 
#                                  prot_exp =  "PH")
# 3. Check your working directory and you should find pdf files with 
# all the QC plots
}
\keyword{QC,}
\keyword{control,}
\keyword{evidence}
\keyword{quality,}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/enrichments.R
\name{artmsEnrichProfiler}
\alias{artmsEnrichProfiler}
\title{Enrichment analysis using GprofileR}
\usage{
artmsEnrichProfiler(
  x,
  categorySource = c("GO"),
  species,
  background = NA,
  verbose = TRUE
)
}
\arguments{
\item{x}{(list, data.frame) List of protein ids. It can be anything:
either a list of ids, or you could also send a data.frame and it will find
the columns with the IDs. Is not cool? Multiple list can be also sent
simultaneously, as for example running:
\code{tmp <- split(enrichment$Gene, enrichment$cl_number, drop= TRUE)}}

\item{categorySource}{(vector) Resources providing the terms on which
the enrichment will be performed. The supported resources by gprofiler are:
\itemize{
\item GO (GO:BP, GO:MF, GO:CC): Gene Ontology (see more below)
\item KEGG: Biological pathways
\item REAC: Biological pathways (Reactome)
\item TF: Regulatory motifs in DNA (TRANSFAC TFBS)
\item MI: Regulatory motifs in DNA (miRBase microRNAs)
\item CORUM: protein complexes database
\item HP: Human Phenotype Ontology
\item HPA: Protein databases (Human Protein Atlas)
\item OMIM: Online Mendelian Inheritance in Man annotations:
\item BIOGRID: BioGRID protein-protein interactions
The type of annotations for Gene Ontology:
\item Inferred from experiment (IDA, IPI, IMP, IGI, IEP)
\item Direct assay (IDA) / Mutant phenotype (IMP]
\item Genetic interaction (IGI) / Physical interaction (IPI)
\item Traceable author (TAS) / Non-traceable author (NAS) /
Inferred by curator (IC)
\item Expression pattern (IEP) / Sequence or structural similarity (ISS)
/ Genomic context (IGC)
\item Biological aspect of ancestor (IBA) / Rapid divergence (IRD)
\item Reviewed computational analysis (RCA) / Electronic annotation (IEA)
\item No biological data (ND) / Not annotated or not in background (NA)
}}

\item{species}{(char) Specie code: Organism names are constructed by
concatenating the first letter of the name and the family name.
Example: human - ’hsapiens’, mouse - ’mmusculus’. Check gProfileR to find out
more about supported species.}

\item{background}{(vector) gene list to use as background for the enrichment
analysis. Default: \code{NA}}

\item{verbose}{(logical) \code{TRUE} (default) shows function messages}
}
\value{
The enrichment results as provided by gprofiler
}
\description{
This function simplifies the enrichment analysis performed by
the excellent tool GprofileR.
}
\details{
This function uses the following \code{gprofiler} arguments as default:
\itemize{
\item ordered_query = FALSE
\item significant = TRUE
\item exclude_iea = TRUE
\item underrep = FALSE
\item evcodes = FALSE
\item region_query = FALSE
\item max_p_value = 0.05
\item min_set_size = 0
\item max_set_size = 0
\item min_isect_size = 0
\item correction_method = "analytical" #Options: "gSCS", "fdr", "bonferroni"
\item hier_filtering = "none"
\item domain_size = "known" # annotated or known
\item numeric_ns = ""
\item png_fn = NULL
\item include_graph = TRUE
}
}
\examples{
\dontrun{
# annotate the MSstats results to get the Gene name
data_annotated <- artmsAnnotationUniprot(
                                     x = artms_data_ph_msstats_results,
                                     columnid = "Protein",
                                     species = "human")

# Filter the list of genes with a log2fc > 2
filtered_data <- 
unique(data_annotated$Gene[which(data_annotated$log2FC > 2)])

# And perform enrichment analysis
data_annotated_enrich <- artmsEnrichProfiler(
                                   x = filtered_data,
                                   categorySource = c('KEGG'),
                                   species = "hsapiens",
                                   background = unique(data_annotated$Gene))
}
}
\keyword{enrichment}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkIfFile.R
\name{artmsIsEvidenceNewVersion}
\alias{artmsIsEvidenceNewVersion}
\title{Check if a given evidencee file was generated by a new version of
MaxQuant (v>1)}
\usage{
artmsIsEvidenceNewVersion(evidence_file)
}
\arguments{
\item{evidence_file}{the evidence file name}
}
\value{
(logical) \code{TRUE} if it is a newer version of MaxQuant,
\code{FALSE} otherwise
}
\description{
MaxQuant introduced changes in the column names and number
of columns for the evidence file in version 1 (we think).
This function check whether the evidence comes from the latest version
of MaxQuant.
}
\examples{
artmsIsEvidenceNewVersion(evidence_file = artms_data_ph_evidence)
}
\keyword{check}
\keyword{evidence,}
\keyword{file,}
\keyword{input,}
\keyword{version}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/analysisQuantifications.R
\name{artmsAnalysisQuantifications}
\alias{artmsAnalysisQuantifications}
\title{Analysis of the Relative Quantifications}
\usage{
artmsAnalysisQuantifications(
  log2fc_file,
  modelqc_file,
  species,
  output_dir = "analysis_quant",
  outliers = c("keep", "iqr", "std"),
  enrich = TRUE,
  l2fc_thres = 1,
  choosePvalue = c("adjpvalue", "pvalue"),
  isBackground = "nobackground",
  isPtm = "global",
  mnbr = 2,
  pathogen = "nopathogen",
  plotPvaluesLog2fcDist = TRUE,
  plotAbundanceStats = TRUE,
  plotReproAbundance = TRUE,
  plotCorrConditions = TRUE,
  plotCorrQuant = TRUE,
  plotPCAabundance = TRUE,
  plotFinalDistributions = TRUE,
  plotPropImputation = TRUE,
  plotHeatmapsChanges = TRUE,
  plotTotalQuant = TRUE,
  plotClusteringAnalysis = TRUE,
  data_object = FALSE,
  printPDF = TRUE,
  verbose = TRUE
)
}
\arguments{
\item{log2fc_file}{(char) MSstats results file location}

\item{modelqc_file}{(char) MSstats modelqc file location}

\item{species}{(char) Select one species. Species currently supported for
a full analysis (including enrichment analysis):
\itemize{
\item HUMAN
\item MOUSE
To find out species supported only for annotation check
\code{?artmsIsSpeciesSupported()}
}}

\item{output_dir}{(char) Name for the folder to output the results from the
function. Default is current directory (recommended to provide a new folder
name).}

\item{outliers}{(char) It allows to keep or remove outliers. Options:
\itemize{
\item \code{keep} (default): it keeps outliers 'keep', 'iqr', 'std'
\item \code{iqr} (recommended): remove outliers +/- 6 x Interquartile Range (IQR)
\item \code{std} : 6 x standard deviation
}}

\item{enrich}{(logical) Performed enrichment analysis using GprofileR?
Only available for species HUMAN and MOUSE.
\code{TRUE} (default if "human" or "mouse" are the species) or \code{FALSE}}

\item{l2fc_thres}{(int) log2fc cutoff for enrichment analysis (default,
\code{l2fc_thres = 1.5})}

\item{choosePvalue}{(char) specify whether \code{pvalue} or \code{adjpvalue} should
use for the analysis. The default option is \code{adjpvalue}
(multiple testing correction).
But if the number of biological replicates for a given experiment is
too low (for example n = 2), then \code{choosePvalue = pvalue} is recommended.}

\item{isBackground}{(char) background of gene names for enrichment analysis.
\code{nobackground} (default) will use the total number of genes detected.
Alternatively provided the file path name to the background gene list.}

\item{isPtm}{(char) Is a ptm-site quantification?
\itemize{
\item \code{global} (default),
\item \code{ptmsites} (for site specific analysis),
\item \code{ptmph} (Jeff Johnson script output evidence file)
}}

\item{mnbr}{(int) \strong{PARAMETER FOR NAIVE IMPUTATION}:
"minimal number of biological replicates" for "naive
imputation" and filtering. Default: \code{mnbr = 2}. Details:
Intensity values for proteins/PTMs that are completely missed in one
of the two conditions compared ("condition A"), but are found in
at least 2 biological replicates (\code{mnbr = 2}) of the other "condition B",
are imputed (values artificially assigned) and the log2FC values calculated.
The goal is to keep those proteins/PTMs that are consistently found in
one of the two conditions (in this case "condition B") and facilitate the
inclusion in downstream analysis (if wished). The imputed intensity values
are sampled from the lowest intensity values detected in the experiment,
and (\strong{WARNING}) the p-values are just randomly assigned between
0.05 and 0.01 for illustration purposes (when generating a volcano
plot with the output of \code{artmsAnalysisQuantifications}) or to include
them when making a cutoff of p-value < 0.05 for enrichment analysis
\strong{CAUTION}: \code{mnbr} would also add the constraint that any protein
must be identified in at least \code{nmbr} biological replicates of the
\strong{same} condition or it will be filtered out. That is,
if \code{mnbr = 2}, a protein found in two conditions but only in one
biological replicate in each of them, it would be removed.}

\item{pathogen}{(char) Is there a pathogen in the dataset as well?
if it does not, then use \code{pathogen = nopathogen} (default).
Pathogens available: \code{tb} (Tuberculosis), \code{lpn} (Legionella)}

\item{plotPvaluesLog2fcDist}{(logical) If \code{TRUE} (default) plots pvalues
and log2fc distributions}

\item{plotAbundanceStats}{(logical) If \code{TRUE} (default) plots stats graphs
about abundance values}

\item{plotReproAbundance}{(logical) If \code{TRUE} plots reproducibility
based on normalized abundance values}

\item{plotCorrConditions}{(logical) If \code{TRUE} plots correlation
between the different conditions}

\item{plotCorrQuant}{(logical) if \code{TRUE} plots correlation between the
available quantifications (comparisons)}

\item{plotPCAabundance}{(logical) if \code{TRUE} performs PCA analysis of
conditions using normalized abundance values}

\item{plotFinalDistributions}{(logical) if \code{TRUE} plots distribution of both
log2fc and pvalues}

\item{plotPropImputation}{(logical) if \code{TRUE} plots proportion of overall
imputation}

\item{plotHeatmapsChanges}{(logical) if \code{TRUE} plots heatmaps of quantified
changes (both all and significant only). Only if \code{printPDF} is also \code{TRUE}}

\item{plotTotalQuant}{(logical) if \code{TRUE} plots barplot of total number of
quantifications per comparison}

\item{plotClusteringAnalysis}{(logical) if \code{TRUE} performs clustering
analysis between quantified comparisons (more than 1 comparison required)}

\item{data_object}{(logical) flag to indicate whether the required files
are data objects. Default is FALSE}

\item{printPDF}{If \code{TRUE} (default) prints out the pdfs. Warning: plot
objects are not returned due to the large number of them.}

\item{verbose}{(logical) \code{TRUE} (default) shows function messages}
}
\value{
(data.frame) summary of quantifications, including annotations,
enrichments, etc
}
\description{
Analysis of relative quantifications, including:
\itemize{
\item Annotations
\item Summary files in different format (xls, txt) and shapes (long, wide)
\item Numerous summary plots
\item Enrichment analysis using Gprofiler
\item PCA of quantifications
\item Clustering analysis
\item Basic imputation of missing values
}

To run this function, the following packages must be installed on your system:
\itemize{
\item From bioconductor:
\code{BiocManager::install(c("ComplexHeatmap", "org.Mm.eg.db"))}
\item From CRAN:
\code{install.packages(c("factoextra", "FactoMineR", "gProfileR", "PerformanceAnalytics"))}
}
}
\examples{
# Testing that the files cannot be empty
artmsAnalysisQuantifications(log2fc_file = NULL,
                              modelqc_file = NULL,
                              species = NULL,
                              output_dir = NULL)
}
\keyword{analysis,}
\keyword{quantifications}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/evidenceToSaintExpressFormat.R
\name{artmsEvidenceToSaintExpress}
\alias{artmsEvidenceToSaintExpress}
\title{MaxQuant evidence file to SAINTexpress format}
\usage{
artmsEvidenceToSaintExpress(
  evidence_file,
  keys_file,
  ref_proteome_file,
  quant_variable = c("msspc", "msint"),
  output_file,
  verbose = TRUE
)
}
\arguments{
\item{evidence_file}{(char) The evidence file path and name}

\item{keys_file}{(char) Keys file with a SAINT column
specifying test (\code{T}) and control (\code{C}) conditions}

\item{ref_proteome_file}{(char) Reference proteome path file name in
fasta format}

\item{quant_variable}{(char) choose either
\itemize{
\item \code{msspc} (spectral counts, default) or
\item \code{msint} (MS Intensity)
}}

\item{output_file}{(char) Output file name (must have extension .txt)}

\item{verbose}{(logical) \code{TRUE} (default) shows function messages}
}
\value{
The 3 required files by SAINTexpress:
\itemize{
\item \code{interactions.txt}
\item \code{preys.txt}
\item \code{baits.txt}
}
}
\description{
Converts the MaxQuant evidence file to the 3 required files
by SAINTexpress. One can choose to either use the \verb{spectral counts}
(use \code{msspc}) or the \code{intensities} (use \code{msint}) for the analysis.
}
\examples{
# Testing that the files cannot be empty
artmsEvidenceToSaintExpress(evidence_file = NULL, 
keys_file = NULL, ref_proteome_file = NULL)
}
\keyword{APMS}
\keyword{SAINT,}
\keyword{SAINTexpress,}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/analysisQuantifications.R
\name{artmsAnnotateSpecie}
\alias{artmsAnnotateSpecie}
\title{Adding a column with the species name}
\usage{
artmsAnnotateSpecie(df, pathogen = "nopathogen", species)
}
\arguments{
\item{df}{(data.frame) with a \code{Protein} column (of uniprot ids)}

\item{pathogen}{(char) Is there a pathogen in the dataset as well?
if it does not, then use \code{pathogen = nopathogen} (default). Supported\code{tb}
(Tuberculosis),
\code{lpn} (Legionella)}

\item{species}{(char) Host organism (supported for now: \code{human} or \code{mouse})}
}
\value{
(data.frame) The same data.frame but with an extra column
specifying the species
}
\description{
Adding the species name to every protein.
This makes more sense if there are more than one species in the dataset,
which must be specified in the \code{pathogen} option. Influenza is a special
case that it does not need to be specified, as far as the proteins were
originally annotated as \code{INFLUENZAGENE_STRAIN}
(strains covered \code{H1N1}, \code{H3N2}, \code{H5N1}), as for example, \code{NS1_H1N1}
}
\examples{
# Adding a new column with the main species of the data. Easy.
# But the main functionality is to add both the host-species and a pathogen,
# which is not illustrated in this example
data_with_specie <- artmsAnnotateSpecie(df = artms_data_ph_msstats_results,
                                         species = "human")
}
\keyword{annotation,}
\keyword{species}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{artms_data_randomDF}
\alias{artms_data_randomDF}
\title{Random data set}
\format{
A data frame with 100 rows and 10 variables:
\describe{
Dataset generated using this code

\code{data.frame(replicate(10,sample(0:1,100,rep=TRUE)))}
}
}
\usage{
artms_data_randomDF
}
\description{
Dataset randomly generated for testing purposes
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{artms_data_pathogen_LPN}
\alias{artms_data_pathogen_LPN}
\title{LPN PATHOGEN: Legionella pneumophila subsp. pneumophila
(strain Philadelphia 1 / ATCC 33152 / DSM 7513) UNIPROT IDS}
\format{
A data.frame of Entry IDs
}
\usage{
artms_data_pathogen_LPN
}
\description{
LPN PATHOGEN: Legionella pneumophila subsp. pneumophila
(strain Philadelphia 1 / ATCC 33152 / DSM 7513) UNIPROT IDS
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MSstats_functions.R
\name{artmsResultsWide}
\alias{artmsResultsWide}
\title{Reshape the MSstats results file from long to wide format}
\usage{
artmsResultsWide(
  results_msstats,
  output_file = NULL,
  select_pvalues = c("adjpvalue", "pvalue"),
  species,
  verbose = TRUE
)
}
\arguments{
\item{results_msstats}{(char) Input file name and location
(MSstats \code{results.txt} file)}

\item{output_file}{(char) Output file name and location
(e.g. \code{results-wide.txt}). If \code{NULL} (default) returns an
R object (data.frame)}

\item{select_pvalues}{(char) Either
\itemize{
\item \code{pvalue} or
\item \code{adjpvalue} (default)
}}

\item{species}{(char) Specie name for annotation purposes.
Check \code{?artmsMapUniprot2Entrez} to find out more about the
supported species (e.g \code{species = "human"})}

\item{verbose}{(logical) \code{TRUE} (default) shows function messages}
}
\value{
(output file tab delimited) reshaped file with unique protein ids
and as many columns log2fc and adj.pvalues as comparisons available
}
\description{
Converts the normal MSStats results.txt file into "wide" format
where each row represents a unique protein's results, and each column
represents the comparison made by MSStats. The fold change and p-value
of each comparison will be its own column.
}
\examples{
ph_results_wide <- artmsResultsWide(
                         results_msstats = artms_data_ph_msstats_results,
                         output_file = NULL,
                         species = "human")
}
\keyword{msstats,}
\keyword{reshape}
\keyword{results,}
\keyword{wide,}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/qualityControlEvidenceExtended.R
\name{artmsQualityControlEvidenceExtended}
\alias{artmsQualityControlEvidenceExtended}
\title{Extended Quality Control of the MaxQuant evidence.txt file}
\usage{
artmsQualityControlEvidenceExtended(
  evidence_file,
  keys_file,
  output_dir = "qc_extended",
  output_name = "qcExtended_evidence",
  isSILAC = FALSE,
  plotPSM = TRUE,
  plotIONS = TRUE,
  plotTYPE = TRUE,
  plotPEPTIDES = TRUE,
  plotPEPTOVERLAP = TRUE,
  plotPROTEINS = TRUE,
  plotPROTOVERLAP = TRUE,
  plotPIO = TRUE,
  plotCS = TRUE,
  plotME = TRUE,
  plotMOCD = TRUE,
  plotPEPICV = TRUE,
  plotPEPDETECT = TRUE,
  plotPROTICV = TRUE,
  plotPROTDETECT = TRUE,
  plotIDoverlap = TRUE,
  plotPCA = TRUE,
  plotSP = TRUE,
  printPDF = TRUE,
  verbose = TRUE
)
}
\arguments{
\item{evidence_file}{(char or data.frame) The evidence file path and name, or
data.frame}

\item{keys_file}{(char or data.frame) The keys file path and name or
data.frame}

\item{output_dir}{(char) Name for the folder to output the results plots.
Default is "qc_extended".}

\item{output_name}{(char) prefix output name (no extension).
Default: "qcExtended_evidence"}

\item{isSILAC}{if \code{TRUE} processes SILAC input files. Default is \code{FALSE}}

\item{plotPSM}{(logical) \code{TRUE} generates peptide-spectrum-matches (PSMs)
statistics plot: Page 1 shows the number of PSMs confidently identified
in each BioReplicate. If replicates are present, Page 2 shows the mean
number of PSMs per condition with error bar showing the standard error
of the mean. Note that potential contaminant proteins are plotted separately.}

\item{plotIONS}{(logical) \code{TRUE} generates peptide ion statistics plot:
A peptide ion is defined in the context of m/z, in other words, an unique
peptide sequence may give rise to multiple ions with different charge state
and/or amino acid modification. Page 1 shows the number of ions confidently
identified in each BioReplicate . If replicates are present, Page 2 shows
the mean number of peptide ions per condition with error bar showing the
standard error of the mean. Note that potential contaminant proteins are
plotted separately.}

\item{plotTYPE}{(logical) \code{TRUE} generates identification type statistics
plot: MaxQuant classifies each peptide identification into different
categories (e.g., MSMS, MULTI-MSMS, MULTI-SECPEP).
Page 1 shows the distribution of identification type in each BioReplicate}

\item{plotPEPTIDES}{(logical) \code{TRUE} generates peptide statistics plot:
Page 1 shows the number of unique peptide sequences (disregard the charge
state or amino acid modifications) confidently identified in each
BioReplicate. If replicates are present, Page 2 shows the mean number of
peptides per condition with error bar showing the standard error of the
mean. Note that potential contaminant proteins are plotted separately.
Pages 3 and 4 show peptide identification intersection between
BioReplicates (the bars are ordered by degree or frequency, respectively),
and Page 4 shows the intersections across conditions instead of
BioReplicates.}

\item{plotPEPTOVERLAP}{(logical) \code{TRUE} Show peptide identification
intersection between BioReplicates and Conditions}

\item{plotPROTEINS}{(logical) \code{TRUE} generates protein statistics plot:
Page 1 shows the number of protein groups confidently identified in each
BioReplicate. If replicates are present, Page 2 shows the mean number of
protein groups per condition with error bar showing the standard error of
the mean. Note that potential contaminant proteins are plotted separately.
Pages 3 and 4 show peptide identification intersection between
BioReplicates (the bars are ordered by degree or frequency, respectively),
and Page 4 shows the intersections across conditions instead of
BioReplicates.}

\item{plotPROTOVERLAP}{(logical) \code{TRUE} Show protein identification
intersection between BioReplicates and Conditions}

\item{plotPIO}{(logical) \code{TRUE} generates oversampling statistics plot:
Page 1 shows the proportion of all peptide ions (including peptides
matched across runs) fragmented once, twice and thrice or more.
Page 2 shows the  proportion of peptide ions (with intensity detected)
fragmented once, twice and thrice or more. Page 3 shows the proportion of
peptide ions (with intensity detected and MS/MS identification) fragmented
once, twice and thrice or more}

\item{plotCS}{(logical) \code{TRUE} generates charge state plot: Page 1 shows
the charge state distribution of PSMs confidently identified in each
BioReplicate.}

\item{plotME}{(logical) \code{TRUE} generates precursor mass error plot:
Page 1 shows the distribution of precursor error for all PSMs confidently
identified in each BioReplicate.}

\item{plotMOCD}{(logical) \code{TRUE} generates precursor mass-over-charge plot:
Page 1 shows the distribution of precursor mass-over-charge for all PSMs
confidently identified in each BioReplicate.}

\item{plotPEPICV}{(logical) \code{TRUE} generates peptide intensity coefficient
of variance (CV) plot: The CV is calculated for each feature (peptide ion)
identified in more than one replicate. Page 1 shows the distribution of
CV's for each condition, while Page 2 shows the distribution of CV's
within 4 bins of intensity (i.e., 4 quantiles of average intensity).}

\item{plotPEPDETECT}{(logical) \code{TRUE} generates peptide detection
frequency plot: Page 1 summarizes the frequency that each peptide is
detected across BioReplicates of each condition, showing the percentage
of peptides detected once, twice, thrice, and so on (for whatever number
of replicates each condition has).}

\item{plotPROTICV}{(logical) \code{TRUE} generates protein intensity coefficient
of variance (CV) plot: The CV is calculated for each protein (after summing
the peptide intensities) identified in more than one replicate.
Page 1 shows the distribution of CV's for each condition, while Page 2
shows the distribution of CV's within 4 bins of intensity (i.e., 4
quantiles of average intensity).}

\item{plotPROTDETECT}{(logical) \code{TRUE} generates protein detection
frequency plot: Page 1 summarizes the frequency that each protein group
is detected across BioReplicates of each condition, showing the
percentage of proteins detected once, twice, thrice, and so on (for
whatever number of replicates each condition has). Page 2 shows the feature
(peptide ion) intensity distribution within each BioReplicate (potential
contaminant proteins are plot separately). Page 3 shows the density of
feature intensity for different feature types (i.e., MULTI-MSMS,
MULTI-SECPEP).}

\item{plotIDoverlap}{(logical) \code{TRUE} generates pairwise identification
heatmap overlap: Pages 1 and 2 show pairwise peptide and protein overlap
between any 2 BioReplicates, respectively.}

\item{plotPCA}{(logical) \code{TRUE} generates PCA and pairwise intensity correlation:
Page 1 and 3 show pairwise peptide and protein intensity correlation and
scatter plot between any 2 BioReplicates, respectively. Page 2 and 4 show
Principal Component Analysis at the intensity level for both peptide and
proteins, respectively.}

\item{plotSP}{(logical)  \code{TRUE} generates sample quality metrics: Page 1
shows missing cleavage distribution of all peptides confidently identified
in each BioReplicate. Page 2 shows the fraction of peptides with at least
one methionine oxidized in each BioReplicate.}

\item{printPDF}{If \code{TRUE} (default) prints out the pdfs. Warning: plot
objects are not returned due to the large number of them.}

\item{verbose}{(logical) \code{TRUE} (default) shows function messages}
}
\value{
A number of QC plots based on the evidence file
}
\description{
Performs quality control based on the information available in
the MaxQuant \code{evidence.txt} file.
}
\details{
all the plots are generated by default
}
\examples{
# Testing warning if files are not submitted
test <- artmsQualityControlEvidenceExtended(evidence_file = NULL,
keys_file = NULL)
}
\keyword{evidence,}
\keyword{keys}
\keyword{qc,}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/evidenceToSAINTqFormat.R
\name{artmsEvidenceToSAINTq}
\alias{artmsEvidenceToSAINTq}
\title{MaxQuant evidence file to SAINTq format}
\usage{
artmsEvidenceToSAINTq(
  evidence_file,
  keys_file,
  output_dir = "artms_saintq",
  sc_option = c("all", "msspc"),
  fractions = FALSE,
  quant_variable = c("msint", "msspc"),
  verbose = TRUE
)
}
\arguments{
\item{evidence_file}{(char or data.frame) The evidence file path and name,
or data.frame}

\item{keys_file}{(char) Keys file with a SAINT column specifying
test (\code{T}) and control (\code{C}) conditions}

\item{output_dir}{(char) New directory to create and save files.
Default is current directory (recommended to provide a new folder name).}

\item{sc_option}{(char). Filter peptides with spectral counts only.
Two options:
\itemize{
\item \code{msspc}: use only peptides with spectral_counts
\item \code{all} (default): all peptides detected (including the one resulting from
the MaxQuant 'Match between run' algorithm)
}}

\item{fractions}{(logical) \code{TRUE} for 2D proteomics (fractions).
Default: \code{FALSE}}

\item{quant_variable}{(char) Select the quantitative variable.
Two options available:
\itemize{
\item \code{msint}: MS Intensity (default)
\item \code{msspc}: MS.MS.count (Spectral Counts)
}}

\item{verbose}{(logical) \code{TRUE} (default) shows function messages}
}
\value{
The input files requires to run SAINTq
}
\description{
Converts the MaxQuant evidence file to the required files
by SAINTq. The user can choose to use either peptides with \verb{spectral counts}
(use \code{msspc}) or the all the peptides (use \code{all}) for the analysis.
The quantitative can be also chosen (either MS Intensity or Spectral Counts)
}
\details{
After running the script, the new specified folder should contain
the folling files:
\itemize{
\item saintq-config-peptides
\item saintq-config-proteins
\item saintq_input_peptides.txt
\item saintq_input_proteins.txt
}

Then \code{cd} into the new folder and run either of the following two options
(assuming that \code{saintq} is installed in your linux/unix/mac os x system):

\verb{> saintq config-saintq-peptides}

or

\verb{> saintq config-saintq-proteins}
}
\examples{
# Testing that the files cannot be empty
artmsEvidenceToSAINTq   (evidence_file = NULL, 
                                   keys_file = NULL, 
                                   output_dir = NULL)
}
\keyword{APMS}
\keyword{SAINT,}
\keyword{SAINTq,}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{artms_data_ph_config}
\alias{artms_data_ph_config}
\title{artMS configuration for the available PH dataset}
\format{
The configuration (\code{yaml}) file contains the following sections:
\describe{
\item{files}{
\itemize{
\item \code{evidence} : Empty. To test an example, run \code{artms_data_ph_config$files$evidence <- artms_data_ph_evidence}
\item \code{keys} : Empty To test an example datasets run \code{artms_data_ph_config$files$keys <- artms_data_ph_keys}
\item \code{contrasts} : Empty. To test the example datasets, run \code{artms_data_ph_config$files$contrasts <- artms_data_ph_contrast}
\item \code{summary} :
\item \code{output} : "results.txt"
}
}

\item{qc}{
\itemize{
\item basic: 0
\item extended: 0
\item extendedSummary: 0
}
}

\item{data}{
\itemize{
\item enabled : 1
\item silac:
\itemize{
\item enabled : 0
}
\item filters:
\itemize{
\item enabled : 1
}
\item contaminants : 1
\item protein_groups : remove
\item modifications : PH
\item sample_plots : 1
}
}

\item{msstats}{
\itemize{
\item enabled : 1
\item msstats_input : # blank if not previous msstats input file is available
\item profilePlots : none # before, after, before-after, none
\item normalization_method : equalizeMedians
\item normalization_reference :  #should be a value in the Protein column
\item summaryMethod : TMP
\item censoredInt : NA
\item cutoffCensored : minFeature
\item MBimpute : 1
\item feature_subset: all
}
}

\item{output_extras}{
\itemize{
\item output_extras :
\itemize{
\item enabled : 1
}
\item annotate :
\itemize{
\item enabled: 1
}
\item species : HUMAN
\item plots:
\itemize{
\item volcano: 1
\item heatmap: 1
\item LFC : -1 1
\item FDR : 0.05
\item heatmap_cluster_cols : 0
\item heatmap_display : log2FC}
}
}
}
}
\usage{
artms_data_ph_config
}
\description{
The configuration file with default options to run the
available PH dataset with `artmsQuantification()``
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/msstat_summary.R
\name{artmsMsstatsSummary}
\alias{artmsMsstatsSummary}
\title{Summarize the MSStats results and data quantification}
\usage{
artmsMsstatsSummary(
  evidence_file,
  prot_group_file,
  keys_file,
  results_file,
  return_df = FALSE,
  verbose = TRUE
)
}
\arguments{
\item{evidence_file}{(char or data.frame) The filepath to the MaxQuant
searched data (evidence) file (txt tab delimited file).
Only works for the newer versions of the evidence file.}

\item{prot_group_file}{(char) The filepath to the MaxQuant
\code{proteinGroups.txt} file (txt tab delimited file) or data.frame}

\item{keys_file}{(char) The filepath to the keys file used with MSStats
(txt tab delimited file).}

\item{results_file}{(char) The filepath to the MSStats results file in t
he default long format (txt tab delimited file or data.frame).}

\item{return_df}{(data.frame) Whether or not to return the results
to the R environment upon completion. This is useful if this is being
used in an R pipeline and you want to feed the results directly into the
next stage of analysis via an R environment/terminal.
Regardless, the results will be written to file. Default = FALSE}

\item{verbose}{(logical) \code{TRUE} (default) shows function messages}
}
\value{
(data.frame or txt file) with the summary
}
\description{
Converts the MSStats results file to wide format
(unique Protein ID and columns are the comparisons), as well as adds
BioReplicate information about
\itemize{
\item the Number of Unique Peptides,
\item Spectral Counts
\item Intensities
for each protein.
In cases where there are multiple values for a Protein-BioReplicate
pair due to minute changes in sequence, the maximum value is taken for the
pair. Any pairs without a value are assigned a value of NA.
}
}
\examples{
# Testing warning if files are not submitted
test <- artmsMsstatsSummary(evidence_file = NULL,
                      prot_group_file = NULL,
                      keys_file = NULL,
                      results_file = NULL)
}
\keyword{MSStats,}
\keyword{MaxQuant,}
\keyword{evidence,}
\keyword{summary}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/annotations.R
\name{artmsAnnotationUniprot}
\alias{artmsAnnotationUniprot}
\title{Annotate table with Gene Symbol and Name based on Uniprot ID(s)}
\usage{
artmsAnnotationUniprot(x, columnid, species, verbose = TRUE)
}
\arguments{
\item{x}{(data.frame) to be annotated (or file path and name)}

\item{columnid}{(char) The column with the uniprotkb ids}

\item{species}{(char) The species name. Check \code{?artmsMapUniprot2Entrez}
to find out more about supported species.}

\item{verbose}{(logical) \code{TRUE} (default) shows function messages}
}
\value{
(data.frame) with two new columns: \code{Gene} and \code{Protein.name}
}
\description{
Annotate gene name and symbol based on uniprot ids. It will
take the column from your data.frame specified by the \code{columnid} argument,
search for the gene symbol, name, and entrez based on the species (\code{species}
argument) and merge the information back to the input data.frame
}
\examples{
# This example adds annotations to the example evidence file included in
# artMS, based on the column 'Proteins'.

evidence_anno <- artmsAnnotationUniprot(x = artms_data_ph_evidence,
                                         columnid = 'Proteins',
                                         species = 'human')
}
\keyword{annotation,}
\keyword{uniprot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots.R
\name{artmsPlotHeatmapQuant}
\alias{artmsPlotHeatmapQuant}
\title{Outputs a heatmap of the MSStats results created using the log2fold
changes}
\usage{
artmsPlotHeatmapQuant(
  input_file,
  output_file = "quantifications_heatmap.pdf",
  species,
  labels = "*",
  cluster_cols = FALSE,
  display = "log2FC",
  lfc_lower = -2,
  lfc_upper = 2,
  whatPvalue = "adj.pvalue",
  FDR = 0.05,
  verbose = TRUE
)
}
\arguments{
\item{input_file}{(char) MSstats \code{results.txt} file and location (or
data.frame of resuts)}

\item{output_file}{(char) Output file name (pdf format) and location.
Default:"quantifications_heatmap.pdf"}

\item{species}{(char). Specie name to be able to add the Gene name. To find
out more about the supported species check \code{?artmsMapUniprot2Entrez}}

\item{labels}{(vector) of uniprot ids if only specific labes would like to
be plotted. Default: all labels}

\item{cluster_cols}{(boolean) \code{True} or \code{False} to cluster columns.
Default: FALSE}

\item{display}{Metric to be displayed. Options:
\itemize{
\item \code{log2fc} (default)
\item \code{adj.pvalue}
\item \code{pvalue}
}}

\item{lfc_lower}{(int) Lower limit for the log2fc. Default: -2}

\item{lfc_upper}{(int) Upper limit for the log2fc. Default: +2}

\item{whatPvalue}{(char) \code{pvalue} or \code{adj.pvalue} (default)}

\item{FDR}{(int) Upper limit false discovery rate (or pvalue). Default: 0.05}

\item{verbose}{(logical) \code{TRUE} (default) shows function messages}
}
\value{
(pdf or ggplot2 object) heatmap of the MSStats results using the
selected metric
}
\description{
Heatmap of the Relative Quantifications (MSStats results)
}
\examples{
# Unfortunately, the example does not contain any significant hits
# Use for illustration purposes
artmsPlotHeatmapQuant(input_file = artms_data_ph_msstats_results,
                       species = "human",
                       output_file = NULL,
                       whatPvalue = "pvalue",
                       lfc_lower = -1,
                       lfc_upper = 1)
}
\keyword{heatmap,}
\keyword{log2fc}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MSstats_functions.R
\name{artmsMergeEvidenceAndKeys}
\alias{artmsMergeEvidenceAndKeys}
\title{Merge evidence.txt (or summary.txt) with keys.txt files}
\usage{
artmsMergeEvidenceAndKeys(
  x,
  keys,
  by = c("RawFile"),
  isSummary = FALSE,
  verbose = TRUE
)
}
\arguments{
\item{x}{(data.frame or char) The evidence data, either as data.frame or
the file name (and path). It also works for the summary.txt file}

\item{keys}{The keys data, either as a data.frame or file name (and path)}

\item{by}{(vector) specifying the columns use to merge the evidence and keys.
Default: \code{by=c('RawFile')}}

\item{isSummary}{(logical) TRUE or FALSE (default)}

\item{verbose}{(logical) \code{TRUE} (default) shows function messages}
}
\value{
(data.frame) with the evidence and keys merged
}
\description{
Merge the evidence and keys files on the given columns
}
\examples{
evidenceKeys <- artmsMergeEvidenceAndKeys(x = artms_data_ph_evidence,
                                           keys = artms_data_ph_keys)
}
\keyword{evidence,}
\keyword{keys}
\keyword{merge,}
\keyword{summary,}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{artms_data_ph_keys}
\alias{artms_data_ph_keys}
\title{Keys File Example}
\format{
Tab delimited file with the following columns:
\describe{
\item{Raw.file}{Raw file processed. Each one should be a unique
biological (or technical) replicate}

\item{IsotopeLabelType}{Type of labeling. \code{L} is used for label free
experiments}

\item{Condition}{Label for conditions. VERY IMPORTANT: Only alpha-numeric
characters and \verb{underscore (_)} are allowed}

\item{BioReplicate}{Label for the Biological replicates. VERY IMPORTANT:
Use the same labeling for bioreplicate as the Condition, but adding a
\verb{dash (-)} corresponding to the number of biological replicate.
For example, for \code{Condition} \code{"Cal"}, use \code{Cal-1}, \code{Cal-2}, \code{Cal-3}, etc
for the bioreplicates}

\item{Run}{The MS run number}
}
}
\usage{
artms_data_ph_keys
}
\description{
the \code{artMS} keys file provides the details of the experimental
design for any given proteomics experiment.

This particular example belongs to a PH experiment consisting of two
head and neck cancer cell lines ("Conditions" \code{"Cal33"} and \code{"HSC6"}),
with 2 biological replicates each (in this reduced version)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metabolomics.R
\name{artmsConvertMetabolomics}
\alias{artmsConvertMetabolomics}
\title{Convert Markview Metabolomics file (alignment table) into
a artMS compatible format}
\usage{
artmsConvertMetabolomics(input_file, out_file, id_file = NULL, verbose = TRUE)
}
\arguments{
\item{input_file}{(char) Markview input file}

\item{out_file}{(char) Output file name}

\item{id_file}{(char) KEGG database}

\item{verbose}{(logical) \code{TRUE} (default) shows function messages}
}
\value{
(text file) Outputs the converted output name
}
\description{
\code{artMS} enables the relative quantification of untargeted
polar metabolites using the alignment table generated by Markview.
MarkerView is an ABSciex software that supports the files
generated by Analyst software (\code{.wiff}) used to run our specific mass
spectrometer (ABSciex Triple TOF 5600+).
It also supports \code{.t2d} files generated by the
Applied Biosystems 4700/4800 MALDI-TOF.
MarkerView software is used to align mass spectrometry data from several
samples for comparison. Using the import feature in the software, \code{.wiff}
files (also \code{.t2d} MALDI-TOF files and tab-delimited \code{.txt} mass spectra data
in mass-intensity format) are loaded for retention time alignment.
Once the data files are selected, a series of windows will appear wherein
peak finding, alignment, and filtering options can be entered and selected.
These options include minimum spectral peak width, minimum retention time
peak width, retention time and mass tolerance, and the ability to filter
out peaks that do not appear in more than a user selected number of samples.

`artmsConvertMetabolomics`` processes the markview file to enable
QC analysis and relative quantification using the artMS functions
}
\examples{
# Testing that the arguments cannot be null
artmsConvertMetabolomics(input_file = NULL, 
                         out_file = NULL)
}
\keyword{convert}
\keyword{metabolomics,}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{artms_data_pathogen_TB}
\alias{artms_data_pathogen_TB}
\title{TB PATHOGEN: Mycobacterium tuberculosis
(strain ATCC 35801 / TMC 107 / Erdman) UNIPROTS IDS}
\format{
A data.frame of Entry IDs
}
\usage{
artms_data_pathogen_TB
}
\description{
TB PATHOGEN: Mycobacterium tuberculosis
(strain ATCC 35801 / TMC 107 / Erdman) UNIPROTS IDS
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/analysisQuantifications.R
\name{artmsGeneratePhSiteExtended}
\alias{artmsGeneratePhSiteExtended}
\title{Generate ph-site specific detailed file}
\usage{
artmsGeneratePhSiteExtended(
  df,
  pathogen = "nopathogen",
  species,
  ptmType,
  output_name
)
}
\arguments{
\item{df}{(data.frame) of log2fc and imputed values}

\item{pathogen}{(char) Is there a pathogen in the dataset as well? Available
pathogens are \code{tb} (Tuberculosis), \code{lpn} (Legionella). If it is not,
then use \code{nopathogen} (default).}

\item{species}{(char) Main organism (supported for now: \code{human} or \code{mouse})}

\item{ptmType}{(char) It must be a ptm-site quantification dataset. Either:
yes: \code{ptmsites} (for site specific analysis), or
\code{ptmph} (Jeff's script output evidence file).}

\item{output_name}{(char) A output file name (extension \code{.txt} required)}
}
\value{
(data.frame) extended version of the ph-site
}
\description{
Generate extended detailed ph-site file, where every line is a
ph site instead of a peptide. Therefore, if one peptide has multiple ph sites
it will be breaking down in each of the sites. This file will help generate
input files for tools as \href{http://phosfate.com/}{Phosfate} or
\href{https://github.com/jdrudolph/photon}{PHOTON}
}
\examples{
\dontrun{
artmsGeneratePhSiteExtended(df = dfobject, 
                             species = "mouse", 
                             ptmType = "ptmsites",
                             output_name = log2fc_file)
}
}
\keyword{external,}
\keyword{phosfate}
\keyword{tools,}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots.R
\name{artmsVolcanoPlot}
\alias{artmsVolcanoPlot}
\title{Volcano plot (log2fc / pvalues)}
\usage{
artmsVolcanoPlot(
  mss_results,
  output_name = "volcano_plot.pdf",
  lfc_upper = 1,
  lfc_lower = -1,
  whatPvalue = "adj.pvalue",
  FDR = 0.05,
  PDF = TRUE,
  decimal_threshold = 16,
  verbose = TRUE
)
}
\arguments{
\item{mss_results}{(data.frame or file) Selected MSstats results}

\item{output_name}{(char) Name for the output file (don't forget the \code{.pdf}
extension)}

\item{lfc_upper}{(numeric) log2fc upper threshold (positive value)}

\item{lfc_lower}{(numeric) log2fc lower threshold (negative value)}

\item{whatPvalue}{(char) \code{pvalue} or \code{adj.pvalue} (default)}

\item{FDR}{(numeric) False Discovery Rate threshold}

\item{PDF}{(logical) Option to generate pdf format. Default: \code{T}}

\item{decimal_threshold}{(numeric) Decimal threshold for the pvalue.
Default: 16 (10^-16)}

\item{verbose}{(logical) \code{TRUE} (default) shows function messages}
}
\value{
(pdf) of a volcano plot
}
\description{
It generates a scatter-plot used to quickly identify changes
}
\examples{
artmsVolcanoPlot(mss_results = artms_data_ph_msstats_results,
                  whatPvalue = "pvalue",
                  PDF = FALSE)
}
\keyword{plot,}
\keyword{volcano}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{artms_data_ph_msstats_modelqc}
\alias{artms_data_ph_msstats_modelqc}
\title{MSstats modelQC example}
\format{
A data frame resulting from running the latest version of
\code{MSstats::groupComparison} function required as input for
artmsAnalysisQuantifications()
}
\usage{
artms_data_ph_msstats_modelqc
}
\description{
Normalized data obtained from the \code{artmsQuantification()} step
of the PH dataset (global analysis)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MSstats_functions.R
\name{artmsChangeColumnName}
\alias{artmsChangeColumnName}
\title{Change a specific column name in a given data.frame}
\usage{
artmsChangeColumnName(dataset, oldname, newname)
}
\arguments{
\item{dataset}{(data.frame) with the column name you want to change}

\item{oldname}{(char) the old column name}

\item{newname}{(char) the new name for that column}
}
\value{
(data.frame) with the new specified column name
}
\description{
Making easier to change a column name in any data.frame
}
\examples{
artms_data_ph_evidence <- artmsChangeColumnName(
                               dataset = artms_data_ph_evidence,
                               oldname = "Phospho..STY.",
                               newname = "PH_STY")
}
\keyword{columns}
\keyword{data.frame,}
\keyword{rename,}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MSstats_functions.R
\name{artmsFilterEvidenceContaminants}
\alias{artmsFilterEvidenceContaminants}
\title{Remove contaminants and empty proteins from the MaxQuant evidence file}
\usage{
artmsFilterEvidenceContaminants(x, verbose = TRUE)
}
\arguments{
\item{x}{(data.frame) of the Evidence file}

\item{verbose}{(logical) \code{TRUE} (default) shows function messages}
}
\value{
(data.frame) without REV__ and CON__ Protein ids
}
\description{
Remove contaminants and erronously identified 'reverse'
sequences by MaxQuant, in addition to empty protein ids
}
\examples{
ef <- artmsFilterEvidenceContaminants(x = artms_data_ph_evidence)
}
\keyword{cleanup,}
\keyword{contaminants}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{artms_config}
\alias{artms_config}
\title{artMS configuration template}
\format{
The configuration (\code{yaml}) file contains the following sections:
\describe{
\item{files}{
\itemize{
\item \code{evidence} : /path/to/the/evidence.txt
\item \code{keys} : /path/to/the/keys.txt
\item \code{contrasts} : /path/to/the/contrast.txt
\item \code{summary} : /path/to/the/summary.txt
\item \code{output} : /path/to/the/output/results/results.txt}
}

\item{qc}{
\itemize{
\item basic: 1 # 1 = yes; 0 = no
\item extended: 1 # 1 = yes; 0 = no
\item extendedSummary: 0 # 1 = yes; 0 = no
}
}

\item{data}{
\itemize{
\item enabled : 1 # 1 = yes; 0 = no
\item silac:
\itemize{
\item enabled : 0 # 1 for SILAC experiments
}
\item filters:
\itemize{
\item enabled : 1
}
\item contaminants : 1
\item protein_groups : remove #remove, keep
\item modifications : ab # PH, UB, AB, APMS
\item sample_plots : 1 # correlation plots }
}

\item{msstats}{
\itemize{
\item enabled : 1
\item msstats_input : # blank if not previous msstats input file is available
\item profilePlots : none # before, after, before-after, none
\item normalization_method : equalizeMedians # globalStandards
(include a reference protein(s) ), equalizeMedians, quantile, 0
\item normalization_reference :  #should be a value in the Protein column
\item summaryMethod : TMP # "TMP"(default) means Tukey's median polish,
which is robust estimation method. "linear" uses linear mixed model.
"logOfSum" conducts log2 (sum of intensities) per run.
\item censoredInt : NA  # Missing values are censored or at random. 'NA'
(default) assumes that all 'NA's in 'Intensity' column are censored.
'0' uses zero intensities as censored intensity. In this case,
NA intensities are missing at random. The output from Skyline
should use '0'. Null assumes that all NA intensites are randomly missing.
\item MBimpute : 1 # only for summaryMethod="TMP" and censoredInt='NA' or '0'.
TRUE (default) imputes 'NA' or '0' (depending on censoredInt option) by
Accelated failure model. FALSE uses the values assigned by cutoffCensored
\item For all othe features, please check documentation for MSstats' dataProcess function
}
}

\item{output_extras}{
\itemize{
\item output_extras :
\itemize{
\item enabled : 1 # if 0, it wont do anything in this section
}
\item annotate :
\itemize{
\item enabled: 1 # 1|0 whether to annotate the proteins in the results or not
}
\item species : HUMAN  # Supported species: HUMAN, MOUSE, ANOPHELES, ARABIDOPSIS,
BOVINE, WORM, CANINE, FLY, ZEBRAFISH, ECOLI_STRAIN_K12, ECOLI_STRAIN_SAKAI,
CHICKEN, RHESUS, MALARIA, CHIMP, RAT, YEAST, PIG, XENOPUS
\item plots:
\itemize{
\item volcano: 1
\item heatmap: 1
\item LFC : -1.5 1.5 # Range of minimal log2fc
\item FDR : 0.05
\item heatmap_cluster_cols : 0
\item heatmap_display : log2FC # log2FC or pvalue}
}
}
}
}
\usage{
artms_config
}
\description{
The configuration file in \code{yaml} format contains
the configuration details required to run \code{artmsQuantification()}, which
includes quality control functions
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots.R
\name{artmsDataPlots}
\alias{artmsDataPlots}
\title{Individual Normalized abundance dot plots for every protein}
\usage{
artmsDataPlots(input_file, output_file, verbose = TRUE)
}
\arguments{
\item{input_file}{(char) File path and name to the \code{-normalized.txt} output
file from MSstats}

\item{output_file}{(char) Output file (path) name (add the \code{.pdf} extension)}

\item{verbose}{(logical) \code{TRUE} (default) shows function messages}
}
\value{
(pdf) file with each individual protein abundance plot for each
conditions
}
\description{
Protein abundance dot plots for each unique uniprot id. It can
take a long time
}
\examples{
\dontrun{
artmsDataPlots(input_file = "results/ab-results-mss-normalized.txt",
               output_file = "results/ab-results-mss-normalized.pdf")
}
}
\keyword{abundance,}
\keyword{dotplots,}
\keyword{plot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{artms_data_ph_evidence}
\alias{artms_data_ph_evidence}
\title{Evidence file example}
\format{
A data frame with all the columns available in an evidence file
generated with MaxQuant version 1.6.2.3
}
\usage{
artms_data_ph_evidence
}
\description{
Evidence file from a PH experiment consisting of two
head and neck cancer cell lines ("Conditions" \code{"Cal33"} and \code{"HSC6"}).

Unfortunately, the number of lines was reduced to 1/20 due
to bioconductor limitations on data size, but it should be enough to test the
qc and quantification functions. The number of total columns
from the original evidence file was also reduced to 36
(out of the original 66 columns).
Check \code{colnames(artms_data_ph_evidence)} for details
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/qualityControlSummaryExtended.R
\name{artmsQualityControlSummaryExtended}
\alias{artmsQualityControlSummaryExtended}
\title{Quality Control of the MaxQuant summary.txt file}
\usage{
artmsQualityControlSummaryExtended(
  summary_file,
  keys_file,
  output_dir = "qc_summary",
  output_name = "qcExtended_summary",
  isFractions = FALSE,
  plotMS1SCANS = TRUE,
  plotMS2 = TRUE,
  plotMSMS = TRUE,
  plotISOTOPE = TRUE,
  printPDF = TRUE,
  verbose = TRUE
)
}
\arguments{
\item{summary_file}{(char or data.frame) The evidence file path and name, or
data.frame}

\item{keys_file}{(char or data.frame) The keys file path and name or
data.frame}

\item{output_dir}{(char) Name for the folder to output the results plots.
Default is "qc_summary".}

\item{output_name}{(char) prefix output name (no extension).
Default: "qcExtended_summary"}

\item{isFractions}{(logical) \code{TRUE} if it is a 2D experiment (fractions).
Default: \code{FALSE}}

\item{plotMS1SCANS}{(logical) \code{TRUE} generates MS1 scan counts plot:
Page 1 shows the number of MS1 scans in each BioReplicate.
If replicates are present, Page 2 shows the mean number of MS1 scans
per condition with error bar showing the standard error of the mean.
If isFractions \code{TRUE}, each fraction is a stack on the individual bar graphs.}

\item{plotMS2}{(logical) \code{TRUE} generates MS2 scan counts plot:
Page 1 shows the number of MSs scans in each BioReplicate.
If replicates are present, Page 2 shows the mean number of MS1 scans per
condition with error bar showing the standard error of the mean.
If isFractions \code{TRUE}, each fraction is a stack on the individual bar graphs.}

\item{plotMSMS}{(logical) \code{TRUE} generates MS2 identification rate (\%) plot:
Page 1 shows the fraction of MS2 scans confidently identified in each
BioReplicate. If replicates are present, Page 2 shows the mean rate of MS2
scans confidently identified per condition with error bar showing the
standard error of the mean.
If isFractions \code{TRUE}, each fraction is a stack on the individual bar graphs.}

\item{plotISOTOPE}{(logical) \code{TRUE} generates Isotope Pattern counts plot:
Page 1 shows the number of Isotope Patterns with charge greater than 1 in
each BioReplicate. If replicates are present, Page 2 shows the mean number
of Isotope Patterns with charge greater than 1 per condition with error bar
showing the standard error of the mean.
If isFractions \code{TRUE}, each fraction is a stack on the individual bar graphs.}

\item{printPDF}{If \code{TRUE} (default) prints out the pdfs. Warning: plot
objects are not returned due to the large number of them.}

\item{verbose}{(logical) \code{TRUE} (default) shows function messages}
}
\value{
A number of plots from the summary file
}
\description{
Performs quality control based on the information available in
the MaxQuant summary.txt file.
}
\examples{
# Testing warning if files are not submitted
test <- artmsQualityControlSummaryExtended(summary_file = NULL,
keys_file = NULL)
}
\keyword{keys}
\keyword{qc,}
\keyword{summary,}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/main.R
\name{artmsQuantification}
\alias{artmsQuantification}
\title{Relative quantification using MSstats}
\usage{
artmsQuantification(
  yaml_config_file,
  data_object = FALSE,
  printPDF = TRUE,
  printTables = TRUE,
  display_msstats = FALSE,
  return_results_object = FALSE,
  verbose = TRUE
)
}
\arguments{
\item{yaml_config_file}{(char, required) The yaml file name and location}

\item{data_object}{(logical) flag to indicate whether the configuration file
is a string to a file that should be opened or config object (yaml).
Default is \code{FALSE}. Choose \code{TRUE} if \code{yaml_config_file} is a yaml object}

\item{printPDF}{(logical) if \code{TRUE} (default), prints out pdf}

\item{printTables}{(logical) \code{TRUE} (default) print results tables}

\item{display_msstats}{(logical) if \code{TRUE}, prints MSstats outputs (default is \code{FALSE})}

\item{return_results_object}{(logical) Default is \code{FALSE}.
If \code{TRUE}, it returns a list of data frames with MSstats results, including:
\itemize{
\item \code{comparisonResult}: comparison results
\item \code{ModelQC}
\item \code{FittedModel}: fit model details
\item \code{power}: power calculations
\item \code{sample_size}: sample size estimations
}}

\item{verbose}{(logical) \code{TRUE} (default) shows function messages}
}
\value{
The relative quantification of the conditions and comparisons
specified in the keys/contrast file resulting from running MSstats, in
addition to quality control plots (if selected)
}
\description{
Relative quantification using MSstats including:
\itemize{
\item plots
\item quantifications (log2fc, pvalues, etc)
\item normalized abundance values
}
}
\examples{
# Recommended
# artmsQuantification(yaml_config_file = "your-config-file.yaml")

# Example to test this function using the example dataset available in artMS
# Step 1: Add evidence, keys, and contrast to configuration object
artms_data_ph_config$files$evidence <- artms_data_ph_evidence
artms_data_ph_config$files$keys <- artms_data_ph_keys
artms_data_ph_config$files$contrasts <- artms_data_ph_contrast

# Step 2: Run the quantification step
quant_results <- artmsQuantification(yaml_config_file = artms_data_ph_config, 
                                     data_object = TRUE, 
                                     display_msstats = FALSE,
                                     printPDF = FALSE,
                                     printTables = FALSE)
# Check the list of data frames "quant_results". Nothing should be printed out. 
}
\keyword{driver,}
\keyword{function}
\keyword{main,}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/enrichments.R
\name{artmsEnrichLog2fc}
\alias{artmsEnrichLog2fc}
\title{Enrichment of changes in protein abundance or PTMs}
\usage{
artmsEnrichLog2fc(
  dataset,
  species,
  background,
  heatmaps = FALSE,
  output_name = "enrichment.txt",
  verbose = TRUE
)
}
\arguments{
\item{dataset}{(data.frame) with a \code{Gene} and \verb{Comparison or Label} (with
the name of the comparisons specified in the contrast file) columns}

\item{species}{(char) Specie, only supported "human" or "mouse"}

\item{background}{(vector) Background genes for the enrichment analysis.}

\item{heatmaps}{(logical) if \code{TRUE} generates heatmaps (pdf),
\code{FALSE} (default) otherwise.}

\item{output_name}{(char) Name of the annotation files, which will be used
as well for the heatmaps (if \code{heatmaps} is selected)
Default \code{output_name = "enrichment.txt"}}

\item{verbose}{(logical) \code{TRUE} (default) shows function messages}
}
\value{
(data.frame) Results from the enrichment analysis using Gprofiler
and heatmaps (if selected)
}
\description{
Enrichment analysis of the selected proteins
}
\examples{
\dontrun{
# The data must be annotated (Protein and Gene columns)
data_annotated <- artmsAnnotationUniprot(
                      x = artms_data_ph_msstats_results,
                      columnid = "Protein",
                      species = "human")
# And then the enrichment
enrich_set <- artmsEnrichLog2fc(
                   dataset = data_annotated,
                   species = "human",
                   background = unique(data_annotated$Gene))
}
}
\keyword{enrichment}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/qualityControlMetabolomics.R
\name{artmsQualityControlMetabolomics}
\alias{artmsQualityControlMetabolomics}
\title{Quality Control analysis of the evidence-like metabolomics dataset}
\usage{
artmsQualityControlMetabolomics(
  evidence_file,
  keys_file,
  met_exp = c("MV"),
  output_name = "qcPlots_metab",
  plotINTDIST = FALSE,
  plotCORMAT = TRUE,
  plotINTMISC = TRUE,
  printPDF = TRUE,
  verbose = TRUE
)
}
\arguments{
\item{evidence_file}{(char or data.frame) The evidence file path and name, or
data.frame}

\item{keys_file}{(char or data.frame) The keys file path and name or
data.frame}

\item{met_exp}{(char) Metabolomics experiment. Only one option available
(so far):
\itemize{
\item \code{MV}: Markview output
}}

\item{output_name}{(char) prefix output name (no extension).
Default: "qcPlots_metab"}

\item{plotINTDIST}{if \code{TRUE} (default) plots both \emph{Box-dot plot}
and \emph{Jitter plot} of biological replicates based on MS (raw)
intensity values.}

\item{plotCORMAT}{if \code{TRUE} (default) generates up to 3 pdf files for
technical replicates, biological replicates, and conditions. Each pdf file
contains:
\itemize{
\item \emph{Correlation matrix} for all the biological replicates using
MS Intensity values,
\item \emph{Clustering matrix} of the MS Intensities and correlation distribution
\item \emph{histogram} of the distribution of correlations
}}

\item{plotINTMISC}{if \code{TRUE} (default) plots several pages, including
bar plots of \emph{Total Sum of Intensities in BioReplicates},
\emph{Total Sum of Intensities in Conditions},
\emph{Total Feature Counts in BioReplicates},
\emph{Total Feature Counts in conditions} separated by categories
(INT: has a intensity value NOINT: no intensity value )
\emph{Box plots} of MS Intensity values per
biological replicates and conditions; \emph{bar plots} of total intensity
by bioreplicates and conditions; Barplots of
\emph{total feature counts} by bioreplicates and conditions.}

\item{printPDF}{If \code{TRUE} (default) prints out the pdfs. Warning: plot
objects are not returned due to the large number of them.}

\item{verbose}{(logical) \code{TRUE} (default) shows function messages}
}
\value{
Quality control files and plots for metabolomics
}
\description{
Quality Control analysis of the evidence-like metabolomics
dataset
}
\examples{
# Testing that input arguments cannot be null
artmsQualityControlMetabolomics(evidence_file = NULL,
                 keys_file = NULL,
                 met_exp = "MV")
}
\keyword{QC,}
\keyword{control,}
\keyword{evidence,}
\keyword{metabolomics}
\keyword{quality,}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/avg_intensity_RT.R
\name{artmsAvgIntensityRT}
\alias{artmsAvgIntensityRT}
\title{Summarize average intensity and retention time per protein}
\usage{
artmsAvgIntensityRT(
  evidence_file,
  protein_file = NULL,
  output_file = FALSE,
  verbose = TRUE
)
}
\arguments{
\item{evidence_file}{(char) The filepath to the MaxQuant searched data
(evidence) file (txt tab delimited file).}

\item{protein_file}{(char) The file path to a file or vector containing
a list of proteins of interest.}

\item{output_file}{(char) The file name for the results
(must have the extension \code{.txt}). If empty, then the
results will be returned as an R object.}

\item{verbose}{(logical) \code{TRUE} (default) shows function messages}
}
\value{
An R object with the results and a file with the results (if the
output_file argument is provided). It contains averages of Intensity,
Retention Time, Caliberated Retention Time
}
\description{
Input an evidence file from MaxQuant and a file
containing a list of proteins of interest (optional).
The function will summarize from the evidence file and report back the
average intensity, average retention time, and the average caliberated
retention time. If a list of proteins is provided, then only those proteins
will be summarized and returned.
}
\examples{
ave_int <- artmsAvgIntensityRT(evidence_file = artms_data_ph_evidence)
}
\keyword{MaxQuant,}
\keyword{caliberated}
\keyword{evidence,}
\keyword{intensity,}
\keyword{retention}
\keyword{summary,}
\keyword{time,}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/main.R
\name{artmsWriteConfigYamlFile}
\alias{artmsWriteConfigYamlFile}
\title{Write out a template file of the artMS configuration file (yaml)}
\usage{
artmsWriteConfigYamlFile(
  config_file_name = "artms_config_file.yaml",
  overwrite = FALSE,
  verbose = TRUE
)
}
\arguments{
\item{config_file_name}{(char) The name for the configuration file. It must
have a \code{.yaml} extension. If \code{NULL}, it returns the config as a yaml object}

\item{overwrite}{(logical) Default FALSE}

\item{verbose}{(logical) \code{TRUE} (default) shows function messages}
}
\value{
A file (or yaml data object) of the artMS configuration file
}
\description{
Creates a template file of the artMS configuration file, which
is required to run \code{artmsQuantification}. Check \code{?artms_config} and the
vignettes to find out more about the details of the structure of the file
and how to fill it up
}
\examples{
config_empty <- artmsWriteConfigYamlFile(config_file_name = NULL)
}
\keyword{config,}
\keyword{yaml}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/annotations.R
\name{artmsIsSpeciesSupported}
\alias{artmsIsSpeciesSupported}
\title{Check if a species is supported and available}
\usage{
artmsIsSpeciesSupported(species, verbose = TRUE)
}
\arguments{
\item{species}{(char) The species name. Species currently supported
as part of artMS:
\itemize{
\item HUMAN
\item MOUSE
}

And the following species can be used as well, but the user needs to
install the corresponding org.db package:
\itemize{
\item ANOPHELES (\code{install.packages(org.Ag.eg.db)})
\item BOVINE (\code{install.packages(org.Bt.eg.db)})
\item WORM (\code{install.packages(org.Ce.eg.db)})
\item CANINE (\code{install.packages(org.Cf.eg.db)})
\item FLY (\code{install.packages(org.Dm.eg.db)})
\item ZEBRAFISH (\code{install.packages(org.Dr.eg.db)})
\item CHICKEN (\code{install.packages(org.Gg.eg.db)})
\item RHESUS (\code{install.packages(org.Mmu.eg.db)})
\item CHIMP (\code{install.packages(org.Pt.eg.db)})
\item RAT (\code{install.packages(org.Rn.eg.db)})
\item YEAST (\code{install.packages(org.Sc.sgd.db)})
\item PIG (\code{install.packages(org.Ss.eg.db)})
\item XENOPUS (\code{install.packages(org.Xl.eg.db)})
}}

\item{verbose}{(logical) \code{TRUE} (default) shows function messages}
}
\value{
(string) Name of the package for the given species
}
\description{
Given a species name, it checkes whether is supported, and
if supported, check whether the annotation package is installed.
}
\examples{
# Should return TRUE
artmsIsSpeciesSupported(species = "HUMAN")
artmsIsSpeciesSupported(species = "CHIMP")
}
\keyword{annotation,}
\keyword{species}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/otherApps.R
\name{artmsPhotonOutput}
\alias{artmsPhotonOutput}
\title{Generate PHOTON Input file}
\usage{
artmsPhotonOutput(inputFile, output_dir = ".", verbose = TRUE)
}
\arguments{
\item{inputFile}{(char) the \code{imputedL2fcExtended.txt} file name and location}

\item{output_dir}{(char) Name of the folder to output results
(Default: current. Recommended: "photon_input_files" or similar)}

\item{verbose}{(logical) \code{TRUE} (default) to show function messages}
}
\value{
Multiple output files (inputs of phosfate)
}
\description{
It takes as input the \code{imputedL2fcExtended.txt} results
generated by the \code{artmsAnalysisQuantifications()} function and generates
the \href{https://github.com/jdrudolph/photon}{PHOTON} input file.
Please, notice that the only species suported by PHOTON is humans.
}
\examples{
\dontrun{
artmsPhotonOutput(inputFile)
}
}
\keyword{files}
\keyword{generate,}
\keyword{outputs,}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{artms_data_ph_msstats_results}
\alias{artms_data_ph_msstats_results}
\title{MSstats results example}
\format{
A data frame resulting from running the latest version of MSstats
}
\usage{
artms_data_ph_msstats_results
}
\description{
Relative quantification results obtained running MSstats
on the available PH datasets (global analysis).
Changes in protein phosphorylation were quantified between two conditions
(check \code{artms_data_ph_contrast})
}
\keyword{datasets}
